@@ -58,10 +58,10 @@
 #include <string.h>
 #include <unistd.h>
 
-#include "vapi/vsl.h"
 #include "vapi/vsm.h"
 #include "vas.h"
 #include "vcli.h"
+#include "vdef.h"
 #include "vnum.h"
 #include "vtcp.h"
 
@@ -113,18 +113,18 @@ cli_sock(const char *T_arg, const char *S_arg)
 	if (status == CLIS_AUTH) {
 		if (S_arg == NULL) {
 			fprintf(stderr, "Authentication required\n");
-			AZ(close(sock));
+			closefd(&sock);
 			return(-1);
 		}
 		fd = open(S_arg, O_RDONLY);
 		if (fd < 0) {
 			fprintf(stderr, "Cannot open \"%s\": %s\n",
 			    S_arg, strerror(errno));
-			AZ(close(sock));
+			closefd(&sock);
 			return (-1);
 		}
 		VCLI_AuthResponse(fd, answer, buf);
-		AZ(close(fd));
+		closefd(&fd);
 		free(answer);
 
 		cli_write(sock, "auth ");
@@ -134,7 +134,7 @@ cli_sock(const char *T_arg, const char *S_arg)
 	}
 	if (status != CLIS_OK) {
 		fprintf(stderr, "Rejected %u\n%s\n", status, answer);
-		AZ(close(sock));
+		closefd(&sock);
 		free(answer);
 		return (-1);
 	}
@@ -144,7 +144,7 @@ cli_sock(const char *T_arg, const char *S_arg)
 	(void)VCLI_ReadResult(sock, &status, &answer, timeout);
 	if (status != CLIS_OK || strstr(answer, "PONG") == NULL) {
 		fprintf(stderr, "No pong received from server\n");
-		AZ(close(sock));
+		closefd(&sock);
 		free(answer);
 		return (-1);
 	}
@@ -185,7 +185,8 @@ do_args(int sock, int argc, char * const *argv)
  * to have a global variable.
  */
 static int _line_sock;
-static void send_line(char *l)
+static void __match_proto__()
+send_line(char *l)
 {
 	if (l) {
 		cli_write(_line_sock, l);
@@ -302,7 +303,7 @@ interactive(int sock)
 				RL_EXIT(1);
 			}
 
-			sprintf(buf, "%u\n", status);
+			bprintf(buf, "%u\n", status);
 			u = write(1, buf, strlen(buf));
 			if (answer) {
 				u = write(1, answer, strlen(answer));
@@ -352,7 +353,7 @@ pass(int sock)
 				RL_EXIT(1);
 			}
 
-			sprintf(buf, "%u\n", status);
+			bprintf(buf, "%u\n", status);
 			u = write(1, buf, strlen(buf));
 			if (answer) {
 				u = write(1, answer, strlen(answer));
@@ -388,42 +389,32 @@ usage(int status)
 }
 
 static int
-n_arg_sock(const char *n_arg)
+n_arg_sock(const char *n_arg, const char *t_arg)
 {
 	char *T_arg = NULL, *T_start = NULL;
 	char *S_arg = NULL;
-	struct VSM_data *vsm;
+	struct vsm *vsm;
 	char *p;
 	int sock;
-	struct VSM_fantom vt;
 
 	vsm = VSM_New();
 	AN(vsm);
-	if (VSM_n_Arg(vsm, n_arg) < 0) {
+	if (VSM_Arg(vsm, 'n', n_arg) < 0 ||
+	    VSM_Arg(vsm, 't', t_arg) < 0 ||
+	    VSM_Attach(vsm, STDERR_FILENO) < 0) {
 		fprintf(stderr, "%s\n", VSM_Error(vsm));
-		VSM_Delete(vsm);
-		return (-1);
-	}
-	if (VSM_Open(vsm)) {
-		fprintf(stderr, "%s\n", VSM_Error(vsm));
-		VSM_Delete(vsm);
+		VSM_Destroy(&vsm);
 		return (-1);
 	}
 
-	if (!VSM_Get(vsm, &vt, "Arg", "-T", "")) {
-		fprintf(stderr, "No -T arg in shared memory\n");
-		VSM_Delete(vsm);
+	T_start = T_arg = VSM_Dup(vsm, "Arg", "-T");
+	S_arg = VSM_Dup(vsm, "Arg", "-S");
+	VSM_Destroy(&vsm);
+
+	if (T_arg == NULL) {
+		fprintf(stderr, "No -T in shared memory\n");
 		return (-1);
 	}
-	AN(vt.b);
-	T_start = T_arg = strdup(vt.b);
-
-	if (VSM_Get(vsm, &vt, "Arg", "-S", "")) {
-		AN(vt.b);
-		S_arg = strdup(vt.b);
-	}
-
-	VSM_Delete(vsm);
 
 	sock = -1;
 	while (*T_arg) {
@@ -446,6 +437,7 @@ main(int argc, char * const *argv)
 	const char *T_arg = NULL;
 	const char *S_arg = NULL;
 	const char *n_arg = NULL;
+	const char *t_arg = NULL;
 	int opt, sock;
 
 	/*
@@ -471,9 +463,7 @@ main(int argc, char * const *argv)
 			T_arg = optarg;
 			break;
 		case 't':
-			timeout = VNUM(optarg);
-			if (isnan(timeout))
-				usage(1);
+			t_arg = optarg;
 			break;
 		default:
 			usage(1);
@@ -483,28 +473,23 @@ main(int argc, char * const *argv)
 	argc -= optind;
 	argv += optind;
 
-	if (n_arg != NULL) {
-		if (T_arg != NULL || S_arg != NULL) {
+	if (T_arg != NULL) {
+		if (n_arg != NULL)
 			usage(1);
-		}
-		sock = n_arg_sock(n_arg);
-	} else if (T_arg == NULL) {
-		sock = n_arg_sock("");
-	} else {
-		assert(T_arg != NULL);
 		sock = cli_sock(T_arg, S_arg);
+	} else {
+		if (S_arg != NULL)
+			usage(1);
+		sock = n_arg_sock(n_arg, t_arg);
 	}
 	if (sock < 0)
 		exit(2);
 
 	if (argc > 0)
 		do_args(sock, argc, argv);
-	else {
-		if (isatty(0)) {
-			interactive(sock);
-		} else {
-			pass(sock);
-		}
-	}
+	else if (isatty(0))
+		interactive(sock);
+	else
+		pass(sock);
 	exit(0);
 }
