@@ -33,14 +33,13 @@
 
 #include "config.h"
 
-#include <sys/types.h>
+#include "cache.h"
 
 #include <errno.h>
 #include <stdlib.h>
 #include <netinet/in.h>
 #include <netinet/tcp.h>
 
-#include "cache.h"
 #include "cache_transport.h"
 #include "cache_pool.h"
 #include "common/heritage.h"
@@ -69,6 +68,7 @@ struct wrk_accept {
 struct poolsock {
 	unsigned			magic;
 #define POOLSOCK_MAGIC			0x1b0a2d38
+	VTAILQ_ENTRY(poolsock)		list;
 	struct listen_sock		*lsock;
 	struct pool_task		task;
 	struct pool			*pool;
@@ -150,7 +150,7 @@ vca_tcp_opt_init(void)
 	for (n = 0; n < n_tcp_opts; n++) {
 		to = &tcp_opts[n];
 		if (to->ptr == NULL)
-			to->ptr = calloc(to->sz, 1);
+			to->ptr = calloc(1, to->sz);
 		AN(to->ptr);
 		if (!strcmp(to->strname, "SO_LINGER")) {
 			assert(to->sz == sizeof linger);
@@ -212,7 +212,7 @@ vca_tcp_opt_test(int sock)
 	for (n = 0; n < n_tcp_opts; n++) {
 		to = &tcp_opts[n];
 		to->need = 1;
-		ptr = calloc(to->sz, 1);
+		ptr = calloc(1, to->sz);
 		AN(ptr);
 		l = to->sz;
 		i = getsockopt(sock, to->level, to->optname, ptr, &l);
@@ -306,7 +306,7 @@ vca_make_session(struct worker *wrk, void *arg)
 	CAST_OBJ_NOTNULL(wa, arg, WRK_ACCEPT_MAGIC);
 
 	if (VTCP_blocking(wa->acceptsock)) {
-		AZ(close(wa->acceptsock));
+		closefd(&wa->acceptsock);
 		wrk->stats->sess_drop++;	// XXX Better counter ?
 		WS_Release(wrk->aws, 0);
 		return;
@@ -323,7 +323,7 @@ vca_make_session(struct worker *wrk, void *arg)
 		 */
 		vca_pace_bad();
 		(void)VTCP_nonblocking(wa->acceptsock);
-		AZ(close(wa->acceptsock));
+		closefd(&wa->acceptsock);
 		wrk->stats->sess_drop++;
 		WS_Release(wrk->aws, 0);
 		return;
@@ -358,7 +358,10 @@ vca_make_session(struct worker *wrk, void *arg)
 	VSL(SLT_Begin, sp->vxid, "sess 0 %s",
 	    wa->acceptlsock->transport->name);
 	VSL(SLT_SessOpen, sp->vxid, "%s %s %s %s %s %.6f %d",
-	    raddr, rport, wa->acceptlsock->name, laddr, lport,
+	    raddr, rport,
+	    wa->acceptlsock->name != NULL ?
+		wa->acceptlsock->name : wa->acceptlsock->endpoint,
+	    laddr, lport,
 	    sp->t_open, sp->fd);
 
 	WS_Release(wrk->aws, 0);
@@ -374,7 +377,7 @@ vca_make_session(struct worker *wrk, void *arg)
 
 	req = Req_New(wrk, sp);
 	CHECK_OBJ_NOTNULL(req, REQ_MAGIC);
-	req->htc->fd = sp->fd;
+	req->htc->rfd = &sp->fd;
 
 	SES_SetTransport(wrk, sp, req, wa->acceptlsock->transport);
 }
@@ -403,7 +406,7 @@ vca_accept_task(struct worker *wrk, void *arg)
 	while (!pool_accepting)
 		VTIM_sleep(.1);
 
-	while (1) {
+	while (!ps->pool->die) {
 		INIT_OBJ(&wa, WRK_ACCEPT_MAGIC);
 		wa.acceptlsock = ls;
 
@@ -415,6 +418,12 @@ vca_accept_task(struct worker *wrk, void *arg)
 				   &wa.acceptaddrlen);
 		} while (i < 0 && errno == EAGAIN);
 
+		if (i < 0 && ps->pool->die) {
+			VSL(SLT_Debug, 0, "XXX Accept thread dies %p", ps);
+			FREE_OBJ(ps);
+			return;
+		}
+
 		if (i < 0 && ls->sock == -2) {
 			/* Shut down in progress */
 			sleep(2);
@@ -447,16 +456,21 @@ vca_accept_task(struct worker *wrk, void *arg)
 
 		wa.acceptsock = i;
 
-		if (!Pool_Task_Arg(wrk, vca_make_session, &wa, sizeof wa)) {
+		if (!Pool_Task_Arg(wrk, TASK_QUEUE_VCA,
+		    vca_make_session, &wa, sizeof wa)) {
 			/*
 			 * We couldn't get another thread, so we will handle
 			 * the request in this worker thread, but first we
 			 * must reschedule the listening task so it will be
 			 * taken up by another thread again.
 			 */
-			AZ(Pool_Task(wrk->pool, &ps->task, TASK_QUEUE_VCA));
+			if (!ps->pool->die)
+				AZ(Pool_Task(wrk->pool, &ps->task,
+				    TASK_QUEUE_VCA));
 			return;
 		}
+		if (!ps->pool->die && DO_DEBUG(DBG_SLOW_ACCEPTOR))
+			VTIM_sleep(2.0);
 
 		/*
 		 * We were able to hand off, so release this threads VCL
@@ -485,19 +499,32 @@ VCA_NewPool(struct pool *pp)
 		ps->task.func = vca_accept_task;
 		ps->task.priv = ps;
 		ps->pool = pp;
+		VTAILQ_INSERT_TAIL(&pp->poolsocks, ps, list);
 		AZ(Pool_Task(pp, &ps->task, TASK_QUEUE_VCA));
 	}
 }
 
+void
+VCA_DestroyPool(struct pool *pp)
+{
+	struct poolsock *ps;
+
+	while (!VTAILQ_EMPTY(&pp->poolsocks)) {
+		ps = VTAILQ_FIRST(&pp->poolsocks);
+		VTAILQ_REMOVE(&pp->poolsocks, ps, list);
+	}
+}
+
 /*--------------------------------------------------------------------*/
 
-static void *
+static void * __match_proto__()
 vca_acct(void *arg)
 {
 	struct listen_sock *ls;
 	double t0, now;
 
 	THR_SetName("cache-acceptor");
+	THR_Init();
 	(void)arg;
 
 	(void)vca_tcp_opt_init();
@@ -542,12 +569,12 @@ vca_acct(void *arg)
 		now = VTIM_real();
 		VSC_C_main->uptime = (uint64_t)(now - t0);
 	}
-	NEEDLESS_RETURN(NULL);
+	NEEDLESS(return NULL);
 }
 
 /*--------------------------------------------------------------------*/
 
-static void
+static void __match_proto__(cli_func_t)
 ccf_start(struct cli *cli, const char * const *av, void *priv)
 {
 
@@ -560,7 +587,7 @@ ccf_start(struct cli *cli, const char * const *av, void *priv)
 
 /*--------------------------------------------------------------------*/
 
-static void
+static void __match_proto__(cli_func_t)
 ccf_listen_address(struct cli *cli, const char * const *av, void *priv)
 {
 	struct listen_sock *ls;
@@ -576,7 +603,7 @@ ccf_listen_address(struct cli *cli, const char * const *av, void *priv)
 	 * a race where varnishtest::client would attempt to connect(2)
 	 * before listen(2) has been called.
 	 */
-	while(!pool_accepting)
+	while (!pool_accepting)
 		VTIM_sleep(.1);
 
 	VTAILQ_FOREACH(ls, &heritage.socks, list) {
