@@ -106,6 +106,8 @@ extern int ban_shutdown;
 extern struct banhead_s ban_head;
 extern struct ban * volatile ban_start;
 extern pthread_cond_t	ban_lurker_cond;
+extern uint64_t bans_persisted_bytes;
+extern uint64_t bans_persisted_fragmentation;
 
 void ban_mark_completed(struct ban *b);
 unsigned ban_len(const uint8_t *banspec);
