@@ -389,7 +389,6 @@ vep_mark_pending(struct vep_state *vep, const char *p)
 	AN(vep->ver_p);
 	l = p - vep->ver_p;
 	assert(l > 0);
-	assert(l >= 0);
 	vep->crcp = crc32(vep->crcp, (const void *)vep->ver_p, l);
 	vep->ver_p = p;
 
@@ -480,7 +479,14 @@ vep_do_include(struct vep_state *vep, enum dowhat what)
 	if (l > 7 && !memcmp(p, "http://", 7)) {
 		h = p + 7;
 		p = strchr(h, '/');
-		AN(p);
+		if (p == NULL) {
+			vep_error(vep,
+			    "ESI 1.0 <esi:include> invalid src= URL");
+			vep->state = VEP_TAGERROR;
+			AZ(vep->attr_vsb);
+			VSB_destroy(&vep->include_src);
+			return;
+		}
 		Debug("HOST <%.*s> PATH <%s>\n", (int)(p-h),h, p);
 		VSB_printf(vep->vsb, "%c", VEC_INCL);
 		VSB_printf(vep->vsb, "Host: %.*s%c", (int)(p-h), h, 0);
@@ -497,7 +503,14 @@ vep_do_include(struct vep_state *vep, enum dowhat what)
 		    "ESI 1.0 <esi:include> https:// treated as http://");
 		h = p + 8;
 		p = strchr(h, '/');
-		AN(p);
+		if (p == NULL) {
+			vep_error(vep,
+			    "ESI 1.0 <esi:include> invalid src= URL");
+			vep->state = VEP_TAGERROR;
+			AZ(vep->attr_vsb);
+			VSB_destroy(&vep->include_src);
+			return;
+		}
 		VSB_printf(vep->vsb, "%c", VEC_INCL);
 		VSB_printf(vep->vsb, "Host: %.*s%c", (int)(p-h), h, 0);
 	} else if (*p == '/') {
@@ -933,6 +946,13 @@ VEP_Parse(struct vep_state *vep, const char *p, size_t l)
 			memcpy(vep->tag + vep->tag_i, p, i);
 			vm = vep_match(vep, vep->tag,
 			    vep->tag + vep->tag_i + i);
+			Debug("MB (%.*s) tag_i %d i %d = vm %p match %s\n",
+			    vep->tag_i + i, vep->tag,
+			    vep->tag_i,
+			    i,
+			    vm,
+			    vm ? vm->match : "(nil)");
+
 			if (vm == NULL) {
 				vep->tag_i += i;
 				p += i;
@@ -940,8 +960,11 @@ VEP_Parse(struct vep_state *vep, const char *p, size_t l)
 			} else {
 				vep->match_hit = vm;
 				vep->state = *vm->state;
-				if (vm->match != NULL)
-					p += strlen(vm->match) - vep->tag_i;
+				if (vm->match != NULL) {
+					i = strlen(vm->match);
+					if (i > vep->tag_i)
+						p += i - vep->tag_i;
+				}
 				vep->match = NULL;
 				vep->tag_i = 0;
 			}
