@@ -33,6 +33,7 @@
 #include "cache_director.h"
 #include "cache_filter.h"
 #include "hash/hash_slinger.h"
+#include "storage/storage.h"
 #include "vcl.h"
 #include "vtim.h"
 
@@ -46,7 +47,7 @@ static int
 vbf_allocobj(struct busyobj *bo, unsigned l)
 {
 	struct objcore *oc;
-	const char *storage_hint;
+	const struct stevedore *stv;
 	double lifetime;
 
 	CHECK_OBJ_NOTNULL(bo, BUSYOBJ_MAGIC);
@@ -56,16 +57,20 @@ vbf_allocobj(struct busyobj *bo, unsigned l)
 	lifetime = oc->ttl + oc->grace + oc->keep;
 
 	if (bo->uncacheable || lifetime < cache_param->shortlived)
-		storage_hint = TRANSIENT_STORAGE;
+		stv = stv_transient;
 	else
-		storage_hint = bo->storage_hint;
+		stv = bo->storage;
 
+	bo->storage = NULL;
 	bo->storage_hint = NULL;
 
-	if (STV_NewObject(bo->wrk, bo->fetch_objcore, storage_hint, l))
+	if (stv == NULL)
+		return (0);
+
+	if (STV_NewObject(bo->wrk, bo->fetch_objcore, stv, l))
 		return (1);
 
-	if (storage_hint != NULL && !strcmp(storage_hint, TRANSIENT_STORAGE))
+	if (stv == stv_transient)
 		return (0);
 
 	/*
@@ -77,8 +82,7 @@ vbf_allocobj(struct busyobj *bo, unsigned l)
 		oc->ttl = cache_param->shortlived;
 	oc->grace = 0.0;
 	oc->keep = 0.0;
-	return (STV_NewObject(bo->wrk, bo->fetch_objcore,
-	    TRANSIENT_STORAGE, l));
+	return (STV_NewObject(bo->wrk, bo->fetch_objcore, stv_transient, l));
 }
 
 /*--------------------------------------------------------------------
@@ -169,6 +173,7 @@ vbf_stp_mkbereq(struct worker *wrk, struct busyobj *bo)
 	CHECK_OBJ_NOTNULL(bo->req, REQ_MAGIC);
 
 	assert(bo->fetch_objcore->boc->state == BOS_INVALID);
+	AZ(bo->storage);
 	AZ(bo->storage_hint);
 
 	HTTP_Setup(bo->bereq0, bo->ws, bo->vsl, SLT_BereqMethod);
@@ -180,9 +185,12 @@ vbf_stp_mkbereq(struct worker *wrk, struct busyobj *bo)
 		http_ForceField(bo->bereq0, HTTP_HDR_PROTO, "HTTP/1.1");
 		if (cache_param->http_gzip_support)
 			http_ForceHeader(bo->bereq0, H_Accept_Encoding, "gzip");
-		http_CopyHome(bo->bereq0);
-	} else
+	} else {
 		AZ(bo->stale_oc);
+		if (bo->bereq0->protover > 11)
+			http_ForceField(bo->bereq0, HTTP_HDR_PROTO, "HTTP/1.1");
+	}
+	http_CopyHome(bo->bereq0);
 
 	if (bo->stale_oc != NULL &&
 	    ObjCheckFlag(bo->wrk, bo->stale_oc, OF_IMSCAND) &&
@@ -232,6 +240,7 @@ vbf_stp_retry(struct worker *wrk, struct busyobj *bo)
 	assert(bo->director_state == DIR_S_NULL);
 
 	/* reset other bo attributes - See VBO_GetBusyObj */
+	bo->storage = NULL;
 	bo->storage_hint = NULL;
 	bo->do_esi = 0;
 	bo->do_stream = 1;
@@ -260,8 +269,11 @@ vbf_stp_startfetch(struct worker *wrk, struct busyobj *bo)
 	CHECK_OBJ_NOTNULL(wrk, WORKER_MAGIC);
 	CHECK_OBJ_NOTNULL(bo, BUSYOBJ_MAGIC);
 
+	AZ(bo->storage);
 	AZ(bo->storage_hint);
 
+	bo->storage = STV_next();
+
 	if (bo->retries > 0)
 		http_Unset(bo->bereq, "\012X-Varnish:");
 
@@ -270,7 +282,7 @@ vbf_stp_startfetch(struct worker *wrk, struct busyobj *bo)
 	VCL_backend_fetch_method(bo->vcl, wrk, NULL, bo, NULL);
 
 	bo->uncacheable = bo->do_pass;
-	if (wrk->handling == VCL_RET_ABANDON)
+	if (wrk->handling == VCL_RET_ABANDON || wrk->handling == VCL_RET_FAIL)
 		return (F_STP_FAIL);
 
 	assert (wrk->handling == VCL_RET_FETCH);
@@ -432,7 +444,7 @@ vbf_stp_startfetch(struct worker *wrk, struct busyobj *bo)
 
 	VCL_backend_response_method(bo->vcl, wrk, NULL, bo, NULL);
 
-	if (wrk->handling == VCL_RET_ABANDON) {
+	if (wrk->handling == VCL_RET_ABANDON || wrk->handling == VCL_RET_FAIL) {
 		bo->htc->doclose = SC_RESP_CLOSE;
 		VDI_Finish(bo->wrk, bo);
 		return (F_STP_FAIL);
@@ -461,6 +473,11 @@ vbf_stp_startfetch(struct worker *wrk, struct busyobj *bo)
 
 	if (bo->do_esi)
 		bo->do_stream = 0;
+	if (wrk->handling == VCL_RET_PASS) {
+		bo->fetch_objcore->flags |= OC_F_HFP;
+		bo->uncacheable = 1;
+		wrk->handling = VCL_RET_DELIVER;
+	}
 	if (bo->do_pass || bo->uncacheable)
 		bo->fetch_objcore->flags |= OC_F_PASS;
 
@@ -472,8 +489,8 @@ vbf_stp_startfetch(struct worker *wrk, struct busyobj *bo)
 /*--------------------------------------------------------------------
  */
 
-static void
-vbf_fetch_body_helper(struct busyobj *bo)
+static enum fetch_step
+vbf_stp_fetchbody(struct worker *wrk, struct busyobj *bo)
 {
 	ssize_t l;
 	uint8_t *ptr;
@@ -499,9 +516,8 @@ vbf_fetch_body_helper(struct busyobj *bo)
 			 * objects to be created.
 			 */
 			AN(vfc->oc->flags & OC_F_PASS);
-			VSLb(vfc->wrk->vsl, SLT_FetchError,
+			VSLb(wrk->vsl, SLT_FetchError,
 			    "Pass delivery abandoned");
-			vfps = VFP_END;
 			bo->htc->doclose = SC_RX_BODY;
 			break;
 		}
@@ -525,13 +541,23 @@ vbf_fetch_body_helper(struct busyobj *bo)
 		}
 	} while (vfps == VFP_OK);
 
-	if (vfps == VFP_ERROR) {
-		AN(vfc->failed);
+	if (vfc->failed) {
 		(void)VFP_Error(vfc, "Fetch pipeline failed to process");
 		bo->htc->doclose = SC_RX_BODY;
+		VFP_Close(vfc);
+		VDI_Finish(wrk, bo);
+		if (!bo->do_stream) {
+			assert(bo->fetch_objcore->boc->state < BOS_STREAM);
+			// XXX: doclose = ?
+			return (F_STP_ERROR);
+		} else {
+			wrk->stats->fetch_failed++;
+			return (F_STP_FAIL);
+		}
 	}
 
-	ObjTrimStore(bo->wrk, vfc->oc);
+	ObjTrimStore(wrk, vfc->oc);
+	return (F_STP_FETCHEND);
 }
 
 /*--------------------------------------------------------------------
@@ -685,22 +711,19 @@ vbf_stp_fetch(struct worker *wrk, struct busyobj *bo)
 
 	if (bo->htc->body_status != BS_NONE) {
 		assert(bo->htc->body_status != BS_ERROR);
-		vbf_fetch_body_helper(bo);
+		return (F_STP_FETCHBODY);
 	}
+	AZ(bo->vfc->failed);
+	return (F_STP_FETCHEND);
+}
 
+static enum fetch_step
+vbf_stp_fetchend(struct worker *wrk, struct busyobj *bo)
+{
+
+	AZ(bo->vfc->failed);
 	VFP_Close(bo->vfc);
 
-	if (bo->vfc->failed) {
-		VDI_Finish(bo->wrk, bo);
-		if (!bo->do_stream) {
-			assert(bo->fetch_objcore->boc->state < BOS_STREAM);
-			// XXX: doclose = ?
-			return (F_STP_ERROR);
-		} else {
-			return (F_STP_FAIL);
-		}
-	}
-
 	AZ(ObjSetU64(wrk, bo->fetch_objcore, OA_LEN,
 	    bo->fetch_objcore->boc->len_so_far));
 
@@ -779,24 +802,10 @@ vbf_stp_condfetch(struct worker *wrk, struct busyobj *bo)
 		(void)VFP_Error(bo->vfc, "Template object failed");
 	if (bo->vfc->failed) {
 		VDI_Finish(bo->wrk, bo);
+		wrk->stats->fetch_failed++;
 		return (F_STP_FAIL);
 	}
-
-	AZ(ObjSetU64(wrk, bo->fetch_objcore, OA_LEN,
-	    bo->fetch_objcore->boc->len_so_far));
-
-	if (!bo->do_stream)
-		HSH_Unbusy(wrk, bo->fetch_objcore);
-
-	HSH_Kill(bo->stale_oc);
-
-	/* Recycle the backend connection before setting BOS_FINISHED to
-	   give predictable backend reuse behavior for varnishtest */
-	VDI_Finish(bo->wrk, bo);
-
-	ObjSetState(wrk, bo->fetch_objcore, BOS_FINISHED);
-	VSLb_ts_busyobj(bo, "BerespBody", W_TIM_real(wrk));
-	return (F_STP_DONE);
+	return (F_STP_FETCHEND);
 }
 
 /*--------------------------------------------------------------------
@@ -817,10 +826,12 @@ vbf_stp_error(struct worker *wrk, struct busyobj *bo)
 	AN(bo->fetch_objcore->flags & OC_F_BUSY);
 	assert(bo->director_state == DIR_S_NULL);
 
+	wrk->stats->fetch_failed++;
+
 	now = W_TIM_real(wrk);
 	VSLb_ts_busyobj(bo, "Error", now);
 
-	if(bo->fetch_objcore->stobj->stevedore != NULL)
+	if (bo->fetch_objcore->stobj->stevedore != NULL)
 		ObjFreeObj(bo->wrk, bo->fetch_objcore);
 
 	// XXX: reset all beresp flags ?
@@ -855,7 +866,7 @@ vbf_stp_error(struct worker *wrk, struct busyobj *bo)
 
 	AZ(VSB_finish(synth_body));
 
-	if (wrk->handling == VCL_RET_ABANDON) {
+	if (wrk->handling == VCL_RET_ABANDON || wrk->handling == VCL_RET_FAIL) {
 		VSB_destroy(&synth_body);
 		return (F_STP_FAIL);
 	}
@@ -888,6 +899,8 @@ vbf_stp_error(struct worker *wrk, struct busyobj *bo)
 		l = ll;
 		if (VFP_GetStorage(bo->vfc, &l, &ptr) != VFP_OK)
 			break;
+		if (l > ll)
+			l = ll;
 		memcpy(ptr, VSB_data(synth_body) + o, l);
 		VFP_Extend(bo->vfc, l);
 		ll -= l;
@@ -915,7 +928,6 @@ vbf_stp_fail(struct worker *wrk, const struct busyobj *bo)
 	if (!(bo->fetch_objcore->flags & OC_F_BUSY))
 		HSH_Kill(bo->fetch_objcore);
 	ObjSetState(wrk, bo->fetch_objcore, BOS_FAILED);
-	wrk->stats->fetch_failed++;
 	return (F_STP_DONE);
 }
 
@@ -926,7 +938,7 @@ static enum fetch_step
 vbf_stp_done(void)
 {
 	WRONG("Just plain wrong");
-	NEEDLESS_RETURN(F_STP_DONE);
+	NEEDLESS(return(F_STP_DONE));
 }
 
 static void __match_proto__(task_func_t)
@@ -961,13 +973,12 @@ vbf_fetch_thread(struct worker *wrk, void *priv)
 	while (stp != F_STP_DONE) {
 		CHECK_OBJ_NOTNULL(bo, BUSYOBJ_MAGIC);
 		assert(bo->fetch_objcore->boc->refcount >= 1);
-		switch(stp) {
+		switch (stp) {
 #define FETCH_STEP(l, U, arg)						\
 		case F_STP_##U:						\
 			stp = vbf_stp_##l arg;				\
 			break;
 #include "tbl/steps.h"
-#undef FETCH_STEP
 		default:
 			WRONG("Illegal fetch_step");
 		}
@@ -1012,20 +1023,28 @@ VBF_Fetch(struct worker *wrk, struct req *req, struct objcore *oc,
 	AN(oc->flags & OC_F_BUSY);
 	CHECK_OBJ_ORNULL(oldoc, OBJCORE_MAGIC);
 
-
-	switch(mode) {
-	case VBF_PASS:		how = "pass"; break;
-	case VBF_NORMAL:	how = "fetch"; break;
-	case VBF_BACKGROUND:	how = "bgfetch"; break;
-	default:		WRONG("Wrong fetch mode");
-	}
-
 	bo = VBO_GetBusyObj(wrk, req);
 	CHECK_OBJ_NOTNULL(bo, BUSYOBJ_MAGIC);
 
 	boc = HSH_RefBoc(oc);
 	CHECK_OBJ_NOTNULL(boc, BOC_MAGIC);
 
+	switch (mode) {
+	case VBF_PASS:
+		how = "pass";
+		bo->do_pass = 1;
+		break;
+	case VBF_NORMAL:
+		how = "fetch";
+		break;
+	case VBF_BACKGROUND:
+		how = "bgfetch";
+		bo->is_bgfetch = 1;
+		break;
+	default:
+		WRONG("Wrong fetch mode");
+	}
+
 	VSLb(bo->vsl, SLT_Begin, "bereq %u %s", VXID(req->vsl->wid), how);
 	VSLb(req->vsl, SLT_Link, "bereq %u %s", VXID(bo->vsl->wid), how);
 
@@ -1036,9 +1055,6 @@ VBF_Fetch(struct worker *wrk, struct req *req, struct objcore *oc,
 
 	AN(bo->vcl);
 
-	if (mode == VBF_PASS)
-		bo->do_pass = 1;
-
 	oc->boc->vary = req->vary_b;
 	req->vary_b = NULL;
 
@@ -1071,6 +1087,7 @@ VBF_Fetch(struct worker *wrk, struct req *req, struct objcore *oc,
 		bo = NULL; /* ref transferred to fetch thread */
 		if (mode == VBF_BACKGROUND) {
 			ObjWaitState(oc, BOS_REQ_DONE);
+			(void)VRB_Ignore(req);
 		} else {
 			ObjWaitState(oc, BOS_STREAM);
 			if (oc->boc->state == BOS_FAILED) {
