@@ -35,11 +35,13 @@
 
 #include "config.h"
 
+#include "cache.h"
+
 #include <errno.h>
 #include <stdlib.h>
 #include <stdio.h>
 
-#include "cache.h"
+#include "VSC_lck.h"
 
 struct ilck {
 	unsigned		magic;
@@ -48,7 +50,7 @@ struct ilck {
 	pthread_mutex_t		mtx;
 	pthread_t		owner;
 	const char		*w;
-	struct VSC_C_lck	*stat;
+	struct VSC_lck		*stat;
 };
 
 static pthread_mutexattr_t attr;
@@ -165,26 +167,29 @@ Lck__Trylock(struct lock *lck, const char *p, int l)
 	return (r);
 }
 
-void
-Lck__Assert(const struct lock *lck, int held)
+int
+Lck__Held(const struct lock *lck)
 {
 	struct ilck *ilck;
 
 	CAST_OBJ_NOTNULL(ilck, lck->priv, ILCK_MAGIC);
-	if (held) {
-		assert(ilck->held);
-		assert(pthread_equal(ilck->owner, pthread_self()));
-	} else {
-		AZ(ilck->held);
-		AZ(pthread_equal(ilck->owner, pthread_self()));
-	}
+	return (ilck->held);
+}
+
+int
+Lck__Owned(const struct lock *lck)
+{
+	struct ilck *ilck;
+
+	CAST_OBJ_NOTNULL(ilck, lck->priv, ILCK_MAGIC);
+	AN(ilck->held);
+	return (pthread_equal(ilck->owner, pthread_self()));
 }
 
 int __match_proto__()
 Lck_CondWait(pthread_cond_t *cond, struct lock *lck, double when)
 {
 	struct ilck *ilck;
-	int retval = 0;
 	struct timespec ts;
 	double t;
 
@@ -193,23 +198,25 @@ Lck_CondWait(pthread_cond_t *cond, struct lock *lck, double when)
 	assert(pthread_equal(ilck->owner, pthread_self()));
 	ilck->held = 0;
 	if (when == 0) {
-		AZ(pthread_cond_wait(cond, &ilck->mtx));
+		errno = pthread_cond_wait(cond, &ilck->mtx);
+		AZ(errno);
 	} else {
+		assert(when > 1e9);
 		ts.tv_nsec = (long)(modf(when, &t) * 1e9);
 		ts.tv_sec = (long)t;
-		retval = pthread_cond_timedwait(cond, &ilck->mtx, &ts);
-		assert(retval == 0 ||
-		    retval == ETIMEDOUT ||
-		    retval == EINTR);
+		errno = pthread_cond_timedwait(cond, &ilck->mtx, &ts);
+		assert(errno == 0 ||
+		    errno == ETIMEDOUT ||
+		    errno == EINTR);
 	}
 	AZ(ilck->held);
 	ilck->held = 1;
 	ilck->owner = pthread_self();
-	return (retval);
+	return (errno);
 }
 
 void
-Lck__New(struct lock *lck, struct VSC_C_lck *st, const char *w)
+Lck__New(struct lock *lck, struct VSC_lck *st, const char *w)
 {
 	struct ilck *ilck;
 
@@ -237,16 +244,20 @@ Lck_Delete(struct lock *lck)
 	FREE_OBJ(ilck);
 }
 
-struct VSC_C_lck *
+struct VSC_lck *
 Lck_CreateClass(const char *name)
 {
-	return(VSM_Alloc(sizeof(struct VSC_C_lck),
-	   VSC_CLASS, VSC_type_lck, name));
+	return(VSC_lck_New(name));
 }
 
-#define LOCK(nam) struct VSC_C_lck *lck_##nam;
+void
+Lck_DestroyClass(struct VSC_lck **vsclck)
+{
+	VSC_lck_Destroy(vsclck);
+}
+
+#define LOCK(nam) struct VSC_lck *lck_##nam;
 #include "tbl/locks.h"
-#undef LOCK
 
 void
 LCK_Init(void)
@@ -258,5 +269,4 @@ LCK_Init(void)
 #endif
 #define LOCK(nam)	lck_##nam = Lck_CreateClass(#nam);
 #include "tbl/locks.h"
-#undef LOCK
 }
