@@ -29,14 +29,17 @@
  */
 
 #include "config.h"
+
 #include <stddef.h>
+#include "cache.h"
+
 #include <stdio.h>
 #include <stdlib.h>
 
-#include "cache.h"
-
 #include "vtim.h"
 
+#include "VSC_mempool.h"
+
 struct memitem {
 	unsigned			magic;
 #define MEMITEM_MAGIC			0x42e55401
@@ -57,7 +60,7 @@ struct mempool {
 	volatile struct poolparam	*param;
 	volatile unsigned		*cur_size;
 	uint64_t			live;
-	struct VSC_C_mempool		*vsc;
+	struct VSC_mempool		*vsc;
 	unsigned			n_pool;
 	pthread_t			thread;
 	double				t_now;
@@ -75,7 +78,7 @@ mpl_alloc(const struct mempool *mpl)
 
 	CHECK_OBJ_NOTNULL(mpl, MEMPOOL_MAGIC);
 	tsz = *mpl->cur_size;
-	mi = calloc(tsz, 1);
+	mi = calloc(1, tsz);
 	AN(mi);
 	mi->magic = MEMITEM_MAGIC;
 	mi->size = tsz;
@@ -101,6 +104,7 @@ mpl_guard(void *priv)
 
 	CAST_OBJ_NOTNULL(mpl, priv, MEMPOOL_MAGIC);
 	THR_SetName(mpl->name);
+	THR_Init();
 	mpl_slp = 0.15;	// random
 	while (1) {
 		VTIM_sleep(mpl_slp);
@@ -157,7 +161,7 @@ mpl_guard(void *priv)
 				FREE_OBJ(mi);
 				mi = NULL;
 			}
-			VSM_Free(mpl->vsc);
+			VSC_mempool_Destroy(&mpl->vsc);
 			Lck_Unlock(&mpl->mtx);
 			Lck_Delete(&mpl->mtx);
 			FREE_OBJ(mpl);
@@ -235,8 +239,7 @@ MPL_New(const char *name,
 	VTAILQ_INIT(&mpl->surplus);
 	Lck_New(&mpl->mtx, lck_mempool);
 	/* XXX: prealloc min_pool */
-	mpl->vsc = VSM_Alloc(sizeof *mpl->vsc,
-	    VSC_CLASS, VSC_type_mempool, mpl->name + 4);
+	mpl->vsc = VSC_mempool_New(mpl->name + 4);
 	AN(mpl->vsc);
 	AZ(pthread_create(&mpl->thread, NULL, mpl_guard, mpl));
 	AZ(pthread_detach(mpl->thread));
@@ -335,7 +338,7 @@ MPL_Free(struct mempool *mpl, void *item)
 }
 
 void
-MPL_AssertSane(void *item)
+MPL_AssertSane(const void *item)
 {
 	struct memitem *mi;
 	mi = (void*)((uintptr_t)item - sizeof(*mi));
