@@ -29,16 +29,14 @@
 
 #include "config.h"
 
-#ifndef HAVE_EXECINFO_H
-#include "compat/execinfo.h"
-#else
 #include <execinfo.h>
-#endif
-
 #include <errno.h>
+#include <stdio.h>
 #include <stdlib.h>
 #include <signal.h>
 
+#include "vtim.h"
+
 #include "cache.h"
 #include "cache_transport.h"
 
@@ -60,7 +58,7 @@
  */
 
 static struct vsb pan_vsb_storage, *pan_vsb;
-static pthread_mutex_t panicstr_mtx = PTHREAD_MUTEX_INITIALIZER;
+static pthread_mutex_t panicstr_mtx;
 
 static void pan_sess(struct vsb *, const struct sess *);
 
@@ -69,10 +67,9 @@ static void pan_sess(struct vsb *, const struct sess *);
 const char *
 body_status_2str(enum body_status e)
 {
-	switch(e) {
+	switch (e) {
 #define BODYSTATUS(U,l)	case BS_##U: return (#l);
 #include "tbl/body_status.h"
-#undef BODYSTATUS
 	default:
 		return ("?");
 	}
@@ -86,7 +83,6 @@ reqbody_status_2str(enum req_body_state_e e)
 	switch (e) {
 #define REQ_BODY(U) case REQ_BODY_##U: return("R_BODY_" #U);
 #include "tbl/req_body.h"
-#undef REQ_BODY
 	default:
 		return("?");
 	}
@@ -100,7 +96,6 @@ boc_state_2str(enum boc_state_e e)
 	switch (e) {
 #define BOC_STATE(U,l) case BOS_##U: return(#l);
 #include "tbl/boc_state.h"
-#undef BOC_STATE
 	default:
 		return ("?");
 	}
@@ -116,7 +111,6 @@ sess_close_2str(enum sess_close sc, int want_desc)
 #define SESS_CLOSE(nm, s, err, desc)			\
 	case SC_##nm: return(want_desc ? desc : #nm);
 #include "tbl/sess_close.h"
-#undef SESS_CLOSE
 
 	default:		return(want_desc ? "(invalid)" : "INVALID");
 	}
@@ -128,11 +122,15 @@ sess_close_2str(enum sess_close sc, int want_desc)
 static const void *already_list[N_ALREADY];
 static int already_idx;
 
-static int
-pan_already(struct vsb *vsb, const void *ptr)
+int
+PAN_already(struct vsb *vsb, const void *ptr)
 {
 	int i;
 
+	if (ptr == NULL) {
+		VSB_printf(vsb, "},\n");
+		return (1);
+	}
 	for (i = 0; i < already_idx; i++) {
 		if (already_list[i] == ptr) {
 			VSB_printf(vsb, "  [Already dumped, see above]\n");
@@ -145,7 +143,6 @@ pan_already(struct vsb *vsb, const void *ptr)
 	return (0);
 }
 
-
 /*--------------------------------------------------------------------*/
 
 static void
@@ -153,7 +150,7 @@ pan_ws(struct vsb *vsb, const struct ws *ws)
 {
 
 	VSB_printf(vsb, "ws = %p {\n", ws);
-	if (pan_already(vsb, ws))
+	if (PAN_already(vsb, ws))
 		return;
 	VSB_indent(vsb, 2);
 	PAN_CheckMagic(vsb, ws, WS_MAGIC);
@@ -161,15 +158,15 @@ pan_ws(struct vsb *vsb, const struct ws *ws)
 		VSB_printf(vsb, "OVERFLOWED ");
 	VSB_printf(vsb, "id = \"%s\",\n", ws->id);
 	VSB_printf(vsb, "{s, f, r, e} = {%p", ws->s);
-	if (ws->f > ws->s)
+	if (ws->f >= ws->s)
 		VSB_printf(vsb, ", +%ld", (long) (ws->f - ws->s));
 	else
 		VSB_printf(vsb, ", %p", ws->f);
-	if (ws->r > ws->s)
+	if (ws->r >= ws->s)
 		VSB_printf(vsb, ", +%ld", (long) (ws->r - ws->s));
 	else
 		VSB_printf(vsb, ", %p", ws->r);
-	if (ws->e > ws->s)
+	if (ws->e >= ws->s)
 		VSB_printf(vsb, ", +%ld", (long) (ws->e - ws->s));
 	else
 		VSB_printf(vsb, ", %p", ws->e);
@@ -185,13 +182,14 @@ pan_htc(struct vsb *vsb, const struct http_conn *htc)
 {
 
 	VSB_printf(vsb, "http_conn = %p {\n", htc);
-	if (pan_already(vsb, htc))
+	if (PAN_already(vsb, htc))
 		return;
 	VSB_indent(vsb, 2);
 	PAN_CheckMagic(vsb, htc, HTTP_CONN_MAGIC);
-	VSB_printf(vsb, "fd = %d,\n", htc->fd);
+	if (htc->rfd != NULL)
+		VSB_printf(vsb, "fd = %d (@%p),\n", *htc->rfd, htc->rfd);
 	VSB_printf(vsb, "doclose = %s,\n", sess_close_2str(htc->doclose, 0));
-	VSB_printf(vsb, "ws = %p,\n", htc->ws);
+	pan_ws(vsb, htc->ws);
 	VSB_printf(vsb, "{rxbuf_b, rxbuf_e} = {%p, %p},\n",
 	    htc->rxbuf_b, htc->rxbuf_e);
 	VSB_printf(vsb, "{pipeline_b, pipeline_e} = {%p, %p},\n",
@@ -216,11 +214,11 @@ pan_http(struct vsb *vsb, const char *id, const struct http *h)
 	int i;
 
 	VSB_printf(vsb, "http[%s] = %p {\n", id, h);
-	if (pan_already(vsb, h))
+	if (PAN_already(vsb, h))
 		return;
 	VSB_indent(vsb, 2);
 	PAN_CheckMagic(vsb, h, HTTP_MAGIC);
-	VSB_printf(vsb, "ws[%s] = %p,\n", h->ws ? h->ws->id : "", h->ws);
+	pan_ws(vsb, h->ws);
 	VSB_printf(vsb, "hdrs {\n");
 	VSB_indent(vsb, 2);
 	for (i = 0; i < h->nhd; ++i) {
@@ -240,7 +238,7 @@ static void
 pan_boc(struct vsb *vsb, const struct boc *boc)
 {
 	VSB_printf(vsb, "boc = %p {\n", boc);
-	if (pan_already(vsb, boc))
+	if (PAN_already(vsb, boc))
 		return;
 	VSB_indent(vsb, 2);
 	PAN_CheckMagic(vsb, boc, BOC_MAGIC);
@@ -260,28 +258,30 @@ pan_objcore(struct vsb *vsb, const char *typ, const struct objcore *oc)
 	const char *p;
 
 	VSB_printf(vsb, "objcore[%s] = %p {\n", typ, oc);
-	if (pan_already(vsb, oc))
+	if (PAN_already(vsb, oc))
 		return;
 	VSB_indent(vsb, 2);
 	PAN_CheckMagic(vsb, oc, OBJCORE_MAGIC);
 	VSB_printf(vsb, "refcnt = %d,\n", oc->refcnt);
 	VSB_printf(vsb, "flags = {");
+
+/*lint -save -esym(438,p) -esym(838,p) -e539 */
 	p = "";
 #define OC_FLAG(U, l, v) \
 	if (oc->flags & v) { VSB_printf(vsb, "%s" #l, p); p = ", "; }
 #include "tbl/oc_flags.h"
-#undef OC_FLAG
 	VSB_printf(vsb, "},\n");
 	VSB_printf(vsb, "exp_flags = {");
 	p = "";
 #define OC_EXP_FLAG(U, l, v) \
 	if (oc->exp_flags & v) { VSB_printf(vsb, "%s" #l, p); p = ", "; }
 #include "tbl/oc_exp_flags.h"
-#undef OC_EXP_FLAG
+/*lint -restore */
 	VSB_printf(vsb, "},\n");
+
 	if (oc->boc != NULL)
 		pan_boc(vsb, oc->boc);
-	VSB_printf(vsb, "exp = {%f, %f, %f, %f}\n",
+	VSB_printf(vsb, "exp = {%f, %f, %f, %f},\n",
 	    oc->t_origin, oc->ttl, oc->grace, oc->keep);
 	VSB_printf(vsb, "objhead = %p,\n", oc->objhead);
 	VSB_printf(vsb, "stevedore = %p", oc->stobj->stevedore);
@@ -313,7 +313,7 @@ pan_wrk(struct vsb *vsb, const struct worker *wrk)
 	const char *p;
 
 	VSB_printf(vsb, "worker = %p {\n", wrk);
-	if (pan_already(vsb, wrk))
+	if (PAN_already(vsb, wrk))
 		return;
 	VSB_indent(vsb, 2);
 	PAN_CheckMagic(vsb, wrk, WORKER_MAGIC);
@@ -363,7 +363,7 @@ pan_busyobj(struct vsb *vsb, const struct busyobj *bo)
 	const char *p;
 
 	VSB_printf(vsb, "busyobj = %p {\n", bo);
-	if (pan_already(vsb, bo))
+	if (PAN_already(vsb, bo))
 		return;
 	VSB_indent(vsb, 2);
 	PAN_CheckMagic(vsb, bo, BUSYOBJ_MAGIC);
@@ -372,12 +372,11 @@ pan_busyobj(struct vsb *vsb, const struct busyobj *bo)
 	VSB_printf(vsb, "failed = %d, ", bo->vfc->failed);
 	VSB_printf(vsb, "flags = {");
 	p = "";
-	/*lint -save -esym(438,p) */
+/*lint -save -esym(438,p) -e539 */
 #define BO_FLAG(l, r, w, d) \
-	if(bo->l) { VSB_printf(vsb, "%s" #l, p); p = ", "; }
+	if (bo->l) { VSB_printf(vsb, "%s" #l, p); p = ", "; }
 #include "tbl/bo_flags.h"
-#undef BO_FLAG
-	/*lint -restore */
+/*lint -restore */
 	VSB_printf(vsb, "},\n");
 
 	if (VALID_OBJ(bo->htc, HTTP_CONN_MAGIC))
@@ -405,6 +404,7 @@ pan_busyobj(struct vsb *vsb, const struct busyobj *bo)
 	if (bo->stale_oc)
 		pan_objcore(vsb, "ims", bo->stale_oc);
 	VCL_Panic(vsb, bo->vcl);
+	VMOD_Panic(vsb);
 	VSB_indent(vsb, -2);
 	VSB_printf(vsb, "},\n");
 }
@@ -418,7 +418,7 @@ pan_req(struct vsb *vsb, const struct req *req)
 	const struct transport *xp;
 
 	VSB_printf(vsb, "req = %p {\n", req);
-	if (pan_already(vsb, req))
+	if (PAN_already(vsb, req))
 		return;
 	VSB_indent(vsb, 2);
 	PAN_CheckMagic(vsb, req, REQ_MAGIC);
@@ -437,7 +437,6 @@ pan_req(struct vsb *vsb, const struct req *req)
 	switch (req->req_step) {
 #define REQ_STEP(l, u, arg) case R_STP_##u: stp = "R_STP_" #u; break;
 #include "tbl/steps.h"
-#undef REQ_STEP
 		default: stp = NULL;
 	}
 	if (stp != NULL)
@@ -470,6 +469,7 @@ pan_req(struct vsb *vsb, const struct req *req)
 		pan_http(vsb, "resp", req->resp);
 
 	VCL_Panic(vsb, req->vcl);
+	VMOD_Panic(vsb);
 
 	if (req->body_oc != NULL)
 		pan_objcore(vsb, "BODY", req->body_oc);
@@ -478,9 +478,8 @@ pan_req(struct vsb *vsb, const struct req *req)
 
 	VSB_printf(vsb, "flags = {\n");
 	VSB_indent(vsb, 2);
-#define REQ_FLAG(l, r, w, d) if(req->l) VSB_printf(vsb, #l ",\n");
+#define REQ_FLAG(l, r, w, d) if (req->l) VSB_printf(vsb, #l ",\n");
 #include "tbl/req_flags.h"
-#undef REQ_FLAG
 	VSB_indent(vsb, -2);
 	VSB_printf(vsb, "},\n");
 
@@ -498,7 +497,7 @@ pan_sess(struct vsb *vsb, const struct sess *sp)
 	const struct transport *xp;
 
 	VSB_printf(vsb, "sp = %p {\n", sp);
-	if (pan_already(vsb, sp))
+	if (PAN_already(vsb, sp))
 		return;
 	VSB_indent(vsb, 2);
 	PAN_CheckMagic(vsb, sp, SESS_MAGIC);
@@ -516,9 +515,12 @@ pan_sess(struct vsb *vsb, const struct sess *sp)
 		VSB_indent(vsb, -2);
 		VSB_printf(vsb, "}");
 	}
+	VSB_printf(vsb, "\n");
 	ci = SES_Get_String_Attr(sp, SA_CLIENT_IP);
 	cp = SES_Get_String_Attr(sp, SA_CLIENT_PORT);
-	VSB_printf(vsb, "\nclient = %s %s,\n", ci, cp);
+	VSB_printf(vsb, "client = %s %s,\n", ci, cp);
+
+	pan_privs(vsb, sp->privs);
 
 	VSB_indent(vsb, -2);
 	VSB_printf(vsb, "},\n");
@@ -536,6 +538,7 @@ pan_backtrace(struct vsb *vsb)
 	size_t i;
 	char **strings;
 	char *p;
+	char buf[32];
 
 	size = backtrace (array, BACKTRACE_LEVELS);
 	if (size > BACKTRACE_LEVELS) {
@@ -545,18 +548,21 @@ pan_backtrace(struct vsb *vsb)
 	VSB_printf(vsb, "Backtrace:\n");
 	VSB_indent(vsb, 2);
 	for (i = 0; i < size; i++) {
-		if (Symbol_Lookup(vsb, array[i]) < 0) {
-			strings = backtrace_symbols(&array[i], 1);
-			if (strings == NULL || strings[0] == NULL) {
-				VSB_printf(vsb, "%p: (?)", array[i]);
-			} else {
-				p = strrchr(strings[0], '/');
-				if (p == NULL)
-					p = strings[0];
-				else
+		bprintf(buf, "%p", array[i]);
+		VSB_printf(vsb, "%s: ", buf);
+		strings = backtrace_symbols(&array[i], 1);
+		if (strings == NULL || strings[0] == NULL) {
+			VSB_printf(vsb, "(?)");
+		} else {
+			p = strings[0];
+			if (!memcmp(buf, p, strlen(buf))) {
+				p += strlen(buf);
+				if (*p == ':')
+					p++;
+				while (*p == ' ')
 					p++;
-				VSB_printf(vsb, "%p: %s", array[i], p);
 			}
+			VSB_printf(vsb, "%s", p);
 		}
 		VSB_printf (vsb, "\n");
 	}
@@ -591,7 +597,7 @@ pan_ic(const char *func, const char *file, int line, const char *cond,
 	   desired effect */
 	(void)sigaction(SIGABRT, &sa, NULL);
 
-	switch(kind) {
+	switch (kind) {
 	case VAS_WRONG:
 		VSB_printf(pan_vsb,
 		    "Wrong turn at %s:%d:\n%s\n", file, line, cond);
@@ -603,7 +609,7 @@ pan_ic(const char *func, const char *file, int line, const char *cond,
 	case VAS_MISSING:
 		VSB_printf(pan_vsb,
 		    "Missing errorhandling code in %s(), %s line %d:\n"
-		    "  Condition(%s) not true.",
+		    "  Condition(%s) not true.\n",
 		    func, file, line, cond);
 		break;
 	case VAS_INCOMPLETE:
@@ -619,6 +625,15 @@ pan_ic(const char *func, const char *file, int line, const char *cond,
 		    func, file, line, cond);
 		break;
 	}
+	VSB_printf(pan_vsb, "version = %s, vrt api = %u.%u\n",
+	    VCS_version, VRT_MAJOR_VERSION, VRT_MINOR_VERSION);
+	VSB_printf(pan_vsb, "ident = %s,%s\n",
+	    VSB_data(vident) + 1, Waiter_GetName());
+	VSB_printf(pan_vsb, "now = %f (mono), %f (real)\n",
+	    VTIM_mono(), VTIM_real());
+
+	pan_backtrace(pan_vsb);
+
 	if (err)
 		VSB_printf(pan_vsb, "errno = %d (%s)\n", err, strerror(err));
 
@@ -626,23 +641,20 @@ pan_ic(const char *func, const char *file, int line, const char *cond,
 	if (q != NULL)
 		VSB_printf(pan_vsb, "thread = (%s)\n", q);
 
-	VSB_printf(pan_vsb, "version = %s\n", VCS_version);
-	VSB_printf(pan_vsb, "ident = %s,%s\n",
-	    VSB_data(vident) + 1, Waiter_GetName());
-
-	pan_backtrace(pan_vsb);
-
 	if (!FEATURE(FEATURE_SHORT_PANIC)) {
 		req = THR_GetRequest();
-		if (req != NULL) {
-			pan_req(pan_vsb, req);
+		VSB_cat(pan_vsb, "thr.");
+		pan_req(pan_vsb, req);
+		if (req != NULL)
 			VSL_Flush(req->vsl, 0);
-		}
 		bo = THR_GetBusyobj();
-		if (bo != NULL) {
-			pan_busyobj(pan_vsb, bo);
+		VSB_cat(pan_vsb, "thr.");
+		pan_busyobj(pan_vsb, bo);
+		if (bo != NULL)
 			VSL_Flush(bo->vsl, 0);
-		}
+	} else {
+		VSB_printf(pan_vsb,
+		    "Feature short panic supressed details.\n");
 	}
 	VSB_printf(pan_vsb, "\n");
 	VSB_putc(pan_vsb, '\0');	/* NUL termination */
@@ -678,11 +690,16 @@ void
 PAN_Init(void)
 {
 
-	VAS_Fail = pan_ic;
+	AZ(pthread_mutex_init(&panicstr_mtx, NULL));
+	VAS_Fail_Func = pan_ic;
 	pan_vsb = &pan_vsb_storage;
 	AN(heritage.panic_str);
 	AN(heritage.panic_str_len);
 	AN(VSB_new(pan_vsb, heritage.panic_str, heritage.panic_str_len,
 	    VSB_FIXEDLEN));
+	VSB_printf(pan_vsb, "This is a test\n");
+	AZ(VSB_finish(pan_vsb));
+	VSB_clear(pan_vsb);
+	heritage.panic_str[0] = '\0';
 	CLI_AddFuncs(debug_cmds);
 }
