@@ -1,6 +1,6 @@
 /*-
  * Copyright (c) 2006 Verdens Gang AS
- * Copyright (c) 2006-2015 Varnish Software AS
+ * Copyright (c) 2006-2017 Varnish Software AS
  * All rights reserved.
  *
  * Author: Poul-Henning Kamp <phk@phk.freebsd.dk>
@@ -44,10 +44,64 @@
 #include "cache_transport.h"
 
 #include "hash/hash_slinger.h"
+#include "storage/storage.h"
 #include "vcl.h"
 #include "vsha256.h"
 #include "vtim.h"
 
+/*--------------------------------------------------------------------
+ * Handle "Expect:" and "Connection:" on incoming request
+ */
+
+static enum req_fsm_nxt
+cnt_transport(struct worker *wrk, struct req *req)
+{
+	const char *p;
+
+	CHECK_OBJ_NOTNULL(wrk, WORKER_MAGIC);
+	CHECK_OBJ_NOTNULL(req, REQ_MAGIC);
+	CHECK_OBJ_NOTNULL(req->http, HTTP_MAGIC);
+	CHECK_OBJ_NOTNULL(req->transport, TRANSPORT_MAGIC);
+	assert(req->req_body_status != REQ_BODY_INIT);
+	AN(req->transport->minimal_response);
+
+	if (http_GetHdr(req->http, H_Expect, &p)) {
+		if (strcasecmp(p, "100-continue")) {
+			req->doclose = SC_RX_JUNK;
+			(void)req->transport->minimal_response(req, 417);
+			wrk->stats->client_req_417++;
+			return (REQ_FSM_DONE);
+		}
+		if (req->http->protover >= 11 &&
+		    req->htc->pipeline_b == NULL)	// XXX: HTTP1 vs 2 ?
+			req->want100cont = 1;
+		http_Unset(req->http, H_Expect);
+	}
+
+	wrk->stats->client_req++;
+
+	AZ(req->err_code);
+
+	req->doclose = http_DoConnection(req->http);
+	if (req->doclose == SC_RX_BAD) {
+		(void)req->transport->minimal_response(req, 400);
+		return (REQ_FSM_DONE);
+	}
+
+	if (req->req_body_status < REQ_BODY_TAKEN) {
+		AN(req->transport->req_body != NULL);
+		VFP_Setup(req->vfc);
+		req->vfc->http = req->http;
+		req->vfc->wrk = wrk;
+		req->transport->req_body(req);
+	}
+
+	req->ws_req = WS_Snapshot(req->ws);
+	HTTP_Copy(req->http0, req->http);	// For ESI & restart
+	req->req_step = R_STP_RECV;
+	return (REQ_FSM_MORE);
+}
+
 /*--------------------------------------------------------------------
  * Deliver an object to client
  */
@@ -92,9 +146,9 @@ cnt_deliver(struct worker *wrk, struct req *req)
 	 * age. Truncate to zero in that case).
 	 */
 	http_PrintfHeader(req->resp, "Age: %.0f",
-	    fmax(0., req->t_prev - req->objcore->t_origin));
+	    floor(fmax(0., req->t_prev - req->objcore->t_origin)));
 
-	http_SetHeader(req->resp, "Via: 1.1 varnish (Varnish/5.0)");
+	http_SetHeader(req->resp, "Via: 1.1 varnish (Varnish/5.2)");
 
 	if (cache_param->http_gzip_support &&
 	    ObjCheckFlag(req->wrk, req->objcore, OF_GZIPED) &&
@@ -116,6 +170,9 @@ cnt_deliver(struct worker *wrk, struct req *req)
 		case VCL_RET_RESTART:
 			req->req_step = R_STP_RESTART;
 			break;
+		case VCL_RET_FAIL:
+			req->req_step = R_STP_VCLFAIL;
+			break;
 		case VCL_RET_SYNTH:
 			req->req_step = R_STP_SYNTH;
 			break;
@@ -128,8 +185,7 @@ cnt_deliver(struct worker *wrk, struct req *req)
 
 	assert(wrk->handling == VCL_RET_DELIVER);
 
-	if (!(req->objcore->flags & OC_F_PASS)
-	    && req->esi_level == 0
+	if (req->esi_level == 0
 	    && http_IsStatus(req->resp, 200)
 	    && req->http->conds && RFC2616_Do_Cond(req))
 		http_PutResponse(req->resp, "HTTP/1.1", 304, NULL);
@@ -138,6 +194,26 @@ cnt_deliver(struct worker *wrk, struct req *req)
 	return (REQ_FSM_MORE);
 }
 
+/*--------------------------------------------------------------------
+ * VCL failed, die horribly
+ */
+
+static enum req_fsm_nxt
+cnt_vclfail(const struct worker *wrk, struct req *req)
+{
+
+	CHECK_OBJ_NOTNULL(wrk, WORKER_MAGIC);
+	CHECK_OBJ_NOTNULL(req, REQ_MAGIC);
+
+	HTTP_Copy(req->http, req->http0);
+	WS_Reset(req->ws, req->ws_req);
+	req->err_code = 503;
+	req->err_reason = "VCL failed";
+	req->req_step = R_STP_SYNTH;
+	req->doclose = SC_VCL_FAILURE;
+	return (REQ_FSM_MORE);
+}
+
 /*--------------------------------------------------------------------
  * Emit a synthetic response
  */
@@ -159,7 +235,7 @@ cnt_synth(struct worker *wrk, struct req *req)
 	now = W_TIM_real(wrk);
 	VSLb_ts_req(req, "Process", now);
 
-	if (req->err_code < 100 || req->err_code > 999)
+	if (req->err_code < 100)
 		req->err_code = 501;
 
 	HTTP_Setup(req->resp, req->ws, req->vsl, SLT_RespMethod);
@@ -169,6 +245,15 @@ cnt_synth(struct worker *wrk, struct req *req)
 	http_PrintfHeader(req->resp, "X-Varnish: %u", VXID(req->vsl->wid));
 	http_PutResponse(h, "HTTP/1.1", req->err_code, req->err_reason);
 
+	/*
+	 * For late 100-continue, we suggest to VCL to close the connection to
+	 * neither send a 100-continue nor drain-read the request. But VCL has
+	 * the option to veto by removing Connection: close
+	 */
+	if (req->want100cont) {
+		http_SetHeader(h, "Connection: close");
+	}
+
 	synth_body = VSB_new_auto();
 	AN(synth_body);
 
@@ -176,14 +261,19 @@ cnt_synth(struct worker *wrk, struct req *req)
 
 	AZ(VSB_finish(synth_body));
 
-	http_Unset(h, H_Content_Length);
-	http_PrintfHeader(req->resp, "Content-Length: %zd",
-	    VSB_len(synth_body));
-
-	/* Discard any lingering request body before delivery */
-	(void)VRB_Ignore(req);
+	if (wrk->handling == VCL_RET_FAIL) {
+		VSB_destroy(&synth_body);
+		req->doclose = SC_VCL_FAILURE;
+		VSLb_ts_req(req, "Resp", W_TIM_real(wrk));
+		http_Teardown(req->resp);
+		return (REQ_FSM_DONE);
+	}
 
 	if (wrk->handling == VCL_RET_RESTART) {
+		/*
+		 * XXX: Should we reset req->doclose = SC_VCL_FAILURE
+		 * XXX: If so, to what ?
+		 */
 		HTTP_Setup(h, req->ws, req->vsl, SLT_RespMethod);
 		VSB_destroy(&synth_body);
 		req->req_step = R_STP_RESTART;
@@ -191,10 +281,20 @@ cnt_synth(struct worker *wrk, struct req *req)
 	}
 	assert(wrk->handling == VCL_RET_DELIVER);
 
+	http_Unset(h, H_Content_Length);
+	http_PrintfHeader(req->resp, "Content-Length: %zd",
+	    VSB_len(synth_body));
+
+	if (!req->doclose && http_HdrIs(req->resp, H_Connection, "close"))
+		req->doclose = SC_RESP_CLOSE;
+
+	/* Discard any lingering request body before delivery */
+	(void)VRB_Ignore(req);
+
 	req->objcore = HSH_Private(wrk);
 	CHECK_OBJ_NOTNULL(req->objcore, OBJCORE_MAGIC);
 	szl = -1;
-	if (STV_NewObject(wrk, req->objcore, TRANSIENT_STORAGE, 1024)) {
+	if (STV_NewObject(wrk, req->objcore, stv_transient, 1024)) {
 		szl = VSB_len(synth_body);
 		assert(szl >= 0);
 		sz = szl;
@@ -391,10 +491,13 @@ cnt_lookup(struct worker *wrk, struct req *req)
 	if (lr == HSH_MISS) {
 		/* Found nothing */
 		AZ(oc);
-		AN(busy);
-		AN(busy->flags & OC_F_BUSY);
-		req->objcore = busy;
-		req->req_step = R_STP_MISS;
+		if (busy != NULL) {
+			AN(busy->flags & OC_F_BUSY);
+			req->objcore = busy;
+			req->req_step = R_STP_MISS;
+		} else {
+			req->req_step = R_STP_PASS;
+		}
 		return (REQ_FSM_MORE);
 	}
 
@@ -403,7 +506,11 @@ cnt_lookup(struct worker *wrk, struct req *req)
 	req->objcore = oc;
 	AZ(oc->flags & OC_F_PASS);
 
-	VSLb(req->vsl, SLT_Hit, "%u", ObjGetXID(wrk, req->objcore));
+	VSLb(req->vsl, SLT_Hit, "%u %.6f %.6f %.6f",
+	    ObjGetXID(wrk, req->objcore),
+	    EXP_Dttl(req, req->objcore),
+	    req->objcore->grace,
+	    req->objcore->keep);
 
 	VCL_hit_method(req->vcl, wrk, req, NULL, NULL);
 
@@ -441,6 +548,9 @@ cnt_lookup(struct worker *wrk, struct req *req)
 	case VCL_RET_RESTART:
 		req->req_step = R_STP_RESTART;
 		break;
+	case VCL_RET_FAIL:
+		req->req_step = R_STP_VCLFAIL;
+		break;
 	case VCL_RET_SYNTH:
 		req->req_step = R_STP_SYNTH;
 		break;
@@ -486,6 +596,9 @@ cnt_miss(struct worker *wrk, struct req *req)
 			(void)HSH_DerefObjCore(wrk, &req->stale_oc, 0);
 		req->req_step = R_STP_FETCH;
 		return (REQ_FSM_MORE);
+	case VCL_RET_FAIL:
+		req->req_step = R_STP_VCLFAIL;
+		break;
 	case VCL_RET_SYNTH:
 		req->req_step = R_STP_SYNTH;
 		break;
@@ -520,6 +633,9 @@ cnt_pass(struct worker *wrk, struct req *req)
 
 	VCL_pass_method(req->vcl, wrk, req, NULL, NULL);
 	switch (wrk->handling) {
+	case VCL_RET_FAIL:
+		req->req_step = R_STP_VCLFAIL;
+		break;
 	case VCL_RET_SYNTH:
 		req->req_step = R_STP_SYNTH;
 		break;
@@ -563,11 +679,20 @@ cnt_pipe(struct worker *wrk, struct req *req)
 	HTTP_Setup(bo->bereq, bo->ws, bo->vsl, SLT_BereqMethod);
 	http_FilterReq(bo->bereq, req->http, 0);	// XXX: 0 ?
 	http_PrintfHeader(bo->bereq, "X-Varnish: %u", VXID(req->vsl->wid));
-	http_SetHeader(bo->bereq, "Connection: close");
+	http_ForceHeader(bo->bereq, H_Connection, "close");
+
+	if (req->want100cont) {
+		http_SetHeader(bo->bereq, "Expect: 100-continue");
+		req->want100cont = 0;
+	}
 
 	VCL_pipe_method(req->vcl, wrk, req, bo, NULL);
 
 	switch (wrk->handling) {
+	case VCL_RET_FAIL:
+		req->req_step = R_STP_VCLFAIL;
+		nxt = REQ_FSM_MORE;
+		break;
 	case VCL_RET_SYNTH:
 		req->req_step = R_STP_SYNTH;
 		nxt = REQ_FSM_MORE;
@@ -667,29 +792,37 @@ cnt_recv(struct worker *wrk, struct req *req)
 	req->director_hint = VCL_DefaultDirector(req->vcl);
 	AN(req->director_hint);
 
+	req->vdp_retval = 0;
 	req->d_ttl = -1;
 	req->disable_esi = 0;
 	req->hash_always_miss = 0;
 	req->hash_ignore_busy = 0;
 	req->client_identity = NULL;
+	req->storage = NULL;
 
 	http_CollectHdr(req->http, H_Cache_Control);
 
-	VFP_Setup(req->htc->vfc);
-	req->htc->vfc->http = req->http;
-	req->htc->vfc->wrk = wrk;
-	if (req->transport->req_body != NULL) {
-		req->transport->req_body(req);
-
-		if (req->req_body_status == REQ_BODY_FAIL) {
-			req->doclose = SC_OVERLOAD;
-			return (REQ_FSM_DONE);
-		}
+	if (req->req_body_status == REQ_BODY_FAIL) {
+		req->doclose = SC_OVERLOAD;
+		return (REQ_FSM_DONE);
 	}
 
 	VCL_recv_method(req->vcl, wrk, req, NULL, NULL);
-	if (wrk->handling == VCL_RET_VCL)
+	if (wrk->handling == VCL_RET_VCL && req->restarts == 0) {
+		req->director_hint = VCL_DefaultDirector(req->vcl);
+		HTTP_Copy(req->http, req->http0);
+		WS_Reset(req->ws, req->ws_req);
+		AN(req->director_hint);
 		VCL_recv_method(req->vcl, wrk, req, NULL, NULL);
+	}
+
+	if (req->want100cont && !req->late100cont) {
+		req->want100cont = 0;
+		if (req->transport->minimal_response(req, 100)) {
+			req->doclose = SC_REM_CLOSE;
+			return (REQ_FSM_DONE);
+		}
+	}
 
 	/* Attempts to cache req.body may fail */
 	if (req->req_body_status == REQ_BODY_FAIL) {
@@ -712,13 +845,18 @@ cnt_recv(struct worker *wrk, struct req *req)
 
 	SHA256_Init(&sha256ctx);
 	VCL_hash_method(req->vcl, wrk, req, NULL, &sha256ctx);
-	assert(wrk->handling == VCL_RET_LOOKUP);
+	if (wrk->handling == VCL_RET_FAIL)
+		recv_handling = wrk->handling;
+	else
+		assert(wrk->handling == VCL_RET_LOOKUP);
 	SHA256_Final(req->digest, &sha256ctx);
 
-	switch(recv_handling) {
+	switch (recv_handling) {
 	case VCL_RET_VCL:
 		VSLb(req->vsl, SLT_VCL_Error,
-		    "return(vcl) only allowed from active (top) VCL.");
+		    "Illegal return(vcl): %s",
+		    req->restarts ? "Not after restarts" :
+		    "Only from active VCL");
 		req->err_code = 503;
 		req->req_step = R_STP_SYNTH;
 		return (REQ_FSM_MORE);
@@ -729,21 +867,29 @@ cnt_recv(struct worker *wrk, struct req *req)
 		req->req_step = R_STP_LOOKUP;
 		return (REQ_FSM_MORE);
 	case VCL_RET_PIPE:
-		if (req->esi_level == 0) {
+		if (req->esi_level > 0) {
+			VSLb(req->vsl, SLT_VCL_Error,
+			    "vcl_recv{} returns pipe for ESI included object."
+			    "  Doing pass.");
+			req->req_step = R_STP_PASS;
+		} else if (req->http0->protover > 11) {
+			VSLb(req->vsl, SLT_VCL_Error,
+			    "vcl_recv{} returns pipe for HTTP/2 request."
+			    "  Doing pass.");
+			req->req_step = R_STP_PASS;
+		} else {
 			req->req_step = R_STP_PIPE;
-			return (REQ_FSM_MORE);
 		}
-		VSLb(req->vsl, SLT_VCL_Error,
-		    "vcl_recv{} returns pipe for ESI included object."
-		    "  Doing pass.");
-		req->req_step = R_STP_PASS;
-		return (REQ_FSM_DONE);
+		return (REQ_FSM_MORE);
 	case VCL_RET_PASS:
 		req->req_step = R_STP_PASS;
 		return (REQ_FSM_MORE);
 	case VCL_RET_SYNTH:
 		req->req_step = R_STP_SYNTH;
 		return (REQ_FSM_MORE);
+	case VCL_RET_FAIL:
+		req->req_step = R_STP_VCLFAIL;
+		return (REQ_FSM_MORE);
 	default:
 		WRONG("Illegal return from vcl_recv{}");
 	}
@@ -776,7 +922,7 @@ cnt_purge(struct worker *wrk, struct req *req)
 	CHECK_OBJ_NOTNULL(boc, OBJCORE_MAGIC);
 	VRY_Finish(req, DISCARD);
 
-	HSH_Purge(wrk, boc->objhead, 0, 0, 0);
+	(void)HSH_Purge(wrk, boc->objhead, 0, 0, 0);
 
 	AZ(HSH_DerefObjCore(wrk, &boc, 1));
 
@@ -785,6 +931,9 @@ cnt_purge(struct worker *wrk, struct req *req)
 	case VCL_RET_RESTART:
 		req->req_step = R_STP_RESTART;
 		break;
+	case VCL_RET_FAIL:
+		req->req_step = R_STP_VCLFAIL;
+		break;
 	case VCL_RET_SYNTH:
 		req->req_step = R_STP_SYNTH;
 		break;
@@ -825,13 +974,13 @@ CNT_Request(struct worker *wrk, struct req *req)
 	 */
 	assert(
 	    req->req_step == R_STP_LOOKUP ||
+	    req->req_step == R_STP_TRANSPORT ||
 	    req->req_step == R_STP_RECV);
 
 	AN(req->vsl->wid & VSL_CLIENTMARKER);
 
-	req->wrk = wrk;
+	req->vfc->wrk = req->wrk = wrk;
 	wrk->vsl = req->vsl;
-
 	for (nxt = REQ_FSM_MORE; nxt == REQ_FSM_MORE; ) {
 		/*
 		 * This is a good place to be paranoid about the various
@@ -847,7 +996,7 @@ CNT_Request(struct worker *wrk, struct req *req)
 		 * empty on state-transitions.
 		 */
 		WS_Assert(wrk->aws);
-		assert(wrk->aws->s == wrk->aws->f);
+		AZ(WS_Snapshot(wrk->aws));
 
 		switch (req->req_step) {
 #define REQ_STEP(l,u,arg) \
@@ -857,7 +1006,6 @@ CNT_Request(struct worker *wrk, struct req *req)
 			nxt = cnt_##l arg; \
 			break;
 #include "tbl/steps.h"
-#undef REQ_STEP
 		default:
 			WRONG("State engine misfire");
 		}
@@ -870,32 +1018,6 @@ CNT_Request(struct worker *wrk, struct req *req)
 		VRB_Free(req);
 		req->wrk = NULL;
 	}
+	assert(nxt == REQ_FSM_DISEMBARK || req->ws->r == NULL);
 	return (nxt);
 }
-
-void
-CNT_AcctLogCharge(struct dstat *ds, struct req *req)
-{
-	struct acct_req *a;
-
-	AN(ds);
-	CHECK_OBJ_NOTNULL(req, REQ_MAGIC);
-
-	a = &req->acct;
-
-	if (req->vsl->wid && !(req->res_mode & RES_PIPE)) {
-		VSLb(req->vsl, SLT_ReqAcct, "%ju %ju %ju %ju %ju %ju",
-		    (uintmax_t)a->req_hdrbytes,
-		    (uintmax_t)a->req_bodybytes,
-		    (uintmax_t)(a->req_hdrbytes + a->req_bodybytes),
-		    (uintmax_t)a->resp_hdrbytes,
-		    (uintmax_t)a->resp_bodybytes,
-		    (uintmax_t)(a->resp_hdrbytes + a->resp_bodybytes));
-	}
-
-#define ACCT(foo)			\
-	ds->s_##foo += a->foo;		\
-	a->foo = 0;
-#include "tbl/acct_fields_req.h"
-#undef ACCT
-}
