@@ -30,14 +30,13 @@
 
 #include "config.h"
 
+#include "cache.h"
+
 #include <errno.h>
 #include <dlfcn.h>
-#include <pthread.h>
 #include <stdio.h>
 #include <stdlib.h>
 
-#include "cache.h"
-
 #include "vcl.h"
 #include "vrt.h"
 
@@ -73,6 +72,7 @@ struct vcl {
 	pthread_rwlock_t	temp_rwl;
 	VTAILQ_HEAD(,backend)	backend_list;
 	VTAILQ_HEAD(,vclref)	ref_list;
+	int			nrefs;
 	struct vcl		*label;
 	int			nlabels;
 };
@@ -98,7 +98,7 @@ static struct vcl		*vcl_active; /* protected by vcl_mtx */
 static struct vrt_ctx ctx_cli;
 static unsigned handling_cli;
 static struct ws ws_cli;
-static char *ws_snapshot_cli;
+static uintptr_t ws_snapshot_cli;
 
 /*--------------------------------------------------------------------*/
 
@@ -107,6 +107,7 @@ vcl_get_ctx(unsigned method, int msg)
 {
 	AZ(ctx_cli.handling);
 	INIT_OBJ(&ctx_cli, VRT_CTX_MAGIC);
+	handling_cli = 0;
 	ctx_cli.handling = &handling_cli;
 	ctx_cli.method = method;
 	if (msg) {
@@ -148,6 +149,7 @@ vcl_send_event(VRT_CTX, enum vcl_event_e ev)
 	       ev == VCL_EVENT_COLD ||
 	       ev == VCL_EVENT_DISCARD);
 	AN(ctx->handling);
+	*ctx->handling = 0;
 	AN(ctx->ws);
 
 	if (ev == VCL_EVENT_LOAD || ev == VCL_EVENT_WARM)
@@ -191,8 +193,8 @@ VCL_Panic(struct vsb *vsb, const struct vcl *vcl)
 	VSB_printf(vsb, "vcl = {\n");
 	VSB_indent(vsb, 2);
 	PAN_CheckMagic(vsb, vcl, VCL_MAGIC);
-	VSB_printf(vsb, "name = \"%s\"\n", vcl->loaded_name);
-	VSB_printf(vsb, "busy = %u\n", vcl->busy);
+	VSB_printf(vsb, "name = \"%s\",\n", vcl->loaded_name);
+	VSB_printf(vsb, "busy = %u,\n", vcl->busy);
 	VSB_printf(vsb, "discard = %u,\n", vcl->discard);
 	VSB_printf(vsb, "state = %s,\n", vcl->state);
 	VSB_printf(vsb, "temp = %s,\n", (const volatile char *)vcl->temp);
@@ -226,7 +228,6 @@ VCL_Return_Name(unsigned r)
 	case VCL_RET_##U:	\
 		return(#l);
 #include "tbl/vcl_returns.h"
-#undef VCL_RET_MAC
 	default:
 		return (NULL);
 	}
@@ -241,7 +242,6 @@ VCL_Method_Name(unsigned m)
 	case VCL_MET_##upper:			\
 		return (#upper);
 #include "tbl/vcl_returns.h"
-#undef VCL_MET_MAC
 	default:
 		return (NULL);
 	}
@@ -252,13 +252,13 @@ VCL_Method_Name(unsigned m)
 static void
 vcl_get(struct vcl **vcc, struct vcl *vcl)
 {
+	AN(vcc);
 
-	CHECK_OBJ_NOTNULL(vcl, VCL_MAGIC);
-	AZ(errno=pthread_rwlock_rdlock(&vcl->temp_rwl));
-	assert(VCL_WARM(vcl));
-	AZ(errno=pthread_rwlock_unlock(&vcl->temp_rwl));
 	Lck_Lock(&vcl_mtx);
-	AN(vcl);
+	if (vcl == NULL)
+		vcl = vcl_active; /* Sample vcl_active under lock to avoid
+				   * race */
+	CHECK_OBJ_NOTNULL(vcl, VCL_MAGIC);
 	if (vcl->label == NULL) {
 		AN(strcmp(vcl->state, VCL_TEMP_LABEL));
 		*vcc = vcl;
@@ -266,16 +266,18 @@ vcl_get(struct vcl **vcc, struct vcl *vcl)
 		AZ(strcmp(vcl->state, VCL_TEMP_LABEL));
 		*vcc = vcl->label;
 	}
-	AN(*vcc);
+	CHECK_OBJ_NOTNULL(*vcc, VCL_MAGIC);
 	AZ((*vcc)->discard);
 	(*vcc)->busy++;
 	Lck_Unlock(&vcl_mtx);
+	AZ(errno=pthread_rwlock_rdlock(&(*vcc)->temp_rwl));
+	assert(VCL_WARM(*vcc));
+	AZ(errno=pthread_rwlock_unlock(&(*vcc)->temp_rwl));
 }
 
 void
 VCL_Refresh(struct vcl **vcc)
 {
-	CHECK_OBJ_NOTNULL(vcl_active, VCL_MAGIC);
 	if (*vcc == vcl_active)
 		return;
 	if (*vcc != NULL)
@@ -284,7 +286,7 @@ VCL_Refresh(struct vcl **vcc)
 	while (vcl_active == NULL)
 		(void)usleep(100000);
 
-	vcl_get(vcc, vcl_active);
+	vcl_get(vcc, NULL);
 }
 
 void
@@ -519,20 +521,37 @@ VRT_count(VRT_CTX, unsigned u)
 	CHECK_OBJ_NOTNULL(ctx->vcl->conf, VCL_CONF_MAGIC);
 	assert(u < ctx->vcl->conf->nref);
 	if (ctx->vsl != NULL)
-		VSLb(ctx->vsl, SLT_VCL_trace, "%u %u.%u", u,
-		    ctx->vcl->conf->ref[u].line, ctx->vcl->conf->ref[u].pos);
+		VSLb(ctx->vsl, SLT_VCL_trace, "%s %u %u.%u.%u",
+		    ctx->vcl->loaded_name, u, ctx->vcl->conf->ref[u].source, ctx->vcl->conf->ref[u].line, ctx->vcl->conf->ref[u].pos);
+	else
+		VSL(SLT_VCL_trace, 0, "%s %u %u.%u.%u",
+		    ctx->vcl->loaded_name, u, ctx->vcl->conf->ref[u].source, ctx->vcl->conf->ref[u].line, ctx->vcl->conf->ref[u].pos);
 }
 
 VCL_VCL
-VRT_vcl_lookup(const char *name)
+VRT_vcl_get(VRT_CTX, const char *name)
 {
 	VCL_VCL vcl;
 
+	CHECK_OBJ_NOTNULL(ctx, VRT_CTX_MAGIC);
 	vcl = vcl_find(name);
 	AN(vcl);
+	Lck_Lock(&vcl_mtx);
+	vcl->nrefs++;
+	Lck_Unlock(&vcl_mtx);
 	return (vcl);
 }
 
+void
+VRT_vcl_rel(VRT_CTX, VCL_VCL vcl)
+{
+	CHECK_OBJ_NOTNULL(ctx, VRT_CTX_MAGIC);
+	AN(vcl);
+	Lck_Lock(&vcl_mtx);
+	vcl->nrefs--;
+	Lck_Unlock(&vcl_mtx);
+}
+
 void
 VRT_vcl_select(VRT_CTX, VCL_VCL vcl)
 {
@@ -567,6 +586,7 @@ VRT_ref_vcl(VRT_CTX, const char *desc)
 
 	Lck_Lock(&vcl_mtx);
 	VTAILQ_INSERT_TAIL(&vcl->ref_list, ref, list);
+	vcl->nrefs++;
 	Lck_Unlock(&vcl_mtx);
 
 	return (ref);
@@ -598,6 +618,7 @@ VRT_rel_vcl(VRT_CTX, struct vclref **refp)
 	Lck_Lock(&vcl_mtx);
 	assert(!VTAILQ_EMPTY(&vcl->ref_list));
 	VTAILQ_REMOVE(&vcl->ref_list, ref, list);
+	vcl->nrefs--;
 	/* No garbage collection here, for the same reasons as in VCL_Rel. */
 	Lck_Unlock(&vcl_mtx);
 
@@ -642,7 +663,7 @@ vcl_set_state(VRT_CTX, const char *state)
 	AZ(errno=pthread_rwlock_wrlock(&vcl->temp_rwl));
 	AN(vcl->temp);
 
-	switch(state[0]) {
+	switch (state[0]) {
 	case '0':
 		assert(vcl->temp != VCL_TEMP_COLD);
 		if (vcl->busy == 0 && VCL_WARM(vcl)) {
@@ -699,8 +720,10 @@ vcl_cancel_load(VRT_CTX, struct cli *cli, const char *name, const char *step)
 	VCLI_Out(cli, "VCL \"%s\" Failed %s", name, step);
 	if (VSB_len(ctx->msg))
 		VCLI_Out(cli, "\nMessage:\n\t%s", VSB_data(ctx->msg));
+	*ctx->handling = 0;
 	AZ(vcl->conf->event_vcl(ctx, VCL_EVENT_DISCARD));
 	vcl_KillBackends(vcl);
+	free(vcl->loaded_name);
 	VCL_Close(&vcl);
 }
 
@@ -735,10 +758,11 @@ vcl_load(struct cli *cli, struct vrt_ctx *ctx,
 
 	VSB_clear(ctx->msg);
 	i = vcl_send_event(ctx, VCL_EVENT_LOAD);
-	if (i) {
+	if (i || *ctx->handling != VCL_RET_OK) {
 		vcl_cancel_load(ctx, cli, name, "initialization");
 		return;
 	}
+	assert(*ctx->handling == VCL_RET_OK);
 	VSB_clear(ctx->msg);
 	i = vcl_set_state(ctx, state);
 	if (i) {
@@ -747,7 +771,6 @@ vcl_load(struct cli *cli, struct vrt_ctx *ctx,
 		return;
 	}
 	bprintf(vcl->state, "%s", state + 1);
-	assert(*ctx->handling == VCL_RET_OK);
 	VCLI_Out(cli, "Loaded \"%s\" as \"%s\"", fn , name);
 	VTAILQ_INSERT_TAIL(&vcl_head, vcl, list);
 	Lck_Lock(&vcl_mtx);
@@ -815,12 +838,15 @@ vcl_cli_list(struct cli *cli, const char * const *av, void *priv)
 			flg = "available";
 		VCLI_Out(cli, "%-10s %5s/%-8s %6u %s",
 		    flg, vcl->state, vcl->temp, vcl->busy, vcl->loaded_name);
-		if (vcl->label != NULL)
+		if (vcl->label != NULL) {
 			VCLI_Out(cli, " -> %s", vcl->label->loaded_name);
-		else if (vcl->nlabels > 1)
-			VCLI_Out(cli, " (%d labels)", vcl->nlabels);
-		else if (vcl->nlabels > 0)
-			VCLI_Out(cli, " (%d label)", vcl->nlabels);
+			if (vcl->nrefs)
+				VCLI_Out(cli, " (%d return(vcl)%s)",
+				    vcl->nrefs, vcl->nrefs > 1 ? "'s" : "");
+		} else if (vcl->nlabels > 0) {
+			VCLI_Out(cli, " (%d label%s)",
+			    vcl->nlabels, vcl->nlabels > 1 ? "s" : "");
+		}
 		VCLI_Out(cli, "\n");
 	}
 }
@@ -888,6 +914,8 @@ vcl_cli_discard(struct cli *cli, const char * const *av, void *priv)
 	if (!strcmp(vcl->state, VCL_TEMP_LABEL)) {
 		VTAILQ_REMOVE(&vcl_head, vcl, list);
 		free(vcl->loaded_name);
+		AZ(errno=pthread_rwlock_destroy(&vcl->temp_rwl));
+		FREE_OBJ(vcl);
 	} else if (vcl->temp == VCL_TEMP_COLD)
 		VCL_Poll();
 }
@@ -988,7 +1016,7 @@ static void
 vcl_call_method(struct worker *wrk, struct req *req, struct busyobj *bo,
     void *specific, unsigned method, vcl_func_f *func)
 {
-	char *aws;
+	uintptr_t aws;
 	struct vsl_log *vsl = NULL;
 	struct vrt_ctx ctx;
 
@@ -1024,16 +1052,18 @@ vcl_call_method(struct worker *wrk, struct req *req, struct busyobj *bo,
 	ctx.vsl = vsl;
 	ctx.specific = specific;
 	ctx.method = method;
+	wrk->handling = 0;
 	ctx.handling = &wrk->handling;
 	aws = WS_Snapshot(wrk->aws);
-	wrk->handling = 0;
 	wrk->cur_method = method;
 	wrk->seen_methods |= method;
 	AN(vsl);
 	VSLb(vsl, SLT_VCL_call, "%s", VCL_Method_Name(method));
-	(void)func(&ctx);
+	func(&ctx);
 	VSLb(vsl, SLT_VCL_return, "%s", VCL_Return_Name(wrk->handling));
 	wrk->cur_method |= 1;		// Magic marker
+	if (wrk->handling == VCL_RET_FAIL)
+		wrk->stats->vcl_fail++;
 
 	/*
 	 * VCL/Vmods are not allowed to make permanent allocations from
@@ -1057,7 +1087,6 @@ VCL_##func##_method(struct vcl *vcl, struct worker *wrk,		\
 }
 
 #include "tbl/vcl_returns.h"
-#undef VCL_MET_MAC
 
 /*--------------------------------------------------------------------*/
 
