@@ -54,11 +54,11 @@ VRT_synth(VRT_CTX, unsigned code, const char *reason)
 
 	CHECK_OBJ_NOTNULL(ctx, VRT_CTX_MAGIC);
 	CHECK_OBJ_NOTNULL(ctx->req, REQ_MAGIC);
-	if (code < 100 || code > 999)
+	if (code < 100)
 		code = 503;
 	ctx->req->err_code = (uint16_t)code;
-	ctx->req->err_reason =
-	    reason ? reason : http_Status2Reason(ctx->req->err_code, NULL);
+	ctx->req->err_reason = reason ? reason
+	    : http_Status2Reason(ctx->req->err_code % 1000, NULL);
 }
 
 /*--------------------------------------------------------------------*/
@@ -85,6 +85,26 @@ VRT_acl_match(VRT_CTX, VCL_ACL acl, VCL_IP ip)
 	return (acl->match(ctx, ip));
 }
 
+void
+VRT_hit_for_pass(VRT_CTX, VCL_DURATION d)
+{
+	struct objcore *oc;
+
+	CHECK_OBJ_NOTNULL(ctx, VRT_CTX_MAGIC);
+	if (ctx->bo == NULL) {
+		VSLb(ctx->vsl, SLT_Error,
+		    "Note: Ignoring DURATION argument to return(pass);");
+		return;
+	}
+	CHECK_OBJ_NOTNULL(ctx->bo, BUSYOBJ_MAGIC);
+	oc = ctx->bo->fetch_objcore;
+	oc->ttl = d;
+	oc->grace = 0.0;
+	oc->keep = 0.0;
+	VSLb(ctx->vsl, SLT_TTL, "HFP %.0f %.0f %.0f %.0f",
+	    oc->ttl, oc->grace, oc->keep, oc->t_origin);
+}
+
 /*--------------------------------------------------------------------*/
 
 struct http *
@@ -150,7 +170,7 @@ VRT_StringList(char *d, unsigned dl, const char *p, va_list ap)
 	b = d;
 	e = b + dl;
 	while (p != vrt_magic_string_end && b < e) {
-		if (p != NULL) {
+		if (p != NULL && *p != '\0') {
 			x = strlen(p);
 			if (b + x < e)
 				memcpy(b, p, x);
@@ -252,10 +272,32 @@ VRT_handling(VRT_CTX, unsigned hand)
 {
 
 	CHECK_OBJ_NOTNULL(ctx, VRT_CTX_MAGIC);
+	assert(hand > 0);
 	assert(hand < VCL_RET_MAX);
+	// XXX:NOTYET assert(*ctx->handling == 0);
 	*ctx->handling = hand;
 }
 
+/*--------------------------------------------------------------------*/
+
+void
+VRT_fail(VRT_CTX, const char *fmt, ...)
+{
+	va_list ap;
+
+	assert(ctx->vsl != NULL || ctx->msg != NULL);
+	AZ(strchr(fmt, '\n'));
+	va_start(ap, fmt);
+	if (ctx->vsl != NULL)
+		VSLbv(ctx->vsl, SLT_VCL_Error, fmt, ap);
+	else {
+		VSB_vprintf(ctx->msg, fmt, ap);
+		VSB_putc(ctx->msg, '\n');
+	}
+	va_end(ap);
+	VRT_handling(ctx, VCL_RET_FAIL);
+}
+
 /*--------------------------------------------------------------------
  * Feed data into the hash calculation
  */
@@ -415,7 +457,11 @@ VRT_ban_string(VRT_CTX, const char *str)
 
 	CHECK_OBJ_NOTNULL(ctx, VRT_CTX_MAGIC);
 	AN(ctx->vsl);
-	AN(str);
+
+	if (str == NULL) {
+		VSLb(ctx->vsl, SLT_VCL_Error, "ban(): Null argument");
+		return;
+	}
 
 	bp = BAN_Build();
 	if (bp == NULL) {
@@ -492,16 +538,23 @@ VRT_CacheReqBody(VRT_CTX, VCL_BYTES maxsize)
  * purges
  */
 
-void
+unsigned
 VRT_purge(VRT_CTX, double ttl, double grace, double keep)
 {
 
 	CHECK_OBJ_NOTNULL(ctx, VRT_CTX_MAGIC);
+
+	if (ctx->method != VCL_MET_HIT && ctx->method != VCL_MET_MISS) {
+		VSLb(ctx->vsl, SLT_VCL_Error,
+		    "purge can only happen in vcl_hit{} or vcl_miss{}");
+		VRT_handling(ctx, VCL_RET_FAIL);
+		return (0);
+	}
+
 	CHECK_OBJ_NOTNULL(ctx->req, REQ_MAGIC);
 	CHECK_OBJ_NOTNULL(ctx->req->wrk, WORKER_MAGIC);
-	if (ctx->method == VCL_MET_HIT || ctx->method == VCL_MET_MISS)
-		HSH_Purge(ctx->req->wrk, ctx->req->objcore->objhead,
-		    ttl, grace, keep);
+	return (HSH_Purge(ctx->req->wrk, ctx->req->objcore->objhead,
+	    ttl, grace, keep));
 }
 
 /*--------------------------------------------------------------------
@@ -522,3 +575,11 @@ VRT_memmove(void *dst, const void *src, unsigned len)
 
 	(void)memmove(dst, src, len);
 }
+
+int
+VRT_ipcmp(const struct suckaddr *sua1, const struct suckaddr *sua2)
+{
+	if (sua1 == NULL || sua2 == NULL)
+		return(1);
+	return (VSA_Compare_IP(sua1, sua2));
+}
