@@ -48,7 +48,39 @@ struct vrt_priv {
 	uintptr_t			vmod_id;
 };
 
-struct vmod_priv cli_task_priv;
+static struct vmod_priv cli_task_priv;
+
+/*--------------------------------------------------------------------
+ */
+
+void
+pan_privs(struct vsb *vsb, const struct vrt_privs *privs)
+{
+	struct vrt_priv *vp;
+
+	VSB_printf(vsb, "privs = %p {\n", privs);
+	if (PAN_already(vsb, privs))
+		return;
+	VSB_indent(vsb, 2);
+	PAN_CheckMagic(vsb, privs, VRT_PRIVS_MAGIC);
+	if (privs->magic == VRT_PRIVS_MAGIC) {
+		VTAILQ_FOREACH(vp, &privs->privs, list) {
+			PAN_CheckMagic(vsb, vp, VRT_PRIV_MAGIC);
+			VSB_printf(vsb,
+			    "priv {p %p l %d f %p} vcl %p id %jx vmod %jx\n",
+			    vp->priv->priv,
+			    vp->priv->len,
+			    vp->priv->free,
+			    vp->vcl,
+			    (uintmax_t)vp->id,
+			    (uintmax_t)vp->vmod_id
+			);
+		}
+	}
+	VSB_indent(vsb, -2);
+	VSB_printf(vsb, "},\n");
+}
+
 
 /*--------------------------------------------------------------------
  */
@@ -118,7 +150,7 @@ VRTPRIV_dynamic_kill(struct vrt_privs *privs, uintptr_t id)
 }
 
 struct vmod_priv *
-VRT_priv_task(VRT_CTX, void *vmod_id)
+VRT_priv_task(VRT_CTX, const void *vmod_id)
 {
 	uintptr_t id;
 
@@ -137,7 +169,7 @@ VRT_priv_task(VRT_CTX, void *vmod_id)
 }
 
 struct vmod_priv *
-VRT_priv_top(VRT_CTX, void *vmod_id)
+VRT_priv_top(VRT_CTX, const void *vmod_id)
 {
 	uintptr_t id;
 
@@ -149,7 +181,7 @@ VRT_priv_top(VRT_CTX, void *vmod_id)
 		return (vrt_priv_dynamic(ctx, id, (uintptr_t)vmod_id));
 	} else
 		WRONG("PRIV_TOP is only accessible in client VCL context");
-	NEEDLESS_RETURN(NULL);
+	NEEDLESS(return NULL);
 }
 
 /*--------------------------------------------------------------------
