@@ -30,9 +30,10 @@
  */
 #include "config.h"
 
+#include <stdio.h>
+
 #include "cache.h"
 #include "common/heritage.h"
-#include "hash/hash_slinger.h"
 
 #include "cache_director.h"
 #include "vrt.h"
@@ -45,7 +46,7 @@ static char vrt_hostname[255] = "";
  */
 
 static void
-vrt_do_string(const struct http *hp, int fld,
+vrt_do_string(VRT_CTX, struct http *hp, int fld,
     const char *err, const char *p, va_list ap)
 {
 	const char *b;
@@ -53,29 +54,33 @@ vrt_do_string(const struct http *hp, int fld,
 	CHECK_OBJ_NOTNULL(hp, HTTP_MAGIC);
 
 	b = VRT_String(hp->ws, NULL, p, ap);
-	if (b == NULL || *b == '\0') {
-		VSLb(hp->vsl, SLT_LostHeader, "%s", err);
+	if (b == NULL) {
+		VRT_fail(ctx, "Workspace overflow (%s)", err);
 		WS_MarkOverflow(hp->ws);
 		return;
 	}
+	if (*b == '\0') {
+		VRT_fail(ctx, "Setting %s to empty string", err);
+		return;
+	}
 	http_SetH(hp, fld, b);
 }
 
 #define VRT_HDR_L(obj, hdr, fld)					\
 void									\
-VRT_l_##obj##_##hdr(VRT_CTX, const char *p, ...)	\
+VRT_l_##obj##_##hdr(VRT_CTX, const char *p, ...)			\
 {									\
 	va_list ap;							\
 									\
 	CHECK_OBJ_NOTNULL(ctx, VRT_CTX_MAGIC);				\
 	va_start(ap, p);						\
-	vrt_do_string(ctx->http_##obj, fld, #obj "." #hdr, p, ap);	\
+	vrt_do_string(ctx, ctx->http_##obj, fld, #obj "." #hdr, p, ap);	\
 	va_end(ap);							\
 }
 
 #define VRT_HDR_R(obj, hdr, fld)					\
 const char *								\
-VRT_r_##obj##_##hdr(VRT_CTX)				\
+VRT_r_##obj##_##hdr(VRT_CTX)						\
 {									\
 	CHECK_OBJ_NOTNULL(ctx, VRT_CTX_MAGIC);				\
 	CHECK_OBJ_NOTNULL(ctx->http_##obj, HTTP_MAGIC);			\
@@ -88,31 +93,29 @@ VRT_r_##obj##_##hdr(VRT_CTX)				\
 
 #define VRT_STATUS_L(obj)						\
 void									\
-VRT_l_##obj##_status(VRT_CTX, long num)		\
+VRT_l_##obj##_status(VRT_CTX, long num)					\
 {									\
 									\
 	CHECK_OBJ_NOTNULL(ctx, VRT_CTX_MAGIC);				\
 	CHECK_OBJ_NOTNULL(ctx->http_##obj, HTTP_MAGIC);			\
-	if (num > 65535) {						\
-		VSLb(ctx->vsl, SLT_VCL_Error, "%s.status > 65535", #obj); \
-		WS_MarkOverflow(ctx->http_##obj->ws);			\
-	} else if ((num % 1000) < 100) {				\
-		VSLb(ctx->vsl, SLT_VCL_Error, "illegal %s.status (..0##)", \
-		    #obj);						\
-		WS_MarkOverflow(ctx->http_##obj->ws);			\
-	} else {							\
+	if (num < 0)							\
+		VRT_fail(ctx, "%s.status (%ld) is negative", #obj, num); \
+	else if (num > 65535)						\
+		VRT_fail(ctx, "%s.status (%ld) > 65535", #obj, num);	\
+	else if ((num % 1000) < 100)					\
+		VRT_fail(ctx, "illegal %s.status (%ld) (..0##)", #obj, num); \
+	else								\
 		http_SetStatus(ctx->http_##obj, (uint16_t)num);		\
-	}								\
 }
 
 #define VRT_STATUS_R(obj)						\
 long									\
-VRT_r_##obj##_status(VRT_CTX)				\
+VRT_r_##obj##_status(VRT_CTX)						\
 {									\
 									\
 	CHECK_OBJ_NOTNULL(ctx, VRT_CTX_MAGIC);				\
 	CHECK_OBJ_NOTNULL(ctx->http_##obj, HTTP_MAGIC);			\
-	return(ctx->http_##obj->status);				\
+	return (ctx->http_##obj->status);				\
 }
 
 VRT_HDR_LR(req,    method,	HTTP_HDR_METHOD)
@@ -157,7 +160,8 @@ VRT_r_obj_proto(VRT_CTX)
 	CHECK_OBJ_NOTNULL(ctx->req, REQ_MAGIC);
 	CHECK_OBJ_NOTNULL(ctx->req->objcore, OBJCORE_MAGIC);
 
-	return (HTTP_GetHdrPack(ctx->req->wrk, ctx->req->objcore, H__Proto));
+	return (HTTP_GetHdrPack(ctx->req->wrk, ctx->req->objcore,
+	    H__Proto));
 }
 
 const char *
@@ -167,7 +171,8 @@ VRT_r_obj_reason(VRT_CTX)
 	CHECK_OBJ_NOTNULL(ctx->req, REQ_MAGIC);
 	CHECK_OBJ_NOTNULL(ctx->req->objcore, OBJCORE_MAGIC);
 
-	return (HTTP_GetHdrPack(ctx->req->wrk, ctx->req->objcore, H__Reason));
+	return (HTTP_GetHdrPack(ctx->req->wrk, ctx->req->objcore,
+	    H__Reason));
 }
 
 /*--------------------------------------------------------------------
@@ -177,7 +182,7 @@ VRT_r_obj_reason(VRT_CTX)
 #define VBERESPW0(field)
 #define VBERESPW1(field)						\
 void									\
-VRT_l_beresp_##field(VRT_CTX, unsigned a)		\
+VRT_l_beresp_##field(VRT_CTX, unsigned a)				\
 {									\
 	CHECK_OBJ_NOTNULL(ctx, VRT_CTX_MAGIC);				\
 	CHECK_OBJ_NOTNULL(ctx->bo, BUSYOBJ_MAGIC);			\
@@ -187,7 +192,7 @@ VRT_l_beresp_##field(VRT_CTX, unsigned a)		\
 #define VBERESPR0(field)
 #define VBERESPR1(field)						\
 unsigned								\
-VRT_r_beresp_##field(VRT_CTX)				\
+VRT_r_beresp_##field(VRT_CTX)						\
 {									\
 	CHECK_OBJ_NOTNULL(ctx, VRT_CTX_MAGIC);				\
 	CHECK_OBJ_NOTNULL(ctx->bo, BUSYOBJ_MAGIC);			\
@@ -198,10 +203,17 @@ VRT_r_beresp_##field(VRT_CTX)				\
 	VBERESPR##r(l) \
 	VBERESPW##w(l)
 #include "tbl/bo_flags.h"
-#undef BO_FLAG
 
 /*--------------------------------------------------------------------*/
 
+unsigned
+VRT_r_bereq_is_bgfetch(VRT_CTX)
+{
+	CHECK_OBJ_NOTNULL(ctx, VRT_CTX_MAGIC);
+	CHECK_OBJ_NOTNULL(ctx->bo, BUSYOBJ_MAGIC);
+	return (ctx->bo->is_bgfetch);
+}
+
 unsigned
 VRT_r_bereq_uncacheable(VRT_CTX)
 {
@@ -242,7 +254,7 @@ VRT_r_client_identity(VRT_CTX)
 	CHECK_OBJ_NOTNULL(ctx->req, REQ_MAGIC);
 	if (ctx->req->client_identity != NULL)
 		return (ctx->req->client_identity);
-	return(SES_Get_String_Attr(ctx->req->sp, SA_CLIENT_IP));
+	return (SES_Get_String_Attr(ctx->req->sp, SA_CLIENT_IP));
 }
 
 void
@@ -268,7 +280,7 @@ VRT_l_client_identity(VRT_CTX, const char *str, ...)
 
 #define BEREQ_TIMEOUT(which)					\
 void								\
-VRT_l_bereq_##which(VRT_CTX, double num)	\
+VRT_l_bereq_##which(VRT_CTX, double num)			\
 {								\
 								\
 	CHECK_OBJ_NOTNULL(ctx, VRT_CTX_MAGIC);			\
@@ -277,7 +289,7 @@ VRT_l_bereq_##which(VRT_CTX, double num)	\
 }								\
 								\
 double								\
-VRT_r_bereq_##which(VRT_CTX)			\
+VRT_r_bereq_##which(VRT_CTX)					\
 {								\
 								\
 	CHECK_OBJ_NOTNULL(ctx, VRT_CTX_MAGIC);			\
@@ -333,6 +345,7 @@ VRT_r_beresp_storage_hint(VRT_CTX)
 void
 VRT_l_beresp_storage_hint(VRT_CTX, const char *str, ...)
 {
+	const struct stevedore *stv;
 	va_list ap;
 	const char *b;
 
@@ -342,11 +355,50 @@ VRT_l_beresp_storage_hint(VRT_CTX, const char *str, ...)
 	b = VRT_String(ctx->bo->ws, NULL, str, ap);	// XXX: ctx->ws ?
 	va_end(ap);
 	if (b == NULL) {
-		VSLb(ctx->vsl, SLT_LostHeader, "storage.hint");
+		VSLb(ctx->vsl, SLT_LostHeader, "storage_hint");
 		WS_MarkOverflow(ctx->bo->beresp->ws);
 		return;
 	}
 	ctx->bo->storage_hint = b;
+	stv = STV_find(b);
+	if (stv != NULL)
+		ctx->bo->storage = stv;
+}
+
+/*--------------------------------------------------------------------*/
+
+VCL_STEVEDORE
+VRT_r_req_storage(VRT_CTX)
+{
+	CHECK_OBJ_NOTNULL(ctx, VRT_CTX_MAGIC);
+	CHECK_OBJ_NOTNULL(ctx->req, REQ_MAGIC);
+	return (ctx->req->storage);
+}
+
+void
+VRT_l_req_storage(VRT_CTX, VCL_STEVEDORE stv)
+{
+	CHECK_OBJ_NOTNULL(ctx, VRT_CTX_MAGIC);
+	CHECK_OBJ_NOTNULL(ctx->req, REQ_MAGIC);
+	ctx->req->storage = stv;
+}
+
+/*--------------------------------------------------------------------*/
+
+VCL_STEVEDORE
+VRT_r_beresp_storage(VRT_CTX)
+{
+	CHECK_OBJ_NOTNULL(ctx, VRT_CTX_MAGIC);
+	CHECK_OBJ_NOTNULL(ctx->bo, BUSYOBJ_MAGIC);
+	return (ctx->bo->storage);
+}
+
+void
+VRT_l_beresp_storage(VRT_CTX, VCL_STEVEDORE stv)
+{
+	CHECK_OBJ_NOTNULL(ctx, VRT_CTX_MAGIC);
+	CHECK_OBJ_NOTNULL(ctx->bo, BUSYOBJ_MAGIC);
+	ctx->bo->storage = stv;
 }
 
 /*--------------------------------------------------------------------*/
@@ -354,7 +406,7 @@ VRT_l_beresp_storage_hint(VRT_CTX, const char *str, ...)
 #define REQ_VAR_L(nm, elem, type,extra)					\
 									\
 void									\
-VRT_l_req_##nm(VRT_CTX, type arg)			\
+VRT_l_req_##nm(VRT_CTX, type arg)					\
 {									\
 	CHECK_OBJ_NOTNULL(ctx, VRT_CTX_MAGIC);				\
 	CHECK_OBJ_NOTNULL(ctx->req, REQ_MAGIC);				\
@@ -365,11 +417,11 @@ VRT_l_req_##nm(VRT_CTX, type arg)			\
 #define REQ_VAR_R(nm, elem, type)					\
 									\
 type									\
-VRT_r_req_##nm(VRT_CTX)				\
+VRT_r_req_##nm(VRT_CTX)							\
 {									\
 	CHECK_OBJ_NOTNULL(ctx, VRT_CTX_MAGIC);				\
 	CHECK_OBJ_NOTNULL(ctx->req, REQ_MAGIC);				\
-	return(ctx->req->elem);						\
+	return (ctx->req->elem);					\
 }
 
 REQ_VAR_L(backend_hint, director_hint, const struct director *,)
@@ -433,7 +485,7 @@ VRT_l_req_esi(VRT_CTX, unsigned process_esi)
 	 * Only allow you to turn of esi in the main request
 	 * else everything gets confused
 	 */
-	if(ctx->req->esi_level == 0)
+	if (ctx->req->esi_level == 0)
 		ctx->req->disable_esi = !process_esi;
 }
 
@@ -452,7 +504,7 @@ VRT_r_req_esi_level(VRT_CTX)
 
 	CHECK_OBJ_NOTNULL(ctx, VRT_CTX_MAGIC);
 	CHECK_OBJ_NOTNULL(ctx->req, REQ_MAGIC);
-	return(ctx->req->esi_level);
+	return (ctx->req->esi_level);
 }
 
 /*--------------------------------------------------------------------*/
@@ -523,7 +575,7 @@ VRT_r_##which##_##fld(VRT_CTX)					\
 	if (d <= 0.0)						\
 		d = 0.0;					\
 	d -= (offset);						\
-	return(d);						\
+	return (d);						\
 }
 
 VRT_DO_EXP_R(obj, ctx->req->objcore, ttl,
@@ -546,11 +598,11 @@ VRT_DO_EXP_R(beresp, ctx->bo->fetch_objcore, keep, 0)
 #define VRT_DO_AGE_R(which, oc)					\
 								\
 double								\
-VRT_r_##which##_##age(VRT_CTX)		\
+VRT_r_##which##_##age(VRT_CTX)					\
 {								\
 								\
 	CHECK_OBJ_NOTNULL(ctx, VRT_CTX_MAGIC);			\
-	return(ctx->now - oc->t_origin);			\
+	return (ctx->now - oc->t_origin);			\
 }
 
 VRT_DO_AGE_R(obj, ctx->req->objcore)
@@ -567,7 +619,8 @@ VRT_r_req_xid(VRT_CTX)
 	CHECK_OBJ_NOTNULL(ctx, VRT_CTX_MAGIC);
 	CHECK_OBJ_NOTNULL(ctx->req, REQ_MAGIC);
 
-	return (WS_Printf(ctx->req->http->ws, "%u", VXID(ctx->req->vsl->wid)));
+	return (WS_Printf(ctx->req->http->ws, "%u",
+	    VXID(ctx->req->vsl->wid)));
 }
 
 const char *
@@ -577,7 +630,8 @@ VRT_r_bereq_xid(VRT_CTX)
 	CHECK_OBJ_NOTNULL(ctx, VRT_CTX_MAGIC);
 	CHECK_OBJ_NOTNULL(ctx->bo, BUSYOBJ_MAGIC);
 
-	return (WS_Printf(ctx->bo->bereq->ws, "%u", VXID(ctx->bo->vsl->wid)));
+	return (WS_Printf(ctx->bo->bereq->ws, "%u",
+	    VXID(ctx->bo->vsl->wid)));
 }
 
 /*--------------------------------------------------------------------
@@ -585,22 +639,22 @@ VRT_r_bereq_xid(VRT_CTX)
  */
 
 #define VREQW0(field)
-#define VREQW1(field)						\
+#define VREQW1(field)							\
 void									\
-VRT_l_req_##field(VRT_CTX, unsigned a)		\
+VRT_l_req_##field(VRT_CTX, unsigned a)					\
 {									\
 	CHECK_OBJ_NOTNULL(ctx, VRT_CTX_MAGIC);				\
-	CHECK_OBJ_NOTNULL(ctx->req, REQ_MAGIC);			\
+	CHECK_OBJ_NOTNULL(ctx->req, REQ_MAGIC);				\
 	ctx->req->field = a ? 1 : 0;					\
 }
 
 #define VREQR0(field)
-#define VREQR1(field)						\
+#define VREQR1(field)							\
 unsigned								\
-VRT_r_req_##field(VRT_CTX)				\
+VRT_r_req_##field(VRT_CTX)						\
 {									\
 	CHECK_OBJ_NOTNULL(ctx, VRT_CTX_MAGIC);				\
-	CHECK_OBJ_NOTNULL(ctx->req, REQ_MAGIC);			\
+	CHECK_OBJ_NOTNULL(ctx->req, REQ_MAGIC);				\
 	return (ctx->req->field);					\
 }
 
@@ -608,7 +662,6 @@ VRT_r_req_##field(VRT_CTX)				\
 	VREQR##r(l) \
 	VREQW##w(l)
 #include "tbl/req_flags.h"
-#undef REQ_FLAG
 
 /*--------------------------------------------------------------------*/
 
@@ -640,7 +693,7 @@ VRT_r_server_identity(VRT_CTX)
 	if (heritage.identity != NULL)
 		return (heritage.identity);
 	else
-		return (heritage.name);
+		return ("varnishd");
 }
 
 const char*
@@ -714,9 +767,47 @@ VRT_BODY_L(resp)
 
 /*--------------------------------------------------------------------*/
 
+static struct vmod_priv *
+vrt_do_blob(VRT_CTX, const char *err, const uint8_t *src, size_t len)
+{
+	struct vmod_priv *p;
+	void *d;
+
+	p = (void *)WS_Alloc(ctx->ws, sizeof *p);
+	d = WS_Copy(ctx->ws, src, len);
+	if (p == NULL || d == NULL) {
+		VRT_fail(ctx, "Workspace overflow (%s)", err);
+		return (NULL);
+	}
+	memset(p, 0, sizeof *p);
+	p->len = len;
+	p->priv = d;
+	return (p);
+}
+
+VCL_BLOB
+VRT_r_req_hash(VRT_CTX)
+{
+	CHECK_OBJ_NOTNULL(ctx, VRT_CTX_MAGIC);
+	CHECK_OBJ_NOTNULL(ctx->req, REQ_MAGIC);
+	return (vrt_do_blob(ctx, "req.hash", ctx->req->digest,
+	    DIGEST_LEN));
+}
+
+VCL_BLOB
+VRT_r_bereq_hash(VRT_CTX)
+{
+	CHECK_OBJ_NOTNULL(ctx, VRT_CTX_MAGIC);
+	CHECK_OBJ_NOTNULL(ctx->bo, BUSYOBJ_MAGIC);
+	return (vrt_do_blob(ctx, "bereq.hash", ctx->bo->digest,
+	    DIGEST_LEN));
+}
+
+/*--------------------------------------------------------------------*/
+
 #define HTTP_VAR(x)						\
 struct http *							\
-VRT_r_##x(VRT_CTX)				\
+VRT_r_##x(VRT_CTX)						\
 {								\
 	CHECK_OBJ_NOTNULL(ctx, VRT_CTX_MAGIC);			\
 	CHECK_OBJ_NOTNULL(ctx->http_##x, HTTP_MAGIC);		\
