@@ -61,16 +61,20 @@ wrk_bgthread(void *arg)
 {
 	struct bgthread *bt;
 	struct worker wrk;
+	struct dstat ds;
 
 	CAST_OBJ_NOTNULL(bt, arg, BGTHREAD_MAGIC);
 	THR_SetName(bt->name);
+	THR_Init();
 	INIT_OBJ(&wrk, WORKER_MAGIC);
+	memset(&ds, 0, sizeof ds);
+	wrk.stats = &ds;
 
 	(void)bt->func(&wrk, bt->priv);
 
 	WRONG("BgThread terminated");
 
-	NEEDLESS_RETURN(NULL);
+	NEEDLESS(return NULL);
 }
 
 void
@@ -93,6 +97,7 @@ static void
 WRK_Thread(struct pool *qp, size_t stacksize, unsigned thread_workspace)
 {
 	struct worker *w, ww;
+	struct dstat ds;
 	unsigned char ws[thread_workspace];
 	uintptr_t u;
 
@@ -104,6 +109,8 @@ WRK_Thread(struct pool *qp, size_t stacksize, unsigned thread_workspace)
 	w = &ww;
 	INIT_OBJ(w, WORKER_MAGIC);
 	w->lastused = NAN;
+	memset(&ds, 0, sizeof ds);
+	w->stats = &ds;
 	AZ(pthread_cond_init(&w->cond, NULL));
 
 	WS_Init(w->aws, "wrk", ws, thread_workspace);
@@ -138,27 +145,39 @@ pool_addstat(struct dstat *dst, struct dstat *src)
 {
 
 	dst->summs++;
-#define L0(n)
-#define L1(n) (dst->n += src->n)
-#define VSC_F(n,t,l,s,f,v,d,e)	L##l(n);
-#include "tbl/vsc_f_main.h"
-#undef VSC_F
-#undef L0
-#undef L1
+	VSC_main_Summ(dst, src);
 	memset(src, 0, sizeof *src);
 }
 
+static inline int
+pool_reserve(void)
+{
+	unsigned lim;
+
+	if (cache_param->wthread_reserve == 0)
+		return (cache_param->wthread_min / 20 + 1);
+	lim = cache_param->wthread_min * 950 / 1000;
+	if (cache_param->wthread_reserve > lim)
+		return (lim);
+	return (cache_param->wthread_reserve);
+}
+
 /*--------------------------------------------------------------------*/
 
 static struct worker *
-pool_getidleworker(struct pool *pp)
+pool_getidleworker(struct pool *pp, enum task_prio prio)
 {
-	struct pool_task *pt;
+	struct pool_task *pt = NULL;
 	struct worker *wrk;
 
 	CHECK_OBJ_NOTNULL(pp, POOL_MAGIC);
 	Lck_AssertHeld(&pp->mtx);
-	pt = VTAILQ_FIRST(&pp->idle_queue);
+	if (prio <= TASK_QUEUE_RESERVE || pp->nidle > pool_reserve()) {
+		pt = VTAILQ_FIRST(&pp->idle_queue);
+		if (pt == NULL)
+			AZ(pp->nidle);
+	}
+
 	if (pt == NULL) {
 		if (pp->nthr < cache_param->wthread_max) {
 			pp->dry++;
@@ -179,7 +198,7 @@ pool_getidleworker(struct pool *pp)
  */
 
 int
-Pool_Task_Arg(struct worker *wrk, task_func_t *func,
+Pool_Task_Arg(struct worker *wrk, enum task_prio prio, task_func_t *func,
     const void *arg, size_t arg_len)
 {
 	struct pool *pp;
@@ -193,9 +212,11 @@ Pool_Task_Arg(struct worker *wrk, task_func_t *func,
 	CHECK_OBJ_NOTNULL(pp, POOL_MAGIC);
 
 	Lck_Lock(&pp->mtx);
-	wrk2 = pool_getidleworker(pp);
+	wrk2 = pool_getidleworker(pp, prio);
 	if (wrk2 != NULL) {
+		AN(pp->nidle);
 		VTAILQ_REMOVE(&pp->idle_queue, &wrk2->task, list);
+		pp->nidle--;
 		retval = 1;
 	} else {
 		wrk2 = wrk;
@@ -218,23 +239,24 @@ Pool_Task_Arg(struct worker *wrk, task_func_t *func,
  */
 
 int
-Pool_Task(struct pool *pp, struct pool_task *task, enum task_how how)
+Pool_Task(struct pool *pp, struct pool_task *task, enum task_prio prio)
 {
 	struct worker *wrk;
 	int retval = 0;
-
 	CHECK_OBJ_NOTNULL(pp, POOL_MAGIC);
 	AN(task);
 	AN(task->func);
-	assert(how < TASK_QUEUE_END);
+	assert(prio < TASK_QUEUE_END);
 
 	Lck_Lock(&pp->mtx);
 
 	/* The common case first:  Take an idle thread, do it. */
 
-	wrk = pool_getidleworker(pp);
+	wrk = pool_getidleworker(pp, prio);
 	if (wrk != NULL) {
+		AN(pp->nidle);
 		VTAILQ_REMOVE(&pp->idle_queue, &wrk->task, list);
+		pp->nidle--;
 		AZ(wrk->task.func);
 		wrk->task.func = task->func;
 		wrk->task.priv = task->priv;
@@ -243,15 +265,21 @@ Pool_Task(struct pool *pp, struct pool_task *task, enum task_how how)
 		return (0);
 	}
 
-	/* Acceptors are not subject to queue limits */
-	if (how == TASK_QUEUE_VCA ||
-	    pp->lqueue < cache_param->wthread_max +
-	    cache_param->wthread_queue_limit + pp->nthr) {
+	/*
+	 * queue limits only apply to client threads - all other
+	 * work is vital and needs do be done at the earliest
+	 */
+	if (!TASK_QUEUE_CLIENT(prio) ||
+	    pp->lqueue + pp->nthr < cache_param->wthread_max +
+	    cache_param->wthread_queue_limit) {
 		pp->nqueued++;
 		pp->lqueue++;
-		VTAILQ_INSERT_TAIL(&pp->queues[how], task, list);
+		VTAILQ_INSERT_TAIL(&pp->queues[prio], task, list);
 	} else {
-		pp->ndropped++;
+		if (prio == TASK_QUEUE_REQ)
+			pp->sdropped++;
+		else
+			pp->rdropped++;
 		retval = -1;
 	}
 	Lck_Unlock(&pp->mtx);
@@ -277,9 +305,9 @@ pool_kiss_of_death(struct worker *wrk, void *priv)
 static void
 Pool_Work_Thread(struct pool *pp, struct worker *wrk)
 {
-	struct pool_task *tp;
+	struct pool_task *tp = NULL;
 	struct pool_task tpx, tps;
-	int i;
+	int i, prio_lim;
 
 	CHECK_OBJ_NOTNULL(pp, POOL_MAGIC);
 	wrk->pool = pp;
@@ -288,10 +316,15 @@ Pool_Work_Thread(struct pool *pp, struct worker *wrk)
 
 		CHECK_OBJ_NOTNULL(wrk, WORKER_MAGIC);
 
-		WS_Reset(wrk->aws, NULL);
+		WS_Reset(wrk->aws, 0);
 		AZ(wrk->vsl);
 
-		for (i = 0; i < TASK_QUEUE_END; i++) {
+		if (pp->nidle < pool_reserve())
+			prio_lim = TASK_QUEUE_RESERVE + 1;
+		else
+			prio_lim = TASK_QUEUE_END;
+
+		for (i = 0; i < prio_lim; i++) {
 			tp = VTAILQ_FIRST(&pp->queues[i]);
 			if (tp != NULL) {
 				pp->lqueue--;
@@ -320,6 +353,7 @@ Pool_Work_Thread(struct pool *pp, struct worker *wrk)
 			wrk->task.func = NULL;
 			wrk->task.priv = wrk;
 			VTAILQ_INSERT_HEAD(&pp->idle_queue, &wrk->task, list);
+			pp->nidle++;
 			do {
 				i = Lck_CondWait(&wrk->cond, &pp->mtx,
 				    wrk->vcl == NULL ?  0 : wrk->lastused+60.);
@@ -368,6 +402,7 @@ pool_thread(void *priv)
 	struct pool_info *pi;
 
 	CAST_OBJ_NOTNULL(pi, priv, POOL_INFO_MAGIC);
+	THR_Init();
 	WRK_Thread(pi->qp, pi->stacksize, cache_param->workspace_thread);
 	FREE_OBJ(pi);
 	return (NULL);
@@ -416,13 +451,14 @@ pool_breed(struct pool *qp)
 /*--------------------------------------------------------------------
  * Herd a single pool
  *
- * This thread wakes up whenever a pool queues.
+ * This thread wakes up every thread_pool_timeout seconds, whenever a pool
+ * queues and when threads need to be destroyed
  *
- * The trick here is to not be too aggressive about creating threads.
- * We do this by only examining one pool at a time, and by sleeping
- * a short while whenever we create a thread and a little while longer
- * whenever we fail to, hopefully missing a lot of cond_signals in
- * the meantime.
+ * The trick here is to not be too aggressive about creating threads.  In
+ * pool_breed(), we sleep whenever we create a thread and a little while longer
+ * whenever we fail to, hopefully missing a lot of cond_signals in the meantime.
+ *
+ * Idle threads are destroyed at a rate determined by wthread_destroy_delay
  *
  * XXX: probably need a lot more work.
  *
@@ -435,43 +471,59 @@ pool_herder(void *priv)
 	struct pool_task *pt;
 	double t_idle;
 	struct worker *wrk;
+	double delay;
+	int wthread_min;
 
 	CAST_OBJ_NOTNULL(pp, priv, POOL_MAGIC);
 
-	while (1) {
+	THR_SetName("pool_herder");
+	THR_Init();
+
+	while (!pp->die || pp->nthr > 0) {
+		wthread_min = cache_param->wthread_min;
+		if (pp->die)
+			wthread_min = 0;
+
 		/* Make more threads if needed and allowed */
-		if (pp->nthr < cache_param->wthread_min ||
+		if (pp->nthr < wthread_min ||
 		    (pp->dry && pp->nthr < cache_param->wthread_max)) {
 			pool_breed(pp);
 			continue;
 		}
-		assert(pp->nthr >= cache_param->wthread_min);
 
-		if (pp->nthr > cache_param->wthread_min) {
+		delay = cache_param->wthread_timeout;
+		assert(pp->nthr >= wthread_min);
+
+		if (pp->nthr > wthread_min) {
 
 			t_idle = VTIM_real() - cache_param->wthread_timeout;
 
 			Lck_Lock(&pp->mtx);
 			/* XXX: unsafe counters */
 			VSC_C_main->sess_queued += pp->nqueued;
-			VSC_C_main->sess_dropped += pp->ndropped;
-			pp->nqueued = pp->ndropped = 0;
+			VSC_C_main->sess_dropped += pp->sdropped;
+			VSC_C_main->req_dropped += pp->rdropped;
+			pp->nqueued = pp->sdropped = pp->rdropped = 0;
 
 			wrk = NULL;
 			pt = VTAILQ_LAST(&pp->idle_queue, taskhead);
 			if (pt != NULL) {
+				AN(pp->nidle);
 				AZ(pt->func);
 				CAST_OBJ_NOTNULL(wrk, pt->priv, WORKER_MAGIC);
 
-				if (wrk->lastused < t_idle ||
+				if (pp->die || wrk->lastused < t_idle ||
 				    pp->nthr > cache_param->wthread_max) {
 					/* Give it a kiss on the cheek... */
 					VTAILQ_REMOVE(&pp->idle_queue,
 					    &wrk->task, list);
+					pp->nidle--;
 					wrk->task.func = pool_kiss_of_death;
 					AZ(pthread_cond_signal(&wrk->cond));
-				} else
+				} else {
+					delay = wrk->lastused - t_idle;
 					wrk = NULL;
+				}
 			}
 			Lck_Unlock(&pp->mtx);
 
@@ -481,15 +533,25 @@ pool_herder(void *priv)
 				VSC_C_main->threads--;
 				VSC_C_main->threads_destroyed++;
 				Lck_Unlock(&pool_mtx);
-				VTIM_sleep(cache_param->wthread_destroy_delay);
-				continue;
-			}
+				delay = cache_param->wthread_destroy_delay;
+			} else if (delay < cache_param->wthread_destroy_delay)
+				delay = cache_param->wthread_destroy_delay;
 		}
 
+		if (pp->die) {
+			if (delay < 2)
+				delay = 10e-3;
+			else
+				delay = 1;
+			VTIM_sleep(delay);
+			continue;
+		}
 		Lck_Lock(&pp->mtx);
 		if (!pp->dry) {
+			if (DO_DEBUG(DBG_VTC_MODE))
+				delay = 0.5;
 			(void)Lck_CondWait(&pp->herder_cond, &pp->mtx,
-				VTIM_real() + 5);
+				VTIM_real() + delay);
 		} else {
 			/* XXX: unsafe counters */
 			VSC_C_main->threads_limited++;
@@ -497,5 +559,5 @@ pool_herder(void *priv)
 		}
 		Lck_Unlock(&pp->mtx);
 	}
-	NEEDLESS_RETURN(NULL);
+	return (NULL);
 }
