@@ -29,15 +29,29 @@
  * This file contains the heritage passed when mgt forks cache
  */
 
-struct vsm_sc;
+struct vsmw;
 struct suckaddr;
+struct listen_sock;
+struct transport;
+
+struct listen_arg {
+	unsigned			magic;
+#define LISTEN_ARG_MAGIC		0xbb2fc333
+	VTAILQ_ENTRY(listen_arg)	list;
+	const char			*endpoint;
+	const char			*name;
+	VTAILQ_HEAD(,listen_sock)	socks;
+	const struct transport		*transport;
+};
 
 struct listen_sock {
 	unsigned			magic;
 #define LISTEN_SOCK_MAGIC		0x999e4b57
 	VTAILQ_ENTRY(listen_sock)	list;
+	VTAILQ_ENTRY(listen_sock)	arglist;
 	int				sock;
-	char				*name;
+	char				*endpoint;
+	const char			*name;
 	struct suckaddr			*addr;
 	const struct transport		*transport;
 };
@@ -53,17 +67,16 @@ struct heritage {
 	/* File descriptor for stdout/stderr */
 	int				std_fd;
 
+	int				vsm_fd;
+
 	/* Sockets from which to accept connections */
 	struct listen_sock_head		socks;
 
 	/* Hash method */
 	const struct hash_slinger	*hash;
 
-	struct vsm_sc			*vsm;
-
 	struct params			*param;
 
-	char				*name;
 	const char			*identity;
 
 	char				*panic_str;
