@@ -69,12 +69,12 @@ void HSH_Init(const struct hash_slinger *slinger);
 void HSH_AddString(struct req *, void *ctx, const char *str);
 void HSH_Insert(struct worker *, const void *hash, struct objcore *,
     struct ban *);
-void HSH_Purge(struct worker *, struct objhead *, double ttl, double grace,
+unsigned HSH_Purge(struct worker *, struct objhead *, double ttl, double grace,
     double keep);
 void HSH_config(const char *h_arg);
 struct boc *HSH_RefBoc(const struct objcore *);
 void HSH_DerefBoc(struct worker *wrk, struct objcore *);
-struct objcore *HSH_Private(struct worker *wrk);
+struct objcore *HSH_Private(const struct worker *wrk);
 void HSH_Abandon(struct objcore *oc);
 int HSH_Snipe(const struct worker *, struct objcore *);
 void HSH_Kill(struct objcore *);
@@ -107,7 +107,7 @@ struct objhead {
 
 void HSH_Fail(struct objcore *);
 void HSH_Unbusy(struct worker *, struct objcore *);
-void HSH_DeleteObjHead(struct worker *, struct objhead *);
+void HSH_DeleteObjHead(const struct worker *, struct objhead *);
 int HSH_DerefObjHead(struct worker *, struct objhead **);
 int HSH_DerefObjCore(struct worker *, struct objcore **, int);
 #define HSH_RUSH_POLICY -1
