@@ -99,8 +99,6 @@ V1D_Deliver(struct req *req, struct boc *boc, int sendbody)
 		}
 	}
 
-	VSLb(req->vsl, SLT_Debug, "RES_MODE %x", req->res_mode);
-
 	if (!req->doclose && http_HdrIs(req->resp, H_Connection, "close")) {
 		req->doclose = SC_RESP_CLOSE;
 	} else if (req->doclose) {
@@ -136,7 +134,7 @@ V1D_Deliver(struct req *req, struct boc *boc, int sendbody)
 	}
 
 	if ((V1L_FlushRelease(req->wrk) || err) && req->sp->fd >= 0)
-		SES_Close(req->sp, SC_REM_CLOSE);
+		Req_Fail(req, SC_REM_CLOSE);
 	AZ(req->wrk->v1l);
 	VDP_close(req);
 }
