@@ -31,15 +31,15 @@
 
 #include "config.h"
 
+#include "cache/cache.h"
+
 #include <poll.h>
 #include <stdio.h>
 
-#include "cache/cache.h"
-
-#include "vrt.h"
-
 #include "cache_http1.h"
 
+#include "VSC_vbe.h"
+
 static struct lock pipestat_mtx;
 
 static int
@@ -63,7 +63,7 @@ rdf(int fd0, int fd1, uint64_t *pcnt)
 }
 
 void
-V1P_Charge(struct req *req, const struct v1p_acct *a, struct VSC_C_vbe *b)
+V1P_Charge(struct req *req, const struct v1p_acct *a, struct VSC_vbe *b)
 {
 
 	AN(b);
