@@ -46,6 +46,8 @@
 #include "config.h"
 
 #include "cache/cache.h"
+#include "cache/cache_transport.h"
+
 #include "cache_http1.h"
 
 #include "vct.h"
@@ -66,11 +68,16 @@ enum htc_status_e __match_proto__(htc_complete_f)
 HTTP1_Complete(struct http_conn *htc)
 {
 	char *p;
+	enum htc_status_e retval;
 
 	CHECK_OBJ_NOTNULL(htc, HTTP_CONN_MAGIC);
 
 	assert(htc->rxbuf_e >= htc->rxbuf_b);
-	assert(*htc->rxbuf_e == '\0');
+	assert(htc->rxbuf_e <= htc->ws->r);
+
+	if (htc->rxbuf_e == htc->ws->r)
+		return (HTC_S_OVERFLOW);		// No space for NUL
+	*htc->rxbuf_e = '\0';
 
 	/* Skip any leading white space */
 	for (p = htc->rxbuf_b ; vct_islws(*p); p++)
@@ -78,6 +85,11 @@ HTTP1_Complete(struct http_conn *htc)
 	if (p == htc->rxbuf_e)
 		return (HTC_S_EMPTY);
 
+	/* Do not return a partial H2 connection preface */
+	retval = H2_prism_complete(htc);
+	if (retval != HTC_S_JUNK)
+		return (retval);
+
 	/*
 	 * Here we just look for NL[CR]NL to see that reception
 	 * is completed.  More stringent validation happens later.
@@ -277,6 +289,8 @@ http1_splitline(struct http *hp, struct http_conn *htc, const int *hf,
 	*p = '\0';
 	p += i;
 
+	http_Proto(hp);
+
 	return (http1_dissect_hdrs(hp, p, htc, maxhdr));
 }
 
@@ -332,19 +346,6 @@ http1_body_status(const struct http *hp, struct http_conn *htc, int request)
 
 /*--------------------------------------------------------------------*/
 
-static int8_t
-http1_proto_ver(const struct http *hp)
-{
-	if (!strcasecmp(hp->hd[HTTP_HDR_PROTO].b, "HTTP/1.0"))
-		return (10);
-	else if (!strcasecmp(hp->hd[HTTP_HDR_PROTO].b, "HTTP/1.1"))
-		return (11);
-	else
-		return (0);
-}
-
-/*--------------------------------------------------------------------*/
-
 uint16_t
 HTTP1_DissectRequest(struct http_conn *htc, struct http *hp)
 {
@@ -359,8 +360,8 @@ HTTP1_DissectRequest(struct http_conn *htc, struct http *hp)
 	    HTTP1_Req, cache_param->http_req_hdr_len);
 	if (retval != 0)
 		return (retval);
-	hp->protover = http1_proto_ver(hp);
-	if (hp->protover == 0)
+
+	if (hp->protover < 10 || hp->protover > 11)
 		return (400);
 
 	if (http_CountHdr(hp, H_Host) > 1)
@@ -410,29 +411,24 @@ HTTP1_DissectRequest(struct http_conn *htc, struct http *hp)
 
 uint16_t
 HTTP1_DissectResponse(struct http_conn *htc, struct http *hp,
-    const struct http *req)
+    const struct http *rhttp)
 {
 	uint16_t retval = 0;
 	const char *p;
-	int8_t rv;
-
 
 	CHECK_OBJ_NOTNULL(htc, HTTP_CONN_MAGIC);
 	CHECK_OBJ_NOTNULL(hp, HTTP_MAGIC);
-	CHECK_OBJ_NOTNULL(req, HTTP_MAGIC);
+	CHECK_OBJ_NOTNULL(rhttp, HTTP_MAGIC);
 
 	if (http1_splitline(hp, htc,
 	    HTTP1_Resp, cache_param->http_resp_hdr_len))
 		retval = 503;
 
-	if (retval == 0) {
-		hp->protover = http1_proto_ver(hp);
-		if (hp->protover == 0)
-			retval = 503;
-		rv = http1_proto_ver(req);
-		if (hp->protover > rv)
-			hp->protover = rv;
-	}
+	if (retval == 0 && hp->protover < 10)
+		retval = 503;
+
+	if (retval == 0 && hp->protover > rhttp->protover)
+		http_SetH(hp, HTTP_HDR_PROTO, rhttp->hd[HTTP_HDR_PROTO].b);
 
 	if (retval == 0 && Tlen(hp->hd[HTTP_HDR_STATUS]) != 3)
 		retval = 503;
