@@ -34,7 +34,6 @@
 #include <sys/types.h>
 #include <sys/socket.h>
 #include <errno.h>
-#include <fcntl.h>
 #include <stdio.h>
 #include <stdlib.h>
 #include <string.h>
@@ -49,26 +48,26 @@
 #include "vss.h"
 #include "vtcp.h"
 
+static VTAILQ_HEAD(,listen_arg) listen_args =
+    VTAILQ_HEAD_INITIALIZER(listen_args);
+
 static int
-mac_opensocket(struct listen_sock *ls, struct cli *cli)
+mac_opensocket(struct listen_sock *ls)
 {
 	int fail;
 
 	CHECK_OBJ_NOTNULL(ls, LISTEN_SOCK_MAGIC);
 	if (ls->sock > 0) {
-		mgt_child_inherit(ls->sock, NULL);
-		AZ(close(ls->sock));
+		MCH_Fd_Inherit(ls->sock, NULL);
+		closefd(&ls->sock);
 	}
 	ls->sock = VTCP_bind(ls->addr, NULL);
 	fail = errno;
 	if (ls->sock < 0) {
-		if (cli != NULL)
-			VCLI_Out(cli, "Could not get socket %s: %s\n",
-			    ls->name, strerror(errno));
 		AN(fail);
 		return (fail);
 	}
-	mgt_child_inherit(ls->sock, "sock");
+	MCH_Fd_Inherit(ls->sock, "sock");
 	return (0);
 }
 
@@ -80,119 +79,109 @@ void
 MAC_reopen_sockets(struct cli *cli)
 {
 	struct listen_sock *ls;
+	int fail;
 
-	VJ_master(JAIL_MASTER_PRIVPORT);
-	VTAILQ_FOREACH(ls, &heritage.socks, list)
-		(void)mac_opensocket(ls, cli);
-	VJ_master(JAIL_MASTER_LOW);
-}
-
-/*=====================================================================
- * Make sure we have all our sockets (and try once more to get them)
- */
-
-int
-MAC_sockets_ready(struct cli *cli)
-{
-	int retval = 1;
-	struct listen_sock *ls;
-
-	VJ_master(JAIL_MASTER_PRIVPORT);
 	VTAILQ_FOREACH(ls, &heritage.socks, list) {
-		if (ls->sock < 0)
-			(void)mac_opensocket(ls, cli);
-		if (ls->sock < 0)
-			retval = 0;
+		VJ_master(JAIL_MASTER_PRIVPORT);
+		fail = mac_opensocket(ls);
+		VJ_master(JAIL_MASTER_LOW);
+		if (fail == 0)
+			continue;
+		if (cli == NULL)
+			MGT_Complain(C_ERR,
+			    "Could not reopen listen socket %s: %s",
+			    ls->endpoint, strerror(fail));
+		else
+			VCLI_Out(cli,
+			    "Could not reopen listen socket %s: %s\n",
+			    ls->endpoint, strerror(fail));
 	}
-	VJ_master(JAIL_MASTER_LOW);
-	return (retval);
 }
 
 /*--------------------------------------------------------------------*/
 
-struct mac_help {
-	unsigned		magic;
-#define MAC_HELP_MAGIC		0x1e00a9d9
-	int			good;
-	const char		*name;
-	const struct transport	*transport;
-};
-
 static int __match_proto__(vss_resolved_f)
 mac_callback(void *priv, const struct suckaddr *sa)
 {
-	struct mac_help *mh;
+	struct listen_arg *la;
 	struct listen_sock *ls;
+	char abuf[VTCP_ADDRBUFSIZE], pbuf[VTCP_PORTBUFSIZE];
+	char nbuf[VTCP_ADDRBUFSIZE+VTCP_PORTBUFSIZE+2];
+	int fail;
 
-	CAST_OBJ_NOTNULL(mh, priv, MAC_HELP_MAGIC);
+	CAST_OBJ_NOTNULL(la, priv, LISTEN_ARG_MAGIC);
 
 	VTAILQ_FOREACH(ls, &heritage.socks, list) {
 		if (!VSA_Compare(sa, ls->addr))
 			ARGV_ERR("-a arguments %s and %s have same address\n",
-			    ls->name, mh->name);
+			    ls->endpoint, la->endpoint);
 	}
 	ALLOC_OBJ(ls, LISTEN_SOCK_MAGIC);
 	AN(ls);
 	ls->sock = -1;
 	ls->addr = VSA_Clone(sa);
 	AN(ls->addr);
-	ls->name = strdup(mh->name);
-	AN(ls->name);
-	ls->transport = mh->transport;
+	ls->endpoint = strdup(la->endpoint);
+	AN(ls->endpoint);
+	ls->name = la->name;
+	ls->transport = la->transport;
+	VJ_master(JAIL_MASTER_PRIVPORT);
+	fail = mac_opensocket(ls);
+	VJ_master(JAIL_MASTER_LOW);
+	if (fail) {
+		free(ls->addr);
+		free(ls->endpoint);
+		FREE_OBJ(ls);
+		if (fail != EAFNOSUPPORT)
+			ARGV_ERR("Could not get socket %s: %s\n",
+			    la->endpoint, strerror(fail));
+		return(0);
+	}
+	if (VSA_Port(ls->addr) == 0) {
+		/*
+		 * If the argv port number is zero, we adopt whatever
+		 * port number this VTCP_bind() found us, as if
+		 * it was specified by the argv.
+		 */
+		free(ls->addr);
+		ls->addr = VTCP_my_suckaddr(ls->sock);
+		VTCP_myname(ls->sock, abuf, sizeof abuf,
+		    pbuf, sizeof pbuf);
+		bprintf(nbuf, "%s:%s", abuf, pbuf);
+		REPLACE(ls->endpoint, nbuf);
+	}
+	VTAILQ_INSERT_TAIL(&la->socks, ls, arglist);
 	VTAILQ_INSERT_TAIL(&heritage.socks, ls, list);
-	mh->good++;
 	return (0);
 }
 
 void
-MAC_Validate(void)
-{
-	struct listen_sock *ls;
-	int fail;
-	char abuf[VTCP_ADDRBUFSIZE], pbuf[VTCP_PORTBUFSIZE];
-	char nbuf[VTCP_ADDRBUFSIZE+VTCP_PORTBUFSIZE+2];
-
-	VTAILQ_FOREACH(ls, &heritage.socks, list) {
-		VJ_master(JAIL_MASTER_PRIVPORT);
-		fail = mac_opensocket(ls, NULL);
-		VJ_master(JAIL_MASTER_LOW);
-		if (fail)
-			ARGV_ERR("Cannot open socket: %s: %s\n",
-			    ls->name, strerror(fail));
-		if (VSA_Port(ls->addr) == 0) {
-			/*
-			 * If the port number is zero, we adopt whatever
-			 * port number this VTCP_bind() found us, as if
-			 * specified by argument.
-			 */
-			free(ls->addr);
-			ls->addr = VTCP_my_suckaddr(ls->sock);
-			VTCP_myname(ls->sock, abuf, sizeof abuf,
-			    pbuf, sizeof pbuf);
-			bprintf(nbuf, "%s:%s", abuf, pbuf);
-			REPLACE(ls->name, nbuf);
-		}
-	}
-}
-
-void
-MAC_Arg(const char *arg)
+MAC_Arg(const char *spec)
 {
 	char **av;
-	struct mac_help *mh;
+	struct listen_arg *la;
 	const char *err;
 	int error;
 	const struct transport *xp;
+	const char *name;
+	char name_buf[8];
+	static unsigned seq = 0;
 
-	av = VAV_Parse(arg, NULL, ARGV_COMMA);
-	if (av == NULL)
-		ARGV_ERR("Parse error: out of memory\n");
-	if (av[0] != NULL)
-		ARGV_ERR("%s\n", av[0]);
+	av = MGT_NamedArg(spec, &name, "-a");
+	AN(av);
 
-	ALLOC_OBJ(mh, MAC_HELP_MAGIC);
-	AN(mh);
-	mh->name = av[1];
+	ALLOC_OBJ(la, LISTEN_ARG_MAGIC);
+	AN(la);
+	VTAILQ_INIT(&la->socks);
+	VTAILQ_INSERT_TAIL(&listen_args, la, list);
+	la->endpoint = av[1];
+
+	if (name == NULL) {
+		bprintf(name_buf, "a%u", seq++);
+		name = strdup(name_buf);
+		AN(name);
+	}
+	la->name = name;
 
 	if (av[2] == NULL) {
 		xp = XPORT_Find("http/1");
@@ -204,11 +193,10 @@ MAC_Arg(const char *arg)
 			ARGV_ERR("Too many sub-arguments to -a(%s)\n", av[2]);
 	}
 	AN(xp);
-	mh->transport = xp;
+	la->transport = xp;
 
-	error = VSS_resolver(av[1], "80", mac_callback, mh, &err);
-	if (mh->good == 0 || error)
-		ARGV_ERR("socket %s didn't resolve\n", av[1]);
+	error = VSS_resolver(av[1], "80", mac_callback, la, &err);
+	if (VTAILQ_EMPTY(&la->socks) || error)
+		ARGV_ERR("Got no socket(s) for %s\n", av[1]);
 	VAV_Free(av);
-	FREE_OBJ(mh);
 }
