@@ -34,25 +34,29 @@
 #include <sys/types.h>
 #include <sys/wait.h>
 
+#include <errno.h>
 #include <fcntl.h>
 #include <poll.h>
 #include <signal.h>
+#include <stdarg.h>
 #include <stdio.h>
-#include <stdlib.h>
 #include <string.h>
 #include <syslog.h>
 #include <unistd.h>
 
 #include "mgt/mgt.h"
 #include "common/heritage.h"
+#include "common/common_vsm.h"
 
 #include "vbm.h"
 #include "vcli_serve.h"
 #include "vev.h"
+#include "vfil.h"
 #include "vlu.h"
 #include "vtim.h"
+#include "vsmw.h"
 
-pid_t			child_pid = -1;
+static pid_t		child_pid = -1;
 
 static struct vbitmap	*fd_map;
 
@@ -81,23 +85,14 @@ static struct vev	*ev_listen;
 static struct vlu	*child_std_vlu;
 
 static struct vsb *child_panic = NULL;
-static double mgt_uptime_t0 = 0.;
+
+#ifdef HAVE_SIGALTSTACK
+#include <sys/mman.h>
+stack_t altstack;
+#endif
 
 static void mgt_reap_child(void);
 
-/*---------------------------------------------------------------------
- * A handy little function
- */
-
-static inline void
-closex(int *fd)
-{
-
-	assert(*fd >= 0);
-	AZ(close(*fd));
-	*fd = -1;
-}
-
 /*=====================================================================
  * Panic string evacuation and handling
  */
@@ -117,7 +112,7 @@ mgt_panic_record(pid_t r)
 	    strnlen(heritage.panic_str, heritage.panic_str_len),
 	    VSB_QUOTE_NONL);
 	AZ(VSB_finish(child_panic));
-	MGT_complain(C_ERR, "Child (%jd) %s",
+	MGT_Complain(C_ERR, "Child (%jd) %s",
 	    (intmax_t)r, VSB_data(child_panic));
 }
 
@@ -128,7 +123,7 @@ mgt_panic_clear(void)
 }
 
 static void __match_proto__(cli_func_t)
-mcf_panic_show(struct cli *cli, const char * const *av, void *priv)
+mch_cli_panic_show(struct cli *cli, const char * const *av, void *priv)
 {
 	(void)av;
 	(void)priv;
@@ -144,7 +139,7 @@ mcf_panic_show(struct cli *cli, const char * const *av, void *priv)
 }
 
 static void __match_proto__(cli_func_t)
-mcf_panic_clear(struct cli *cli, const char * const *av, void *priv)
+mch_cli_panic_clear(struct cli *cli, const char * const *av, void *priv)
 {
 	(void)priv;
 
@@ -153,7 +148,7 @@ mcf_panic_clear(struct cli *cli, const char * const *av, void *priv)
 		VCLI_Out(cli, "Unknown parameter \"%s\".", av[2]);
 		return;
 	} else if (av[2] != NULL) {
-		VSC_C_mgt->child_panic = static_VSC_C_mgt.child_panic = 0;
+		VSC_C_mgt->child_panic = 0;
 		if (child_panic == NULL)
 			return;
 	}
@@ -181,7 +176,7 @@ static int		mgt_max_fd;
 #define CLOSE_FD_UP_TO	(mgt_max_fd + 100)
 
 void
-mgt_got_fd(int fd)
+MCH_TrackHighFd(int fd)
 {
 	/*
 	 * Assert > 0, to catch bogus opens, we know where stdin goes
@@ -198,7 +193,7 @@ mgt_got_fd(int fd)
  */
 
 void
-mgt_child_inherit(int fd, const char *what)
+MCH_Fd_Inherit(int fd, const char *what)
 {
 
 	assert(fd >= 0);
@@ -215,12 +210,12 @@ mgt_child_inherit(int fd, const char *what)
  * Listen to stdout+stderr from the child
  */
 
-static int
+static int __match_proto__(vlu_f)
 child_line(void *priv, const char *p)
 {
 	(void)priv;
 
-	MGT_complain(C_INFO, "Child (%jd) said %s", (intmax_t)child_pid, p);
+	MGT_Complain(C_INFO, "Child (%jd) said %s", (intmax_t)child_pid, p);
 	return (0);
 }
 
@@ -258,33 +253,40 @@ child_poker(const struct vev *e, int what)
 	if (child_pid < 0)
 		return (0);
 	if (mgt_cli_askchild(&status, &r, "ping\n") || strncmp("PONG ", r, 5)) {
-		MGT_complain(C_ERR, "Unexpected reply from ping: %u %s",
+		MGT_Complain(C_ERR, "Unexpected reply from ping: %u %s",
 		    status, r);
 		if (status != CLIS_COMMS)
-			MGT_Child_Cli_Fail();
+			MCH_Cli_Fail();
 	}
 	free(r);
 	return 0;
 }
 
 /*=====================================================================
- * SIGSEGV handler for child process
+ * signal handler for child process
  */
 
 static void __match_proto__()
-child_sigsegv_handler(int s, siginfo_t *si, void *c)
+child_signal_handler(int s, siginfo_t *si, void *c)
 {
 	char buf[1024];
+	struct sigaction sa;
 
-	(void)s;
 	(void)c;
 
-	sprintf(buf, "Segmentation fault by instruction at %p", si->si_addr);
+	/* Don't come back */
+	memset(&sa, 0, sizeof sa);
+	sa.sa_handler = SIG_DFL;
+	(void)sigaction(SIGSEGV, &sa, NULL);
+	(void)sigaction(SIGABRT, &sa, NULL);
+
+	bprintf(buf, "Signal %d (%s) received at %p si_code %d",
+		s, strsignal(s), si->si_addr, si->si_code);
 	VAS_Fail(__func__,
 		 __FILE__,
 		 __LINE__,
 		 buf,
-		 VAS_ASSERT);
+		 VAS_WRONG);
 }
 
 /*=====================================================================
@@ -298,35 +300,24 @@ mgt_launch_child(struct cli *cli)
 	unsigned u;
 	char *p;
 	struct vev *e;
-	int i, j, k, cp[2];
+	int i, cp[2];
 	struct sigaction sa;
 
 	if (child_state != CH_STOPPED && child_state != CH_DIED)
 		return;
 
-	if (!MAC_sockets_ready(cli)) {
-		child_state = CH_STOPPED;
-		if (cli != NULL) {
-			VCLI_SetResult(cli, CLIS_CANT);
-			return;
-		}
-		MGT_complain(C_ERR,
-		    "Child start failed: could not open sockets");
-		return;
-	}
-
 	child_state = CH_STARTING;
 
-	/* Open pipe for mgr->child CLI */
+	/* Open pipe for mgt->child CLI */
 	AZ(pipe(cp));
 	heritage.cli_in = cp[0];
-	mgt_child_inherit(heritage.cli_in, "cli_in");
+	MCH_Fd_Inherit(heritage.cli_in, "cli_in");
 	child_cli_out = cp[1];
 
-	/* Open pipe for child->mgr CLI */
+	/* Open pipe for child->mgt CLI */
 	AZ(pipe(cp));
 	heritage.cli_out = cp[1];
-	mgt_child_inherit(heritage.cli_out, "cli_out");
+	MCH_Fd_Inherit(heritage.cli_out, "cli_out");
 	child_cli_in = cp[0];
 
 	/*
@@ -337,20 +328,19 @@ mgt_launch_child(struct cli *cli)
 	heritage.std_fd = cp[1];
 	child_output = cp[0];
 
-	AN(heritage.vsm);
-	mgt_SHM_Size_Adjust();
-	AN(heritage.vsm);
+	mgt_SHM_ChildNew();
+
 	AN(heritage.param);
+	AN(heritage.panic_str);
 	if ((pid = fork()) < 0) {
-		/* XXX */
 		perror("Could not fork child");
-		exit(1);
+		exit(1);		// XXX Harsh ?
 	}
 	if (pid == 0) {
 
+
 		/* Redirect stdin/out/err */
-		AZ(close(STDIN_FILENO));
-		assert(open("/dev/null", O_RDONLY) == STDIN_FILENO);
+		VFIL_null_fd(STDIN_FILENO);
 		assert(dup2(heritage.std_fd, STDOUT_FILENO) == STDOUT_FILENO);
 		assert(dup2(heritage.std_fd, STDERR_FILENO) == STDERR_FILENO);
 
@@ -370,47 +360,58 @@ mgt_launch_child(struct cli *cli)
 		for (i = STDERR_FILENO + 1; i < CLOSE_FD_UP_TO; i++) {
 			if (vbit_test(fd_map, i))
 				continue;
-			if (close(i) == 0) {
-				k = open("/dev/null", O_RDONLY);
-				assert(k >= 0);
-				j = dup2(k, i);
-				assert(j == i);
-				AZ(close(k));
-			}
+			if (close(i) == 0)
+				VFIL_null_fd(i);
 		}
-#ifdef HAVE_SETPROCTITLE
-		setproctitle("Varnish-Chld %s", heritage.name);
-#endif
-
+		mgt_ProcTitle("Child");
 		if (mgt_param.sigsegv_handler) {
 			memset(&sa, 0, sizeof sa);
-			sa.sa_sigaction = child_sigsegv_handler;
+			sa.sa_sigaction = child_signal_handler;
 			sa.sa_flags = SA_SIGINFO;
-			(void)sigaction(SIGSEGV, &sa, NULL);
 			(void)sigaction(SIGBUS, &sa, NULL);
 			(void)sigaction(SIGABRT, &sa, NULL);
+
+#ifdef HAVE_SIGALTSTACK
+			size_t sz = SIGSTKSZ + 4096;
+			if (sz < mgt_param.wthread_stacksize)
+				sz = mgt_param.wthread_stacksize;
+			altstack.ss_sp = mmap(NULL, sz,  PROT_READ | PROT_WRITE,
+					      MAP_PRIVATE | MAP_ANONYMOUS,
+					      -1, 0);
+			AN(altstack.ss_sp != MAP_FAILED);
+			AN(altstack.ss_sp);
+			altstack.ss_size = sz;
+			altstack.ss_flags = 0;
+			AZ(sigaltstack(&altstack, NULL));
+			sa.sa_flags |= SA_ONSTACK;
+#endif
+			(void)sigaction(SIGSEGV, &sa, NULL);
+
 		}
 		(void)signal(SIGINT, SIG_DFL);
 		(void)signal(SIGTERM, SIG_DFL);
 
 		VJ_subproc(JAIL_SUBPROC_WORKER);
 
+		proc_vsmw = VSMW_New(heritage.vsm_fd, 0640, "_.index");
+		AN(proc_vsmw);
+
 		child_main();
 
 		exit(0);
 	}
 	assert(pid > 1);
-	MGT_complain(C_DEBUG, "Child (%jd) Started", (intmax_t)pid);
-	VSC_C_mgt->child_start = ++static_VSC_C_mgt.child_start;
+	MGT_Complain(C_DEBUG, "Child (%jd) Started", (intmax_t)pid);
+	VSC_C_mgt->child_start++;
 
 	/* Close stuff the child got */
-	closex(&heritage.std_fd);
+	closefd(&heritage.std_fd);
 
-	mgt_child_inherit(heritage.cli_in, NULL);
-	closex(&heritage.cli_in);
+	MCH_Fd_Inherit(heritage.cli_in, NULL);
+	closefd(&heritage.cli_in);
 
-	mgt_child_inherit(heritage.cli_out, NULL);
-	closex(&heritage.cli_out);
+	MCH_Fd_Inherit(heritage.cli_out, NULL);
+	closefd(&heritage.cli_out);
 
 	child_std_vlu = VLU_New(NULL, child_line, 0);
 	AN(child_std_vlu);
@@ -439,11 +440,11 @@ mgt_launch_child(struct cli *cli)
 	child_pid = pid;
 	if (mgt_push_vcls_and_start(cli, &u, &p)) {
 		VCLI_SetResult(cli, u);
-		MGT_complain(C_ERR, "Child (%jd) Pushing vcls failed:\n%s",
+		MGT_Complain(C_ERR, "Child (%jd) Pushing vcls failed:\n%s",
 		    (intmax_t)child_pid, p);
 		free(p);
 		child_state = CH_RUNNING;
-		mgt_stop_child();
+		MCH_Stop_Child();
 	} else
 		child_state = CH_RUNNING;
 }
@@ -452,6 +453,22 @@ mgt_launch_child(struct cli *cli)
  * Cleanup when child dies.
  */
 
+static int
+kill_child(void)
+{
+	int i, error;
+
+	VJ_master(JAIL_MASTER_KILL);
+	if (MGT_FEATURE(FEATURE_NO_COREDUMP))
+		i = kill(child_pid, SIGKILL);
+	else
+		i = kill(child_pid, SIGQUIT);
+	error = errno;
+	VJ_master(JAIL_MASTER_LOW);
+	errno = error;
+	return (i);
+}
+
 static void
 mgt_reap_child(void)
 {
@@ -468,9 +485,9 @@ mgt_reap_child(void)
 	 */
 	mgt_cli_stop_child();
 	if (child_cli_out >= 0)
-		closex(&child_cli_out);
+		closefd(&child_cli_out);
 	if (child_cli_in >= 0)
-		closex(&child_cli_in);
+		closefd(&child_cli_in);
 
 	/* Stop the poker */
 	if (ev_poker != NULL) {
@@ -501,10 +518,7 @@ mgt_reap_child(void)
 		VSB_printf(vsb, "Child (%jd) not dying, killing", (intmax_t)r);
 
 		/* Kick it Jim... */
-		if (MGT_FEATURE(FEATURE_NO_COREDUMP))
-			(void)kill(child_pid, SIGKILL);
-		else
-			(void)kill(child_pid, SIGQUIT);
+		(void)kill_child();
 		r = waitpid(child_pid, &status, 0);
 	}
 	if (r != child_pid)
@@ -519,48 +533,45 @@ mgt_reap_child(void)
 		VSB_printf(vsb, " status=%d", WEXITSTATUS(status));
 		exit_status |= 0x20;
 		if (WEXITSTATUS(status) == 1)
-			VSC_C_mgt->child_exit = ++static_VSC_C_mgt.child_exit;
+			VSC_C_mgt->child_exit++;
 		else
-			VSC_C_mgt->child_stop = ++static_VSC_C_mgt.child_stop;
+			VSC_C_mgt->child_stop++;
 	}
 	if (WIFSIGNALED(status)) {
 		VSB_printf(vsb, " signal=%d", WTERMSIG(status));
 		exit_status |= 0x40;
-		VSC_C_mgt->child_died = ++static_VSC_C_mgt.child_died;
+		VSC_C_mgt->child_died++;
 	}
 #ifdef WCOREDUMP
 	if (WCOREDUMP(status)) {
 		VSB_printf(vsb, " (core dumped)");
 		exit_status |= 0x80;
-		VSC_C_mgt->child_dump = ++static_VSC_C_mgt.child_dump;
+		VSC_C_mgt->child_dump++;
 	}
 #endif
 	AZ(VSB_finish(vsb));
-	MGT_complain(status ? C_ERR : C_INFO, "%s", VSB_data(vsb));
+	MGT_Complain(status ? C_ERR : C_INFO, "%s", VSB_data(vsb));
 	VSB_destroy(&vsb);
 
 	/* Dispose of shared memory but evacuate panic messages first */
 	if (heritage.panic_str[0] != '\0') {
 		mgt_panic_record(r);
-		mgt_SHM_Destroy(1);
-		VSC_C_mgt->child_panic = ++static_VSC_C_mgt.child_panic;
-	} else {
-		mgt_SHM_Destroy(MGT_DO_DEBUG(DBG_VSM_KEEP));
+		VSC_C_mgt->child_panic++;
 	}
-	mgt_SHM_Create();
-	mgt_SHM_Commit();
+
+	mgt_SHM_ChildDestroy();
 
 	if (child_state == CH_RUNNING)
 		child_state = CH_DIED;
 
 	/* Pick up any stuff lingering on stdout/stderr */
 	(void)child_listener(NULL, EV_RD);
-	closex(&child_output);
+	closefd(&child_output);
 	VLU_Destroy(child_std_vlu);
 
 	child_pid = -1;
 
-	MGT_complain(C_DEBUG, "Child cleanup complete");
+	MGT_Complain(C_DEBUG, "Child cleanup complete");
 
 	if (child_state == CH_DIED && mgt_param.auto_restart)
 		mgt_launch_child(NULL);
@@ -581,19 +592,19 @@ mgt_reap_child(void)
  */
 
 void
-MGT_Child_Cli_Fail(void)
+MCH_Cli_Fail(void)
 {
 
 	if (child_state != CH_RUNNING)
 		return;
 	if (child_pid < 0)
 		return;
-	MGT_complain(C_ERR, "Child (%jd) not responding to CLI, killing it.",
-	    (intmax_t)child_pid);
-	if (MGT_FEATURE(FEATURE_NO_COREDUMP))
-		(void)kill(child_pid, SIGKILL);
+	if (kill_child() == 0)
+		MGT_Complain(C_ERR, "Child (%jd) not responding to CLI,"
+		    " killed it.", (intmax_t)child_pid);
 	else
-		(void)kill(child_pid, SIGQUIT);
+		MGT_Complain(C_ERR, "Failed to kill child with PID %jd: %s",
+		    (intmax_t)child_pid, strerror(errno));
 }
 
 /*=====================================================================
@@ -603,7 +614,7 @@ MGT_Child_Cli_Fail(void)
  */
 
 void
-mgt_stop_child(void)
+MCH_Stop_Child(void)
 {
 
 	if (child_state != CH_RUNNING)
@@ -611,17 +622,40 @@ mgt_stop_child(void)
 
 	child_state = CH_STOPPING;
 
-	MGT_complain(C_DEBUG, "Stopping Child");
+	MGT_Complain(C_DEBUG, "Stopping Child");
 
 	mgt_reap_child();
 }
 
 /*=====================================================================
- * CLI commands to start/stop child
+ */
+
+int
+MCH_Start_Child(void)
+{
+	mgt_launch_child(NULL);
+	if (child_state != CH_RUNNING)
+		return (2);
+	return(0);
+}
+
+/*====================================================================
+ * Query if the child is running
+ */
+
+int
+MCH_Running(void)
+{
+
+	return (child_pid > 0);
+}
+
+/*=====================================================================
+ * CLI commands
  */
 
 static void __match_proto__(cli_func_t)
-mcf_server_start(struct cli *cli, const char * const *av, void *priv)
+mch_cli_server_start(struct cli *cli, const char * const *av, void *priv)
 {
 
 	(void)av;
@@ -640,68 +674,33 @@ mcf_server_start(struct cli *cli, const char * const *av, void *priv)
 }
 
 static void __match_proto__(cli_func_t)
-mcf_server_stop(struct cli *cli, const char * const *av, void *priv)
+mch_cli_server_stop(struct cli *cli, const char * const *av, void *priv)
 {
 
 	(void)av;
 	(void)priv;
 	if (child_state == CH_RUNNING) {
-		mgt_stop_child();
+		MCH_Stop_Child();
 	} else {
 		VCLI_SetResult(cli, CLIS_CANT);
 		VCLI_Out(cli, "Child in state %s", ch_state[child_state]);
 	}
 }
 
-/*--------------------------------------------------------------------*/
-
-static void
-mcf_server_status(struct cli *cli, const char * const *av, void *priv)
+static void __match_proto__(cli_func_t)
+mch_cli_server_status(struct cli *cli, const char * const *av, void *priv)
 {
 	(void)av;
 	(void)priv;
 	VCLI_Out(cli, "Child in state %s", ch_state[child_state]);
 }
 
-/*--------------------------------------------------------------------*/
-
-static int __match_proto__(vev_cb_f)
-mgt_sigint(const struct vev *e, int what)
-{
-
-	(void)e;
-	(void)what;
-	MGT_complain(C_ERR, "Manager got SIGINT");
-	(void)fflush(stdout);
-	if (child_pid >= 0)
-		mgt_stop_child();
-	exit(0);
-}
-
-/*--------------------------------------------------------------------*/
-
-static int __match_proto__(vev_cb_f)
-mgt_uptime(const struct vev *e, int what)
-{
-
-	(void)e;
-	(void)what;
-	AN(VSC_C_mgt);
-	VSC_C_mgt->uptime = static_VSC_C_mgt.uptime =
-	    (uint64_t)(VTIM_real() - mgt_uptime_t0);
-	if (heritage.vsm != NULL)
-		VSM_common_ageupdate(heritage.vsm);
-	return (0);
-}
-
-/*--------------------------------------------------------------------*/
-
-static struct cli_proto cli_child[] = {
-	{ CLICMD_SERVER_STATUS,		"", mcf_server_status },
-	{ CLICMD_SERVER_START,		"", mcf_server_start },
-	{ CLICMD_SERVER_STOP,		"", mcf_server_stop },
-	{ CLICMD_PANIC_SHOW,		"", mcf_panic_show },
-	{ CLICMD_PANIC_CLEAR,		"", mcf_panic_clear },
+static struct cli_proto cli_mch[] = {
+	{ CLICMD_SERVER_STATUS,		"", mch_cli_server_status },
+	{ CLICMD_SERVER_START,		"", mch_cli_server_start },
+	{ CLICMD_SERVER_STOP,		"", mch_cli_server_stop },
+	{ CLICMD_PANIC_SHOW,		"", mch_cli_panic_show },
+	{ CLICMD_PANIC_CLEAR,		"", mch_cli_panic_clear },
 	{ NULL }
 };
 
@@ -712,67 +711,8 @@ static struct cli_proto cli_child[] = {
  */
 
 void
-MGT_Run(void)
+MCH_Init(void)
 {
-	struct sigaction sac;
-	struct vev *e;
-	int i;
 
-	VCLS_AddFunc(mgt_cls, MCF_AUTH, cli_child);
-
-	mgt_uptime_t0 = VTIM_real();
-	e = vev_new();
-	XXXAN(e);
-	e->callback = mgt_uptime;
-	e->timeout = 1.0;
-	e->name = "mgt_uptime";
-	AZ(vev_add(mgt_evb, e));
-
-	e = vev_new();
-	XXXAN(e);
-	e->sig = SIGTERM;
-	e->callback = mgt_sigint;
-	e->name = "mgt_sigterm";
-	AZ(vev_add(mgt_evb, e));
-
-	e = vev_new();
-	XXXAN(e);
-	e->sig = SIGINT;
-	e->callback = mgt_sigint;
-	e->name = "mgt_sigint";
-	AZ(vev_add(mgt_evb, e));
-
-#ifdef HAVE_SETPROCTITLE
-	setproctitle("Varnish-Mgr %s", heritage.name);
-#endif
-
-	memset(&sac, 0, sizeof sac);
-	sac.sa_handler = SIG_IGN;
-	sac.sa_flags = SA_RESTART;
-
-	AZ(sigaction(SIGPIPE, &sac, NULL));
-	AZ(sigaction(SIGHUP, &sac, NULL));
-
-	if (!d_flag && !mgt_has_vcl())
-		MGT_complain(C_ERR, "No VCL loaded yet");
-	else if (!d_flag) {
-		mgt_launch_child(NULL);
-		if (child_state != CH_RUNNING) {
-			// XXX correct? or 0?
-			exit_status = 2;
-			return;
-		}
-	}
-
-	i = mgt_SHM_Commit();
-	if (i != 0) {
-		MGT_complain(C_ERR, "Could not commit SHM file");
-		return;
-	}
-
-	i = vev_schedule(mgt_evb);
-	if (i != 0)
-		MGT_complain(C_ERR, "vev_schedule() = %d", i);
-
-	MGT_complain(C_INFO, "manager dies");
+	VCLS_AddFunc(mgt_cls, MCF_AUTH, cli_mch);
 }
