@@ -55,12 +55,10 @@
 #define CLI_CMD(U,l,s,h,d,m,M) \
 const struct cli_cmd_desc CLICMD_##U[1] = {{ l, s, h, d, m, M }};
 #include "tbl/cli_cmds.h"
-#undef CLI_CMD
 
 static const struct cli_cmd_desc *cmds[] = {
 #define CLI_CMD(U,l,s,h,d,m,M) CLICMD_##U,
 #include "tbl/cli_cmds.h"
-#undef CLI_CMD
 };
 
 static const int ncmds = sizeof cmds / sizeof cmds[0];
@@ -69,9 +67,11 @@ static int		cli_i = -1, cli_o = -1;
 struct VCLS		*mgt_cls;
 static const char	*secret_file;
 
+static struct vsb	*cli_buf = NULL;
+
 /*--------------------------------------------------------------------*/
 
-static void
+static void __match_proto__(cli_func_t)
 mcf_banner(struct cli *cli, const char *const *av, void *priv)
 {
 
@@ -85,7 +85,7 @@ mcf_banner(struct cli *cli, const char *const *av, void *priv)
 	VCLI_Out(cli, "\n");
 	VCLI_Out(cli, "Type 'help' for command list.\n");
 	VCLI_Out(cli, "Type 'quit' to close CLI session.\n");
-	if (child_pid < 0)
+	if (!MCH_Running())
 		VCLI_Out(cli, "Type 'start' to launch worker process.\n");
 	VCLI_SetResult(cli, CLIS_OK);
 }
@@ -99,7 +99,7 @@ static struct cli_proto cli_proto[] = {
 
 /*--------------------------------------------------------------------*/
 
-static void
+static void __match_proto__(cli_func_t)
 mcf_panic(struct cli *cli, const char * const *av, void *priv)
 {
 
@@ -116,7 +116,7 @@ static struct cli_proto cli_debug[] = {
 
 /*--------------------------------------------------------------------*/
 
-static void
+static void __match_proto__(cli_func_t)
 mcf_askchild(struct cli *cli, const char * const *av, void *priv)
 {
 	int i;
@@ -149,12 +149,12 @@ mcf_askchild(struct cli *cli, const char * const *av, void *priv)
 		VSB_destroy(&vsb);
 		VCLI_SetResult(cli, CLIS_COMMS);
 		VCLI_Out(cli, "CLI communication error");
-		MGT_Child_Cli_Fail();
+		MCH_Cli_Fail();
 		return;
 	}
 	VSB_destroy(&vsb);
 	if (VCLI_ReadResult(cli_i, &u, &q, mgt_param.cli_timeout))
-		MGT_Child_Cli_Fail();
+		MCH_Cli_Fail();
 	VCLI_SetResult(cli, u);
 	VCLI_Out(cli, "%s", q);
 	free(q);
@@ -174,39 +174,46 @@ static struct cli_proto cli_askchild[] = {
  */
 
 int
-mgt_cli_askchild(unsigned *status, char **resp, const char *fmt, ...) {
+mgt_cli_askchild(unsigned *status, char **resp, const char *fmt, ...)
+{
 	int i, j;
 	va_list ap;
 	unsigned u;
-	char buf[mgt_param.cli_buffer], *p;
+
+	if (cli_buf == NULL) {
+		cli_buf = VSB_new_auto();
+		AN(cli_buf);
+	} else {
+		VSB_clear(cli_buf);
+	}
 
 	if (resp != NULL)
 		*resp = NULL;
 	if (status != NULL)
 		*status = 0;
-	if (cli_i < 0|| cli_o < 0) {
+	if (cli_i < 0 || cli_o < 0) {
 		if (status != NULL)
 			*status = CLIS_CANT;
 		return (CLIS_CANT);
 	}
 	va_start(ap, fmt);
-	vbprintf(buf, fmt, ap);
+	AZ(VSB_vprintf(cli_buf, fmt, ap));
 	va_end(ap);
-	p = strchr(buf, '\0');
-	assert(p != NULL && p > buf && p[-1] == '\n');
-	i = p - buf;
-	j = write(cli_o, buf, i);
+	AZ(VSB_finish(cli_buf));
+	i = VSB_len(cli_buf);
+	assert(i > 0 && VSB_data(cli_buf)[i - 1] == '\n');
+	j = write(cli_o, VSB_data(cli_buf), i);
 	if (j != i) {
 		if (status != NULL)
 			*status = CLIS_COMMS;
 		if (resp != NULL)
 			*resp = strdup("CLI communication error");
-		MGT_Child_Cli_Fail();
+		MCH_Cli_Fail();
 		return (CLIS_COMMS);
 	}
 
 	if (VCLI_ReadResult(cli_i, &u, resp, mgt_param.cli_timeout))
-		MGT_Child_Cli_Fail();
+		MCH_Cli_Fail();
 	if (status != NULL)
 		*status = u;
 	return (u == CLIS_OK ? 0 : u);
@@ -282,11 +289,11 @@ mcf_auth(struct cli *cli, const char *const *av, void *priv)
 		return;
 	}
 	VJ_master(JAIL_MASTER_LOW);
-	mgt_got_fd(fd);
+	MCH_TrackHighFd(fd);
 	VCLI_AuthResponse(fd, cli->challenge, buf);
-	AZ(close(fd));
+	closefd(&fd);
 	if (strcasecmp(buf, av[2])) {
-		MGT_complain(C_SECURITY,
+		MGT_Complain(C_SECURITY,
 		    "CLI Authentication failure from %s", cli->ident);
 		VCLI_SetResult(cli, CLIS_CLOSE);
 		return;
@@ -331,15 +338,23 @@ static void
 mgt_cli_cb_before(const struct cli *cli)
 {
 
-	MGT_complain(C_CLI, "CLI %s Rd %s", cli->ident, cli->cmd);
+	if (cli->priv == stderr)
+		fprintf(stderr, "> %s\n", cli->cmd);
+	MGT_Complain(C_CLI, "CLI %s Rd %s", cli->ident, cli->cmd);
 }
 
 static void
 mgt_cli_cb_after(const struct cli *cli)
 {
 
-	MGT_complain(C_CLI, "CLI %s Wr %03u %s",
+	MGT_Complain(C_CLI, "CLI %s Wr %03u %s",
 	    cli->ident, cli->result, VSB_data(cli->sb));
+	if (cli->priv == stderr &&
+	    cli->result != CLIS_OK && cli->cmd[0] != '-') {
+		MGT_Complain(C_ERR, "-I file CLI command failed (%d)\n%s\n",
+		    cli->result, VSB_data(cli->sb));
+		exit(2);
+	}
 }
 
 /*--------------------------------------------------------------------*/
@@ -386,20 +401,17 @@ mgt_cli_callback2(const struct vev *e, int what)
 /*--------------------------------------------------------------------*/
 
 void
-mgt_cli_setup(int fdi, int fdo, int verbose, const char *ident,
+mgt_cli_setup(int fdi, int fdo, int auth, const char *ident,
     mgt_cli_close_f *closefunc, void *priv)
 {
 	struct cli *cli;
 	struct vev *ev;
 
-	(void)ident;
-	(void)verbose;
-
 	cli = VCLS_AddFd(mgt_cls, fdi, fdo, closefunc, priv);
 
-	cli->ident = strdup(ident);
+	REPLACE(cli->ident, ident);
 
-	if (fdi != 0 && secret_file != NULL) {
+	if (!auth && secret_file != NULL) {
 		cli->auth = MCF_NOAUTH;
 		mgt_cli_challenge(cli);
 	} else {
@@ -409,7 +421,6 @@ mgt_cli_setup(int fdi, int fdo, int verbose, const char *ident,
 	AZ(VSB_finish(cli->sb));
 	(void)VCLI_WriteResult(fdo, cli->result, VSB_data(cli->sb));
 
-
 	ev = vev_new();
 	AN(ev);
 	ev->name = cli->ident;
@@ -486,7 +497,7 @@ telnet_accept(const struct vev *ev, int what)
 	if (i < 0)
 		return (0);
 
-	mgt_got_fd(i);
+	MCH_TrackHighFd(i);
 	tn = telnet_new(i);
 	vsb = sock_id("telnet", i);
 	mgt_cli_setup(i, i, 0, VSB_data(vsb), telnet_close, tn);
@@ -501,7 +512,7 @@ mgt_cli_secret(const char *S_arg)
 	char buf[BUFSIZ];
 
 	/* Save in shmem */
-	mgt_SHM_static_alloc(S_arg, strlen(S_arg) + 1L, "Arg", "-S", "");
+	mgt_SHM_static_alloc(S_arg, strlen(S_arg) + 1L, "Arg", "-S");
 
 	VJ_master(JAIL_MASTER_FILE);
 	fd = open(S_arg, O_RDONLY);
@@ -510,7 +521,7 @@ mgt_cli_secret(const char *S_arg)
 		exit(2);
 	}
 	VJ_master(JAIL_MASTER_LOW);
-	mgt_got_fd(fd);
+	MCH_TrackHighFd(fd);
 	i = read(fd, buf, sizeof buf);
 	if (i == 0) {
 		fprintf(stderr, "Empty secret-file \"%s\"\n", S_arg);
@@ -520,7 +531,7 @@ mgt_cli_secret(const char *S_arg)
 		fprintf(stderr, "Can not read secret-file \"%s\"\n", S_arg);
 		exit(2);
 	}
-	AZ(close(fd));
+	closefd(&fd);
 	secret_file = S_arg;
 }
 
@@ -569,9 +580,9 @@ mgt_cli_telnet(const char *T_arg)
 	AZ(error);
 	AZ(VSB_finish(vsb));
 	if (VSB_len(vsb) == 0)
-		ARGV_ERR("-T %s could not be listened on.", T_arg);
+		ARGV_ERR("-T %s could not be listened on.\n", T_arg);
 	/* Save in shmem */
-	mgt_SHM_static_alloc(VSB_data(vsb), VSB_len(vsb) + 1, "Arg", "-T", "");
+	mgt_SHM_static_alloc(VSB_data(vsb), VSB_len(vsb) + 1, "Arg", "-T");
 	VSB_destroy(&vsb);
 }
 
@@ -591,7 +602,7 @@ static double M_poll = 0.1;
 static VTAILQ_HEAD(,m_addr)	m_addr_list =
     VTAILQ_HEAD_INITIALIZER(m_addr_list);
 
-static void
+static void __match_proto__(mgt_cli_close_f)
 Marg_closer(void *priv)
 {
 
@@ -611,7 +622,8 @@ Marg_connect(const struct vev *e, int what)
 
 	M_fd = VTCP_connected(M_fd);
 	if (M_fd < 0) {
-		MGT_complain(C_INFO, "Could not connect to CLI-master: %m");
+		MGT_Complain(C_INFO, "Could not connect to CLI-master: %s",
+			strerror(errno));
 		ma = VTAILQ_FIRST(&m_addr_list);
 		AN(ma);
 		VTAILQ_REMOVE(&m_addr_list, ma, list);
@@ -648,7 +660,7 @@ Marg_poker(const struct vev *e, int what)
 	if (s < 0)
 		return (0);
 
-	mgt_got_fd(s);
+	MCH_TrackHighFd(s);
 
 	M_conn = vev_new();
 	AN(M_conn);
