@@ -100,7 +100,7 @@ VJ_Init(const char *j_arg)
 			ARGV_ERR("-j argument: %s\n", av[0]);
 		if (av[1] == NULL)
 			ARGV_ERR("-j argument is emtpy\n");
-		vjt = pick(vj_choice, av[1], "jail");
+		vjt = MGT_Pick(vj_choice, av[1], "jail");
 		CHECK_OBJ_NOTNULL(vjt, JAIL_TECH_MAGIC);
 		(void)vjt->init(av + 2);
 		VAV_Free(av);
@@ -157,7 +157,7 @@ VJ_make_workdir(const char *dname)
 		ARGV_ERR("Cannot create test-file in %s (%s)\n"
 		    "Check permissions (or delete old directory)\n",
 		    dname, strerror(errno));
-	AZ(close(fd));
+	closefd(&fd);
 	AZ(unlink("_.testfile"));
 	VJ_master(JAIL_MASTER_LOW);
 	return (0);
@@ -173,7 +173,7 @@ VJ_make_vcldir(const char *dname)
 		return (vjt->make_vcldir(dname));
 
 	if (mkdir(dname, 0755) < 0 && errno != EEXIST) {
-		MGT_complain(C_ERR, "Cannot create VCL directory '%s': %s",
+		MGT_Complain(C_ERR, "Cannot create VCL directory '%s': %s",
 		    dname, strerror(errno));
 		return (1);
 	}
@@ -181,19 +181,10 @@ VJ_make_vcldir(const char *dname)
 }
 
 void
-VJ_fix_storage_file(int fd)
+VJ_fix_fd(int fd, enum jail_fixfd_e what)
 {
 
 	CHECK_OBJ_NOTNULL(vjt, JAIL_TECH_MAGIC);
-	if (vjt->storage_file != NULL)
-		vjt->storage_file(fd);
-}
-
-void
-VJ_fix_vsm_file(int fd)
-{
-
-	CHECK_OBJ_NOTNULL(vjt, JAIL_TECH_MAGIC);
-	if (vjt->vsm_file != NULL)
-		vjt->vsm_file(fd);
+	if (vjt->fixfd != NULL)
+		vjt->fixfd(fd, what);
 }
