@@ -34,9 +34,9 @@
 #include <fcntl.h>
 #include <grp.h>
 #include <pwd.h>
-#include <string.h>
 #include <stdio.h>
 #include <stdlib.h>
+#include <string.h>
 #include <unistd.h>
 #include <sys/stat.h>
 
@@ -46,7 +46,7 @@
 #include <sys/prctl.h>
 #endif
 
-static gid_t vju_mgr_gid;
+static gid_t vju_mgt_gid;
 static uid_t vju_uid;
 static gid_t vju_gid;
 static const char *vju_user;
@@ -166,10 +166,12 @@ vju_init(char **args)
 
 	AN(vju_user);
 
-	vju_mgr_gid = getgid();
+	vju_mgt_gid = getgid();
 
-	if (vju_wrkuser == NULL)
-		(void)vju_getwrkuid(VCACHE_USER);
+	if (vju_wrkuser == NULL && vju_getwrkuid(VCACHE_USER)) {
+		vju_wrkuid = vju_uid;
+		vju_wrkgid = vju_gid;
+	}
 
 	if (vju_wrkuser != NULL && vju_wrkgid != vju_gid)
 		ARGV_ERR("Unix jail: user %s and %s have "
@@ -189,7 +191,7 @@ vju_master(enum jail_master_e jme)
 		AZ(seteuid(vju_uid));
 	} else {
 		AZ(seteuid(0));
-		AZ(setegid(vju_mgr_gid));
+		AZ(setegid(vju_mgt_gid));
 	}
 }
 
@@ -230,7 +232,7 @@ vju_subproc(enum jail_subproc_e jse)
 	 * reenable them again.
 	 */
 	if (prctl(PR_SET_DUMPABLE, 1) != 0) {
-		MGT_complain(C_INFO,
+		MGT_Complain(C_INFO,
 		    "Could not set dumpable bit.  Core dumps turned off");
 	}
 #endif
@@ -242,7 +244,7 @@ vju_make_vcldir(const char *dname)
 	AZ(seteuid(0));
 
 	if (mkdir(dname, 0755) < 0 && errno != EEXIST) {
-		MGT_complain(C_ERR, "Cannot create VCL directory '%s': %s",
+		MGT_Complain(C_ERR, "Cannot create VCL directory '%s': %s",
 		    dname, strerror(errno));
 		return (1);
 	}
@@ -252,22 +254,27 @@ vju_make_vcldir(const char *dname)
 }
 
 
-static void __match_proto__(jail_fixfile_f)
-vju_vsm_file(int fd)
+static void __match_proto__(jail_fixfd_f)
+vju_fixfd(int fd, enum jail_fixfd_e what)
 {
 	/* Called under JAIL_MASTER_FILE */
 
-	AZ(fchmod(fd, 0640));
-	AZ(fchown(fd, 0, vju_gid));
-}
-
-static void __match_proto__(jail_fixfile_f)
-vju_storage_file(int fd)
-{
-	/* Called under JAIL_MASTER_STORAGE */
-
-	AZ(fchmod(fd, 0600));
-	AZ(fchown(fd, vju_uid, vju_gid));
+	switch (what) {
+	case JAIL_FIXFD_FILE:
+		AZ(fchmod(fd, 0750));
+		AZ(fchown(fd, vju_wrkuid, vju_wrkgid));
+		break;
+	case JAIL_FIXFD_VSMMGT:
+		AZ(fchmod(fd, 0750));
+		AZ(fchown(fd, vju_uid, vju_gid));
+		break;
+	case JAIL_FIXFD_VSMWRK:
+		AZ(fchmod(fd, 0750));
+		AZ(fchown(fd, vju_wrkuid, vju_wrkgid));
+		break;
+	default:
+		WRONG("Ain't Fixin'");
+	}
 }
 
 const struct jail_tech jail_tech_unix = {
@@ -276,7 +283,6 @@ const struct jail_tech jail_tech_unix = {
 	.init =		vju_init,
 	.master =	vju_master,
 	.make_vcldir =	vju_make_vcldir,
-	.vsm_file =	vju_vsm_file,
-	.storage_file =	vju_storage_file,
+	.fixfd =	vju_fixfd,
 	.subproc =	vju_subproc,
 };
