@@ -97,6 +97,14 @@ static const char ONLY_ROOT_TEXT[] =
 	"\n\n"
 	"NB: This parameter only works if varnishd is run as root.";
 
+static const char NOT_IMPLEMENTED_TEXT[] =
+	"This parameter depends on a feature which is not available"
+	" on this platform.";
+
+static const char NOT_IMPLEMENTED_DOC[] =
+	"NB: This parameter depends on a feature which is not available"
+	" on all platforms.";
+
 /*--------------------------------------------------------------------*/
 
 static struct parspec *
@@ -257,19 +265,34 @@ mcf_param_show(struct cli *cli, const char * const *av, void *priv)
 		if (chg && pp->def != NULL && !strcmp(pp->def, VSB_data(vsb)))
 			continue;
 
-		if (lfmt) {
-			VCLI_Out(cli, "%s\n", pp->name);
-			VCLI_Out(cli, "%-*sValue is: ", margin1, " ");
+		if (pp->flags & NOT_IMPLEMENTED) {
+			if (lfmt) {
+				VCLI_Out(cli, "%s\n", pp->name);
+				VCLI_Out(cli, "%-*sNot available", margin1, " ");
+			} else {
+				VCLI_Out(cli, "%-*s-", margin2, pp->name);
+			}
 		} else {
-			VCLI_Out(cli, "%-*s", margin2, pp->name);
+			if (lfmt) {
+				VCLI_Out(cli, "%s\n", pp->name);
+				VCLI_Out(cli, "%-*sValue is: ", margin1, " ");
+			} else {
+				VCLI_Out(cli, "%-*s", margin2, pp->name);
+			}
+
+			VCLI_Out(cli, "%s", VSB_data(vsb));
+			if (pp->units != NULL && *pp->units != '\0')
+				VCLI_Out(cli, " [%s]", pp->units);
+			if (pp->def != NULL && !strcmp(pp->def, VSB_data(vsb)))
+				VCLI_Out(cli, " (default)");
 		}
-		VCLI_Out(cli, "%s", VSB_data(vsb));
-		if (pp->units != NULL && *pp->units != '\0')
-			VCLI_Out(cli, " [%s]", pp->units);
-		if (pp->def != NULL && !strcmp(pp->def, VSB_data(vsb)))
-			VCLI_Out(cli, " (default)");
 		VCLI_Out(cli, "\n");
-		if (lfmt) {
+
+		if (lfmt && pp->flags & NOT_IMPLEMENTED) {
+			VCLI_Out(cli, "\n");
+			mcf_wrap(cli, NOT_IMPLEMENTED_TEXT);
+			VCLI_Out(cli, "\n\n");
+		} else if (lfmt) {
 			if (pp->def != NULL && strcmp(pp->def, VSB_data(vsb)))
 				VCLI_Out(cli, "%-*sDefault is: %s\n",
 				    margin1, "", pp->def);
@@ -365,7 +388,7 @@ MCF_ParamSet(struct cli *cli, const char *param, const char *val)
 	if (cli->result != CLIS_OK) {
 		VCLI_Out(cli, "\n(attempting to set param '%s' to '%s')",
 		    pp->name, val);
-	} else if (child_pid >= 0 && pp->flags & MUST_RESTART) {
+	} else if (MCH_Running() && pp->flags & MUST_RESTART) {
 		VCLI_Out(cli,
 		    "\nChange will take effect when child is restarted");
 	} else if (pp->flags & MUST_RELOAD) {
@@ -466,6 +489,8 @@ MCF_InitParams(struct cli *cli)
 	VTAILQ_FOREACH(pl, &phead, list) {
 		pp = pl->spec;
 
+		if (pp->flags & NOT_IMPLEMENTED)
+			continue;
 		if (pp->min != NULL)
 			mcf_wash_param(cli, pp, &pp->min, "minimum", vsb);
 		if (pp->max != NULL)
@@ -532,7 +557,7 @@ MCF_DumpRstParam(void)
 	int j;
 
 	printf("\n.. The following is the autogenerated "
-	    "output from varnishd -x dumprstparam\n\n");
+	    "output from varnishd -x parameter\n\n");
 	VTAILQ_FOREACH(pl, &phead, list) {
 		pp = pl->spec;
 		printf(".. _ref_param_%s:\n\n", pp->name);
@@ -540,6 +565,10 @@ MCF_DumpRstParam(void)
 		for (j = 0; j < strlen(pp->name); j++)
 			printf("~");
 		printf("\n");
+
+		if (pp->flags && pp->flags & NOT_IMPLEMENTED)
+			printf("\n%s\n\n", NOT_IMPLEMENTED_DOC);
+
 		if (pp->units != NULL && *pp->units != '\0')
 			printf("\t* Units: %s\n", pp->units);
 		printf("\t* Default: %s\n", pp->def);
@@ -555,6 +584,7 @@ MCF_DumpRstParam(void)
 		if (pp->flags) {
 			printf("\t* Flags: ");
 			q = "";
+
 			if (pp->flags & DELAYED_EFFECT) {
 				printf("%sdelayed", q);
 				q = ", ";
@@ -594,6 +624,7 @@ MCF_DumpRstParam(void)
 			t1 = strchr(p, '\t');
 			if (t1 != NULL && t1 < q) {
 				t2 = strchr(t1 + 1, '\t');
+				AN(t2);
 				printf("\n\t*");
 				(void)fwrite(t1 + 1, (t2 - 1) - t1, 1, stdout);
 				printf("*\n\t\t");
