@@ -48,6 +48,8 @@ struct parspec {
 #define PROTECTED	(1<<5)
 #define OBJ_STICKY	(1<<6)
 #define ONLY_ROOT	(1<<7)
+#define NOT_IMPLEMENTED	(1<<8)
+
 	const char	*def;
 	const char	*units;
 };
@@ -60,7 +62,6 @@ tweak_t tweak_poolparam;
 tweak_t tweak_string;
 tweak_t tweak_timeout;
 tweak_t tweak_uint;
-tweak_t tweak_waiter;
 tweak_t tweak_vsl_buffer;
 tweak_t tweak_vsl_reclen;
 
