@@ -46,7 +46,6 @@ struct parspec mgt_parspec[] = {
 #define PARAM(nm, ty, mi, ma, de, un, fl, st, lt, fn)		\
 	{ #nm, tweak_##ty, &mgt_param.nm, mi, ma, st, fl, de, un },
 #include "tbl/params.h"
-#undef PARAM
 
 	{ "cc_command", tweak_string, &mgt_cc_cmd,
 		NULL, NULL,
@@ -147,6 +146,12 @@ struct parspec mgt_parspec[] = {
 		MEMPOOL_TEXT,
 		0,
 		"10,100,10", ""},
+	{ "shm_reclen", tweak_vsl_reclen, &mgt_param.vsl_reclen,
+		"16b", NULL,
+		"Old name for vsl_reclen, use that instead.",
+		0,
+		"255b",
+		"bytes" },
 
 	{ NULL, NULL, NULL }
 };
