@@ -31,7 +31,6 @@
 
 #include "config.h"
 
-#include <dlfcn.h>
 #include <errno.h>
 #include <fcntl.h>
 #include <stdio.h>
@@ -73,9 +72,11 @@ unsigned mgt_vcc_unsafe_path;
 
 /*--------------------------------------------------------------------*/
 
-static const char * const builtin_vcl =
-#include "builtin_vcl.h"
-    ""	;
+void
+mgt_DumpBuiltin(void)
+{
+	printf("%s\n", builtin_vcl);
+}
 
 /*--------------------------------------------------------------------
  * Invoke system VCC compiler in a sub-process
@@ -104,10 +105,9 @@ run_vcc(void *priv)
 	VCC_Err_Unref(vcc, mgt_vcc_err_unref);
 	VCC_Allow_InlineC(vcc, mgt_vcc_allow_inline_c);
 	VCC_Unsafe_Path(vcc, mgt_vcc_unsafe_path);
-	VTAILQ_FOREACH(stv, &stv_stevedores, list)
+	STV_Foreach(stv)
 		VCC_Predef(vcc, "VCL_STEVEDORE", stv->ident);
 	mgt_vcl_export_labels(vcc);
-	VCC_Predef(vcc, "VCL_STEVEDORE", stv_transient->ident);
 	csrc = VCC_Compile(vcc, &sb, vp->vclsrc, vp->vclsrcfile);
 	AZ(VSB_finish(sb));
 	if (VSB_len(sb))
@@ -127,7 +127,7 @@ run_vcc(void *priv)
 		fprintf(stderr, "VCC cannot write %s", vp->csrcfile);
 		exit(2);
 	}
-	AZ(close(fd));
+	closefd(&fd);
 	VSB_destroy(&csrc);
 	exit(0);
 }
@@ -218,7 +218,7 @@ mgt_vcc_touchfile(const char *fn, struct vsb *sb)
 		if (geteuid() == 0)
 			VSB_printf(sb, "Failed to change owner on %s: %s\n",
 			    fn, strerror(errno));
-	AZ(close(i));
+	closefd(&i);
 	return (0);
 }
 
@@ -369,8 +369,12 @@ mgt_VccCompile(struct cli *cli, struct vclprog *vcl, const char *vclname,
 		AZ(av[0]);
 		AZ(strcmp(av[1], "/*"));
 		AZ(strcmp(av[ac-1], "*/"));
-		AZ(strcmp(av[3], "VCL"));
-		mgt_vcl_depends(vcl, av[4]);
+		if (!strcmp(av[3], "VCL"))
+			mgt_vcl_depends(vcl, av[4]);
+		else if (!strcmp(av[3], "VMOD"))
+			mgt_vcl_vmod(vcl, av[4], av[5]);
+		else
+			WRONG("Wrong VCCINFO");
 		VAV_Free(av);
 	}
 	AZ(fclose(fcs));
