@@ -31,6 +31,7 @@
 
 #include "config.h"
 
+#include <errno.h>
 #include <fcntl.h>
 #include <stdio.h>
 #include <stdlib.h>
@@ -52,6 +53,15 @@ static const char * const VCL_STATE_AUTO = "auto";
 static const char * const VCL_STATE_LABEL = "label";
 
 struct vclprog;
+struct vmodfile;
+
+struct vmoddep {
+	unsigned		magic;
+#define VMODDEP_MAGIC		0xc1490542
+	VTAILQ_ENTRY(vmoddep)	lfrom;
+	struct vmodfile		*to;
+	VTAILQ_ENTRY(vmoddep)	lto;
+};
 
 struct vcldep {
 	unsigned		magic;
@@ -74,9 +84,20 @@ struct vclprog {
 	VTAILQ_HEAD(, vcldep)	dfrom;
 	VTAILQ_HEAD(, vcldep)	dto;
 	int			nto;
+	int			loaded;
+	VTAILQ_HEAD(, vmoddep)	vmods;
+};
+
+struct vmodfile {
+	unsigned		magic;
+#define VMODFILE_MAGIC		0xffa1a0d5
+	char			*fname;
+	VTAILQ_ENTRY(vmodfile)	list;
+	VTAILQ_HEAD(, vmoddep)	vcls;
 };
 
 static VTAILQ_HEAD(, vclprog)	vclhead = VTAILQ_HEAD_INITIALIZER(vclhead);
+static VTAILQ_HEAD(, vmodfile)	vmodhead = VTAILQ_HEAD_INITIALIZER(vmodhead);
 static struct vclprog		*active_vcl;
 static struct vev *e_poker;
 
@@ -94,30 +115,23 @@ mcf_vcl_byname(const char *name)
 }
 
 static int
-mcf_bad_vclname(struct cli *cli, const char *name)
+mcf_invalid_vclname(struct cli *cli, const char *name)
 {
-	const char *p;
-	int bad = 0;
+	const char *bad;
 
 	AN(name);
-	p = name;
-	if (!vct_isalpha(*p))
-		bad = *p;
-	for (p++; bad == 0 && *p != '\0'; p++)
-		if (!vct_isalpha(*p) &&
-		    !vct_isdigit(*p) &&
-		    *p != '_' &&
-		    *p != '-')
-			bad = *p;
-	if (bad) {
+	bad = VCT_invalid_name(name, NULL);
+
+	if (bad != NULL) {
 		VCLI_SetResult(cli, CLIS_PARAM);
 		VCLI_Out(cli, "Illegal character in VCL name ");
-		if (bad > 0x20 && bad < 0x7f)
-			VCLI_Out(cli, "('%c')", bad);
+		if (*bad > 0x20 && *bad < 0x7f)
+			VCLI_Out(cli, "('%c')", *bad);
 		else
-			VCLI_Out(cli, "(0x%02x)", bad & 0xff);
+			VCLI_Out(cli, "(0x%02x)", *bad & 0xff);
+		return (-1);
 	}
-	return (bad);
+	return (0);
 }
 
 static struct vclprog *
@@ -125,7 +139,7 @@ mcf_find_vcl(struct cli *cli, const char *name)
 {
 	struct vclprog *vp;
 
-	if (mcf_bad_vclname(cli, name))
+	if (mcf_invalid_vclname(cli, name))
 		return (NULL);
 
 	vp = mcf_vcl_byname(name);
@@ -140,7 +154,7 @@ static int
 mcf_find_no_vcl(struct cli *cli, const char *name)
 {
 
-	if (mcf_bad_vclname(cli, name))
+	if (mcf_invalid_vclname(cli, name))
 		return (0);
 
 	if (mcf_vcl_byname(name) != NULL) {
@@ -202,6 +216,7 @@ mgt_vcl_add(const char *name, const char *state)
 	REPLACE(vp->name, name);
 	VTAILQ_INIT(&vp->dfrom);
 	VTAILQ_INIT(&vp->dto);
+	VTAILQ_INIT(&vp->vmods);
 	vp->state = state;
 
 	if (vp->state != VCL_STATE_COLD)
@@ -215,6 +230,8 @@ static void
 mgt_vcl_del(struct vclprog *vp)
 {
 	char *p;
+	struct vmoddep *vd;
+	struct vmodfile *vf;
 
 	CHECK_OBJ_NOTNULL(vp, VCLPROG_MAGIC);
 	while (!VTAILQ_EMPTY(&vp->dto))
@@ -239,6 +256,23 @@ mgt_vcl_del(struct vclprog *vp)
 		VJ_master(JAIL_MASTER_LOW);
 		free(vp->fname);
 	}
+	while (!VTAILQ_EMPTY(&vp->vmods)) {
+		vd = VTAILQ_FIRST(&vp->vmods);
+		CHECK_OBJ(vd, VMODDEP_MAGIC);
+		vf = vd->to;
+		CHECK_OBJ(vf, VMODFILE_MAGIC);
+		VTAILQ_REMOVE(&vp->vmods, vd, lfrom);
+		VTAILQ_REMOVE(&vf->vcls, vd, lto);
+		FREE_OBJ(vd);
+
+		if (VTAILQ_EMPTY(&vf->vcls)) {
+			if (!MGT_DO_DEBUG(DBG_VMOD_SO_KEEP))
+				AZ(unlink(vf->fname));
+			VTAILQ_REMOVE(&vmodhead, vf, list);
+			free(vf->fname);
+			FREE_OBJ(vf);
+		}
+	}
 	free(vp->name);
 	FREE_OBJ(vp);
 }
@@ -255,6 +289,78 @@ mgt_vcl_depends(struct vclprog *vp1, const char *name)
 	mgt_vcl_dep_add(vp1, vp2);
 }
 
+static int
+mgt_vcl_cache_vmod(const char *nm, const char *fm, const char *to)
+{
+	int fi, fo;
+	int ret = 0;
+	ssize_t sz;
+	char buf[BUFSIZ];
+
+	fo = open(to, O_WRONLY | O_CREAT | O_EXCL, 0744);
+	if (fo < 0 && errno == EEXIST)
+		return (0);
+	if (fo < 0) {
+		fprintf(stderr, "Creating copy of vmod %s: %s\n",
+		    nm, strerror(errno));
+		return (1);
+	}
+	fi = open(fm, O_RDONLY);
+	if (fi < 0) {
+		fprintf(stderr, "Opening vmod %s from %s: %s\n",
+		    nm, fm, strerror(errno));
+		AZ(unlink(to));
+		closefd(&fo);
+		return (1);
+	}
+	while (1) {
+		sz = read(fi, buf, sizeof buf);
+		if (sz == 0)
+			break;
+		if (sz < 0 || sz != write(fo, buf, sz)) {
+			fprintf(stderr, "Copying vmod %s: %s\n",
+			    nm, strerror(errno));
+			AZ(unlink(to));
+			ret = 1;
+			break;
+		}
+	}
+	closefd(&fi);
+	AZ(fchmod(fo, 0444));
+	closefd(&fo);
+	return(ret);
+}
+
+void
+mgt_vcl_vmod(struct vclprog *vp, const char *src, const char *dst)
+{
+	struct vmodfile *vf;
+	struct vmoddep *vd;
+
+	CHECK_OBJ_NOTNULL(vp, VCLPROG_MAGIC);
+	AN(src);
+	AN(dst);
+	assert(!strncmp(dst, "./vmod_cache/", 13));
+
+	VTAILQ_FOREACH(vf, &vmodhead, list)
+		if (!strcmp(vf->fname, dst))
+			break;
+	if (vf == NULL) {
+		ALLOC_OBJ(vf, VMODFILE_MAGIC);
+		AN(vf);
+		REPLACE(vf->fname, dst);
+		AN(vf->fname);
+		VTAILQ_INIT(&vf->vcls);
+		AZ(mgt_vcl_cache_vmod(vp->name, src, dst));
+		VTAILQ_INSERT_TAIL(&vmodhead, vf, list);
+	}
+	ALLOC_OBJ(vd, VMODDEP_MAGIC);
+	AN(vd);
+	vd->to = vf;
+	VTAILQ_INSERT_TAIL(&vp->vmods, vd, lfrom);
+	VTAILQ_INSERT_TAIL(&vf->vcls, vd, lto);
+}
+
 int
 mgt_has_vcl(void)
 {
@@ -276,33 +382,41 @@ mgt_vcl_setstate(struct cli *cli, struct vclprog *vp, const char *vs)
 	}
 	if (vs == VCL_STATE_AUTO) {
 		now = VTIM_mono();
-		vs = vp->warm ? VCL_STATE_WARM : VCL_STATE_COLD;
+		vs = (vp->warm ? VCL_STATE_WARM : VCL_STATE_COLD);
 		if (vp->go_cold > 0 && vp->state == VCL_STATE_AUTO &&
 		    vp->go_cold + mgt_param.vcl_cooldown < now)
 			vs = VCL_STATE_COLD;
 	}
 
 	assert(vs != VCL_STATE_AUTO);
-	warm = vs == VCL_STATE_WARM ? 1 : 0;
+	warm = (vs == VCL_STATE_WARM ? 1 : 0);
 
 	if (vp->warm == warm)
 		return (0);
 
-	vp->warm = warm;
-
-	if (vp->warm == 0)
-		vp->go_cold = 0;
-
-	if (child_pid < 0)
+	if (!MCH_Running()) {
+		vp->warm = warm;
+		if (vp->warm == 0)
+			vp->go_cold = 0;
 		return (0);
+	}
 
 	i = mgt_cli_askchild(&status, &p, "vcl.state %s %d%s\n",
-	    vp->name, vp->warm, vp->state);
-	if (i) {
-		AN(cli);
-		XXXAN(vp->warm);	/* XXX: should restart child instead */
+	    vp->name, warm, vp->state);
+	if (i && cli != NULL) {
 		VCLI_SetResult(cli, status);
 		VCLI_Out(cli, "%s", p);
+	} else if (i) {
+		MGT_Complain(C_ERR,
+		    "Please file ticket: VCL poker problem: "
+		    "'vcl.state %s %d%s' -> %03d '%s'",
+		    vp->name, warm, vp->state, i, p);
+	} else {
+		/* Success, update mgt's VCL state to reflect child's
+		   state */
+		vp->warm = warm;
+		if (vp->warm == 0)
+			vp->go_cold = 0;
 	}
 
 	free(p);
@@ -354,7 +468,7 @@ mgt_new_vcl(struct cli *cli, const char *vclname, const char *vclsrc,
 	if (active_vcl == NULL)
 		active_vcl = vp;
 
-	if (child_pid < 0)
+	if (!MCH_Running())
 		return;
 
 	if (mgt_cli_askchild(&status, &p, "vcl.load %s %s %d%s\n",
@@ -362,32 +476,32 @@ mgt_new_vcl(struct cli *cli, const char *vclname, const char *vclsrc,
 		mgt_vcl_del(vp);
 		VCLI_Out(cli, "%s", p);
 		VCLI_SetResult(cli, status);
+		free(p);
+		return;
 	}
 	free(p);
+
+	if (vp->warm && !strcmp(vp->state, "auto"))
+		vp->go_cold = VTIM_mono();
 }
 
 /*--------------------------------------------------------------------*/
 
 void
-mgt_vcc_startup(struct cli *cli, const char *b_arg, const char *f_arg,
-    const char *vclsrc, int C_flag)
+mgt_vcl_startup(struct cli *cli, const char *vclsrc, const char *vclname,
+    const char *origin, int C_flag)
 {
-	char buf[BUFSIZ];
+	char buf[20];
+	static int n = 0;
 
-	if (b_arg == NULL) {
-		AN(vclsrc);
-		AN(f_arg);
-		mgt_new_vcl(cli, "boot", vclsrc, f_arg, NULL, C_flag);
-		return;
+	AN(vclsrc);
+	AN(origin);
+	if (vclname == NULL) {
+		bprintf(buf, "boot%d", n++);
+		vclname = buf;
 	}
-
-	AZ(vclsrc);
-	bprintf(buf,
-	    "vcl 4.0;\n"
-	    "backend default {\n"
-	    "    .host = \"%s\";\n"
-	    "}\n", b_arg);
-	mgt_new_vcl(cli, "boot", buf, "<-b argument>", NULL, C_flag);
+	mgt_new_vcl(cli, vclname, vclsrc, origin, NULL, C_flag);
+	active_vcl = mcf_vcl_byname(vclname);
 }
 
 /*--------------------------------------------------------------------*/
@@ -409,45 +523,46 @@ mgt_push_vcls_and_start(struct cli *cli, unsigned *status, char **p)
 {
 	struct vclprog *vp;
 	struct vcldep *vd;
+	int done;
 
 	AN(active_vcl);
 
 	/* The VCL has not been loaded yet, it cannot fail */
 	AZ(mgt_vcl_setstate(cli, active_vcl, VCL_STATE_WARM));
 
-	VTAILQ_FOREACH(vp, &vclhead, list) {
-		if (!VTAILQ_EMPTY(&vp->dfrom))
-			continue;
-		if (mcf_is_label(vp))
-			continue;
-		if (mgt_cli_askchild(status, p, "vcl.load \"%s\" %s %d%s\n",
-		    vp->name, vp->fname, vp->warm, vp->state))
-			return (1);
-		free(*p);
-		*p = NULL;
-	}
-	VTAILQ_FOREACH(vp, &vclhead, list) {
-		if (!mcf_is_label(vp))
-			continue;
-		vd = VTAILQ_FIRST(&vp->dfrom);
-		AN(vd);
-		if (mgt_cli_askchild(status, p, "vcl.label %s %s\n",
-		    vp->name, vd->to->name))
-			return (1);
-		free(*p);
-		*p = NULL;
-	}
-	VTAILQ_FOREACH(vp, &vclhead, list) {
-		if (VTAILQ_EMPTY(&vp->dfrom))
-			continue;
-		if (mcf_is_label(vp))
-			continue;
-		if (mgt_cli_askchild(status, p, "vcl.load \"%s\" %s %d%s\n",
-		    vp->name, vp->fname, vp->warm, vp->state))
-			return (1);
-		free(*p);
-		*p = NULL;
-	}
+	VTAILQ_FOREACH(vp, &vclhead, list)
+		vp->loaded = 0;
+
+	do {
+		done = 1;
+		VTAILQ_FOREACH(vp, &vclhead, list) {
+			if (vp->loaded)
+				continue;
+			VTAILQ_FOREACH(vd, &vp->dfrom, lfrom)
+				if (!vd->to->loaded)
+					break;
+			if (vd != NULL) {
+				done = 0;
+				continue;
+			}
+			if (mcf_is_label(vp)) {
+				vd = VTAILQ_FIRST(&vp->dfrom);
+				AN(vd);
+				if (mgt_cli_askchild(status, p,
+				    "vcl.label %s %s\n",
+				    vp->name, vd->to->name))
+					return (1);
+			} else {
+				if (mgt_cli_askchild(status, p,
+				    "vcl.load \"%s\" %s %d%s\n",
+				    vp->name, vp->fname, vp->warm, vp->state))
+					return (1);
+			}
+			vp->loaded = 1;
+			free(*p);
+			*p = NULL;
+		}
+	} while (!done);
 
 	if (mgt_cli_askchild(status, p, "vcl.use \"%s\"\n", active_vcl->name))
 		return (1);
@@ -551,7 +666,7 @@ mcf_vcl_use(struct cli *cli, const char * const *av, void *priv)
 		return;
 	if (mgt_vcl_setstate(cli, vp, VCL_STATE_WARM))
 		return;
-	if (child_pid >= 0 &&
+	if (MCH_Running() &&
 	    mgt_cli_askchild(&status, &p, "vcl.use %s\n", av[2])) {
 		VCLI_SetResult(cli, status);
 		VCLI_Out(cli, "%s", p);
@@ -610,7 +725,7 @@ mcf_vcl_discard(struct cli *cli, const char * const *av, void *priv)
 		AN(vp->warm);
 	else
 		(void)mgt_vcl_setstate(cli, vp, VCL_STATE_COLD);
-	if (child_pid >= 0) {
+	if (MCH_Running()) {
 		/* XXX If this fails the child is crashing, figure that later */
 		(void)mgt_cli_askchild(&status, &p, "vcl.discard %s\n", av[2]);
 		free(p);
@@ -630,7 +745,7 @@ mcf_vcl_list(struct cli *cli, const char * const *av, void *priv)
 
 	(void)av;
 	(void)priv;
-	if (child_pid >= 0) {
+	if (MCH_Running()) {
 		if (!mgt_cli_askchild(&status, &p, "vcl.list\n")) {
 			VCLI_SetResult(cli, status);
 			VCLI_Out(cli, "%s", p);
@@ -643,15 +758,17 @@ mcf_vcl_list(struct cli *cli, const char * const *av, void *priv)
 			    vp->state);
 			VCLI_Out(cli, "/%-8s", vp->warm ?
 			    VCL_STATE_WARM : VCL_STATE_COLD);
-			VCLI_Out(cli, " %6s %s", "", vp->name);
+			VCLI_Out(cli, " %6s %s", "-", vp->name);
 			if (mcf_is_label(vp)) {
 				vd = VTAILQ_FIRST(&vp->dfrom);
 				AN(vd);
 				VCLI_Out(cli, " -> %s", vd->to->name);
-			} else if (vp->nto > 1) {
-				VCLI_Out(cli, " (%d labels)", vp->nto);
+				if (vp->nto > 0)
+					VCLI_Out(cli, " (%d return(vcl)%s)",
+					    vp->nto, vp->nto > 1 ? "'s" : "");
 			} else if (vp->nto > 0) {
-				VCLI_Out(cli, " (%d label)", vp->nto);
+				VCLI_Out(cli, " (%d label%s)",
+				    vp->nto, vp->nto > 1 ? "s" : "");
 			}
 			VCLI_Out(cli, "\n");
 		}
@@ -668,9 +785,9 @@ mcf_vcl_label(struct cli *cli, const char * const *av, void *priv)
 	int i;
 
 	(void)priv;
-	if (mcf_bad_vclname(cli, av[2]))
+	if (mcf_invalid_vclname(cli, av[2]))
 		return;
-	if (mcf_bad_vclname(cli, av[3]))
+	if (mcf_invalid_vclname(cli, av[3]))
 		return;
 	vpt = mcf_find_vcl(cli, av[3]);
 	if (vpt == NULL)
@@ -687,6 +804,18 @@ mcf_vcl_label(struct cli *cli, const char * const *av, void *priv)
 			VCLI_Out(cli, "%s is not a label", vpl->name);
 			return;
 		}
+		if (!VTAILQ_EMPTY(&vpt->dfrom) &&
+		    !VTAILQ_EMPTY(&vpl->dto)) {
+			VCLI_SetResult(cli, CLIS_PARAM);
+			VCLI_Out(cli, "return(vcl) can only be used from"
+			    " the active VCL.\n\n");
+			VCLI_Out(cli,
+			    "Label %s is used in return(vcl) from VCL %s\n",
+			    vpl->name, VTAILQ_FIRST(&vpl->dto)->from->name);
+			VCLI_Out(cli, "and VCL %s also has return(vcl)",
+			    vpt->name);
+			return;
+		}
 		mgt_vcl_dep_del(VTAILQ_FIRST(&vpl->dfrom));
 		AN(VTAILQ_EMPTY(&vpl->dfrom));
 	} else {
@@ -698,7 +827,7 @@ mcf_vcl_label(struct cli *cli, const char * const *av, void *priv)
 	if (vpt->state == VCL_STATE_COLD)
 		vpt->state = VCL_STATE_AUTO;
 	(void)mgt_vcl_setstate(cli, vpt, VCL_STATE_WARM);
-	if (child_pid < 0)
+	if (!MCH_Running())
 		return;
 
 	i = mgt_cli_askchild(&status, &p, "vcl.label %s %s\n", av[2], av[3]);
