@@ -29,21 +29,16 @@
 
 #include "config.h"
 
-#include <sys/types.h>
-#include <sys/socket.h>
+#include "cache/cache.h"
 
 #include <netinet/in.h>
 
 #include <netdb.h>
-#include <stdlib.h>
-#include <string.h>
 
-#include "cache/cache.h"
 #include "cache/cache_transport.h"
 
 #include "vend.h"
 #include "vsa.h"
-#include "vsb.h"
 #include "vtcp.h"
 
 /**********************************************************************
@@ -152,6 +147,7 @@ vpx_proto1(const struct worker *wrk, struct req *req)
 	VSL(SLT_Proxy, req->sp->vxid, "1 %s %s %s %s",
 	    fld[1], fld[3], fld[2], fld[4]);
 	HTC_RxPipeline(req->htc, q);
+	WS_Reset(req->htc->ws, 0);
 	return (0);
 }
 
@@ -186,6 +182,7 @@ vpx_proto2(const struct worker *wrk, struct req *req)
 	l = vbe16dec(req->htc->rxbuf_b + 14);
 	assert(req->htc->rxbuf_e - req->htc->rxbuf_b >= 16L + l);
 	HTC_RxPipeline(req->htc, req->htc->rxbuf_b + 16L + l);
+	WS_Reset(req->ws, 0);
 	p = (const void *)req->htc->rxbuf_b;
 
 	/* Version @12 top half */
@@ -196,7 +193,7 @@ vpx_proto2(const struct worker *wrk, struct req *req)
 	}
 
 	/* Command @12 bottom half */
-	switch(p[12] & 0x0f) {
+	switch (p[12] & 0x0f) {
 	case 0x0:
 		/* Local connection from proxy, ignore addresses */
 		return (0);
@@ -210,7 +207,7 @@ vpx_proto2(const struct worker *wrk, struct req *req)
 	}
 
 	/* Address family & protocol @13 */
-	switch(p[13]) {
+	switch (p[13]) {
 	case 0x00:
 		/* UNSPEC|UNSPEC, ignore proxy header */
 		VSL(SLT_ProxyGarbage, req->sp->vxid,
@@ -301,6 +298,9 @@ vpx_complete(struct http_conn *htc)
 
 	CHECK_OBJ_NOTNULL(htc, HTTP_CONN_MAGIC);
 
+	assert(htc->rxbuf_e >= htc->rxbuf_b);
+	assert(htc->rxbuf_e <= htc->ws->r);
+
 	l = htc->rxbuf_e - htc->rxbuf_b;
 	p = htc->rxbuf_b;
 	j = 0x3;
@@ -312,6 +312,8 @@ vpx_complete(struct http_conn *htc)
 		if (j == 0)
 			return (HTC_S_JUNK);
 		if (j == 1 && i == sizeof vpx1_sig) {
+			assert (htc->rxbuf_e < htc->ws->r);
+			*htc->rxbuf_e = '\0';
 			q = strchr(p + i, '\n');
 			if (q != NULL && (q - htc->rxbuf_b) > 107)
 				return (HTC_S_OVERFLOW);
