@@ -33,50 +33,39 @@
 
 #include "config.h"
 
+#include "cache/cache.h"
+
 #include <stdio.h>
 #include <stdlib.h>
 
-#include "cache/cache.h"
-#include "hash/hash_slinger.h"
-
 #include "storage/storage.h"
 #include "vrt.h"
 #include "vrt_obj.h"
 
-static const struct stevedore * volatile stv_next;
+
+static pthread_mutex_t stv_mtx;
 
 /*--------------------------------------------------------------------
  * XXX: trust pointer writes to be atomic
  */
 
-static struct stevedore *
-stv_pick_stevedore(struct vsl_log *vsl, const char **hint)
+const struct stevedore *
+STV_next()
 {
-	struct stevedore *stv;
+	static struct stevedore *stv;
+	struct stevedore *r;
 
-	AN(hint);
-	if (*hint != NULL && **hint != '\0') {
-		VTAILQ_FOREACH(stv, &stv_stevedores, list) {
-			if (!strcmp(stv->ident, *hint))
-				return (stv);
-		}
-		if (!strcmp(TRANSIENT_STORAGE, *hint))
-			return (stv_transient);
-
-		/* Hint was not valid, nuke it */
-		VSLb(vsl, SLT_Debug, "Storage hint not usable");
-		*hint = NULL;
+	AZ(pthread_mutex_lock(&stv_mtx));
+	if (!STV__iter(&stv))
+		AN(STV__iter(&stv));
+	if (stv == stv_transient) {
+		stv = NULL;
+		AN(STV__iter(&stv));
 	}
-	if (stv_next == NULL)
-		return (stv_transient);
-	/* pick a stevedore and bump the head along */
-	stv = VTAILQ_NEXT(stv_next, list);
-	if (stv == NULL)
-		stv = VTAILQ_FIRST(&stv_stevedores);
-	AN(stv);
-	AN(stv->name);
-	stv_next = stv;
-	return (stv);
+	r = stv;
+	AZ(pthread_mutex_unlock(&stv_mtx));
+	AN(r);
+	return (r);
 }
 
 /*-------------------------------------------------------------------
@@ -87,31 +76,16 @@ stv_pick_stevedore(struct vsl_log *vsl, const char **hint)
 
 int
 STV_NewObject(struct worker *wrk, struct objcore *oc,
-    const char *hint, unsigned wsl)
+    const struct stevedore *stv, unsigned wsl)
 {
-	struct stevedore *stv, *stv0;
-	int j;
-
 	CHECK_OBJ_NOTNULL(oc, OBJCORE_MAGIC);
 	CHECK_OBJ_NOTNULL(wrk, WORKER_MAGIC);
+	CHECK_OBJ_NOTNULL(stv, STEVEDORE_MAGIC);
 	assert(wsl > 0);
 
-	stv = stv0 = stv_pick_stevedore(wrk->vsl, &hint);
+	wrk->strangelove = cache_param->nuke_limit;
 	AN(stv->allocobj);
-	j = stv->allocobj(wrk, stv, oc, wsl, 0);
-	if (j == 0 && hint == NULL) {
-		do {
-			stv = stv_pick_stevedore(wrk->vsl, &hint);
-			AN(stv->allocobj);
-			j = stv->allocobj(wrk, stv, oc, wsl, 0);
-		} while (j == 0 && stv != stv0);
-	}
-	if (j == 0 && cache_param->nuke_limit > 0) {
-		/* no luck; try to free some space and keep trying */
-		j = stv->allocobj(wrk, stv, oc, wsl, cache_param->nuke_limit);
-	}
-
-	if (j == 0)
+	if (stv->allocobj(wrk, stv, oc, wsl) == 0)
 		return (0);
 
 	wrk->stats->n_object++;
@@ -129,20 +103,14 @@ STV_open(void)
 	char buf[1024];
 
 	ASSERT_CLI();
-	VTAILQ_FOREACH(stv, &stv_stevedores, list) {
+	AZ(pthread_mutex_init(&stv_mtx, NULL));
+	STV_Foreach(stv) {
 		bprintf(buf, "storage.%s", stv->ident);
 		stv->vclname = strdup(buf);
 		AN(stv->vclname);
 		if (stv->open != NULL)
 			stv->open(stv);
 	}
-	stv = stv_transient;
-	bprintf(buf, "storage.%s", stv->ident);
-	stv->vclname = strdup(buf);
-	AN(stv->vclname);
-	if (stv->open != NULL)
-		stv->open(stv);
-	stv_next = VTAILQ_FIRST(&stv_stevedores);
 }
 
 void
@@ -154,11 +122,7 @@ STV_close(void)
 	ASSERT_CLI();
 	for (i = 1; i >= 0; i--) {
 		/* First send close warning */
-		VTAILQ_FOREACH(stv, &stv_stevedores, list)
-			if (stv->close != NULL)
-				stv->close(stv, i);
-		stv = stv_transient;
-		if (stv->close != NULL)
+		STV_Foreach(stv)
 			if (stv->close != NULL)
 				stv->close(stv, i);
 	}
@@ -176,7 +140,7 @@ STV_BanInfoDrop(const uint8_t *ban, unsigned len)
 	struct stevedore *stv;
 	int r = 0;
 
-	VTAILQ_FOREACH(stv, &stv_stevedores, list)
+	STV_Foreach(stv)
 		if (stv->baninfo != NULL)
 			r |= stv->baninfo(stv, BI_DROP, ban, len);
 
@@ -189,7 +153,7 @@ STV_BanInfoNew(const uint8_t *ban, unsigned len)
 	struct stevedore *stv;
 	int r = 0;
 
-	VTAILQ_FOREACH(stv, &stv_stevedores, list)
+	STV_Foreach(stv)
 		if (stv->baninfo != NULL)
 			r |= stv->baninfo(stv, BI_NEW, ban, len);
 
@@ -207,7 +171,7 @@ STV_BanExport(const uint8_t *bans, unsigned len)
 {
 	struct stevedore *stv;
 
-	VTAILQ_FOREACH(stv, &stv_stevedores, list)
+	STV_Foreach(stv)
 		if (stv->banexport != NULL)
 			stv->banexport(stv, bans, len);
 }
@@ -216,16 +180,14 @@ STV_BanExport(const uint8_t *bans, unsigned len)
  * VRT functions for stevedores
  */
 
-static const struct stevedore *
-stv_find(const char *nm)
+const struct stevedore *
+STV_find(const char *nm)
 {
-	const struct stevedore *stv;
+	struct stevedore *stv;
 
-	VTAILQ_FOREACH(stv, &stv_stevedores, list)
+	STV_Foreach(stv)
 		if (!strcmp(stv->ident, nm))
 			return (stv);
-	if (!strcmp(TRANSIENT_STORAGE, nm))
-		return (stv_transient);
 	return (NULL);
 }
 
@@ -233,7 +195,7 @@ int
 VRT_Stv(const char *nm)
 {
 
-	if (stv_find(nm) != NULL)
+	if (STV_find(nm) != NULL)
 		return (1);
 	return (0);
 }
@@ -250,7 +212,7 @@ VRT_STEVEDORE_string(VCL_STEVEDORE s)
 VCL_STEVEDORE
 VRT_stevedore(const char *nm)
 {
-	return (stv_find(nm));
+	return (STV_find(nm));
 }
 
 #define VRTSTVVAR(nm, vtype, ctype, dval)	\
@@ -259,7 +221,7 @@ VRT_Stv_##nm(const char *nm)			\
 {						\
 	const struct stevedore *stv;		\
 						\
-	stv = stv_find(nm);			\
+	stv = STV_find(nm);			\
 	if (stv == NULL)			\
 		return (dval);			\
 	if (stv->var_##nm == NULL)		\
@@ -268,4 +230,3 @@ VRT_Stv_##nm(const char *nm)			\
 }
 
 #include "tbl/vrt_stv_var.h"
-#undef VRTSTVVAR
