@@ -31,18 +31,7 @@
 
 #include "config.h"
 
-#include <sys/types.h>
 #include <sys/stat.h>
-#ifdef HAVE_SYS_MOUNT_H
-#  include <sys/param.h>
-#  include <sys/mount.h>
-#endif
-#ifdef HAVE_SYS_STATVFS_H
-#  include <sys/statvfs.h>
-#endif
-#ifdef HAVE_SYS_VFS_H
-#  include <sys/vfs.h>
-#endif
 
 #include <errno.h>
 #include <fcntl.h>
@@ -95,7 +84,7 @@ STV_GetFile(const char *fn, int *fdp, const char **fnp, const char *ctx)
 	VJ_master(JAIL_MASTER_STORAGE);
 	fd = open(fn, O_RDWR | O_CREAT | O_EXCL | O_LARGEFILE, 0600);
 	if (fd >= 0) {
-		VJ_fix_storage_file(fd);
+		VJ_fix_fd(fd, JAIL_FIXFD_FILE);
 		*fdp = fd;
 		*fnp = fn;
 		VJ_master(JAIL_MASTER_LOW);
@@ -134,7 +123,7 @@ STV_GetFile(const char *fn, int *fdp, const char **fnp, const char *ctx)
 		    ctx, fn);
 
 	*fdp = fd;
-	VJ_fix_storage_file(fd);
+	VJ_fix_fd(fd, JAIL_FIXFD_FILE);
 	VJ_master(JAIL_MASTER_LOW);
 	return (retval);
 }
@@ -199,7 +188,7 @@ STV_FileSize(int fd, const char *size, unsigned *granularity, const char *ctx)
 	 * just add OFF_MAX to <limits.h>...
 	 */
 	i = 0;
-	while(1) {
+	while (1) {
 		o = l;
 		if (o == l && o > 0)
 			break;
