@@ -31,19 +31,22 @@
 
 #include "config.h"
 
+#include "cache/cache.h"
+
 #include <sys/mman.h>
 
 #include <errno.h>
 #include <stdio.h>
 #include <stdlib.h>
 
-#include "cache/cache.h"
 #include "storage/storage.h"
 #include "storage/storage_simple.h"
 
 #include "vnum.h"
 #include "vfil.h"
 
+#include "VSC_smf.h"
+
 #ifndef MAP_NOCORE
 #define MAP_NOCORE 0 /* XXX Linux */
 #endif
@@ -63,7 +66,7 @@
  */
 #define NBUCKET			(128 / 4 + 1)
 
-static struct VSC_C_lck *lck_smf;
+static struct VSC_lck *lck_smf;
 
 /*--------------------------------------------------------------------*/
 
@@ -90,7 +93,7 @@ struct smf_sc {
 	unsigned		magic;
 #define SMF_SC_MAGIC		0x52962ee7
 	struct lock		mtx;
-	struct VSC_C_smf	*stats;
+	struct VSC_smf		*stats;
 
 	const char		*filename;
 	int			fd;
@@ -155,7 +158,7 @@ smf_init(struct stevedore *parent, int ac, char * const *av)
 	parent->priv = sc;
 
 	(void)STV_GetFile(fn, &sc->fd, &sc->filename, "-sfile");
-	mgt_child_inherit(sc->fd, "storage_file");
+	MCH_Fd_Inherit(sc->fd, "storage_file");
 	sc->filesize = STV_FileSize(sc->fd, size, &sc->pagesize, "-sfile");
 	if (VFIL_allocate(sc->fd, (off_t)sc->filesize, 0))
 		ARGV_ERR("(-sfile) allocation error: %s\n", strerror(errno));
@@ -378,7 +381,7 @@ smf_open_chunk(struct smf_sc *sc, off_t sz, off_t off, off_t *fail, off_t *sum)
 		p = mmap(NULL, sz, PROT_READ|PROT_WRITE,
 		    MAP_NOCORE | MAP_NOSYNC | MAP_SHARED, sc->fd, off);
 		if (p != MAP_FAILED) {
-			(void) madvise(p, sz, sc->advice);
+			(void)madvise(p, sz, sc->advice);
 			(*sum) += sz;
 			new_smf(sc, p, off, sz);
 			return;
@@ -409,8 +412,7 @@ smf_open(struct stevedore *st)
 	if (lck_smf == NULL)
 		lck_smf = Lck_CreateClass("smf");
 	CAST_OBJ_NOTNULL(sc, st->priv, SMF_SC_MAGIC);
-	sc->stats = VSM_Alloc(sizeof *sc->stats,
-	    VSC_CLASS, VSC_type_smf, st->ident);
+	sc->stats = VSC_smf_New(st->ident);
 	Lck_New(&sc->mtx, lck_smf);
 	Lck_Lock(&sc->mtx);
 	smf_open_chunk(sc, sc->filesize, 0, &fail, &sum);
@@ -435,8 +437,8 @@ smf_alloc(const struct stevedore *st, size_t size)
 
 	CAST_OBJ_NOTNULL(sc, st->priv, SMF_SC_MAGIC);
 	assert(size > 0);
-	size += (sc->pagesize - 1L);
-	size &= ~(sc->pagesize - 1L);
+	size += (sc->pagesize - 1UL);
+	size &= ~(sc->pagesize - 1UL);
 	Lck_Lock(&sc->mtx);
 	sc->stats->c_req++;
 	smf = alloc_smf(sc, size);
