@@ -35,14 +35,13 @@
 
 #include "config.h"
 
-#include <sys/param.h>
+#include "cache/cache.h"
+
 #include <sys/mman.h>
 
-#include <stdint.h>
 #include <stdio.h>
 #include <stdlib.h>
 
-#include "cache/cache.h"
 #include "cache/cache_obj.h"
 #include "storage/storage.h"
 #include "storage/storage_simple.h"
@@ -56,7 +55,7 @@
 
 static struct obj_methods smp_oc_realmethods;
 
-static struct VSC_C_lck *lck_smp;
+static struct VSC_lck *lck_smp;
 
 static void smp_init(void);
 
@@ -73,7 +72,7 @@ static VTAILQ_HEAD(,smp_sc)	silos = VTAILQ_HEAD_INITIALIZER(silos);
  */
 
 static int
-smp_appendban(struct smp_sc *sc, struct smp_signspace *spc,
+smp_appendban(const struct smp_sc *sc, struct smp_signspace *spc,
     uint32_t len, const uint8_t *ban)
 {
 
@@ -136,7 +135,7 @@ smp_banexport(const struct stevedore *stv, const uint8_t *bans, unsigned len)
  */
 
 static int
-smp_open_bans(struct smp_sc *sc, struct smp_signspace *spc)
+smp_open_bans(const struct smp_sc *sc, struct smp_signspace *spc)
 {
 	uint8_t *ptr, *pe;
 	int i;
@@ -232,7 +231,7 @@ smp_open_segs(struct smp_sc *sc, struct smp_signspace *spc)
 
 	sg1 = NULL;
 	sg2 = NULL;
-	for(; ss <= se; ss++) {
+	for (; ss <= se; ss++) {
 		ALLOC_OBJ(sg, SMP_SEG_MAGIC);
 		AN(sg);
 		VTAILQ_INIT(&sg->objcores);
@@ -313,7 +312,7 @@ smp_thread(struct worker *wrk, void *priv)
 	Lck_Unlock(&sc->mtx);
 	pthread_exit(0);
 
-	NEEDLESS_RETURN(NULL);
+	NEEDLESS(return NULL);
 }
 
 /*--------------------------------------------------------------------
@@ -505,7 +504,7 @@ smp_allocx(const struct stevedore *st, size_t min_size, size_t max_size,
 
 static int __match_proto__(storage_allocobj_f)
 smp_allocobj(struct worker *wrk, const struct stevedore *stv,
-    struct objcore *oc, unsigned wsl, int really)
+    struct objcore *oc, unsigned wsl)
 {
 	struct object *o;
 	struct storage *st;
@@ -518,7 +517,6 @@ smp_allocobj(struct worker *wrk, const struct stevedore *stv,
 	CHECK_OBJ_NOTNULL(wrk, WORKER_MAGIC);
 	CHECK_OBJ_NOTNULL(oc, OBJCORE_MAGIC);
 	CAST_OBJ_NOTNULL(sc, stv->priv, SMP_SC_MAGIC);
-	assert(really >= 0);
 
 	/* Don't entertain already dead objects */
 	if (oc->flags & OC_F_DYING)
@@ -531,24 +529,24 @@ smp_allocobj(struct worker *wrk, const struct stevedore *stv,
 	ltot = sizeof(struct object) + PRNDUP(wsl);
 	ltot = IRNUP(sc, ltot);
 
-	while (1) {
-		if (really > 0) {
-			if (!LRU_NukeOne(wrk, stv->lru))
-				return (0);
-			really--;
-		}
+	st = NULL;
+	sg = NULL;
+	so = NULL;
+	objidx = 0;
+
+	do {
 		st = smp_allocx(stv, ltot, ltot, &so, &objidx, &sg);
 		if (st != NULL && st->space < ltot) {
 			stv->sml_free(st);		// NOP
 			st = NULL;
 		}
-		if (st != NULL)
-			break;
-		if (!really)
-			return (0);
-	}
+	} while (st == NULL && LRU_NukeOne(wrk, stv->lru));
+	if (st == NULL)
+		return (0);
 
 	AN(st);
+	AN(sg);
+	AN(so);
 	assert(st->space >= ltot);
 
 	o = SML_MkObject(stv, oc, st->ptr);
@@ -607,7 +605,7 @@ const struct stevedore smp_stevedore = {
 };
 
 /*--------------------------------------------------------------------
- * Persistence is a bear to test unadultered, so we cheat by adding
+ * Persistence is a bear to test unadulterated, so we cheat by adding
  * a cli command we can use to make it do tricks for us.
  */
 
@@ -632,7 +630,7 @@ debug_report_silo(struct cli *cli, const struct smp_sc *sc)
 	}
 }
 
-static void
+static void __match_proto__(cli_func_t)
 debug_persistent(struct cli *cli, const char * const * av, void *priv)
 {
 	struct smp_sc *sc;
