@@ -30,14 +30,12 @@
 
 #include "config.h"
 
-#include <errno.h>
-#include <stdio.h>
+#include "cache/cache.h"
+
 #include <stdlib.h>
 
 #include "binary_heap.h"
 
-#include "cache/cache.h"
-
 #include "waiter/waiter_priv.h"
 #include "waiter/mgt_waiter.h"
 
@@ -72,10 +70,9 @@ Wait_Call(const struct waiter *w, struct waited *wp,
 {
 	CHECK_OBJ_NOTNULL(w, WAITER_MAGIC);
 	CHECK_OBJ_NOTNULL(wp, WAITED_MAGIC);
-	CHECK_OBJ_NOTNULL(wp->waitfor, WAITFOR_MAGIC);
-	AN(wp->waitfor->func);
+	AN(wp->func);
 	assert(wp->idx == BINHEAP_NOIDX);
-	wp->waitfor->func(wp, ev, now);
+	wp->func(wp, ev, now);
 }
 
 /**********************************************************************/
@@ -97,13 +94,15 @@ Wait_HeapInsert(const struct waiter *w, struct waited *wp)
  * XXX: any harm to come from it.  Caveat Emptor.
  */
 
-void
+int
 Wait_HeapDelete(const struct waiter *w, const struct waited *wp)
 {
 	CHECK_OBJ_NOTNULL(w, WAITER_MAGIC);
 	CHECK_OBJ_NOTNULL(wp, WAITED_MAGIC);
-	if (wp->idx != BINHEAP_NOIDX)
-		binheap_delete(w->heap, wp->idx);
+	if (wp->idx == BINHEAP_NOIDX)
+		return (0);
+	binheap_delete(w->heap, wp->idx);
+	return (1);
 }
 
 double
@@ -132,7 +131,8 @@ Wait_Enter(const struct waiter *w, struct waited *wp)
 	CHECK_OBJ_NOTNULL(w, WAITER_MAGIC);
 	CHECK_OBJ_NOTNULL(wp, WAITED_MAGIC);
 	assert(wp->fd > 0);			// stdin never comes here
-	CHECK_OBJ_NOTNULL(wp->waitfor, WAITFOR_MAGIC);
+	AN(wp->func);
+	AN(wp->tmo);
 	wp->idx = BINHEAP_NOIDX;
 	return (w->impl->enter(w->priv, wp));
 }
