@@ -26,9 +26,8 @@
  * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
  * SUCH DAMAGE.
  *
- * XXX: We need to pass sessions back into the event engine when they are
- * reused.  Not sure what the most efficient way is for that.  For now
- * write the session pointer to a pipe which the event engine monitors.
+ * Recommended reading: libev(3) "EVBACKEND_EPOLL" section
+ * - thank you, Marc Alexander Lehmann
  */
 
 //lint -e{766}
@@ -38,15 +37,12 @@
 
 #include <sys/epoll.h>
 
-#include <stdlib.h>
 #include <errno.h>
 
 #include "cache/cache.h"
 
 #include "waiter/waiter_priv.h"
-#include "waiter/mgt_waiter.h"
 #include "vtim.h"
-#include "vfil.h"
 
 #ifndef EPOLLRDHUP
 #  define EPOLLRDHUP 0
@@ -76,7 +72,7 @@ vwe_thread(void *priv)
 	struct waited *wp;
 	struct waiter *w;
 	double now, then;
-	int i, n;
+	int i, n, active;
 	struct vwe *vwe;
 	char c;
 
@@ -84,6 +80,7 @@ vwe_thread(void *priv)
 	w = vwe->waiter;
 	CHECK_OBJ_NOTNULL(w, WAITER_MAGIC);
 	THR_SetName("cache-epoll");
+	THR_Init();
 
 	now = VTIM_real();
 	while (1) {
@@ -104,7 +101,7 @@ vwe_thread(void *priv)
 			CHECK_OBJ_NOTNULL(wp, WAITED_MAGIC);
 			AZ(epoll_ctl(vwe->epfd, EPOLL_CTL_DEL, wp->fd, NULL));
 			vwe->nwaited--;
-			Wait_HeapDelete(w, wp);
+			AN(Wait_HeapDelete(w, wp));
 			Lck_Unlock(&vwe->mtx);
 			Wait_Call(w, wp, WAITER_TIMEOUT, now);
 		}
@@ -128,8 +125,12 @@ vwe_thread(void *priv)
 			}
 			CAST_OBJ_NOTNULL(wp, ep->data.ptr, WAITED_MAGIC);
 			Lck_Lock(&vwe->mtx);
-			Wait_HeapDelete(w, wp);
+			active = Wait_HeapDelete(w, wp);
 			Lck_Unlock(&vwe->mtx);
+			if (!active) {
+				VSL(SLT_Debug, wp->fd, "epoll: spurious event");
+				continue;
+			}
 			AZ(epoll_ctl(vwe->epfd, EPOLL_CTL_DEL, wp->fd, NULL));
 			vwe->nwaited--;
 			if (ep->events & EPOLLIN)
@@ -144,9 +145,9 @@ vwe_thread(void *priv)
 		if (vwe->nwaited == 0 && vwe->die)
 			break;
 	}
-	AZ(close(vwe->pipe[0]));
-	AZ(close(vwe->pipe[1]));
-	AZ(close(vwe->epfd));
+	closefd(&vwe->pipe[0]);
+	closefd(&vwe->pipe[1]);
+	closefd(&vwe->epfd);
 	return (NULL);
 }
 
@@ -219,6 +220,8 @@ vwe_fini(struct waiter *w)
 
 /*--------------------------------------------------------------------*/
 
+#include "waiter/mgt_waiter.h"
+
 const struct waiter_impl waiter_epoll = {
 	.name =		"epoll",
 	.init =		vwe_init,
