@@ -37,7 +37,6 @@
 #include "cache/cache.h"
 
 #include "waiter/waiter_priv.h"
-#include "waiter/mgt_waiter.h"
 #include "vtim.h"
 
 struct vwp {
@@ -162,6 +161,7 @@ vwp_main(void *priv)
 	int i;
 
 	THR_SetName("cache-poll");
+	THR_Init();
 	CAST_OBJ_NOTNULL(vwp, priv, VWP_MAGIC);
 	w = vwp->waiter;
 
@@ -179,7 +179,6 @@ vwp_main(void *priv)
 		if (vwp->pollfd[0].revents)
 			v--;
 		for (i = 1; i < vwp->hpoll;) {
-VSL(SLT_Debug, vwp->pollfd[i].fd, "POLL loop i=%d revents=0x%x", i, vwp->pollfd[i].revents);
 			assert(vwp->pollfd[i].fd != vwp->pipes[0]);
 			wp = vwp->idx[i];
 			CHECK_OBJ_NOTNULL(wp, WAITED_MAGIC);
@@ -190,13 +189,13 @@ VSL(SLT_Debug, vwp->pollfd[i].fd, "POLL loop i=%d revents=0x%x", i, vwp->pollfd[
 				v--;
 			then = Wait_When(wp);
 			if (then <= now) {
-				Wait_HeapDelete(w, wp);
+				AN(Wait_HeapDelete(w, wp));
 				Wait_Call(w, wp, WAITER_TIMEOUT, now);
 				vwp_del(vwp, i);
 			} else if (vwp->pollfd[i].revents & POLLIN) {
 				assert(wp->fd > 0);
 				assert(wp->fd == vwp->pollfd[i].fd);
-				Wait_HeapDelete(w, wp);
+				AN(Wait_HeapDelete(w, wp));
 				Wait_Call(w, wp, WAITER_ACTION, now);
 				vwp_del(vwp, i);
 			} else {
@@ -206,7 +205,7 @@ VSL(SLT_Debug, vwp->pollfd[i].fd, "POLL loop i=%d revents=0x%x", i, vwp->pollfd[
 		if (vwp->pollfd[0].revents)
 			vwp_dopipe(vwp);
 	}
-	NEEDLESS_RETURN(NULL);
+	NEEDLESS(return NULL);
 }
 
 /*--------------------------------------------------------------------*/
@@ -262,14 +261,16 @@ vwp_fini(struct waiter *w)
 	// XXX: set write pipe blocking
 	assert(write(vwp->pipes[1], &vp, sizeof vp) == sizeof vp);
 	AZ(pthread_join(vwp->thread, &vp));
-	AZ(close(vwp->pipes[0]));
-	AZ(close(vwp->pipes[1]));
+	closefd(&vwp->pipes[0]);
+	closefd(&vwp->pipes[1]);
 	free(vwp->pollfd);
 	free(vwp->idx);
 }
 
 /*--------------------------------------------------------------------*/
 
+#include "waiter/mgt_waiter.h"
+
 const struct waiter_impl waiter_poll = {
 	.name =		"poll",
 	.init =		vwp_init,
