@@ -107,9 +107,10 @@ vws_del(struct vws *vws, int fd)
 }
 
 static inline void
-vws_port_ev(struct vws *vws, struct waiter *w, port_event_t *ev, double now) {
+vws_port_ev(struct vws *vws, struct waiter *w, port_event_t *ev, double now)
+{
 	struct waited *wp;
-	if(ev->portev_source == PORT_SOURCE_USER) {
+	if (ev->portev_source == PORT_SOURCE_USER) {
 		CAST_OBJ_NOTNULL(wp, ev->portev_user, WAITED_MAGIC);
 		assert(wp->fd >= 0);
 		vws->nwaited++;
@@ -127,7 +128,7 @@ vws_port_ev(struct vws *vws, struct waiter *w, port_event_t *ev, double now) {
 		 *          threadID=129476&tstart=0
 		 */
 		vws_del(vws, wp->fd);
-		Wait_HeapDelete(w, wp);
+		AN(Wait_HeapDelete(w, wp));
 		Wait_Call(w, wp, ev->portev_events & POLLERR ?
 		    WAITER_REMCLOSE : WAITER_ACTION,
 		    now);
@@ -151,6 +152,7 @@ vws_thread(void *priv)
 	w = vws->waiter;
 	CHECK_OBJ_NOTNULL(w, WAITER_MAGIC);
 	THR_SetName("cache-ports");
+	THR_Init();
 
 	now = VTIM_real();
 
@@ -166,7 +168,7 @@ vws_thread(void *priv)
 			}
 			CHECK_OBJ_NOTNULL(wp, WAITED_MAGIC);
 			vws_del(vws, wp->fd);
-			Wait_HeapDelete(w, wp);
+			AN(Wait_HeapDelete(w, wp));
 			Wait_Call(w, wp, WAITER_TIMEOUT, now);
 		}
 		then = vws->next - now;
@@ -255,7 +257,7 @@ vws_fini(struct waiter *w)
 
 	CAST_OBJ_NOTNULL(vws, w->priv, VWS_MAGIC);
 	vws->die = 1;
-	AZ(close(vws->dport));
+	closefd(&vws->dport);
 	AZ(pthread_join(vws->thread, &vp));
 }
 
