@@ -54,13 +54,6 @@ enum wait_event {
 
 typedef void waiter_handle_f(struct waited *, enum wait_event, double now);
 
-struct waitfor {
-	unsigned		magic;
-#define WAITFOR_MAGIC		0x16b79246
-	waiter_handle_f		*func;
-	volatile double		*tmo;
-};
-
 struct waited {
 	unsigned		magic;
 #define WAITED_MAGIC		0x1743992d
@@ -68,7 +61,8 @@ struct waited {
 	unsigned		idx;
 	void			*priv1;
 	uintptr_t		priv2;
-	const struct waitfor	*waitfor;
+	waiter_handle_f		*func;
+	volatile double		*tmo;
 	double			idle;
 };
 
