@@ -33,33 +33,32 @@
 
 #include "config.h"
 
-#include <sys/types.h>
-
 #include <errno.h>
 #include <limits.h>
 #include <math.h>
 #include <pthread.h>
-#include <regex.h>
 #include <signal.h>
-#include <stdint.h>
+#include <stdarg.h>
 #include <stdlib.h>
 #include <string.h>
 #include <unistd.h>
 
+#define VOPT_DEFINITION
+#define VOPT_INC "varnishhist_options.h"
+
 #include "vdef.h"
 #include "vcurses.h"
 #include "vapi/vsl.h"
 #include "vapi/vsm.h"
 #include "vapi/voptget.h"
 #include "vas.h"
-#include "vcs.h"
 #include "vut.h"
 #include "vtim.h"
 
 #define HIST_N 2000		/* how far back we remember */
 #define HIST_RES 100		/* bucket resolution */
 
-static const char progname[] = "varnishhist";
+static struct VUT *vut;
 
 static int hist_low;
 static int hist_high;
@@ -77,12 +76,13 @@ static unsigned *bucket_miss;
 static unsigned *bucket_hit;
 static char *format;
 static int match_tag;
-double timebend = 0, t0;
-double vsl_t0 = 0, vsl_to, vsl_ts = 0;
-pthread_cond_t timebend_cv;
+static double timebend = 0, t0;
+static double vsl_t0 = 0, vsl_to, vsl_ts = 0;
+static pthread_cond_t timebend_cv;
 static double log_ten;
+static char *ident;
 
-static int scales[] = {
+static const int scales[] = {
 	1,
 	2,
 	3,
@@ -114,13 +114,14 @@ struct profile {
 	int field;
 	int hist_low;
 	int hist_high;
-}
+};
+
 #define HIS_PROF(name,vsl_arg,tag,prefix,field,hist_low,high_high,doc)	\
 	{name,vsl_arg,tag,prefix,field,hist_low,high_high},
 #define HIS_NO_PREFIX	NULL
 #define HIS_CLIENT	'c'
 #define HIS_BACKEND	'b'
-profiles[] = {
+static const struct profile profiles[] = {
 #include "varnishhist_profiles.h"
 	{ NULL }
 };
@@ -129,14 +130,16 @@ profiles[] = {
 #undef HIS_CLIENT
 #undef HIS_PROF
 
-static struct profile *active_profile;
+static const struct profile *active_profile;
+
+static volatile sig_atomic_t quit = 0;
 
 static void
 update(void)
 {
 	char t[VTIM_FORMAT_SIZE];
-	unsigned w = COLS / hist_range;
-	unsigned n = w * hist_range;
+	const unsigned w = COLS / hist_range;
+	const unsigned n = w * hist_range;
 	unsigned bm[n], bh[n];
 	unsigned max;
 	unsigned i, j, scale;
@@ -155,7 +158,7 @@ update(void)
 	if (end_of_file)
 		mvprintw(0, 0, "%*s", COLS - 1, "EOF");
 	else
-		mvprintw(0, 0, "%*s", COLS - 1, VUT.name);
+		mvprintw(0, 0, "%*s", COLS - 1, ident);
 
 	/* count our flock */
 	for (i = 0; i < n; ++i)
@@ -177,14 +180,14 @@ update(void)
 	if (vsl_t0 > 0) {
 		VTIM_format(vsl_ts, t);
 
-		mvprintw(0, 0, "1:%d, n = %d, d = %g @ %s x %g",
+		mvprintw(0, 0, "1:%u, n = %u, d = %g @ %s x %g",
 		    scale, nhist, delay, t, timebend);
 	} else
-		mvprintw(0, 0, "1:%d, n = %d, d = %g",
+		mvprintw(0, 0, "1:%u, n = %u, d = %g",
 		    scale, nhist, delay);
 
 	for (j = 2; j < LINES - 3; j += 5)
-		mvprintw(j, 0, "%d_", (LINES - 3 - j) * scale);
+		mvprintw(j, 0, "%u_", (LINES - 3 - j) * scale);
 
 	/* show them */
 	for (i = 0; i < n; ++i) {
@@ -216,13 +219,13 @@ upd_vsl_ts(const char *p)
 		vsl_ts = t;
 }
 
-static int /*__match_proto__ (VSLQ_dispatch_f)*/
+static int __match_proto__ (VSLQ_dispatch_f)
 accumulate(struct VSL_data *vsl, struct VSL_transaction * const pt[],
     void *priv)
 {
 	int i, tag, skip, match, hit;
 	unsigned u;
-	double value;
+	double value = 0;
 	struct VSL_transaction *tr;
 	double t;
 	const char *tsp;
@@ -355,10 +358,10 @@ accumulate(struct VSL_data *vsl, struct VSL_transaction * const pt[],
 	if (vsl_ts > vsl_to) {
 		double when = VTIM_real() + vsl_ts - vsl_to;
 		struct timespec ts;
-		// Lck_CondWait
 		ts.tv_nsec = (long)(modf(when, &t) * 1e9);
 		ts.tv_sec = (long)t;
-		(void)pthread_cond_timedwait(&timebend_cv, &mtx, &ts);
+		i = pthread_cond_timedwait(&timebend_cv, &mtx, &ts);
+		assert(i == 0 || i == ETIMEDOUT);
 	}
 	AZ(pthread_mutex_unlock(&mtx));
 
@@ -366,12 +369,14 @@ accumulate(struct VSL_data *vsl, struct VSL_transaction * const pt[],
 }
 
 static int __match_proto__(VUT_cb_f)
-sighup(void)
+sighup(struct VUT *v)
 {
-	exit(1);
+	assert(v == vut);
+	quit = 1;
+	return (1);
 }
 
-static void *
+static void * __match_proto__(pthread_t)
 do_curses(void *arg)
 {
 	int ch;
@@ -384,7 +389,7 @@ do_curses(void *arg)
 	intrflush(stdscr, FALSE);
 	curs_set(0);
 	erase();
-	for (;;) {
+	while (!quit) {
 		AZ(pthread_mutex_lock(&mtx));
 		update();
 		AZ(pthread_mutex_unlock(&mtx));
@@ -413,7 +418,7 @@ do_curses(void *arg)
 		case 'q':
 			raise(SIGINT);
 			endwin();
-			pthread_exit(NULL);
+			return (NULL);
 		case '0':
 		case '1':
 		case '2':
@@ -444,7 +449,7 @@ do_curses(void *arg)
 		}
 
 		if (ch == '<' || ch == '>') {
-			pthread_mutex_lock(&mtx);
+			AZ(pthread_mutex_lock(&mtx));
 			vsl_to = vsl_t0 = vsl_ts;
 			t0 = VTIM_mono();
 			if (timebend == 0)
@@ -453,21 +458,22 @@ do_curses(void *arg)
 				timebend /= 2;
 			else
 				timebend *= 2;
-			pthread_cond_broadcast(&timebend_cv);
-			pthread_mutex_unlock(&mtx);
+			AZ(pthread_cond_broadcast(&timebend_cv));
+			AZ(pthread_mutex_unlock(&mtx));
 		}
 	}
-	pthread_exit(NULL);
+	endwin();
+	return (NULL);
 }
 
 /*--------------------------------------------------------------------*/
 
-static void
+static void __attribute__((__noreturn__))
 usage(int status)
 {
 	const char **opt;
 
-	fprintf(stderr, "Usage: %s <options>\n\n", progname);
+	fprintf(stderr, "Usage: %s <options>\n\n", vut->progname);
 	fprintf(stderr, "Options:\n");
 	for (opt = vopt_spec.vopt_usage; *opt != NULL; opt += 2)
 		fprintf(stderr, " %-25s %s\n", *opt, *(opt + 1));
@@ -482,6 +488,12 @@ profile_error(const char *s)
 	exit(1);
 }
 
+static void
+vut_sighandler(int sig)
+{
+	VUT_Signaled(vut, sig);
+}
+
 int
 main(int argc, char **argv)
 {
@@ -493,7 +505,8 @@ main(int argc, char **argv)
 	struct profile cli_p = {0};
 	cli_p.name = 0;
 
-	VUT_Init(progname, argc, argv, &vopt_spec);
+	vut = VUT_InitProg(argc, argv, &vopt_spec);
+	AN(vut);
 	AZ(pthread_cond_init(&timebend_cv, NULL));
 
 	while ((i = getopt(argc, argv, vopt_spec.vopt_optstring)) != -1) {
@@ -504,7 +517,7 @@ main(int argc, char **argv)
 		case 'p':
 			delay = strtod(optarg, NULL);
 			if (delay <= 0)
-				VUT_Error(1, "-p: invalid '%s'", optarg);
+				VUT_Error(vut, 1, "-p: invalid '%s'", optarg);
 			break;
 		case 'P':
 			colon = strchr(optarg, ':');
@@ -533,7 +546,7 @@ main(int argc, char **argv)
 
 			match_tag = VSL_Name2Tag(ptag, colon - ptag);
 			if (match_tag < 0)
-				VUT_Error(1,
+				VUT_Error(vut, 1,
 				    "-P: '%s' is not a valid tag name",
 				    optarg);
 			cli_p.name = "custom";
@@ -545,18 +558,18 @@ main(int argc, char **argv)
 		case 'B':
 			timebend = strtod(optarg, NULL);
 			if (timebend == 0)
-				VUT_Error(1,
+				VUT_Error(vut, 1,
 				    "-B: being able to bend time does not"
 				    " mean we can stop it"
 				    " (invalid factor '%s')", optarg);
 			if (timebend < 0)
-				VUT_Error(1,
+				VUT_Error(vut, 1,
 				    "-B: being able to bend time does not"
 				    " mean we can make it go backwards"
 				    " (invalid factor '%s')", optarg);
 			break;
 		default:
-			if (!VUT_Arg(i, optarg))
+			if (!VUT_Arg(vut, i, optarg))
 				usage(1);
 		}
 	}
@@ -565,11 +578,11 @@ main(int argc, char **argv)
 		usage(1);
 
 	/* Check for valid grouping mode */
-	assert(VUT.g_arg < VSL_g__MAX);
-	if (VUT.g_arg != VSL_g_vxid && VUT.g_arg != VSL_g_request)
-		VUT_Error(1, "Invalid grouping mode: %s"
+	assert(vut->g_arg < VSL_g__MAX);
+	if (vut->g_arg != VSL_g_vxid && vut->g_arg != VSL_g_request)
+		VUT_Error(vut, 1, "Invalid grouping mode: %s"
 		    " (only vxid and request are supported)",
-		    VSLQ_grouping[VUT.g_arg]);
+		    VSLQ_grouping[vut->g_arg]);
 
 	if (profile) {
 		for (active_profile = profiles; active_profile->name;
@@ -578,10 +591,13 @@ main(int argc, char **argv)
 				break;
 		}
 	}
+	AN(active_profile);
 	if (!active_profile->name)
-		VUT_Error(1, "-P: No such profile '%s'", profile);
+		VUT_Error(vut, 1, "-P: No such profile '%s'", profile);
 
-	assert(VUT_Arg(active_profile->VSL_arg, NULL));
+	assert(active_profile->VSL_arg == 'b' ||
+	    active_profile->VSL_arg == 'c');
+	assert(VUT_Arg(vut, active_profile->VSL_arg, NULL));
 	match_tag = active_profile->tag;
 	fnum = active_profile->field;
 	hist_low = active_profile->hist_low;
@@ -589,28 +605,31 @@ main(int argc, char **argv)
 
 	hist_range = hist_high - hist_low;
 	hist_buckets = hist_range * HIST_RES;
-	bucket_hit = calloc(sizeof *bucket_hit, hist_buckets);
-	bucket_miss = calloc(sizeof *bucket_miss, hist_buckets);
+	bucket_hit = calloc(hist_buckets, sizeof *bucket_hit);
+	bucket_miss = calloc(hist_buckets, sizeof *bucket_miss);
 
 	if (timebend > 0)
 		t0 = VTIM_mono();
 
-	format = malloc(4 * fnum);
+	format = malloc(4L * fnum);
+	AN(format);
 	for (i = 0; i < fnum - 1; i++)
 		strcpy(format + 4 * i, "%*s ");
 	strcpy(format + 4 * (fnum - 1), "%lf");
 
 	log_ten = log(10.0);
 
-	VUT_Setup();
+	VUT_Signal(vut_sighandler);
+	VUT_Setup(vut);
+	ident = VSM_Dup(vut->vsm, "Arg", "-i");
 	if (pthread_create(&thr, NULL, do_curses, NULL) != 0)
-		VUT_Error(1, "pthread_create(): %s", strerror(errno));
-	VUT.dispatch_f = &accumulate;
-	VUT.dispatch_priv = NULL;
-	VUT.sighup_f = sighup;
-	VUT_Main();
+		VUT_Error(vut, 1, "pthread_create(): %s", strerror(errno));
+	vut->dispatch_f = accumulate;
+	vut->dispatch_priv = NULL;
+	vut->sighup_f = sighup;
+	VUT_Main(vut);
 	end_of_file = 1;
 	AZ(pthread_join(thr, NULL));
-	VUT_Fini();
+	VUT_Fini(&vut);
 	exit(0);
 }
