@@ -73,16 +73,15 @@ HIS_OPT_g
 VUT_OPT_h
 VSL_OPT_L
 VUT_OPT_n
-VUT_OPT_N
 HIS_OPT_p
 #define HIS_CLIENT	"client"
 #define HIS_BACKEND	"backend"
 #define HIS_NO_PREFIX	""
-#define HIS_PROF(name,cb,tag,prefix,field,hist_low,high_high,doc)	\
+#define HIS_PROF(name,cb,tg,prefix,fld,hist_low,high_high,doc)		\
 	VOPT("P:", "[-P " name "]",					\
 	     "Predefined " cb " profile",				\
 	     "Predefined " cb " profile: " doc				\
-	     " (field " #field " of " #tag " " prefix " VSL tag)."	\
+	     " (field " #fld " of " #tg " " prefix " VSL tag)."		\
 	    )
 #include "varnishhist_profiles.h"
 #undef HIS_NO_PREFIX
@@ -94,4 +93,4 @@ VUT_OPT_q
 VUT_OPT_r
 VUT_OPT_t
 VSL_OPT_T
-VUT_OPT_V
+VUT_GLOBAL_OPT_V
