@@ -30,3 +30,6 @@
 VTC_VMOD(std)
 VTC_VMOD(debug)
 VTC_VMOD(directors)
+VTC_VMOD(purge)
+VTC_VMOD(vtc)
+VTC_VMOD(blob)
