@@ -28,20 +28,18 @@
 
 #include "config.h"
 
-#include <sys/types.h>
 #include <sys/wait.h>
 
 #include <ctype.h>
 #include <errno.h>
-#include <signal.h>
+#include <grp.h>
+#include <math.h>
+#include <pwd.h>
 #include <stdarg.h>
-#include <stdint.h>
 #include <stdio.h>
 #include <stdlib.h>
 #include <string.h>
 #include <unistd.h>
-#include <pwd.h>
-#include <grp.h>
 
 #include "vtc.h"
 
@@ -50,12 +48,17 @@
 #include "vre.h"
 #include "vtim.h"
 
+#ifdef HAVE_SYS_PERSONALITY_H
+#  include <sys/personality.h>
+#endif
+
 #define		MAX_TOKENS		200
 
 volatile sig_atomic_t	vtc_error;	/* Error encountered */
 int			vtc_stop;	/* Stops current test without error */
 pthread_t		vtc_thread;
 static struct vtclog	*vltop;
+static int		ign_unknown_macro = 0;
 
 /**********************************************************************
  * Macro facility
@@ -234,13 +237,18 @@ macro_expand(struct vtclog *vl, const char *text)
 		p += 2;
 		m = macro_get(p, q);
 		if (m == NULL) {
-			VSB_destroy(&vsb);
-			vtc_log(vl, 0, "Macro ${%.*s} not found", (int)(q - p),
-			    p);
-			NEEDLESS_RETURN (NULL);
+			if (!ign_unknown_macro) {
+				VSB_destroy(&vsb);
+				vtc_fatal(vl, "Macro ${%.*s} not found",
+					  (int)(q - p), p);
+				NEEDLESS(return (NULL));
+			}
+			VSB_printf(vsb, "${%.*s}", (int)(q - p), p);
+		}
+		else {
+			VSB_printf(vsb, "%s", m);
+			free(m);
 		}
-		VSB_printf(vsb, "%s", m);
-		free(m);
 		text = q + 1;
 	}
 	AZ(VSB_finish(vsb));
@@ -326,7 +334,7 @@ parse_string(const char *spec, const struct cmds *cmd, void *priv,
 						q++;
 					} else {
 						if (*p == '\n')
-							vtc_log(vl, 0,
+							vtc_fatal(vl,
 				"Unterminated quoted string in line: %*.*s",
 				(int)(p - f), (int)(p - f), f);
 						assert(*p != '\n');
@@ -375,10 +383,9 @@ parse_string(const char *spec, const struct cmds *cmd, void *priv,
 		for (cp = cmd; cp->name != NULL; cp++)
 			if (!strcmp(token_s[0], cp->name))
 				break;
-		if (cp->name == NULL) {
-			vtc_log(vl, 0, "Unknown command: \"%s\"", token_s[0]);
-			NEEDLESS_RETURN;
-		}
+
+		if (cp->name == NULL)
+			vtc_fatal(vl, "Unknown command: \"%s\"", token_s[0]);
 
 		assert(cp->cmd != NULL);
 		cp->cmd(token_s, priv, cmd, vl);
@@ -426,8 +433,9 @@ cmd_varnishtest(CMD_ARGS)
 
 /* SECTION: shell shell
  *
- * Pass the string given as argument to a shell. If you have multiple commands
- * to run, you can use curly barces to describe a multi-lines script, eg::
+ * Pass the string given as argument to a shell. If you have multiple
+ * commands to run, you can use curly barces to describe a multi-lines
+ * script, eg::
  *
  *         shell {
  *                 echo begin
@@ -435,26 +443,132 @@ cmd_varnishtest(CMD_ARGS)
  *                 echo end
  *         }
  *
- * The vtc will fail if the return code of the shell is not 0.
+ * By default a zero exit code is expected, otherwise the vtc will fail.
+ *
+ * Notice that the commandstring is prefixed with "exec 2>&1;" to join
+ * stderr and stdout back to the varnishtest process.
+ *
+ * Optional arguments:
+ *
+ * \-err
+ *	Expect non-zero exit code.
+ *
+ * \-exit N
+ *	Expect exit code N instead of zero.
+ *
+ * \-expect STRING
+ *	Expect string to be found in stdout+err.
+ *
+ * \-match REGEXP
+ *	Expect regexp to match the stdout+err output.
+ */
+/* SECTION: client-server.spec.shell shell
+ *
+ * Same as for the top-level shell.
  */
 
 static void
+cmd_shell_engine(struct vtclog *vl, int ok, const char *cmd,
+    const char *expect, const char *re)
+{
+	struct vsb *vsb;
+	FILE *fp;
+	vre_t *vre = NULL;
+	const char *errptr;
+	int r, c;
+	int err;
+
+	AN(vl);
+	AN(cmd);
+	vsb = VSB_new_auto();
+	AN(vsb);
+	if (re != NULL) {
+		vre = VRE_compile(re, 0, &errptr, &err);
+		if (vre == NULL)
+			vtc_fatal(vl, "shell_match invalid regexp (\"%s\")",
+			    re);
+	}
+	VSB_printf(vsb, "exec 2>&1 ; %s", cmd);
+	AZ(VSB_finish(vsb));
+	vtc_dump(vl, 4, "shell_cmd", VSB_data(vsb), -1);
+	fp = popen(VSB_data(vsb), "r");
+	if (fp == NULL)
+		vtc_fatal(vl, "popen fails: %s", strerror(errno));
+	VSB_clear(vsb);
+	do {
+		c = getc(fp);
+		if (c != EOF)
+			VSB_putc(vsb, c);
+	} while (c != EOF);
+	r = pclose(fp);
+	AZ(VSB_finish(vsb));
+	vtc_dump(vl, 4, "shell_out", VSB_data(vsb), VSB_len(vsb));
+	vtc_log(vl, 4, "shell_status = 0x%04x", WEXITSTATUS(r));
+	if (WIFSIGNALED(r))
+		vtc_log(vl, 4, "shell_signal = %d", WTERMSIG(r));
+
+	if (ok < 0 && !WEXITSTATUS(r) && !WIFSIGNALED(r))
+		vtc_fatal(vl, "shell did not fail as expected");
+	else if (ok >= 0 && WEXITSTATUS(r) != ok)
+		vtc_fatal(vl, "shell_exit not as expected: "
+		    "got 0x%04x wanted 0x%04x", WEXITSTATUS(r), ok);
+
+	if (expect != NULL) {
+		if (strstr(VSB_data(vsb), expect) == NULL)
+			vtc_fatal(vl,
+			    "shell_expect not found: (\"%s\")", expect);
+		else
+			vtc_log(vl, 4, "shell_expect found");
+	} else if (vre != NULL) {
+		if (VRE_exec(vre, VSB_data(vsb), VSB_len(vsb), 0, 0,
+		    NULL, 0, NULL) < 1)
+			vtc_fatal(vl,
+			    "shell_match failed: (\"%s\")", re);
+		else
+			vtc_log(vl, 4, "shell_match succeeded");
+		VRE_free(&vre);
+	}
+	VSB_destroy(&vsb);
+}
+
+
+void
 cmd_shell(CMD_ARGS)
 {
+	const char *expect = NULL;
+	const char *re = NULL;
+	int n;
+	int ok = 0;
+
 	(void)priv;
 	(void)cmd;
-	int r, s;
 
 	if (av == NULL)
 		return;
-	AN(av[1]);
-	AZ(av[2]);
-	vtc_dump(vl, 4, "shell", av[1], -1);
-	r = system(av[1]);
-	s = WEXITSTATUS(r);
-	if (s != 0)
-		vtc_log(vl, 0, "CMD '%s' failed with status %d (%s)",
-		    av[1], s, strerror(errno));
+	for (n = 1; av[n] != NULL; n++) {
+		if (!strcmp(av[n], "-err")) {
+			ok = -1;
+		} else if (!strcmp(av[n], "-exit")) {
+			n += 1;
+			ok = atoi(av[n]);
+		} else if (!strcmp(av[n], "-expect")) {
+			if (re != NULL)
+				vtc_fatal(vl,
+				    "Cannot use -expect with -match");
+			n += 1;
+			expect = av[n];
+		} else if (!strcmp(av[n], "-match")) {
+			if (expect != NULL)
+				vtc_fatal(vl,
+				    "Cannot use -match with -expect");
+			n += 1;
+			re = av[n];
+		} else {
+			break;
+		}
+	}
+	AN(av[n]);
+	cmd_shell_engine(vl, ok, av[n], expect, re);
 }
 
 /* SECTION: err_shell err_shell
@@ -473,52 +587,66 @@ cmd_err_shell(CMD_ARGS)
 {
 	(void)priv;
 	(void)cmd;
-	struct vsb *vsb;
-	FILE *fp;
-	int r, c;
 
 	if (av == NULL)
 		return;
 	AN(av[1]);
 	AN(av[2]);
 	AZ(av[3]);
-	vsb = VSB_new_auto();
-	AN(vsb);
-	vtc_dump(vl, 4, "cmd", av[2], -1);
-	fp = popen(av[2], "r");
-	if (fp == NULL)
-		vtc_log(vl, 0, "popen fails: %s", strerror(errno));
-	do {
-		c = getc(fp);
-		if (c != EOF)
-			VSB_putc(vsb, c);
-	} while (c != EOF);
-	r = pclose(fp);
-	vtc_log(vl, 4, "Status = %d", WEXITSTATUS(r));
-	if (WIFSIGNALED(r))
-		vtc_log(vl, 4, "Signal = %d", WTERMSIG(r));
-	if (WEXITSTATUS(r) == 0) {
-		vtc_log(vl, 0,
-		    "expected error from shell");
-	}
-	AZ(VSB_finish(vsb));
-	vtc_dump(vl, 4, "stdout", VSB_data(vsb), VSB_len(vsb));
-	if (strstr(VSB_data(vsb), av[1]) == NULL)
-		vtc_log(vl, 0,
-		    "Did not find expected string: (\"%s\")", av[1]);
-	else
-		vtc_log(vl, 4,
-		    "Found expected string: (\"%s\")", av[1]);
-	VSB_destroy(&vsb);
+	vtc_log(vl, 1,
+	    "NOTICE: err_shell is deprecated, use 'shell -err -expect'");
+	cmd_shell_engine(vl, -1, av[2], av[1], NULL);
 }
 
-/* SECTION: client-server.spec.delay delay
+/* SECTION: setenv setenv
  *
- * Take a float as argument and sleep for that number of seconds.
+ * Set or change an environment variable::
+ *
+ *         setenv FOO "bar baz"
+ *
+ * The above will set the environment variable $FOO to the value
+ * provided. There is also an ``-ifunset`` argument which will only
+ * set the value if the the environment variable does not already
+ * exist::
+ *
+ *        setenv -ifunset FOO quux
+ */
+static void
+cmd_setenv(CMD_ARGS)
+{
+	int r;
+	int force;
+
+	(void)priv;
+	(void)cmd;
+
+	if (av == NULL)
+		return;
+	AN(av[1]);
+	AN(av[2]);
+
+	force = 1;
+	if (strcmp("-ifunset", av[1]) == 0) {
+		force = 0;
+		av++;
+		AN(av[2]);
+	}
+	if (av[3] != NULL)
+		vtc_fatal(vl, "CMD setenv: Unexpected argument '%s'", av[3]);
+	r = setenv(av[1], av[2], force);
+	if (r != 0)
+		vtc_log(vl, 0, "CMD setenv %s=\"%s\" failed: %s",
+		    av[1], av[2], strerror(errno));
+}
+
+/* SECTION: delay delay
+ *
+ * Sleep for the number of seconds specified in the argument. The number
+ * can include a fractional part, e.g. 1.5.
  */
 /* SECTION: stream.spec.delay delay
  *
- * Take a float as argument and sleep for that number of seconds.
+ * Same as for the top-level delay.
  */
 void
 cmd_delay(CMD_ARGS)
@@ -532,14 +660,17 @@ cmd_delay(CMD_ARGS)
 	AN(av[1]);
 	AZ(av[2]);
 	f = VNUM(av[1]);
+	if (isnan(f))
+		vtc_fatal(vl, "Syntax error in number (%s)", av[1]);
 	vtc_log(vl, 3, "delaying %g second(s)", f);
 	VTIM_sleep(f);
 }
 
 /* SECTION: feature feature
  *
- * Test that the required feature(s) for a test are available, and skip the test
- * otherwise. feature takes any number of arguments from this list:
+ * Test that the required feature(s) for a test are available, and skip
+ * the test otherwise; or change the interpretation of the test, as
+ * documented below. feature takes any number of arguments from this list:
  *
  * SO_RCVTIMEO_WORKS
  *        The SO_RCVTIMEO socket option is working
@@ -561,6 +692,13 @@ cmd_delay(CMD_ARGS)
  *        The varnish group is present
  * cmd <command-line>
  *        A command line that should execute with a zero exit status
+ * ignore_unknown_macro
+ *        Do not fail the test if a string of the form ${...} is not
+ *        recognized as a macro.
+ *
+ * Be careful with the last feature, because it may cause a test with a
+ * misspelled macro to fail silently. You should only need it if you must
+ * run a test with strings of the form "${...}".
  */
 
 static void
@@ -581,7 +719,7 @@ cmd_feature(CMD_ARGS)
 			if (tst) {			\
 				good = 1;		\
 			} else {			\
-				vtc_stop = 1;		\
+				vtc_stop = 2;		\
 			}				\
 		}					\
 	} while (0)
@@ -592,7 +730,7 @@ cmd_feature(CMD_ARGS)
 #ifdef SO_RCVTIMEO_WORKS
 			good = 1;
 #else
-			vtc_stop = 1;
+			vtc_stop = 2;
 #endif
 		}
 
@@ -600,7 +738,7 @@ cmd_feature(CMD_ARGS)
 #if !defined(__APPLE__) || !defined(__MACH__)
 			good = 1;
 #else
-			vtc_stop = 1;
+			vtc_stop = 2;
 #endif
 		}
 		FEATURE("pcre_jit", VRE_has_jit);
@@ -612,28 +750,37 @@ cmd_feature(CMD_ARGS)
 		FEATURE("user_vcache", getpwnam("vcache") != NULL);
 		FEATURE("group_varnish", getgrnam("varnish") != NULL);
 
-		if (!strcmp(*av, "cmd")) {
-			av++;
-			if (*av == NULL) {
-				vtc_log(vl, 0, "Missing the command-line");
-				return;
+		if (!strcmp(*av, "disable_aslr")) {
+			good = 1;
+#ifdef HAVE_SYS_PERSONALITY_H
+			r = personality(0xffffffff);
+			r = personality(r | ADDR_NO_RANDOMIZE);
+			if (r < 0) {
+				good = 0;
+				vtc_stop = 2;
 			}
+#endif
+		} else if (!strcmp(*av, "cmd")) {
+			av++;
+			if (*av == NULL)
+				vtc_fatal(vl, "Missing the command-line");
 			r = system(*av);
 			if (WEXITSTATUS(r) == 0)
 				good = 1;
 			else
-				vtc_stop = 1;
+				vtc_stop = 2;
+		} else if (!strcmp(*av, "ignore_unknown_macro")) {
+			ign_unknown_macro = 1;
+			good = 1;
 		}
 		if (good)
 			continue;
 
-		if (!vtc_stop) {
-			vtc_log(vl, 0,
-			    "FAIL test, unknown feature: %s", *av);
-		} else {
+		if (!vtc_stop)
+			vtc_fatal(vl, "FAIL test, unknown feature: %s", *av);
+		else
 			vtc_log(vl, 1,
 			    "SKIPPING test, lacking feature: %s", *av);
-		}
 		return;
 	}
 }
@@ -643,37 +790,61 @@ cmd_feature(CMD_ARGS)
  */
 
 static const struct cmds cmds[] = {
-#define CMD(n) { #n, cmd_##n }
-	CMD(server),
-	CMD(client),
-	CMD(varnish),
-	CMD(delay),
-	CMD(varnishtest),
-	CMD(shell),
-	CMD(err_shell),
-	CMD(barrier),
-	CMD(feature),
-	CMD(logexpect),
-	CMD(process),
+#define CMD(n) { #n, cmd_##n },
+	CMD(server)
+	CMD(client)
+	CMD(varnish)
+	CMD(delay)
+	CMD(varnishtest)
+	CMD(shell)
+	CMD(err_shell)
+	CMD(barrier)
+	CMD(feature)
+	CMD(logexpect)
+	CMD(process)
+	CMD(setenv)
 #undef CMD
-	{ NULL,		NULL }
+	{ NULL, NULL }
 };
 
+static const char *tfn;
+
+int
+fail_out(void)
+{
+	unsigned old_err;
+
+	old_err = vtc_error;
+	if (!vtc_stop)
+		vtc_stop = 1;
+	vtc_log(vltop, 1, "RESETTING after %s", tfn);
+	reset_cmds(cmds);
+	vtc_error |= old_err;
+
+	if (vtc_error)
+		vtc_log(vltop, 1, "TEST %s FAILED", tfn);
+	else
+		vtc_log(vltop, 1, "TEST %s completed", tfn);
+
+	if (vtc_stop > 1)
+		return (1);
+	return (vtc_error);
+}
+
 int
 exec_file(const char *fn, const char *script, const char *tmpdir,
     char *logbuf, unsigned loglen)
 {
-	unsigned old_err;
 	FILE *f;
 
 	(void)signal(SIGPIPE, SIG_IGN);
 
+	tfn = fn;
 	vtc_loginit(logbuf, loglen);
 	vltop = vtc_logopen("top");
 	AN(vltop);
 
 	init_macro();
-	init_barrier();
 	init_server();
 
 	/* Move into our tmpdir */
@@ -691,16 +862,5 @@ exec_file(const char *fn, const char *script, const char *tmpdir,
 
 	vtc_thread = pthread_self();
 	parse_string(script, cmds, NULL, vltop);
-	old_err = vtc_error;
-	vtc_stop = 1;
-	vtc_log(vltop, 1, "RESETTING after %s", fn);
-	reset_cmds(cmds);
-	vtc_error |= old_err;
-
-	if (vtc_error)
-		vtc_log(vltop, 1, "TEST %s FAILED", fn);
-	else
-		vtc_log(vltop, 1, "TEST %s completed", fn);
-
-	return (vtc_error);
+	return(fail_out());
 }
