@@ -28,7 +28,6 @@
 
 #include "config.h"
 
-#include <sys/types.h>
 #include <sys/socket.h>
 
 #include <errno.h>
@@ -43,13 +42,13 @@
 #include "vtc_http.h"
 
 #include "vct.h"
+#include "vfil.h"
 #include "vgz.h"
 #include "vnum.h"
-#include "vre.h"
 #include "vtcp.h"
 #include "hpack.h"
 
-static const struct cmds http_cmds[];
+extern const struct cmds http_cmds[];
 
 /* SECTION: client-server client/server
  *
@@ -92,7 +91,7 @@ static const struct cmds http_cmds[];
  * \-run (client only)
  *        Equivalent to "-start -wait".
  *
- * \repeat NUMBER
+ * \-repeat NUMBER
  *        Instead of processing the specification only once, do it NUMBER times.
  *
  * \-break (server only)
@@ -139,22 +138,22 @@ static const struct cmds http_cmds[];
 #define ONLY_CLIENT(hp, av)						\
 	do {								\
 		if (hp->h2)						\
-			vtc_log(hp->vl, 0,				\
+			vtc_fatal(hp->vl,				\
 			    "\"%s\" only possible before H/2 upgrade",	\
 					av[0]);				\
 		if (hp->sfd != NULL)					\
-			vtc_log(hp->vl, 0,				\
+			vtc_fatal(hp->vl,				\
 			    "\"%s\" only possible in client", av[0]);	\
 	} while (0)
 
 #define ONLY_SERVER(hp, av)						\
 	do {								\
 		if (hp->h2)						\
-			vtc_log(hp->vl, 0,				\
+			vtc_fatal(hp->vl,				\
 			    "\"%s\" only possible before H/2 upgrade",	\
 					av[0]);				\
 		if (hp->sfd == NULL)					\
-			vtc_log(hp->vl, 0,				\
+			vtc_fatal(hp->vl,				\
 			    "\"%s\" only possible in server", av[0]);	\
 	} while (0)
 
@@ -344,13 +343,9 @@ static void
 cmd_http_expect(CMD_ARGS)
 {
 	struct http *hp;
-	const char *lhs, *clhs;
+	const char *lhs;
 	char *cmp;
-	const char *rhs, *crhs;
-	vre_t *vre;
-	const char *error;
-	int erroroffset;
-	int i, retval = -1;
+	const char *rhs;
 
 	(void)cmd;
 	(void)vl;
@@ -366,41 +361,7 @@ cmd_http_expect(CMD_ARGS)
 	cmp = av[1];
 	rhs = cmd_var_resolve(hp, av[2]);
 
-	clhs = lhs ? lhs : "<undef>";
-	crhs = rhs ? rhs : "<undef>";
-
-	if (!strcmp(cmp, "~") || !strcmp(cmp, "!~")) {
-		vre = VRE_compile(crhs, 0, &error, &erroroffset);
-		if (vre == NULL)
-			vtc_log(hp->vl, 0, "REGEXP error: %s (@%d) (%s)",
-			    error, erroroffset, crhs);
-		i = VRE_exec(vre, clhs, strlen(clhs), 0, 0, NULL, 0, 0);
-		retval = (i >= 0 && *cmp == '~') || (i < 0 && *cmp == '!');
-		VRE_free(&vre);
-	} else if (!strcmp(cmp, "==")) {
-		retval = strcmp(clhs, crhs) == 0;
-	} else if (!strcmp(cmp, "!=")) {
-		retval = strcmp(clhs, crhs) != 0;
-	} else if (lhs == NULL || rhs == NULL) {
-		// fail inequality comparisons if either side is undef'ed
-		retval = 0;
-	} else if (!strcmp(cmp, "<")) {
-		retval = isless(VNUM(lhs), VNUM(rhs));
-	} else if (!strcmp(cmp, ">")) {
-		retval = isgreater(VNUM(lhs), VNUM(rhs));
-	} else if (!strcmp(cmp, "<=")) {
-		retval = islessequal(VNUM(lhs), VNUM(rhs));
-	} else if (!strcmp(cmp, ">=")) {
-		retval = isgreaterequal(VNUM(lhs), VNUM(rhs));
-	}
-
-	if (retval == -1)
-		vtc_log(hp->vl, 0,
-		    "EXPECT %s (%s) %s %s (%s) test not implemented",
-		    av[0], clhs, av[1], av[2], crhs);
-	else
-		vtc_log(hp->vl, retval ? 4 : 0, "EXPECT %s (%s) %s \"%s\" %s",
-		    av[0], clhs, cmp, crhs, retval ? "match" : "failed");
+	vtc_expect(vl, av[0], lhs, cmp, av[2], rhs);
 }
 
 static void
@@ -418,7 +379,7 @@ cmd_http_expect_pattern(CMD_ARGS)
 	AZ(av[0]);
 	for (p = hp->body; *p != '\0'; p++) {
 		if (*p != t)
-			vtc_log(hp->vl, 0,
+			vtc_fatal(hp->vl,
 			    "EXPECT PATTERN FAIL @%zd should 0x%02x is 0x%02x",
 			    p - hp->body, t, *p);
 		t += 1;
@@ -500,7 +461,7 @@ http_splitheader(struct http *hp, int req)
 	assert(*p == '\0');
 
 	for (n = 0; n < 3 || hh[n] != NULL; n++) {
-		sprintf(buf, "http[%2d] ", n);
+		bprintf(buf, "http[%2d] ", n);
 		vtc_dump(hp->vl, 4, buf, hh[n], -1);
 	}
 }
@@ -523,14 +484,18 @@ http_rxchar(struct http *hp, int n, int eof)
 		i = poll(pfd, 1, hp->timeout);
 		if (i < 0 && errno == EINTR)
 			continue;
-		if (i == 0)
+		if (i == 0) {
 			vtc_log(hp->vl, hp->fatal,
 			    "HTTP rx timeout (fd:%d %u ms)",
 			    hp->fd, hp->timeout);
-		if (i < 0)
+			continue;
+		}
+		if (i < 0) {
 			vtc_log(hp->vl, hp->fatal,
 			    "HTTP rx failed (fd:%d poll: %s)",
 			    hp->fd, strerror(errno));
+			continue;
+		}
 		assert(i > 0);
 		assert(hp->prxbuf + n < hp->nrxbuf);
 		i = read(hp->fd, hp->rxbuf + hp->prxbuf, n);
@@ -540,14 +505,18 @@ http_rxchar(struct http *hp, int n, int eof)
 			    hp->fd, pfd[0].revents, n, i);
 		if (i == 0 && eof)
 			return (i);
-		if (i == 0)
+		if (i == 0) {
 			vtc_log(hp->vl, hp->fatal,
 			    "HTTP rx EOF (fd:%d read: %s) %d",
 			    hp->fd, strerror(errno), n);
-		if (i < 0)
+			return (-1);
+		}
+		if (i < 0) {
 			vtc_log(hp->vl, hp->fatal,
 			    "HTTP rx failed (fd:%d read: %s)",
 			    hp->fd, strerror(errno));
+			return (-1);
+		}
 		hp->prxbuf += i;
 		hp->rxbuf[hp->prxbuf] = '\0';
 		n -= i;
@@ -562,9 +531,10 @@ http_rxchunk(struct http *hp)
 	int l, i;
 
 	l = hp->prxbuf;
-	do
-		(void)http_rxchar(hp, 1, 0);
-	while (hp->rxbuf[hp->prxbuf - 1] != '\n');
+	do {
+		if (http_rxchar(hp, 1, 0) < 0)
+			return (-1);
+	} while (hp->rxbuf[hp->prxbuf - 1] != '\n');
 	vtc_dump(hp->vl, 4, "len", hp->rxbuf + l, -1);
 	i = strtoul(hp->rxbuf + l, &q, 16);
 	bprintf(hp->chunklen, "%d", i);
@@ -572,25 +542,31 @@ http_rxchunk(struct http *hp)
 		(*q != '\0' && !vct_islws(*q))) {
 		vtc_log(hp->vl, hp->fatal, "chunked fail %02x @ %td",
 		    *q, q - (hp->rxbuf + l));
+		return (-1);
 	}
 	assert(q != hp->rxbuf + l);
 	assert(*q == '\0' || vct_islws(*q));
 	hp->prxbuf = l;
 	if (i > 0) {
-		(void)http_rxchar(hp, i, 0);
-		vtc_dump(hp->vl, 4, "chunk",
-		    hp->rxbuf + l, i);
+		if (http_rxchar(hp, i, 0) < 0)
+			return (-1);
+		vtc_dump(hp->vl, 4, "chunk", hp->rxbuf + l, i);
 	}
 	l = hp->prxbuf;
-	(void)http_rxchar(hp, 2, 0);
-	if(!vct_iscrlf(hp->rxbuf + l))
+	if (http_rxchar(hp, 2, 0) < 0)
+		return (-1);
+	if (!vct_iscrlf(hp->rxbuf + l)) {
 		vtc_log(hp->vl, hp->fatal,
 		    "Wrong chunk tail[0] = %02x",
 		    hp->rxbuf[l] & 0xff);
-	if(!vct_iscrlf(hp->rxbuf + l + 1))
+		return (-1);
+	}
+	if (!vct_iscrlf(hp->rxbuf + l + 1)) {
 		vtc_log(hp->vl, hp->fatal,
 		    "Wrong chunk tail[1] = %02x",
 		    hp->rxbuf[l + 1] & 0xff);
+		return (-1);
+	}
 	hp->prxbuf = l;
 	hp->rxbuf[l] = '\0';
 	return (i);
@@ -609,18 +585,19 @@ http_swallow_body(struct http *hp, char * const *hh, int body)
 	ll = 0;
 	p = http_find_header(hh, "transfer-encoding");
 	if (p != NULL && !strcasecmp(p, "chunked")) {
-		while (http_rxchunk(hp) != 0)
+		while (http_rxchunk(hp) > 0)
 			continue;
 		vtc_dump(hp->vl, 4, "body", hp->body, ll);
 		ll = hp->rxbuf + hp->prxbuf - hp->body;
 		hp->bodyl = ll;
-		sprintf(hp->bodylen, "%d", ll);
+		bprintf(hp->bodylen, "%d", ll);
 		return;
 	}
 	p = http_find_header(hh, "content-length");
 	if (p != NULL) {
 		l = strtoul(p, NULL, 10);
-		(void)http_rxchar(hp, l, 0);
+		if (http_rxchar(hp, l, 0) < 0)
+			return;
 		vtc_dump(hp->vl, 4, "body", hp->body, l);
 		hp->bodyl = l;
 		bprintf(hp->bodylen, "%d", l);
@@ -629,12 +606,14 @@ http_swallow_body(struct http *hp, char * const *hh, int body)
 	if (body) {
 		do  {
 			i = http_rxchar(hp, 1, 1);
+			if (i < 0)
+				return;
 			ll += i;
 		} while (i > 0);
 		vtc_dump(hp->vl, 4, "rxeof", hp->body, ll);
 	}
 	hp->bodyl = ll;
-	sprintf(hp->bodylen, "%d", ll);
+	bprintf(hp->bodylen, "%d", ll);
 }
 
 /**********************************************************************
@@ -689,16 +668,16 @@ cmd_http_rxresp(CMD_ARGS)
 	AZ(strcmp(av[0], "rxresp"));
 	av++;
 
-	for(; *av != NULL; av++)
+	for (; *av != NULL; av++)
 		if (!strcmp(*av, "-no_obj"))
 			has_obj = 0;
 		else
-			vtc_log(hp->vl, 0,
+			vtc_fatal(hp->vl,
 			    "Unknown http rxresp spec: %s\n", *av);
 	http_rxhdr(hp);
 	http_splitheader(hp, 0);
 	if (http_count_header(hp->resp, "Content-Length") > 1)
-		vtc_log(hp->vl, 0,
+		vtc_fatal(hp->vl,
 		    "Multiple Content-Length headers.\n");
 	if (!has_obj)
 		return;
@@ -709,9 +688,9 @@ cmd_http_rxresp(CMD_ARGS)
 	vtc_log(hp->vl, 4, "bodylen = %s", hp->bodylen);
 }
 
-/* SECTION: client-server.spec.rxreqhdrs
+/* SECTION: client-server.spec.rxresphdrs
  *
- * rxresp (client only)
+ * rxresphdrs (client only)
  *         Receive and parse a response's headers.
  */
 static void
@@ -726,12 +705,12 @@ cmd_http_rxresphdrs(CMD_ARGS)
 	AZ(strcmp(av[0], "rxresphdrs"));
 	av++;
 
-	for(; *av != NULL; av++)
-		vtc_log(hp->vl, 0, "Unknown http rxresp spec: %s\n", *av);
+	for (; *av != NULL; av++)
+		vtc_fatal(hp->vl, "Unknown http rxresp spec: %s\n", *av);
 	http_rxhdr(hp);
 	http_splitheader(hp, 0);
 	if (http_count_header(hp->resp, "Content-Length") > 1)
-		vtc_log(hp->vl, 0,
+		vtc_fatal(hp->vl,
 		    "Multiple Content-Length headers.\n");
 }
 
@@ -743,7 +722,7 @@ cmd_http_rxresphdrs(CMD_ARGS)
 #define OVERHEAD 64L
 
 static void
-cmd_http_gunzip_body(CMD_ARGS)
+cmd_http_gunzip(CMD_ARGS)
 {
 	int i;
 	z_stream vz;
@@ -766,7 +745,7 @@ cmd_http_gunzip_body(CMD_ARGS)
 	vz.avail_in = hp->bodyl;
 
 	l = hp->bodyl * 10;
-	p = calloc(l, 1);
+	p = calloc(1, l);
 	AN(p);
 
 	vz.next_out = TRUST_ME(p);
@@ -811,7 +790,7 @@ gzip_body(const struct http *hp, const char *txt, char **body, int *bodylen)
 	memset(&vz, 0, sizeof vz);
 
 	l = strlen(txt);
-	*body = calloc(l + OVERHEAD, 1);
+	*body = calloc(1, l + OVERHEAD);
 	AN(*body);
 
 	vz.next_in = TRUST_ME(txt);
@@ -853,20 +832,28 @@ http_tx_parse_args(char * const *av, struct vtclog *vl, struct http *hp,
 	char *b, *c;
 	char *nullbody = NULL;
 	int nolen = 0;
+	int l;
 
 	(void)vl;
 	nullbody = body;
 
-	for(; *av != NULL; av++) {
+	for (; *av != NULL; av++) {
 		if (!strcmp(*av, "-nolen")) {
 			nolen = 1;
 		} else if (!strcmp(*av, "-hdr")) {
 			VSB_printf(hp->vsb, "%s%s", av[1], nl);
 			av++;
+		} else if (!strcmp(*av, "-hdrlen")) {
+			VSB_printf(hp->vsb, "%s: ", av[1]);
+			l = atoi(av[2]);
+			while (l-- > 0)
+				VSB_putc(hp->vsb, '0' + (l % 10));
+			VSB_printf(hp->vsb, "%s", nl);
+			av+=2;
 		} else
 			break;
 	}
-	for(; *av != NULL; av++) {
+	for (; *av != NULL; av++) {
 		if (!strcmp(*av, "-body")) {
 			assert(body == nullbody);
 			REPLACE(body, av[1]);
@@ -875,9 +862,9 @@ http_tx_parse_args(char * const *av, struct vtclog *vl, struct http *hp,
 			av++;
 			bodylen = strlen(body);
 			for (b = body; *b != '\0'; b++) {
-				if(*b == '\\' && b[1] == '0') {
+				if (*b == '\\' && b[1] == '0') {
 					*b = '\0';
-					for(c = b+1; *c != '\0'; c++) {
+					for (c = b+1; *c != '\0'; c++) {
 						*c = c[1];
 					}
 					b++;
@@ -921,7 +908,7 @@ http_tx_parse_args(char * const *av, struct vtclog *vl, struct http *hp,
 	return (av);
 }
 
-/* SECTION: client-server.spec.txre
+/* SECTION: client-server.spec.txreq
  *
  * txreq|txresp [...]
  *         Send a minimal request or response, but overload it if necessary.
@@ -945,7 +932,7 @@ http_tx_parse_args(char * const *av, struct vtclog *vl, struct http *hp,
  *         \-status NUMBER (txresp only)
  *                 What status code to return (default 200).
  *
- *         \-msg STRING (txresp only)
+ *         \-reason STRING (txresp only)
  *                 What message to put in the status line (default: "OK").
  *
  *         These three switches can appear in any order but must come before the
@@ -958,6 +945,9 @@ http_tx_parse_args(char * const *av, struct vtclog *vl, struct http *hp,
  *                 Add STRING as a header, it must follow this format:
  *                 "name: value". It can be called multiple times.
  *
+ *         \-hdrlen STRING NUMBER
+ *                 Add STRING as a header with NUMBER bytes of content.
+ *
  *         You can then use the arguments related to the body:
  *
  *         \-body STRING
@@ -991,7 +981,7 @@ cmd_http_txresp(CMD_ARGS)
 	struct http *hp;
 	const char *proto = "HTTP/1.1";
 	const char *status = "200";
-	const char *msg = "OK";
+	const char *reason = "OK";
 	char* body = NULL;
 
 	(void)cmd;
@@ -1003,29 +993,29 @@ cmd_http_txresp(CMD_ARGS)
 
 	VSB_clear(hp->vsb);
 
-	for(; *av != NULL; av++) {
+	for (; *av != NULL; av++) {
 		if (!strcmp(*av, "-proto")) {
 			proto = av[1];
 			av++;
 		} else if (!strcmp(*av, "-status")) {
 			status = av[1];
 			av++;
-		} else if (!strcmp(*av, "-msg")) {
-			msg = av[1];
+		} else if (!strcmp(*av, "-reason")) {
+			reason = av[1];
 			av++;
 			continue;
 		} else
 			break;
 	}
 
-	VSB_printf(hp->vsb, "%s %s %s%s", proto, status, msg, nl);
+	VSB_printf(hp->vsb, "%s %s %s%s", proto, status, reason, nl);
 
 	/* send a "Content-Length: 0" header unless something else happens */
 	REPLACE(body, "");
 
 	av = http_tx_parse_args(av, vl, hp, body);
 	if (*av != NULL)
-		vtc_log(hp->vl, 0, "Unknown http txresp spec: %s\n", *av);
+		vtc_fatal(hp->vl, "Unknown http txresp spec: %s\n", *av);
 
 	http_write(hp, 4, "txresp");
 }
@@ -1042,16 +1032,16 @@ cmd_http_upgrade(CMD_ARGS)
 
 	h = http_find_header(hp->req, "Upgrade");
 	if (!h || strcmp(h, "h2c"))
-		vtc_log(vl, 0, "Req misses \"Upgrade: h2c\" header");
+		vtc_fatal(vl, "Req misses \"Upgrade: h2c\" header");
 
 	h = http_find_header(hp->req, "Connection");
 	if (!h || strcmp(h, "Upgrade, HTTP2-Settings"))
-		vtc_log(vl, 0, "Req misses \"Connection: "
+		vtc_fatal(vl, "Req misses \"Connection: "
 			"Upgrade, HTTP2-Settings\" header");
 
 	h = http_find_header(hp->req, "HTTP2-Settings");
 	if (!h)
-		vtc_log(vl, 0, "Req misses \"HTTP2-Settings\" header");
+		vtc_fatal(vl, "Req misses \"HTTP2-Settings\" header");
 
 
 	parse_string("txresp -status 101 "
@@ -1090,13 +1080,12 @@ cmd_http_rxreq(CMD_ARGS)
 	AZ(strcmp(av[0], "rxreq"));
 	av++;
 
-	for(; *av != NULL; av++)
-		vtc_log(vl, 0, "Unknown http rxreq spec: %s\n", *av);
+	for (; *av != NULL; av++)
+		vtc_fatal(vl, "Unknown http rxreq spec: %s\n", *av);
 	http_rxhdr(hp);
 	http_splitheader(hp, 1);
 	if (http_count_header(hp->req, "Content-Length") > 1)
-		vtc_log(vl, 0,
-		    "Multiple Content-Length headers.\n");
+		vtc_fatal(vl, "Multiple Content-Length headers.\n");
 	http_swallow_body(hp, hp->req, 0);
 	vtc_log(vl, 4, "bodylen = %s", hp->bodylen);
 }
@@ -1118,13 +1107,12 @@ cmd_http_rxreqhdrs(CMD_ARGS)
 	AZ(strcmp(av[0], "rxreqhdrs"));
 	av++;
 
-	for(; *av != NULL; av++)
-		vtc_log(hp->vl, 0, "Unknown http rxreq spec: %s\n", *av);
+	for (; *av != NULL; av++)
+		vtc_fatal(hp->vl, "Unknown http rxreq spec: %s\n", *av);
 	http_rxhdr(hp);
 	http_splitheader(hp, 1);
 	if (http_count_header(hp->req, "Content-Length") > 1)
-		vtc_log(hp->vl, 0,
-		    "Multiple Content-Length headers.\n");
+		vtc_fatal(hp->vl, "Multiple Content-Length headers.\n");
 }
 
 /* SECTION: client-server.spec.rxreqbody
@@ -1145,15 +1133,15 @@ cmd_http_rxreqbody(CMD_ARGS)
 	AZ(strcmp(av[0], "rxreqbody"));
 	av++;
 
-	for(; *av != NULL; av++)
-		vtc_log(hp->vl, 0, "Unknown http rxreq spec: %s\n", *av);
+	for (; *av != NULL; av++)
+		vtc_fatal(hp->vl, "Unknown http rxreq spec: %s\n", *av);
 	http_swallow_body(hp, hp->req, 0);
 	vtc_log(hp->vl, 4, "bodylen = %s", hp->bodylen);
 }
 
 /* SECTION: client-server.spec.rxrespbody
  *
- * rxrespbody
+ * rxrespbody (client only)
  *         Receive a response's body.
  */
 
@@ -1169,8 +1157,8 @@ cmd_http_rxrespbody(CMD_ARGS)
 	AZ(strcmp(av[0], "rxrespbody"));
 	av++;
 
-	for(; *av != NULL; av++)
-		vtc_log(hp->vl, 0, "Unknown http rxrespbody spec: %s\n", *av);
+	for (; *av != NULL; av++)
+		vtc_fatal(hp->vl, "Unknown http rxrespbody spec: %s\n", *av);
 	http_swallow_body(hp, hp->resp, 0);
 	vtc_log(hp->vl, 4, "bodylen = %s", hp->bodylen);
 }
@@ -1178,7 +1166,7 @@ cmd_http_rxrespbody(CMD_ARGS)
 /* SECTION: client-server.spec.rxchunk
  *
  * rxchunk
- *         Receive an HTTP chunk
+ *         Receive an HTTP chunk.
  */
 
 static void
@@ -1196,7 +1184,7 @@ cmd_http_rxchunk(CMD_ARGS)
 	if (i == 0) {
 		ll = hp->rxbuf + hp->prxbuf - hp->body;
 		hp->bodyl = ll;
-		sprintf(hp->bodylen, "%d", ll);
+		bprintf(hp->bodylen, "%d", ll);
 		vtc_log(hp->vl, 4, "bodylen = %s", hp->bodylen);
 	}
 }
@@ -1223,7 +1211,7 @@ cmd_http_txreq(CMD_ARGS)
 
 	VSB_clear(hp->vsb);
 
-	for(; *av != NULL; av++) {
+	for (; *av != NULL; av++) {
 		if (!strcmp(*av, "-url")) {
 			url = av[1];
 			av++;
@@ -1248,7 +1236,7 @@ cmd_http_txreq(CMD_ARGS)
 
 	av = http_tx_parse_args(av, vl, hp, NULL);
 	if (*av != NULL)
-		vtc_log(hp->vl, 0, "Unknown http txreq spec: %s\n", *av);
+		vtc_fatal(hp->vl, "Unknown http txreq spec: %s\n", *av);
 	http_write(hp, 4, "txreq");
 
 	if (up) {
@@ -1387,38 +1375,21 @@ cmd_http_send_urgent(CMD_ARGS)
 static void
 cmd_http_sendhex(CMD_ARGS)
 {
+	struct vsb *vsb;
 	struct http *hp;
-	char buf[3], *q;
-	uint8_t *p;
-	int i, j, l;
+	int j;
 
 	(void)cmd;
 	(void)vl;
 	CAST_OBJ_NOTNULL(hp, priv, HTTP_MAGIC);
 	AN(av[1]);
 	AZ(av[2]);
-	l = strlen(av[1]) / 2;
-	p = malloc(l);
-	AN(p);
-	q = av[1];
-	for (i = 0; i < l; i++) {
-		while (vct_issp(*q))
-			q++;
-		if (*q == '\0')
-			break;
-		memcpy(buf, q, 2);
-		q += 2;
-		buf[2] = '\0';
-		if (!vct_ishex(buf[0]) || !vct_ishex(buf[1]))
-			vtc_log(hp->vl, 0, "Illegal Hex char \"%c%c\"",
-			    buf[0], buf[1]);
-		p[i] = (uint8_t)strtoul(buf, NULL, 16);
-	}
-	vtc_hexdump(hp->vl, 4, "sendhex", (void*)p, i);
-	j = write(hp->fd, p, i);
-	assert(j == i);
-	free(p);
-
+	vsb = vtc_hex_to_bin(hp->vl, av[1]);
+	assert(VSB_len(vsb) >= 0);
+	vtc_hexdump(hp->vl, 4, "sendhex", VSB_data(vsb), VSB_len(vsb));
+	j = write(hp->fd, VSB_data(vsb), VSB_len(vsb));
+	assert(j == VSB_len(vsb));
+	VSB_destroy(&vsb);
 }
 
 /* SECTION: client-server.spec.chunked
@@ -1505,20 +1476,14 @@ cmd_http_timeout(CMD_ARGS)
 	AZ(av[2]);
 	d = VNUM(av[1]);
 	if (isnan(d))
-		vtc_log(vl, 0, "timeout is not a number (%s)", av[1]);
+		vtc_fatal(vl, "timeout is not a number (%s)", av[1]);
 	hp->timeout = (int)(d * 1000.0);
 }
 
 /* SECTION: client-server.spec.expect_close
  *
  * expect_close
- *         Wait for the connected client to close the connection.
- */
-
-/* SECTION: client-server.spec.expect_close expect_close (server)
- *
- * Reads from the connection, expecting nothing to read but an EOF.
- *
+ *	Reads from the connection, expecting nothing to read but an EOF.
  */
 static void
 cmd_http_expect_close(CMD_ARGS)
@@ -1563,14 +1528,8 @@ cmd_http_expect_close(CMD_ARGS)
 /* SECTION: client-server.spec.close
  *
  * close (server only)
- *         Close the active TCP connection
- */
-
-/* SECTION: client-server.spec.close close (server)
- *
- * Close the connection. Not that if operating in H/2 mode, no extra (GOAWAY)
- * frame is sent, it's simply a TCP close.
- *
+ *	Close the connection. Note that if operating in HTTP/2 mode no
+ *	extra (GOAWAY) frame is sent, it's simply a TCP close.
  */
 static void
 cmd_http_close(CMD_ARGS)
@@ -1580,6 +1539,7 @@ cmd_http_close(CMD_ARGS)
 	(void)cmd;
 	(void)vl;
 	CAST_OBJ_NOTNULL(hp, priv, HTTP_MAGIC);
+	ONLY_SERVER(hp, av);
 	AZ(av[1]);
 	assert(hp->sfd != NULL);
 	assert(*hp->sfd >= 0);
@@ -1589,16 +1549,11 @@ cmd_http_close(CMD_ARGS)
 	vtc_log(vl, 4, "Closed");
 }
 
-/* SECTION: client-server.spec.accept_close
+/* SECTION: client-server.spec.accept
  *
  * accept (server only)
- *         Close the active connection (if any) and accept a new one.
- */
-
-/* SECTION: client-server.spec.accept accept (server)
- *
- * Close the potential current connection, and accept a new one. Note that this
- * new connection is H/1.
+ *	Close the current connection, if any, and accept a new one. Note
+ *	that this new connection is HTTP/1.x.
  */
 static void
 cmd_http_accept(CMD_ARGS)
@@ -1608,6 +1563,7 @@ cmd_http_accept(CMD_ARGS)
 	(void)cmd;
 	(void)vl;
 	CAST_OBJ_NOTNULL(hp, priv, HTTP_MAGIC);
+	ONLY_SERVER(hp, av);
 	AZ(av[1]);
 	assert(hp->sfd != NULL);
 	assert(*hp->sfd >= 0);
@@ -1647,7 +1603,7 @@ cmd_http_loop(CMD_ARGS)
 
 /* SECTION: client-server.spec.fatal
  *
- * fatal|non-fatal
+ * fatal|non_fatal
  *         Control whether a failure of this entity should stop the test.
  */
 
@@ -1660,13 +1616,14 @@ cmd_http_fatal(CMD_ARGS)
 	AZ(av[1]);
 	if (!strcmp(av[0], "fatal"))
 		hp->fatal = 0;
-	else if (!strcmp(av[0], "non-fatal"))
+	else if (!strcmp(av[0], "non_fatal"))
 		hp->fatal = -1;
-	else {
-		vtc_log(vl, 0, "XXX: fatal %s", cmd->name);
-	}
+	else
+		vtc_fatal(vl, "XXX: fatal %s", cmd->name);
 }
 
+#define cmd_http_non_fatal cmd_http_fatal
+
 /* SECTION: client-server.spec.delay
  *
  * delay
@@ -1678,12 +1635,17 @@ cmd_http_fatal(CMD_ARGS)
  *	Same as for the top-level barrier
  */
 
-const char PREFACE[] = "PRI * HTTP/2.0\r\n\r\nSM\r\n\r\n";
+static const char PREFACE[24] = {
+	0x50, 0x52, 0x49, 0x20, 0x2a, 0x20, 0x48, 0x54,
+	0x54, 0x50, 0x2f, 0x32, 0x2e, 0x30, 0x0d, 0x0a,
+	0x0d, 0x0a, 0x53, 0x4d, 0x0d, 0x0a, 0x0d, 0x0a
+};
 
-/* SECTION: client-server.spec.txpri txpri (client)
+/* SECTION: client-server.spec.txpri
  *
- * Send an H/2 preface ("PRI * HTTP/2.0\\r\\n\\r\\nSM\\r\\n\\r\\n") and set
- * client to H/2.
+ * txpri (client only)
+ *	Send an HTTP/2 preface ("PRI * HTTP/2.0\\r\\n\\r\\nSM\\r\\n\\r\\n")
+ *	and set client to HTTP/2.
  */
 static void
 cmd_http_txpri(CMD_ARGS)
@@ -1694,19 +1656,27 @@ cmd_http_txpri(CMD_ARGS)
 	CAST_OBJ_NOTNULL(hp, priv, HTTP_MAGIC);
 	ONLY_CLIENT(hp, av);
 
-	vtc_dump(hp->vl, 4, "txpri", PREFACE, sizeof(PREFACE) - 1);
-	l = write(hp->fd, PREFACE, sizeof(PREFACE) - 1);
-	if (l != sizeof(PREFACE) - 1)
+	vtc_dump(hp->vl, 4, "txpri", PREFACE, sizeof(PREFACE));
+	/* Dribble out the preface */
+	l = write(hp->fd, PREFACE, 18);
+	if (l != 18)
 		vtc_log(vl, hp->fatal, "Write failed: (%zd vs %zd) %s",
-		    l, sizeof(PREFACE) - 1, strerror(errno));
+		    l, sizeof(PREFACE), strerror(errno));
+	usleep(10000);
+	l = write(hp->fd, PREFACE + 18, sizeof(PREFACE) - 18);
+	if (l != sizeof(PREFACE) - 18)
+		vtc_log(vl, hp->fatal, "Write failed: (%zd vs %zd) %s",
+		    l, sizeof(PREFACE), strerror(errno));
+
 	start_h2(hp);
 	AN(hp->h2);
 }
 
-/* SECTION: client-server.spec.rxpri rxpri (server)
+/* SECTION: client-server.spec.rxpri
  *
- * Receive a preface, and if it matches, sets the server to H/2, aborts
- * otherwise.
+ * rxpri (server only)
+ *	Receive a preface. If valid set the server to HTTP/2, abort
+ *	otherwise.
  */
 static void
 cmd_http_rxpri(CMD_ARGS)
@@ -1717,10 +1687,10 @@ cmd_http_rxpri(CMD_ARGS)
 	ONLY_SERVER(hp, av);
 
 	hp->prxbuf = 0;
-	if (!http_rxchar(hp, sizeof(PREFACE) - 1, 0))
-		vtc_log(vl, 0, "Couldn't retrieve connection preface");
-	if (strncmp(hp->rxbuf, PREFACE, sizeof(PREFACE) - 1))
-		vtc_log(vl, 0, "Received invalid preface\n");
+	if (!http_rxchar(hp, sizeof(PREFACE), 0))
+		vtc_fatal(vl, "Couldn't retrieve connection preface");
+	if (memcmp(hp->rxbuf, PREFACE, sizeof(PREFACE)))
+		vtc_fatal(vl, "Received invalid preface\n");
 	start_h2(hp);
 	AN(hp->h2);
 }
@@ -1728,8 +1698,8 @@ cmd_http_rxpri(CMD_ARGS)
 /* SECTION: client-server.spec.settings
  *
  * settings -dectbl INT
- *         Force internal H/2 settings to certain values. Currently only
- *         support setting the decoding table size.
+ *	Force internal HTTP/2 settings to certain values. Currently only
+ *	support setting the decoding table size.
  */
 static void
 cmd_http_settings(CMD_ARGS)
@@ -1741,20 +1711,20 @@ cmd_http_settings(CMD_ARGS)
 	(void)cmd;
 
 	if (!hp->h2)
-		vtc_log(hp->vl, 0, "Only possible in H/2 mode");
+		vtc_fatal(hp->vl, "Only possible in H/2 mode");
 
 	CAST_OBJ_NOTNULL(hp, priv, HTTP_MAGIC);
 
-	for(; *av != NULL; av++) {
+	for (; *av != NULL; av++) {
 		if (!strcmp(*av, "-dectbl")) {
 			n = strtoul(av[1], &p, 0);
 			if (*p != '\0')
-				vtc_log(hp->vl, 0, "-dectbl takes an integer as"
-						" argument (found %s)", av[1]);
+				vtc_fatal(hp->vl, "-dectbl takes an integer as "
+				    "argument (found %s)", av[1]);
 			HPK_ResizeTbl(hp->decctx, n);
 			av++;
 		} else
-			vtc_log(vl, 0, "Unknown settings spec: %s\n", *av);
+			vtc_fatal(vl, "Unknown settings spec: %s\n", *av);
 	}
 }
 
@@ -1780,48 +1750,93 @@ cmd_http_stream(CMD_ARGS)
 	cmd_stream(av, hp, cmd, vl);
 }
 
+/* SECTION: client-server.spec.write_body
+ *
+ * write_body STRING
+ *	Write the body of a request or a response to a file. By using the
+ *	shell command, higher-level checks on the body can be performed
+ *	(eg. XML, JSON, ...) provided that such checks can be delegated
+ *	to an external program.
+ */
+static void
+cmd_http_write_body(CMD_ARGS)
+{
+	struct http *hp;
+
+	(void)cmd;
+	(void)vl;
+	CAST_OBJ_NOTNULL(hp, priv, HTTP_MAGIC);
+	AN(av[0]);
+	AN(av[1]);
+	AZ(av[2]);
+	AZ(strcmp(av[0], "write_body"));
+	if (VFIL_writefile(NULL, av[1], hp->body, hp->bodyl) != 0)
+		vtc_fatal(hp->vl, "failed to write body: %s (%d)",
+		    strerror(errno), errno);
+}
+
 /**********************************************************************
  * Execute HTTP specifications
  */
 
-static const struct cmds http_cmds[] = {
-	{ "timeout",		cmd_http_timeout },
-	{ "txreq",		cmd_http_txreq },
+const struct cmds http_cmds[] = {
+#define CMD(n) { #n, cmd_##n },
+#define CMD_HTTP(n) { #n, cmd_http_##n },
+	/* session */
+	CMD_HTTP(accept)
+	CMD_HTTP(close)
+	CMD_HTTP(recv)
+	CMD_HTTP(send)
+	CMD_HTTP(send_n)
+	CMD_HTTP(send_urgent)
+	CMD_HTTP(sendhex)
+	CMD_HTTP(timeout)
 
-	{ "rxreq",		cmd_http_rxreq },
-	{ "rxreqhdrs",		cmd_http_rxreqhdrs },
-	{ "rxreqbody",		cmd_http_rxreqbody },
-	{ "rxchunk",		cmd_http_rxchunk },
+	/* spec */
+	CMD_HTTP(fatal)
+	CMD_HTTP(loop)
+	CMD_HTTP(non_fatal)
 
-	{ "txresp",		cmd_http_txresp },
-	{ "rxresp",		cmd_http_rxresp },
-	{ "rxresphdrs",		cmd_http_rxresphdrs },
-	{ "rxrespbody",		cmd_http_rxrespbody },
-	{ "gunzip",		cmd_http_gunzip_body },
-	{ "expect",		cmd_http_expect },
-	{ "expect_pattern",	cmd_http_expect_pattern },
-	{ "recv",		cmd_http_recv },
-	{ "send",		cmd_http_send },
-	{ "send_n",		cmd_http_send_n },
-	{ "send_urgent",	cmd_http_send_urgent },
-	{ "sendhex",		cmd_http_sendhex },
-	{ "chunked",		cmd_http_chunked },
-	{ "chunkedlen",		cmd_http_chunkedlen },
-	{ "delay",		cmd_delay },
-	{ "barrier",		cmd_barrier },
-	{ "expect_close",	cmd_http_expect_close },
-	{ "close",		cmd_http_close },
-	{ "accept",		cmd_http_accept },
-	{ "loop",		cmd_http_loop },
-	{ "fatal",		cmd_http_fatal },
-	{ "non-fatal",		cmd_http_fatal },
+	/* body */
+	CMD_HTTP(gunzip)
+	CMD_HTTP(write_body)
 
-	{ "rxpri",		cmd_http_rxpri },
-	{ "txpri",		cmd_http_txpri },
-	{ "stream",		cmd_http_stream },
-	{ "settings",		cmd_http_settings },
-	{ "upgrade",		cmd_http_upgrade },
-	{ NULL,			NULL }
+	/* HTTP/1.x */
+	CMD_HTTP(chunked)
+	CMD_HTTP(chunkedlen)
+	CMD_HTTP(rxchunk)
+
+	/* HTTP/2 */
+	CMD_HTTP(stream)
+	CMD_HTTP(settings)
+
+	/* client */
+	CMD_HTTP(rxresp)
+	CMD_HTTP(rxrespbody)
+	CMD_HTTP(rxresphdrs)
+	CMD_HTTP(txpri)
+	CMD_HTTP(txreq)
+
+	/* server */
+	CMD_HTTP(rxpri)
+	CMD_HTTP(rxreq)
+	CMD_HTTP(rxreqbody)
+	CMD_HTTP(rxreqhdrs)
+	CMD_HTTP(txresp)
+	CMD_HTTP(upgrade)
+
+	/* expect */
+	CMD_HTTP(expect)
+	CMD_HTTP(expect_close)
+	CMD_HTTP(expect_pattern)
+
+	/* general purpose */
+	CMD(barrier)
+	CMD(delay)
+	CMD(shell)
+#undef CMD_HTTP
+#undef CMD
+	{ NULL, NULL }
 };
 
 int
@@ -1906,10 +1921,10 @@ xxx(void)
 
 	memset(&vz, 0, sizeof vz);
 
-	for(n = 0;  n < 999999999; n++) {
+	for (n = 0;  n < 999999999; n++) {
 		*ibuf = 0;
 		for (j = 0; j < 7; j++) {
-			sprintf(strchr(ibuf, 0), "%x",
+			snprintf(strchr(ibuf, 0), 5, "%x",
 			    (unsigned)random() & 0xffff);
 			vz.next_in = TRUST_ME(ibuf);
 			vz.avail_in = strlen(ibuf);
