@@ -101,8 +101,10 @@
  *         Tag to match against
  *
  * regex:
- *         regular expression to match against (optional) ('*' is anything, '='
- *         is the value of the last matched record)
+ *         regular expression to match against (optional)
+ *
+ * For skip, vxid and tag, '*' matches anything, '=' expects the value of the
+ * previous matched record.
  */
 
 #include "config.h"
@@ -140,6 +142,7 @@ struct logexp {
 	VTAILQ_ENTRY(logexp)		list;
 
 	char				*name;
+	char				*vname;
 	struct vtclog			*vl;
 	char				run;
 	VTAILQ_HEAD(,logexp_test)	tests;
@@ -153,8 +156,7 @@ struct logexp {
 	int				g_arg;
 	char				*query;
 
-	struct VSM_data			*vsm;
-	struct vsb			*n_arg;
+	struct vsm			*vsm;
 	struct VSL_data			*vsl;
 	struct VSLQ			*vslq;
 	pthread_t			tp;
@@ -190,18 +192,16 @@ logexp_delete(struct logexp *le)
 	logexp_delete_tests(le);
 	free(le->name);
 	free(le->query);
-	VSM_Delete(le->vsm);
-	if (le->n_arg)
-		VSB_destroy(&le->n_arg);
+	VSM_Destroy(&le->vsm);
 	FREE_OBJ(le);
 }
 
 static struct logexp *
-logexp_new(const char *name)
+logexp_new(const char *name, const char *varg)
 {
 	struct logexp *le;
+	struct vsb *vsb, *n_arg;
 
-	AN(name);
 	ALLOC_OBJ(le, LOGEXP_MAGIC);
 	AN(le);
 	REPLACE(le->name, name);
@@ -216,6 +216,23 @@ logexp_new(const char *name)
 	AN(le->vsl);
 
 	VTAILQ_INSERT_TAIL(&logexps, le, list);
+
+	REPLACE(le->vname, varg);
+
+	vsb = VSB_new_auto();
+	AN(vsb);
+	AZ(VSB_printf(vsb, "${tmpdir}/%s", varg));
+	AZ(VSB_finish(vsb));
+	n_arg = macro_expand(le->vl, VSB_data(vsb));
+	VSB_destroy(&vsb);
+	if (n_arg == NULL)
+		vtc_fatal(le->vl, "-v argument problems");
+	if (VSM_Arg(le->vsm, 'n', VSB_data(n_arg)) <= 0)
+		vtc_fatal(le->vl, "-v argument error: %s",
+		    VSM_Error(le->vsm));
+	VSB_destroy(&n_arg);
+	if (VSM_Attach(le->vsm, -1))
+		vtc_fatal(le->vl, "VSM_Attach: %s", VSM_Error(le->vsm));
 	return (le);
 }
 
@@ -232,7 +249,7 @@ logexp_next(struct logexp *le)
 
 	CHECK_OBJ_ORNULL(le->test, LOGEXP_TEST_MAGIC);
 	if (le->test)
-		vtc_log(le->vl, 3, "waitfor| %s", VSB_data(le->test->str));
+		vtc_log(le->vl, 3, "expecting| %s", VSB_data(le->test->str));
 }
 
 static int __match_proto__(VSLQ_dispatch_f)
@@ -336,16 +353,16 @@ logexp_thread(void *priv)
 	AN(le->vslq);
 
 	AZ(le->test);
-	vtc_log(le->vl, 4, "beg|");
+	vtc_log(le->vl, 4, "begin|");
 	if (le->query != NULL)
 		vtc_log(le->vl, 4, "qry| %s", le->query);
 	logexp_next(le);
 	while (le->test) {
 		i = VSLQ_Dispatch(le->vslq, logexp_dispatch, le);
 		if (i == 2)
-			vtc_log(le->vl, 0, "bad| expectation failed");
+			vtc_fatal(le->vl, "bad| expectation failed");
 		else if (i < 0)
-			vtc_log(le->vl, 0, "bad| dispatch failed (%d)", i);
+			vtc_fatal(le->vl, "bad| dispatch failed (%d)", i);
 		else if (i == 0 && le->test)
 			VTIM_sleep(0.01);
 	}
@@ -363,7 +380,6 @@ logexp_close(struct logexp *le)
 	if (le->vslq)
 		VSLQ_Delete(&le->vslq);
 	AZ(le->vslq);
-	VSM_Close(le->vsm);
 }
 
 static void
@@ -375,34 +391,16 @@ logexp_start(struct logexp *le)
 	AN(le->vsl);
 	AZ(le->vslq);
 
-	if (le->n_arg == NULL) {
-		vtc_log(le->vl, 0, "-v argument not given");
-		return;
-	}
-	if (VSM_n_Arg(le->vsm, VSB_data(le->n_arg)) <= 0) {
-		vtc_log(le->vl, 0, "-v argument error: %s",
-		    VSM_Error(le->vsm));
-		return;
-	}
-	if (VSM_Open(le->vsm)) {
-		vtc_log(le->vl, 0, "VSM_Open: %s", VSM_Error(le->vsm));
-		return;
-	}
 	AN(le->vsl);
+	(void)VSM_Status(le->vsm);
 	c = VSL_CursorVSM(le->vsl, le->vsm,
 	    (le->d_arg ? 0 : VSL_COPT_TAIL) | VSL_COPT_BATCH);
-	if (c == NULL) {
-		vtc_log(le->vl, 0, "VSL_CursorVSM: %s", VSL_Error(le->vsl));
-		logexp_close(le);
-		return;
-	}
+	if (c == NULL)
+		vtc_fatal(le->vl, "VSL_CursorVSM: %s", VSL_Error(le->vsl));
 	le->vslq = VSLQ_New(le->vsl, &c, le->g_arg, le->query);
 	if (le->vslq == NULL) {
 		VSL_DeleteCursor(c);
-		vtc_log(le->vl, 0, "VSLQ_New: %s", VSL_Error(le->vsl));
-		AZ(le->vslq);
-		logexp_close(le);
-		return;
+		vtc_fatal(le->vl, "VSLQ_New: %s", VSL_Error(le->vsl));
 	}
 	AZ(c);
 
@@ -423,7 +421,7 @@ logexp_wait(struct logexp *le)
 	AZ(pthread_join(le->tp, &res));
 	logexp_close(le);
 	if (res != NULL && !vtc_stop)
-		vtc_log(le->vl, 0, "logexp returned \"%p\"", (char *)res);
+		vtc_fatal(le->vl, "logexp returned \"%p\"", (char *)res);
 	le->run = 0;
 }
 
@@ -442,19 +440,18 @@ cmd_logexp_expect(CMD_ARGS)
 
 	(void)cmd;
 	CAST_OBJ_NOTNULL(le, priv, LOGEXP_MAGIC);
-	if (av[1] == NULL || av[2] == NULL || av[3] == NULL) {
-		vtc_log(vl, 0, "Syntax error");
-		return;
-	}
+	if (av[1] == NULL || av[2] == NULL || av[3] == NULL)
+		vtc_fatal(vl, "Syntax error");
+
+	if (av[4] != NULL && av[5] != NULL)
+		vtc_fatal(vl, "Syntax error");
 
 	if (!strcmp(av[1], "*"))
 		skip_max = LE_ANY;
 	else {
 		skip_max = (int)strtol(av[1], &end, 10);
-		if (*end != '\0' || skip_max < 0) {
-			vtc_log(vl, 0, "Not a positive integer: '%s'", av[1]);
-			return;
-		}
+		if (*end != '\0' || skip_max < 0)
+			vtc_fatal(vl, "Not a positive integer: '%s'", av[1]);
 	}
 	if (!strcmp(av[2], "*"))
 		vxid = LE_ANY;
@@ -462,10 +459,8 @@ cmd_logexp_expect(CMD_ARGS)
 		vxid = LE_LAST;
 	else {
 		vxid = (int)strtol(av[2], &end, 10);
-		if (*end != '\0' || vxid < 0) {
-			vtc_log(vl, 0, "Not a positive integer: '%s'", av[2]);
-			return;
-		}
+		if (*end != '\0' || vxid < 0)
+			vtc_fatal(vl, "Not a positive integer: '%s'", av[2]);
 	}
 	if (!strcmp(av[3], "*"))
 		tag = LE_ANY;
@@ -473,19 +468,15 @@ cmd_logexp_expect(CMD_ARGS)
 		tag = LE_LAST;
 	else {
 		tag = VSL_Name2Tag(av[3], strlen(av[3]));
-		if (tag < 0) {
-			vtc_log(vl, 0, "Unknown tag name: '%s'", av[3]);
-			return;
-		}
+		if (tag < 0)
+			vtc_fatal(vl, "Unknown tag name: '%s'", av[3]);
 	}
 	vre = NULL;
 	if (av[4]) {
 		vre = VRE_compile(av[4], 0, &err, &pos);
-		if (vre == NULL) {
-			vtc_log(vl, 0, "Regex error (%s): '%s' pos %d",
+		if (vre == NULL)
+			vtc_fatal(vl, "Regex error (%s): '%s' pos %d",
 			    err, av[4], pos);
-			return;
-		}
 	}
 
 	ALLOC_OBJ(test, LOGEXP_TEST_MAGIC);
@@ -522,12 +513,9 @@ void
 cmd_logexpect(CMD_ARGS)
 {
 	struct logexp *le, *le2;
-	const char tmpdir[] = "${tmpdir}";
-	struct vsb *vsb;
 
 	(void)priv;
 	(void)cmd;
-	(void)vl;
 
 	if (av == NULL) {
 		/* Reset and free */
@@ -546,23 +534,26 @@ cmd_logexpect(CMD_ARGS)
 	AZ(strcmp(av[0], "logexpect"));
 	av++;
 
+	VTC_CHECK_NAME(vl, av[0], "Logexpect", 'l');
 	VTAILQ_FOREACH(le, &logexps, list) {
 		if (!strcmp(le->name, av[0]))
 			break;
 	}
-	if (le == NULL)
-		le = logexp_new(av[0]);
+	if (le == NULL) {
+		if (strcmp(av[1], "-v") || av[2] == NULL)
+			vtc_fatal(vl, "new logexp lacks -v");
+		le = logexp_new(av[0], av[2]);
+		av += 2;
+	}
 	av++;
 
 	for (; *av != NULL; av++) {
 		if (vtc_error)
 			break;
 		if (!strcmp(*av, "-wait")) {
-			if (!le->run) {
-				vtc_log(le->vl, 0, "logexp not -started '%s'",
+			if (!le->run)
+				vtc_fatal(le->vl, "logexp not -started '%s'",
 					*av);
-				return;
-			}
 			logexp_wait(le);
 			continue;
 		}
@@ -576,51 +567,31 @@ cmd_logexpect(CMD_ARGS)
 		AZ(le->run);
 
 		if (!strcmp(*av, "-v")) {
-			if (av[1] == NULL) {
-				vtc_log(le->vl, 0, "Missing -v argument");
-				return;
-			}
-			if (le->n_arg != NULL)
-				VSB_destroy(&le->n_arg);
-			vsb = VSB_new_auto();
-			AN(vsb);
-			AZ(VSB_printf(vsb, "%s/%s", tmpdir, av[1]));
-			AZ(VSB_finish(vsb));
-			le->n_arg = macro_expand(le->vl, VSB_data(vsb));
-			VSB_destroy(&vsb);
-			if (le->n_arg == NULL)
-				return;
+			if (av[1] == NULL || strcmp(av[1], le->vname))
+				vtc_fatal(le->vl, "-v argument cannot change");
 			av++;
 			continue;
 		}
 		if (!strcmp(*av, "-d")) {
-			if (av[1] == NULL) {
-				vtc_log(le->vl, 0, "Missing -d argument");
-				return;
-			}
+			if (av[1] == NULL)
+				vtc_fatal(le->vl, "Missing -d argument");
 			le->d_arg = atoi(av[1]);
 			av++;
 			continue;
 		}
 		if (!strcmp(*av, "-g")) {
-			if (av[1] == NULL) {
-				vtc_log(le->vl, 0, "Missing -g argument");
-				return;
-			}
+			if (av[1] == NULL)
+				vtc_fatal(le->vl, "Missing -g argument");
 			le->g_arg = VSLQ_Name2Grouping(av[1], strlen(av[1]));
-			if (le->g_arg < 0) {
-				vtc_log(le->vl, 0, "Unknown grouping '%s'",
+			if (le->g_arg < 0)
+				vtc_fatal(le->vl, "Unknown grouping '%s'",
 				    av[1]);
-				return;
-			}
 			av++;
 			continue;
 		}
 		if (!strcmp(*av, "-q")) {
-			if (av[1] == NULL) {
-				vtc_log(le->vl, 0, "Missing -q argument");
-				return;
-			}
+			if (av[1] == NULL)
+				vtc_fatal(le->vl, "Missing -q argument");
 			REPLACE(le->query, av[1]);
 			av++;
 			continue;
@@ -640,11 +611,9 @@ cmd_logexpect(CMD_ARGS)
 					av++;
 					continue;
 				}
-				vtc_log(le->vl, 0, "%s", VSL_Error(le->vsl));
-				return;
+				vtc_fatal(le->vl, "%s", VSL_Error(le->vsl));
 			}
-			vtc_log(le->vl, 0, "Unknown logexp argument: %s", *av);
-			return;
+			vtc_fatal(le->vl, "Unknown logexp argument: %s", *av);
 		}
 		logexp_spec(le, *av);
 	}
