@@ -30,13 +30,10 @@
 
 #include <sys/mman.h>
 #include <sys/stat.h>
-#include <sys/socket.h>
-#include <sys/types.h>
 #include <sys/wait.h>
 
 #include <ctype.h>
 #include <errno.h>
-#include <fcntl.h>
 #include <poll.h>
 #include <stdio.h>
 #include <stdlib.h>
@@ -45,21 +42,17 @@
 
 #include "vtc.h"
 
+#include "vdef.h"
 #include "vev.h"
 #include "vfil.h"
 #include "vnum.h"
-#include "vqueue.h"
 #include "vrnd.h"
-#include "vsa.h"
 #include "vss.h"
 #include "vsub.h"
 #include "vtcp.h"
 #include "vtim.h"
 #include "vct.h"
 
-#define		MAX_FILESIZE		(1024 * 1024)
-
-
 struct vtc_tst {
 	unsigned		magic;
 #define TST_MAGIC		0x618d8b88
@@ -80,11 +73,13 @@ struct vtc_job {
 	char			*tmpdir;
 	unsigned		bufsiz;
 	double			t0;
+	struct vsb		*diag;
+	int			killed;
 };
 
 int iflg = 0;
 unsigned vtc_maxdur = 60;
-unsigned vtc_bufsiz = 512 * 1024;
+static unsigned vtc_bufsiz = 1024 * 1024;
 
 static VTAILQ_HEAD(, vtc_tst) tst_head = VTAILQ_HEAD_INITIALIZER(tst_head);
 static struct vev_base *vb;
@@ -94,6 +89,7 @@ static int vtc_continue;		/* Continue on error */
 static int vtc_verbosity = 1;		/* Verbosity Level */
 static int vtc_good;
 static int vtc_fail;
+static int vtc_skip;
 static char *tmppath;
 static char *cwd = NULL;
 char *vmod_path = NULL;
@@ -132,7 +128,7 @@ usage(void)
 	fprintf(stderr, "usage: varnishtest [options] file ...\n");
 #define FMT "    %-28s # %s\n"
 	fprintf(stderr, FMT, "-b size",
-	    "Set internal buffer size (default: 512K)");
+	    "Set internal buffer size (default: 1M)");
 	fprintf(stderr, FMT, "-D name=val", "Define macro");
 	fprintf(stderr, FMT, "-i", "Find varnish binaries in build tree");
 	fprintf(stderr, FMT, "-j jobs", "Run this many tests in parallel");
@@ -145,7 +141,6 @@ usage(void)
 	fprintf(stderr, FMT, "-t duration", "Time tests out after this long");
 	fprintf(stderr, FMT, "-v", "Verbose mode: always report test log");
 	fprintf(stderr, FMT, "-W", "Enable the witness facility for locking");
-	fprintf(stderr, "\n");
 	exit(1);
 }
 
@@ -158,25 +153,28 @@ tst_cb(const struct vev *ve, int what)
 {
 	struct vtc_job *jp;
 	char buf[BUFSIZ];
+	int ecode;
 	int i, stx;
 	pid_t px;
 	double t;
 	FILE *f;
+	char *p;
+	struct vsb *v;
 
 	CAST_OBJ_NOTNULL(jp, ve->priv, JOB_MAGIC);
 
-	// printf("%p %s %d\n", ve, jp->tst->filename, what);
-	if (what == 0)
+	// printf("CB %p %s %d\n", ve, jp->tst->filename, what);
+	if (what == 0) {
+		jp->killed = 1;
 		AZ(kill(jp->child, SIGKILL)); /* XXX: Timeout */
-	else
+	} else {
 		assert(what & (EV_RD | EV_HUP));
+	}
 
 	*buf = '\0';
-	i = read(ve->fd, buf, sizeof buf - 1);
-	if (i > 0) {
-		buf[i] = '\0';
-		printf("######## %s ########\n%s", jp->tst->filename, buf);
-	}
+	i = read(ve->fd, buf, sizeof buf);
+	if (i > 0)
+		VSB_bcat(jp->diag, buf, i);
 	if (i == 0) {
 		njob--;
 		px = wait4(jp->child, &stx, 0, NULL);
@@ -184,43 +182,65 @@ tst_cb(const struct vev *ve, int what)
 		t = VTIM_mono() - jp->t0;
 		AZ(close(ve->fd));
 
-		if (stx && vtc_verbosity)
-			printf("%s\n", jp->buf);
-		else if (vtc_verbosity > 1)
-			printf("%s\n", jp->buf);
+		ecode = WTERMSIG(stx);
+		if (ecode == 0)
+			ecode = WEXITSTATUS(stx);
 
-		if (stx)
-			vtc_fail++;
-		else
+		AZ(VSB_finish(jp->diag));
+		v = VSB_new_auto();
+		AN(v);
+		VSB_cat(v, jp->buf);
+		p = strchr(jp->buf, '\0');
+		if (p > jp->buf && p[-1] != '\n')
+			VSB_putc(v, '\n');
+		VSB_quote_pfx(v, "*    diag  0.0 ",
+		    VSB_data(jp->diag), -1, VSB_QUOTE_NONL);
+		AZ(VSB_finish(v));
+		VSB_destroy(&jp->diag);
+		AZ(munmap(jp->buf, jp->bufsiz));
+
+		if ((ecode > 1 && vtc_verbosity) || vtc_verbosity > 1)
+			printf("%s", VSB_data(v));
+
+		if (!ecode)
 			vtc_good++;
+		else if (ecode == 1)
+			vtc_skip++;
+		else
+			vtc_fail++;
 
-		if (leave_temp == 0 || (leave_temp == 1 && !stx)) {
+		if (leave_temp == 0 || (leave_temp == 1 && ecode <= 1)) {
 			bprintf(buf, "rm -rf %s", jp->tmpdir);
 			AZ(system(buf));
 		} else {
 			bprintf(buf, "%s/LOG", jp->tmpdir);
 			f = fopen(buf, "w");
 			AN(f);
-			(void)fprintf(f, "%s\n", jp->buf);
+			(void)fprintf(f, "%s\n", VSB_data(v));
 			AZ(fclose(f));
 		}
 		free(jp->tmpdir);
+		VSB_destroy(&v);
 
-		if (stx) {
-			printf("#     top  TEST %s FAILED (%.3f)",
+		if (jp->killed)
+			printf("#    top  TEST %s TIMED OUT (kill -9)\n",
+			    jp->tst->filename);
+		if (ecode > 1) {
+			printf("#    top  TEST %s FAILED (%.3f)",
 			    jp->tst->filename, t);
 			if (WIFSIGNALED(stx))
-				printf(" signal=%d", WTERMSIG(stx));
-			printf(" exit=%d\n", WEXITSTATUS(stx));
+				printf(" signal=%d\n", WTERMSIG(stx));
+			else if (WIFEXITED(stx))
+				printf(" exit=%d\n", WEXITSTATUS(stx));
 			if (!vtc_continue) {
 				/* XXX kill -9 other jobs ? */
 				exit(2);
 			}
 		} else if (vtc_verbosity) {
-			printf("#     top  TEST %s passed (%.3f)\n",
-			    jp->tst->filename, t);
+			printf("#    top  TEST %s %s (%.3f)\n",
+			    jp->tst->filename,
+			    ecode ? "skipped" : "passed", t);
 		}
-		AZ(munmap(jp->buf, jp->bufsiz));
 		if (jp->evt != NULL)
 			vev_del(vb, jp->evt);
 
@@ -245,6 +265,9 @@ start_test(void)
 	ALLOC_OBJ(jp, JOB_MAGIC);
 	AN(jp);
 
+	jp->diag = VSB_new_auto();
+	AN(jp->diag);
+
 	jp->bufsiz = vtc_bufsiz;
 
 	jp->buf = mmap(NULL, jp->bufsiz, PROT_READ|PROT_WRITE,
@@ -276,8 +299,7 @@ start_test(void)
 	jp->child = fork();
 	assert(jp->child >= 0);
 	if (jp->child == 0) {
-		AZ(close(STDIN_FILENO));
-		assert(open("/dev/null", O_RDONLY) == STDIN_FILENO);
+		VFIL_null_fd(STDIN_FILENO);
 		assert(dup2(p[1], STDOUT_FILENO) == STDOUT_FILENO);
 		assert(dup2(p[1], STDERR_FILENO) == STDERR_FILENO);
 		VSUB_closefrom(STDERR_FILENO + 1);
@@ -285,7 +307,7 @@ start_test(void)
 		    jp->tmpdir, jp->buf, jp->bufsiz);
 		exit(retval);
 	}
-	AZ(close(p[1]));
+	closefd(&p[1]);
 
 	jp->ev = vev_new();
 	AN(jp->ev);
@@ -374,7 +396,7 @@ i_mode(void)
 		sep = ":";						\
 	} while (0);
 #include "programs.h"
-#undef VTC_PROG
+
 	VSB_printf(vsb, ":%s", getenv("PATH"));
 	AZ(VSB_finish(vsb));
 
@@ -402,7 +424,7 @@ i_mode(void)
 	/*
 	 * strict jemalloc checking
 	 */
-	AZ(putenv(strdup("MALLOC_CONF=abort:true,redzone:true,junk:true")));
+	AZ(putenv(strdup("MALLOC_CONF=abort:true,junk:true")));
 }
 
 /**********************************************************************
@@ -442,12 +464,20 @@ dns_works(void)
  * Figure out what IP related magic
  */
 
+static int __match_proto__(vss_resolved_f)
+bind_cb(void *priv, const struct suckaddr *sa)
+{
+	(void)priv;
+	return (VTCP_bind(sa, NULL));
+}
+
 static void
 ip_magic(void)
 {
 	const char *p;
 	int fd;
 	char abuf[VTCP_ADDRBUFSIZE];
+	char pbuf[VTCP_PORTBUFSIZE];
 
 	/*
 	 * In FreeBSD jails localhost/127.0.0.1 becomes the jails IP#
@@ -456,12 +486,14 @@ ip_magic(void)
 	 * XXX: "localhost", but that doesn't work out of the box.
 	 * XXX: Things like "prefer_ipv6" parameter complicates things.
 	 */
-	fd = VTCP_listen_on("127.0.0.1", NULL, 10, &p);
+	fd = VSS_resolver("127.0.0.1", NULL, bind_cb, NULL, &p);
 	assert(fd >= 0);
-	VTCP_myname(fd, abuf, sizeof abuf, NULL, 0);
-	AZ(close(fd));
+	VTCP_myname(fd, abuf, sizeof abuf, pbuf, sizeof(pbuf));
 	extmacro_def("localhost", "%s", abuf);
 
+	/* Expose a backend that is forever down. */
+	extmacro_def("bad_backend", "%s %s", abuf, pbuf);
+
 	/*
 	 * We need an IP number which will not repond, ever, and that is a
 	 * lot harder than it sounds.  This IP# is from RFC5737 and a
@@ -495,6 +527,8 @@ read_file(const char *fn, int ntest)
 		if (*q != '#')
 			break;
 		q = strchr(q, '\n');
+		if (q == NULL)
+			break;
 	}
 
 	if (q == NULL || *q == '\0') {
@@ -507,6 +541,7 @@ read_file(const char *fn, int ntest)
 		fprintf(stderr,
 		    "File \"%s\" doesn't start with 'varnishtest'\n", fn);
 		free(p);
+		vtc_skip++;
 		return(2);
 	}
 	ALLOC_OBJ(tp, TST_MAGIC);
@@ -528,6 +563,7 @@ main(int argc, char * const *argv)
 	int ch, i;
 	int ntest = 1;			/* Run tests this many times */
 	uintmax_t bufsiz;
+	const char *p;
 
 	if (getenv("TMPDIR") != NULL)
 		tmppath = strdup(getenv("TMPDIR"));
@@ -541,6 +577,9 @@ main(int argc, char * const *argv)
 
 	params_vsb = VSB_new_auto();
 	AN(params_vsb);
+	p = getenv("VARNISHTEST_DURATION");
+	if (p != NULL)
+		vtc_maxdur = atoi(p);
 
 	setbuf(stdout, NULL);
 	setbuf(stderr, NULL);
@@ -586,7 +625,7 @@ main(int argc, char * const *argv)
 			break;
 		case 'p':
 			VSB_printf(params_vsb, " -p ");
-			VSB_quote(params_vsb, optarg, -1, VSB_QUOTE_NONL);
+			VSB_quote(params_vsb, optarg, -1, 0);
 			break;
 		case 'q':
 			if (vtc_verbosity > 0)
@@ -609,7 +648,10 @@ main(int argc, char * const *argv)
 	argc -= optind;
 	argv += optind;
 
-	for (;argc > 0; argc--, argv++) {
+	if (argc < 1)
+		usage();
+
+	for (; argc > 0; argc--, argv++) {
 		if (!read_file(*argv, ntest))
 			continue;
 		if (!vtc_continue)
@@ -627,7 +669,7 @@ main(int argc, char * const *argv)
 	vb = vev_new_base();
 
 	i = 0;
-	while(!VTAILQ_EMPTY(&tst_head) || i) {
+	while (!VTAILQ_EMPTY(&tst_head) || i) {
 		if (!VTAILQ_EMPTY(&tst_head) && njob < npar) {
 			start_test();
 			njob++;
@@ -640,9 +682,12 @@ main(int argc, char * const *argv)
 		i = vev_schedule_one(vb);
 	}
 	if (vtc_continue)
-		fprintf(stderr, "%d tests failed, %d tests passed\n",
-		    vtc_fail, vtc_good);
+		fprintf(stderr,
+		    "%d tests failed, %d tests skipped, %d tests passed\n",
+		    vtc_fail, vtc_skip, vtc_good);
 	if (vtc_fail)
 		return (1);
+	if (vtc_skip && !vtc_good)
+		return (77);
 	return (0);
 }
