@@ -49,15 +49,11 @@
 #include "vapi/vsl.h"
 #include "vapi/vsm.h"
 #include "vcli.h"
-#include "vss.h"
+#include "vre.h"
 #include "vsub.h"
 #include "vtcp.h"
 #include "vtim.h"
 
-extern int leave_temp;
-extern char *vmod_path;
-extern struct vsb *params_vsb;
-
 struct varnish {
 	unsigned		magic;
 #define VARNISH_MAGIC		0x208cd8e3
@@ -80,10 +76,14 @@ struct varnish {
 	char			*jail;
 	char			*proto;
 
-	struct VSM_data		*vd;		/* vsc use */
+	struct vsm		*vsm_vsl;
+	struct vsm		*vsm_vsc;
+	struct vsc		*vsc;
+	int			has_a_arg;
 
 	unsigned		vsl_tag_count[256];
 
+	volatile int		vsl_rec;
 	volatile int		vsl_idle;
 };
 
@@ -106,21 +106,18 @@ varnish_ask_cli(const struct varnish *v, const char *cmd, char **repl)
 	if (cmd != NULL) {
 		vtc_dump(v->vl, 4, "CLI TX", cmd, -1);
 		i = write(v->cli_fd, cmd, strlen(cmd));
-		if (i != strlen(cmd))
-			vtc_log(v->vl, 0, "CLI write failed (%s) = %u %s",
+		if (i != strlen(cmd) && !vtc_stop)
+			vtc_fatal(v->vl, "CLI write failed (%s) = %u %s",
 			    cmd, errno, strerror(errno));
 		i = write(v->cli_fd, "\n", 1);
-		if (i != 1)
-			vtc_log(v->vl, 0, "CLI write failed (%s) = %u %s",
+		if (i != 1 && !vtc_stop)
+			vtc_fatal(v->vl, "CLI write failed (%s) = %u %s",
 			    cmd, errno, strerror(errno));
 	}
 	i = VCLI_ReadResult(v->cli_fd, &retval, &r, vtc_maxdur);
-	if (i != 0) {
-		vtc_log(v->vl, 0, "CLI failed (%s) = %d %u %s",
+	if (i != 0 && !vtc_stop)
+		vtc_fatal(v->vl, "CLI failed (%s) = %d %u %s",
 		    cmd, i, retval, r);
-		return ((enum VCLI_status_e)retval);
-	}
-	AZ(i);
 	vtc_log(v->vl, 3, "CLI RX  %u", retval);
 	vtc_dump(v->vl, 4, "CLI RX", r, -1);
 	if (repl != NULL)
@@ -140,11 +137,11 @@ wait_stopped(const struct varnish *v)
 	char *r = NULL;
 	enum VCLI_status_e st;
 
+	vtc_log(v->vl, 3, "wait-stopped");
 	while (1) {
-		vtc_log(v->vl, 3, "wait-stopped");
 		st = varnish_ask_cli(v, "status", &r);
 		if (st != CLIS_OK)
-			vtc_log(v->vl, 0,
+			vtc_fatal(v->vl,
 			    "CLI status command failed: %u %s", st, r);
 		if (!strcmp(r, "Child in state stopped")) {
 			free(r);
@@ -169,14 +166,11 @@ wait_running(const struct varnish *v)
 		vtc_log(v->vl, 3, "wait-running");
 		st = varnish_ask_cli(v, "status", &r);
 		if (st != CLIS_OK)
-			vtc_log(v->vl, 0,
+			vtc_fatal(v->vl,
 			    "CLI status command failed: %u %s", st, r);
-		if (!strcmp(r, "Child in state stopped")) {
-			vtc_log(v->vl, 0,
+		if (!strcmp(r, "Child in state stopped"))
+			vtc_fatal(v->vl,
 			    "Child stopped before running: %u %s", st, r);
-			free(r);
-			break;
-		}
 		if (!strcmp(r, "Child in state running")) {
 			free(r);
 			break;
@@ -191,12 +185,22 @@ wait_running(const struct varnish *v)
  * Varnishlog gatherer thread
  */
 
+static void
+vsl_catchup(const struct varnish *v)
+{
+	int vsl_idle;
+
+	vsl_idle = v->vsl_idle;
+	while (!vtc_error && v->pid && vsl_idle == v->vsl_idle)
+		VTIM_sleep(0.1);
+}
+
 static void *
 varnishlog_thread(void *priv)
 {
 	struct varnish *v;
 	struct VSL_data *vsl;
-	struct VSM_data *vsm;
+	struct vsm *vsm;
 	struct VSL_cursor *c;
 	enum VSL_tag_e tag;
 	uint32_t vxid;
@@ -208,22 +212,19 @@ varnishlog_thread(void *priv)
 
 	vsl = VSL_New();
 	AN(vsl);
-	vsm = VSM_New();
-	AN(vsm);
-	(void)VSM_n_Arg(vsm, v->workdir);
+	vsm = v->vsm_vsl;
 
 	c = NULL;
 	opt = 0;
-	while (v->pid) {
+	while (v->pid || c != NULL) {
 		if (c == NULL) {
-			v->vsl_idle++;
+			if (vtc_error)
+				break;
 			VTIM_sleep(0.1);
-			if (VSM_Open(vsm)) {
-				VSM_ResetError(vsm);
-				continue;
-			}
+			(void)VSM_Status(vsm);
 			c = VSL_CursorVSM(vsl, vsm, opt);
 			if (c == NULL) {
+				vtc_log(v->vl, 3, "vsl|%s", VSL_Error(vsl));
 				VSL_ResetError(vsl);
 				continue;
 			}
@@ -232,45 +233,48 @@ varnishlog_thread(void *priv)
 
 		opt = VSL_COPT_TAIL;
 
-		i = VSL_Next(c);
+		while (1) {
+			i = VSL_Next(c);
+			if (i != 1)
+				break;
+
+			v->vsl_rec = 1;
+
+			tag = VSL_TAG(c->rec.ptr);
+			vxid = VSL_ID(c->rec.ptr);
+			if (tag == SLT__Batch)
+				continue;
+			tagname = VSL_tags[tag];
+			len = VSL_LEN(c->rec.ptr);
+			type = VSL_CLIENT(c->rec.ptr) ?
+			    'c' : VSL_BACKEND(c->rec.ptr) ?
+			    'b' : '-';
+			data = VSL_CDATA(c->rec.ptr);
+			v->vsl_tag_count[tag]++;
+			vtc_log(v->vl, 4, "vsl| %10u %-15s %c %.*s",
+			    vxid, tagname, type, (int)len, data);
+		}
 		if (i == 0) {
-			v->vsl_idle++;
 			/* Nothing to do but wait */
-			VTIM_sleep(0.1);
-			continue;
+			v->vsl_idle++;
+			if (!(VSM_Status(vsm) & VSM_WRK_RUNNING)) {
+				/* Abandoned - try reconnect */
+				VSL_DeleteCursor(c);
+				c = NULL;
+			} else {
+				VTIM_sleep(0.1);
+			}
 		} else if (i == -2) {
 			/* Abandoned - try reconnect */
 			VSL_DeleteCursor(c);
 			c = NULL;
-			VSM_Close(vsm);
-			continue;
-		} else if (i != 1)
+		} else
 			break;
-
-		v->vsl_idle = 0;
-
-		tag = VSL_TAG(c->rec.ptr);
-		vxid = VSL_ID(c->rec.ptr);
-		if (tag != SLT_CLI)
-			v->vsl_idle = 0;
-		if (tag == SLT__Batch)
-			continue;
-		tagname = VSL_tags[tag];
-		len = VSL_LEN(c->rec.ptr);
-		type = VSL_CLIENT(c->rec.ptr) ? 'c' : VSL_BACKEND(c->rec.ptr) ?
-		    'b' : '-';
-		data = VSL_CDATA(c->rec.ptr);
-		v->vsl_tag_count[tag]++;
-		vtc_log(v->vl, 4, "vsl| %10u %-15s %c %.*s", vxid, tagname,
-		    type, (int)len, data);
 	}
 
-	v->vsl_idle = 100;
-
 	if (c)
 		VSL_DeleteCursor(c);
 	VSL_Delete(vsl);
-	VSM_Delete(vsm);
 
 	return (NULL);
 }
@@ -286,7 +290,6 @@ varnish_new(const char *name)
 	struct vsb *vsb;
 	char buf[1024];
 
-	AN(name);
 	ALLOC_OBJ(v, VARNISH_MAGIC);
 	AN(v);
 	REPLACE(v->name, name);
@@ -308,9 +311,6 @@ varnish_new(const char *name)
 	bprintf(buf, "rm -rf %s ; mkdir -p %s", v->workdir, v->workdir);
 	AZ(system(buf));
 
-	if (*v->name != 'v')
-		vtc_log(v->vl, 0, "Varnish name must start with 'v'");
-
 	v->args = VSB_new_auto();
 
 	v->cli_fd = -1;
@@ -332,8 +332,12 @@ varnish_delete(struct varnish *v)
 	vtc_logclose(v->vl);
 	free(v->name);
 	free(v->workdir);
-	if (v->vd != NULL)
-		VSM_Delete(v->vd);
+	if (v->vsc != NULL)
+		VSC_Destroy(&v->vsc, v->vsm_vsc);
+	if (v->vsm_vsc != NULL)
+		VSM_Destroy(&v->vsm_vsc);
+	if (v->vsm_vsl != NULL)
+		VSM_Destroy(&v->vsm_vsl);
 
 	/*
 	 * We do not delete the workdir, it may contain stuff people
@@ -397,12 +401,10 @@ varnish_launch(struct varnish *v)
 	const char *err;
 	char *r = NULL;
 
-	v->vd = VSM_New();
-
 	/* Create listener socket */
 	v->cli_fd = VTCP_listen_on("127.0.0.1:0", NULL, 1, &err);
 	if (err != NULL)
-		vtc_log(v->vl, 0, "Create CLI listen socket failed: %s", err);
+		vtc_fatal(v->vl, "Create CLI listen socket failed: %s", err);
 	assert(v->cli_fd > 0);
 	VTCP_myname(v->cli_fd, abuf, sizeof abuf, pbuf, sizeof pbuf);
 
@@ -416,17 +418,21 @@ varnish_launch(struct varnish *v)
 	VSB_cat(vsb, VSB_data(params_vsb));
 	if (vtc_witness)
 		VSB_cat(vsb, " -p debug=+witness");
-	if (leave_temp)
+	if (leave_temp) {
 		VSB_cat(vsb, " -p debug=+vsm_keep");
-	VSB_printf(vsb, " -l 2m,1m,-");
+		VSB_cat(vsb, " -p debug=+vmod_so_keep");
+	}
+	VSB_printf(vsb, " -l 2m");
 	VSB_printf(vsb, " -p auto_restart=off");
 	VSB_printf(vsb, " -p syslog_cli_traffic=off");
 	VSB_printf(vsb, " -p sigsegv_handler=on");
 	VSB_printf(vsb, " -p thread_pool_min=10");
 	VSB_printf(vsb, " -p debug=+vtc_mode");
-	VSB_printf(vsb, " -a '%s'", "127.0.0.1:0");
-	if (v->proto != NULL)
-		VSB_printf(vsb, ",%s", v->proto);
+	if (!v->has_a_arg) {
+		VSB_printf(vsb, " -a '%s'", "127.0.0.1:0");
+		if (v->proto != NULL)
+			VSB_printf(vsb, ",%s", v->proto);
+	}
 	VSB_printf(vsb, " -M '%s %s'", abuf, pbuf);
 	VSB_printf(vsb, " -P %s/varnishd.pid", v->workdir);
 	if (vmod_path != NULL)
@@ -447,10 +453,10 @@ varnish_launch(struct varnish *v)
 		AZ(dup2(v->fds[0], 0));
 		assert(dup2(v->fds[3], 1) == 1);
 		assert(dup2(1, 2) == 2);
-		AZ(close(v->fds[0]));
-		AZ(close(v->fds[1]));
-		AZ(close(v->fds[2]));
-		AZ(close(v->fds[3]));
+		closefd(&v->fds[0]);
+		closefd(&v->fds[1]);
+		closefd(&v->fds[2]);
+		closefd(&v->fds[3]);
 		VSUB_closefrom(STDERR_FILENO + 1);
 		AZ(execl("/bin/sh", "/bin/sh", "-c", VSB_data(vsb), (char*)0));
 		exit(1);
@@ -459,13 +465,12 @@ varnish_launch(struct varnish *v)
 		macro_def(v->vl, v->name, "pid", "%ld", (long)v->pid);
 		macro_def(v->vl, v->name, "name", "%s", v->workdir);
 	}
-	AZ(close(v->fds[0]));
-	AZ(close(v->fds[3]));
+	closefd(&v->fds[0]);
+	closefd(&v->fds[3]);
 	v->fds[0] = v->fds[2];
 	v->fds[2] = v->fds[3] = -1;
 	VSB_destroy(&vsb);
 	AZ(pthread_create(&v->tp, NULL, varnish_thread, v));
-	AZ(pthread_create(&v->tp_vsl, NULL, varnishlog_thread, v));
 
 	/* Wait for the varnish to call home */
 	memset(fd, 0, sizeof fd);
@@ -473,36 +478,22 @@ varnish_launch(struct varnish *v)
 	fd[0].events = POLLIN;
 	fd[1].fd = v->fds[1];
 	fd[1].events = POLLIN;
-	i = poll(fd, 2, 10000);
+	i = poll(fd, 2, vtc_maxdur * 1000 / 3);
 	vtc_log(v->vl, 4, "CLIPOLL %d 0x%x 0x%x",
 	    i, fd[0].revents, fd[1].revents);
-	if (i == 0) {
-		vtc_log(v->vl, 0, "FAIL timeout waiting for CLI connection");
-		AZ(close(v->cli_fd));
-		v->cli_fd = -1;
-		return;
-	}
-	if (fd[1].revents & POLLHUP) {
-		vtc_log(v->vl, 0, "FAIL debug pipe closed");
-		AZ(close(v->cli_fd));
-		v->cli_fd = -1;
-		return;
-	}
-	if (!(fd[0].revents & POLLIN)) {
-		vtc_log(v->vl, 0, "FAIL CLI connection wait failure");
-		AZ(close(v->cli_fd));
-		v->cli_fd = -1;
-		return;
-	}
+	if (i == 0)
+		vtc_fatal(v->vl, "FAIL timeout waiting for CLI connection");
+	if (fd[1].revents & POLLHUP)
+		vtc_fatal(v->vl, "FAIL debug pipe closed");
+	if (!(fd[0].revents & POLLIN))
+		vtc_fatal(v->vl, "FAIL CLI connection wait failure");
 	nfd = accept(v->cli_fd, NULL, NULL);
 	if (nfd < 0) {
-		AZ(close(v->cli_fd));
-		v->cli_fd = -1;
-		vtc_log(v->vl, 0, "FAIL no CLI connection accepted");
-		return;
+		closefd(&v->cli_fd);
+		vtc_fatal(v->vl, "FAIL no CLI connection accepted");
 	}
 
-	AZ(close(v->cli_fd));
+	closefd(&v->cli_fd);
 	v->cli_fd = nfd;
 
 	vtc_log(v->vl, 3, "CLI connection fd = %d", v->cli_fd);
@@ -513,7 +504,7 @@ varnish_launch(struct varnish *v)
 	if (vtc_error)
 		return;
 	if (u != CLIS_AUTH)
-		vtc_log(v->vl, 0, "CLI auth demand expected: %u %s", u, r);
+		vtc_fatal(v->vl, "CLI auth demand expected: %u %s", u, r);
 
 	bprintf(abuf, "%s/_.secret", v->workdir);
 	nfd = open(abuf, O_RDONLY);
@@ -522,7 +513,7 @@ varnish_launch(struct varnish *v)
 	assert(sizeof abuf >= CLI_AUTH_RESPONSE_LEN + 7);
 	strcpy(abuf, "auth ");
 	VCLI_AuthResponse(nfd, r, abuf + 5);
-	AZ(close(nfd));
+	closefd(&nfd);
 	free(r);
 	r = NULL;
 	strcat(abuf, "\n");
@@ -531,11 +522,21 @@ varnish_launch(struct varnish *v)
 	if (vtc_error)
 		return;
 	if (u != CLIS_OK)
-		vtc_log(v->vl, 0, "CLI auth command failed: %u %s", u, r);
+		vtc_fatal(v->vl, "CLI auth command failed: %u %s", u, r);
 	free(r);
 
-	(void)VSM_n_Arg(v->vd, v->workdir);
-	AZ(VSM_Open(v->vd));
+	v->vsm_vsc = VSM_New();
+	AN(v->vsm_vsc);
+	v->vsc = VSC_New();
+	AN(v->vsc);
+	assert(VSM_Arg(v->vsm_vsc, 'n', v->workdir) > 0);
+	AZ(VSM_Attach(v->vsm_vsc, -1));
+
+	v->vsm_vsl = VSM_New();
+	assert(VSM_Arg(v->vsm_vsl, 'n', v->workdir) > 0);
+	AZ(VSM_Attach(v->vsm_vsl, -1));
+
+	AZ(pthread_create(&v->tp_vsl, NULL, varnishlog_thread, v));
 }
 
 /**********************************************************************
@@ -557,7 +558,7 @@ varnish_start(struct varnish *v)
 	if (vtc_error)
 		return;
 	if (u != CLIS_OK)
-		vtc_log(v->vl, 0, "CLI start command failed: %u %s", u, resp);
+		vtc_fatal(v->vl, "CLI start command failed: %u %s", u, resp);
 	wait_running(v);
 	free(resp);
 	resp = NULL;
@@ -565,7 +566,7 @@ varnish_start(struct varnish *v)
 	if (vtc_error)
 		return;
 	if (u != CLIS_OK)
-		vtc_log(v->vl, 0, "CLI debug.xid command failed: %u %s",
+		vtc_fatal(v->vl, "CLI debug.xid command failed: %u %s",
 		    u, resp);
 	free(resp);
 	resp = NULL;
@@ -573,7 +574,7 @@ varnish_start(struct varnish *v)
 	if (vtc_error)
 		return;
 	if (u != CLIS_OK)
-		vtc_log(v->vl, 0,
+		vtc_fatal(v->vl,
 		    "CLI debug.listen_address command failed: %u %s", u, resp);
 	h = resp;
 	p = strchr(h, '\n');
@@ -586,6 +587,10 @@ varnish_start(struct varnish *v)
 	macro_def(v->vl, v->name, "addr", "%s", h);
 	macro_def(v->vl, v->name, "port", "%s", p);
 	macro_def(v->vl, v->name, "sock", "%s %s", h, p);
+	free(resp);
+	/* Wait for vsl logging to get underway */
+	while (v->vsl_rec == 0)
+		VTIM_sleep(.1);
 }
 
 /**********************************************************************
@@ -595,24 +600,13 @@ varnish_start(struct varnish *v)
 static void
 varnish_stop(struct varnish *v)
 {
-	char *r;
-
 	if (v->cli_fd < 0)
 		varnish_launch(v);
 	if (vtc_error)
 		return;
 	vtc_log(v->vl, 2, "Stop");
 	(void)varnish_ask_cli(v, "stop", NULL);
-	while (1) {
-		r = NULL;
-		(void)varnish_ask_cli(v, "status", &r);
-		AN(r);
-		if (!strcmp(r, "Child in state stopped"))
-			break;
-		free(r);
-		(void)sleep (1);
-		/* XXX: should fail eventually */
-	}
+	wait_stopped(v);
 }
 
 /**********************************************************************
@@ -626,20 +620,15 @@ varnish_cleanup(struct varnish *v)
 	int status, r;
 	struct rusage ru;
 
-	/* Give the VSL log time to finish */
-	while (v->vsl_idle < 10)
-		(void)usleep(200000);
-
 	/* Close the CLI connection */
-	AZ(close(v->cli_fd));
-	v->cli_fd = -1;
+	closefd(&v->cli_fd);
 
 	/* Close the STDIN connection. */
-	AZ(close(v->fds[1]));
+	closefd(&v->fds[1]);
 
 	/* Wait until STDOUT+STDERR closes */
 	AZ(pthread_join(v->tp, &p));
-	AZ(close(v->fds[0]));
+	closefd(&v->fds[0]);
 
 	r = wait4(v->pid, &status, 0, &ru);
 	v->pid = 0;
@@ -654,11 +643,11 @@ varnish_cleanup(struct varnish *v)
 	if (WIFEXITED(status) && (WEXITSTATUS(status) == 0))
 		return;
 #ifdef WCOREDUMP
-	vtc_log(v->vl, 0, "Bad exit code: %04x sig %x exit %x core %x",
+	vtc_fatal(v->vl, "Bad exit code: %04x sig %x exit %x core %x",
 	    status, WTERMSIG(status), WEXITSTATUS(status),
 	    WCOREDUMP(status));
 #else
-	vtc_log(v->vl, 0, "Bad exit code: %04x sig %x exit %x",
+	vtc_fatal(v->vl, "Bad exit code: %04x sig %x exit %x",
 	    status, WTERMSIG(status), WEXITSTATUS(status));
 #endif
 }
@@ -670,19 +659,22 @@ varnish_cleanup(struct varnish *v)
 static void
 varnish_wait(struct varnish *v)
 {
-	char *resp;
-
 	if (v->cli_fd < 0)
 		return;
 
 	vtc_log(v->vl, 2, "Wait");
 
-	/* Do a backend.list to log if child is still running */
-	varnish_ask_cli(v, "backend.list", &resp);
+	if (!vtc_error) {
+		/* Do a backend.list to log if child is still running */
+		(void)varnish_ask_cli(v, "backend.list", NULL);
+	}
 
 	/* Then stop it */
 	varnish_stop(v);
 
+	if (varnish_ask_cli(v, "panic.clear", NULL) != CLIS_CANT)
+		vtc_fatal(v->vl, "Unexpected panic");
+
 	varnish_cleanup(v);
 }
 
@@ -692,18 +684,37 @@ varnish_wait(struct varnish *v)
  */
 
 static void
-varnish_cli(struct varnish *v, const char *cli, unsigned exp)
+varnish_cli(struct varnish *v, const char *cli, unsigned exp, const char *re)
 {
 	enum VCLI_status_e u;
+	vre_t *vre = NULL;
+	char *resp = NULL;
+	const char *errptr;
+	int err;
 
+	if (re != NULL) {
+		vre = VRE_compile(re, 0, &errptr, &err);
+		if (vre == NULL)
+			vtc_fatal(v->vl, "Illegal regexp");
+	}
 	if (v->cli_fd < 0)
 		varnish_launch(v);
-	if (vtc_error)
+	if (vtc_error) {
+		if (vre != NULL)
+			VRE_free(&vre);
 		return;
-	u = varnish_ask_cli(v, cli, NULL);
+	}
+	u = varnish_ask_cli(v, cli, &resp);
 	vtc_log(v->vl, 2, "CLI %03u <%s>", u, cli);
 	if (exp != 0 && exp != (unsigned)u)
-		vtc_log(v->vl, 0, "FAIL CLI response %u expected %u", u, exp);
+		vtc_fatal(v->vl, "FAIL CLI response %u expected %u", u, exp);
+	if (vre != NULL) {
+		err = VRE_exec(vre, resp, strlen(resp), 0, 0, NULL, 0, NULL);
+		if (err < 1)
+			vtc_fatal(v->vl, "Expect failed (%d)", err);
+		VRE_free(&vre);
+	}
+	free(resp);
 }
 
 /**********************************************************************
@@ -728,25 +739,20 @@ varnish_vcl(struct varnish *v, const char *vcl, int fail, char **resp)
 	AZ(VSB_finish(vsb));
 
 	u = varnish_ask_cli(v, VSB_data(vsb), resp);
-	if (u == CLIS_OK && fail) {
-		VSB_destroy(&vsb);
-		vtc_log(v->vl, 0,
-		    "VCL compilation succeeded expected failure");
-		return;
-	} else if (u == CLIS_OK) {
+	if (u == CLIS_OK) {
 		VSB_clear(vsb);
 		VSB_printf(vsb, "vcl.use vcl%d", v->vcl_nbr);
 		AZ(VSB_finish(vsb));
 		u = varnish_ask_cli(v, VSB_data(vsb), NULL);
-		assert(u == CLIS_OK);
-	} else if (!fail) {
-		VSB_destroy(&vsb);
-		vtc_log(v->vl, 0,
-		    "VCL compilation failed expected success");
-		return;
-	} else {
-		vtc_log(v->vl, 2, "VCL compilation failed (as expected)");
 	}
+	if (u == CLIS_OK && fail) {
+		VSB_destroy(&vsb);
+		vtc_fatal(v->vl, "VCL compilation succeeded expected failure");
+	} else if (u != CLIS_OK && !fail) {
+		VSB_destroy(&vsb);
+		vtc_fatal(v->vl, "VCL compilation failed expected success");
+	} else if (fail)
+		vtc_log(v->vl, 2, "VCL compilation failed (as expected)");
 	VSB_destroy(&vsb);
 }
 
@@ -784,8 +790,7 @@ varnish_vclbackend(struct varnish *v, const char *vcl)
 	if (u != CLIS_OK) {
 		VSB_destroy(&vsb);
 		VSB_destroy(&vsb2);
-		vtc_log(v->vl, 0, "FAIL VCL does not compile");
-		return;
+		vtc_fatal(v->vl, "FAIL VCL does not compile");
 	}
 	VSB_clear(vsb);
 	VSB_printf(vsb, "vcl.use vcl%d", v->vcl_nbr);
@@ -810,31 +815,22 @@ do_stat_dump_cb(void *priv, const struct VSC_point * const pt)
 	const struct varnish *v;
 	struct dump_priv *dp;
 	uint64_t u;
-	char buf[1024];
 
 	if (pt == NULL)
 		return (0);
 	dp = priv;
 	v = dp->v;
 
-	if(strcmp(pt->desc->ctype, "uint64_t"))
+	if (strcmp(pt->ctype, "uint64_t"))
 		return (0);
-	u = *(const volatile uint64_t*)pt->ptr;
-
-	strcpy(buf, pt->section->type);
-	if (pt->section->ident[0] != '\0') {
-		strcat(buf, ".");
-		strcat(buf, pt->section->ident);
-	}
-	strcat(buf, ".");
-	strcat(buf, pt->desc->name);
+	u = *pt->ptr;
 
 	if (strcmp(dp->arg, "*")) {
-		if (fnmatch(dp->arg, buf, 0))
+		if (fnmatch(dp->arg, pt->name, 0))
 			return (0);
 	}
 
-	vtc_log(v->vl, 4, "VSC %s %ju",  buf, (uintmax_t)u);
+	vtc_log(v->vl, 4, "VSC %s %ju",  pt->name, (uintmax_t)u);
 	return (0);
 }
 
@@ -846,12 +842,8 @@ varnish_vsc(const struct varnish *v, const char *arg)
 	memset(&dp, 0, sizeof dp);
 	dp.v = v;
 	dp.arg = arg;
-	if (VSM_Abandoned(v->vd)) {
-		VSM_Close(v->vd);
-		VSM_Open(v->vd);
-	}
-
-	(void)VSC_Iter(v->vd, NULL, do_stat_dump_cb, &dp);
+	(void)VSM_Status(v->vsm_vsc);
+	(void)VSC_Iter(v->vsc, v->vsm_vsc, do_stat_dump_cb, &dp);
 }
 
 /**********************************************************************
@@ -859,30 +851,25 @@ varnish_vsc(const struct varnish *v, const char *arg)
  */
 
 struct stat_priv {
-	char target_type[256];
-	char target_ident[256];
 	char target_name[256];
 	uintmax_t val;
 	const struct varnish *v;
 };
 
 static int
-do_stat_cb(void *priv, const struct VSC_point * const pt)
+do_expect_cb(void *priv, const struct VSC_point * const pt)
 {
 	struct stat_priv *sp = priv;
 
 	if (pt == NULL)
 		return(0);
 
-	if (strcmp(pt->section->type, sp->target_type))
-		return(0);
-	if (strcmp(pt->section->ident, sp->target_ident))
-		return(0);
-	if (strcmp(pt->desc->name, sp->target_name))
+	if (strcmp(pt->name, sp->target_name))
 		return(0);
 
-	AZ(strcmp(pt->desc->ctype, "uint64_t"));
-	sp->val = *(const volatile uint64_t*)pt->ptr;
+	AZ(strcmp(pt->ctype, "uint64_t"));
+	AN(pt->ptr);
+	sp->val = *pt->ptr;
 	return (1);
 }
 
@@ -896,7 +883,6 @@ varnish_expect(const struct varnish *v, char * const *av)
 	int good;
 	char *r;
 	char *p;
-	char *q;
 	int i, not = 0;
 	struct stat_priv sp;
 
@@ -909,65 +895,45 @@ varnish_expect(const struct varnish *v, char * const *av)
 		AN(av[1]);
 		AN(av[2]);
 	}
-	p = strchr(r, '.');
+	p = strrchr(r, '.');
 	if (p == NULL) {
-		strcpy(sp.target_type, "MAIN");
-		sp.target_ident[0] = '\0';
-		bprintf(sp.target_name, "%s", r);
+		bprintf(sp.target_name, "MAIN.%s", r);
 	} else {
-		bprintf(sp.target_type, "%.*s", (int)(p - r), r);
-		p++;
-		q = strrchr(p, '.');
-		if (q == NULL) {
-			sp.target_ident[0] = '\0';
-			bprintf(sp.target_name, "%s", p);
-		} else {
-			bprintf(sp.target_ident, "%.*s", (int)(q - p), p);
-			bprintf(sp.target_name, "%s", q + 1);
-		}
+		bprintf(sp.target_name, "%s", r);
 	}
 
 	sp.val = 0;
 	sp.v = v;
 	ref = 0;
 	good = 0;
-	for (i = 0; i < 10; i++, (void)usleep(100000)) {
-		if (VSM_Abandoned(v->vd)) {
-			VSM_Close(v->vd);
-			good = VSM_Open(v->vd);
-		}
-		if (good < 0)
-			continue;
-
-		good = VSC_Iter(v->vd, NULL, do_stat_cb, &sp);
+	for (i = 0; i < 50; i++, (void)usleep(100000)) {
+		(void)VSM_Status(v->vsm_vsc);
+		good = VSC_Iter(v->vsc, v->vsm_vsc, do_expect_cb, &sp);
 		if (!good) {
 			good = -2;
 			continue;
 		}
 
-		if (not) {
-			vtc_log(v->vl, 0, "Found (not expected): %s", av[0]+1);
-			return;
-		}
+		if (not)
+			vtc_fatal(v->vl, "Found (not expected): %s", av[0]+1);
 
 		good = 0;
 		ref = strtoumax(av[2], &p, 0);
 		if (ref == UINTMAX_MAX || *p)
-			vtc_log(v->vl, 0, "Syntax error in number (%s)", av[2]);
+			vtc_fatal(v->vl, "Syntax error in number (%s)", av[2]);
 		if      (!strcmp(av[1], "==")) { if (sp.val == ref) good = 1; }
 		else if (!strcmp(av[1], "!=")) { if (sp.val != ref) good = 1; }
 		else if (!strcmp(av[1], ">"))  { if (sp.val > ref)  good = 1; }
 		else if (!strcmp(av[1], "<"))  { if (sp.val < ref)  good = 1; }
 		else if (!strcmp(av[1], ">=")) { if (sp.val >= ref) good = 1; }
 		else if (!strcmp(av[1], "<=")) { if (sp.val <= ref) good = 1; }
-		else {
-			vtc_log(v->vl, 0, "comparison %s unknown", av[1]);
-		}
+		else
+			vtc_fatal(v->vl, "comparison %s unknown", av[1]);
 		if (good)
 			break;
 	}
 	if (good == -1) {
-		vtc_log(v->vl, 0, "VSM error: %s", VSM_Error(v->vd));
+		vtc_fatal(v->vl, "VSM error: %s", VSM_Error(v->vsm_vsc));
 	}
 	if (good == -2) {
 		if (not) {
@@ -975,14 +941,14 @@ varnish_expect(const struct varnish *v, char * const *av)
 			    av[0] + 1);
 			return;
 		}
-		vtc_log(v->vl, 0, "stats field %s unknown", av[0]);
+		vtc_fatal(v->vl, "stats field %s unknown", av[0]);
 	}
 
 	if (good == 1) {
 		vtc_log(v->vl, 2, "as expected: %s (%ju) %s %s",
 		    av[0], sp.val, av[1], av[2]);
 	} else {
-		vtc_log(v->vl, 0, "Not true: %s (%ju) %s %s (%ju)",
+		vtc_fatal(v->vl, "Not true: %s (%ju) %s %s (%ju)",
 		    av[0], (uintmax_t)sp.val, av[1], av[2], (uintmax_t)ref);
 	}
 }
@@ -1002,7 +968,7 @@ varnish_expect(const struct varnish *v, char * const *av)
  * With:
  *
  * vNAME
- *         Identify the Varnish server with a string, it must starts with 'v'.
+ *	   Identify the Varnish server with a string, it must starts with 'v'.
  *
  * \-arg STRING
  *         Pass an argument to varnishd, for example "-h simple_list".
@@ -1050,7 +1016,7 @@ varnish_expect(const struct varnish *v, char * const *av)
  *
  * \-cleanup
  *         Once Varnish is stopped, clean everything after it. This is only used
- *         in one test and you should never need it.
+ *         in very few tests and you should never need it.
  *
  * Once Varnish is started, you can talk to it (as you would through
  * ``varnishadm``) with these additional switches::
@@ -1058,10 +1024,11 @@ varnish_expect(const struct varnish *v, char * const *av)
  *         varnish vNAME [-cli STRING] [-cliok STRING] [-clierr STRING]
  *                       [-expect STRING OP NUMBER]
  *
- * \-cli STRING|-cliok STRING|-clierr STATUS STRING
- *         All three of these will send STRING to the CLI, the only difference
- *         is what they expect the return code to be. -cli doesn't expect
- *         anything, -cliok expects 200 and -clierr expects STATUS
+ * \-cli STRING|-cliok STRING|-clierr STATUS STRING|-cliexpect REGEXP STRING
+ *         All four of these will send STRING to the CLI, the only difference
+ *         is what they expect the result to be. -cli doesn't expect
+ *         anything, -cliok expects 200, -clierr expects STATUS, and
+ *         -cliexpect expects the REGEXP to match the returned response.
  *
  * \-expect STRING OP NUMBER
  *         Look into the VSM and make sure the counter identified by STRING has
@@ -1074,6 +1041,10 @@ varnish_expect(const struct varnish *v, char * const *av)
  *         style pattern (ie: fnmatch(3)) as used in shell filename expansion.
  *         To see all counters use pattern "*", to see all counters about
  *         requests use "*req*".
+ *
+ * \-vsl_catchup
+ *         Wait until the logging thread has idled to make sure that all
+ *         the generated log is flushed
  */
 
 void
@@ -1083,7 +1054,6 @@ cmd_varnish(CMD_ARGS)
 
 	(void)priv;
 	(void)cmd;
-	(void)vl;
 
 	if (av == NULL) {
 		/* Reset and free */
@@ -1099,6 +1069,7 @@ cmd_varnish(CMD_ARGS)
 	AZ(strcmp(av[0], "varnish"));
 	av++;
 
+	VTC_CHECK_NAME(vl, av[0], "Varnish", 'v');
 	VTAILQ_FOREACH(v, &varnishes, list)
 		if (!strcmp(v->name, av[0]))
 			break;
@@ -1114,30 +1085,39 @@ cmd_varnish(CMD_ARGS)
 			AZ(v->pid);
 			VSB_cat(v->args, " ");
 			VSB_cat(v->args, av[1]);
+			if (av[1][0] == '-' && av[1][1] == 'a')
+				v->has_a_arg = 1;
 			av++;
 			continue;
 		}
-		if (!strcmp(*av, "-cli")) {
-			AN(av[1]);
-			varnish_cli(v, av[1], 0);
-			av++;
-			continue;
-		}
-		if (!strcmp(*av, "-clierr")) {
-			AN(av[1]);
-			AN(av[2]);
-			varnish_cli(v, av[2], atoi(av[1]));
-			av += 2;
-			continue;
-		}
 		if (!strcmp(*av, "-cleanup")) {
 			AZ(av[1]);
 			varnish_cleanup(v);
 			continue;
 		}
+		if (!strcmp(*av, "-cli")) {
+			AN(av[1]);
+			varnish_cli(v, av[1], 0, NULL);
+			av++;
+			continue;
+		}
+		if (!strcmp(*av, "-clierr")) {
+			AN(av[1]);
+			AN(av[2]);
+			varnish_cli(v, av[2], atoi(av[1]), NULL);
+			av += 2;
+			continue;
+		}
+		if (!strcmp(*av, "-cliexpect")) {
+			AN(av[1]);
+			AN(av[2]);
+			varnish_cli(v, av[2], 0, av[1]);
+			av += 2;
+			continue;
+		}
 		if (!strcmp(*av, "-cliok")) {
 			AN(av[1]);
-			varnish_cli(v, av[1], (unsigned)CLIS_OK);
+			varnish_cli(v, av[1], (unsigned)CLIS_OK, NULL);
 			av++;
 			continue;
 		}
@@ -1147,7 +1127,7 @@ cmd_varnish(CMD_ARGS)
 			AN(av[2]);
 			varnish_vcl(v, av[2], 1, &r);
 			if (strstr(r, av[1]) == NULL)
-				vtc_log(v->vl, 0,
+				vtc_fatal(v->vl,
 				    "Did not find expected string: (\"%s\")",
 				    av[1]);
 			else
@@ -1222,6 +1202,10 @@ cmd_varnish(CMD_ARGS)
 			varnish_wait(v);
 			continue;
 		}
-		vtc_log(v->vl, 0, "Unknown varnish argument: %s", *av);
+		if (!strcmp(*av, "-vsl_catchup")) {
+			vsl_catchup(v);
+			continue;
+		}
+		vtc_fatal(v->vl, "Unknown varnish argument: %s", *av);
 	}
 }
