@@ -35,24 +35,25 @@
 
 #include <ctype.h>
 #include <errno.h>
-#include <limits.h>
 #include <pthread.h>
 #include <signal.h>
+#include <stdarg.h>
 #include <stdint.h>
-#include <stdio.h>
 #include <stdlib.h>
 #include <string.h>
 #include <unistd.h>
 
+#define VOPT_DEFINITION
+#define VOPT_INC "varnishtop_options.h"
+
+#include "miniobj.h"
 #include "vcurses.h"
-#include "vapi/vsm.h"
 #include "vapi/vsl.h"
+#include "vapi/vsm.h"
 #include "vapi/voptget.h"
 #include "vas.h"
 #include "vdef.h"
-#include "vcs.h"
 #include "vtree.h"
-#include "vsb.h"
 #include "vut.h"
 
 #if 0
@@ -61,9 +62,7 @@
 #define AC(x) x
 #endif
 
-static const char progname[] = "varnishtop";
-static float period = 60; /* seconds */
-static int end_of_file = 0;
+static struct VUT *vut;
 
 struct top {
 	uint8_t			tag;
@@ -76,6 +75,16 @@ struct top {
 	double			count;
 };
 
+static float period = 60; /* seconds */
+static int end_of_file = 0;
+static unsigned ntop;
+static pthread_mutex_t mtx = PTHREAD_MUTEX_INITIALIZER;
+static int f_flag = 0;
+static unsigned maxfieldlen = 0;
+static char *ident;
+
+static volatile sig_atomic_t quit = 0;
+
 static VRB_HEAD(t_order, top) h_order = VRB_INITIALIZER(&h_order);
 static VRB_HEAD(t_key, top) h_key = VRB_INITIALIZER(&h_key);
 
@@ -101,18 +110,10 @@ cmp_order(const struct top *a, const struct top *b)
 	return (cmp_key(a, b));
 }
 
-static unsigned ntop;
-
-static pthread_mutex_t mtx = PTHREAD_MUTEX_INITIALIZER;
-
-static int f_flag = 0;
-
-static unsigned maxfieldlen = 0;
-
-VRB_PROTOTYPE_STATIC(t_order, top, e_order, cmp_order);
-VRB_GENERATE_STATIC(t_order, top, e_order, cmp_order);
-VRB_PROTOTYPE_STATIC(t_key, top, e_key, cmp_key);
-VRB_GENERATE_STATIC(t_key, top, e_key, cmp_key);
+VRB_PROTOTYPE_STATIC(t_order, top, e_order, cmp_order)
+VRB_GENERATE_STATIC(t_order, top, e_order, cmp_order)
+VRB_PROTOTYPE_STATIC(t_key, top, e_key, cmp_key)
+VRB_GENERATE_STATIC(t_key, top, e_key, cmp_key)
 
 static int __match_proto__(VSLQ_dispatch_f)
 accumulate(struct VSL_data *vsl, struct VSL_transaction * const pt[],
@@ -160,7 +161,7 @@ accumulate(struct VSL_data *vsl, struct VSL_transaction * const pt[],
 				VRB_INSERT(t_order, &h_order, tp);
 			} else {
 				ntop++;
-				tp = calloc(sizeof *tp, 1);
+				tp = calloc(1, sizeof *tp);
 				assert(tp != NULL);
 				tp->hash = u;
 				tp->count = 1.0;
@@ -181,9 +182,18 @@ accumulate(struct VSL_data *vsl, struct VSL_transaction * const pt[],
 }
 
 static int __match_proto__(VUT_cb_f)
-sighup(void)
+sighup(struct VUT *v)
 {
-	exit (1);
+	assert(v == vut);
+	quit = 1;
+	return (1);
+}
+
+static void
+vut_sighandler(int sig)
+{
+	CHECK_OBJ_NOTNULL(vut, VUT_MAGIC);
+	VUT_Signaled(vut, sig);
 }
 
 static void
@@ -194,6 +204,7 @@ update(int p)
 	double t = 0;
 	static time_t last = 0;
 	static unsigned n;
+	const char *q;
 	time_t now;
 
 	now = time(NULL);
@@ -205,11 +216,12 @@ update(int p)
 	if (n < p)
 		n++;
 	AC(erase());
+	q = ident;
+	len = COLS - strlen(q);
 	if (end_of_file)
-		AC(mvprintw(0, COLS - 1 - strlen(VUT.name) - 5, "%s (EOF)",
-			VUT.name));
+		AC(mvprintw(0, len - (1 + 6), "%s (EOF)", q));
 	else
-		AC(mvprintw(0, COLS - 1 - strlen(VUT.name), "%s", VUT.name));
+		AC(mvprintw(0, len - 1, "%s", q));
 	AC(mvprintw(0, 0, "list length %u", ntop));
 	for (tp = VRB_MIN(t_order, &h_order); tp != NULL; tp = tp2) {
 		tp2 = VRB_NEXT(t_order, &h_order, tp);
@@ -258,7 +270,7 @@ do_curses(void *arg)
 	AC(intrflush(stdscr, FALSE));
 	(void)curs_set(0);
 	AC(erase());
-	for (;;) {
+	while (!quit) {
 		AZ(pthread_mutex_lock(&mtx));
 		update(period);
 		AZ(pthread_mutex_unlock(&mtx));
@@ -287,14 +299,14 @@ do_curses(void *arg)
 		case 'q':
 			AZ(raise(SIGINT));
 			AC(endwin());
-			return NULL;
+			return (NULL);
 		default:
 			AC(beep());
 			break;
 		}
 	}
-	return NULL;
-
+	AC(endwin());
+	return (NULL);
 }
 
 static void
@@ -311,12 +323,13 @@ dump(void)
 	}
 }
 
-static void
+//lint -sem(usage, r_no)
+static void __attribute__((__noreturn__))
 usage(int status)
 {
 	const char **opt;
 
-	fprintf(stderr, "Usage: %s <options>\n\n", progname);
+	fprintf(stderr, "Usage: %s <options>\n\n", vut->progname);
 	fprintf(stderr, "Options:\n");
 	for (opt = vopt_spec.vopt_usage; *opt != NULL; opt +=2)
 		fprintf(stderr, " %-25s %s\n", *opt, *(opt + 1));
@@ -329,12 +342,13 @@ main(int argc, char **argv)
 	int o, once = 0;
 	pthread_t thr;
 
-	VUT_Init(progname, argc, argv, &vopt_spec);
+	vut = VUT_InitProg(argc, argv, &vopt_spec);
+	AN(vut);
 
 	while ((o = getopt(argc, argv, vopt_spec.vopt_optstring)) != -1) {
 		switch (o) {
 		case '1':
-			AN(VUT_Arg('d', NULL));
+			AN(VUT_Arg(vut, 'd', NULL));
 			once = 1;
 			break;
 		case 'f':
@@ -353,7 +367,7 @@ main(int argc, char **argv)
 			}
 			break;
 		default:
-			if (!VUT_Arg(o, optarg))
+			if (!VUT_Arg(vut, o, optarg))
 				usage(1);
 		}
 	}
@@ -361,7 +375,9 @@ main(int argc, char **argv)
 	if (optind != argc)
 		usage(1);
 
-	VUT_Setup();
+	VUT_Signal(vut_sighandler);
+	VUT_Setup(vut);
+	ident = VSM_Dup(vut->vsm, "Arg", "-i");
 	if (!once) {
 		if (pthread_create(&thr, NULL, do_curses, NULL) != 0) {
 			fprintf(stderr, "pthread_create(): %s\n",
@@ -369,15 +385,15 @@ main(int argc, char **argv)
 			exit(1);
 		}
 	}
-	VUT.dispatch_f = &accumulate;
-	VUT.dispatch_priv = NULL;
-	VUT.sighup_f = sighup;
-	VUT_Main();
+	vut->dispatch_f = accumulate;
+	vut->dispatch_priv = NULL;
+	vut->sighup_f = sighup;
+	VUT_Main(vut);
 	end_of_file = 1;
 	if (once)
 		dump();
 	else
 		pthread_join(thr, NULL);
-	VUT_Fini();
+	VUT_Fini(&vut);
 	exit(0);
 }
