@@ -65,7 +65,6 @@ VSL_OPT_i
 VSL_OPT_I
 VSL_OPT_L
 VUT_OPT_n
-VUT_OPT_N
 TOP_OPT_p
 VUT_OPT_q
 VUT_OPT_r
@@ -73,4 +72,4 @@ VUT_OPT_t
 VSL_OPT_T
 VSL_OPT_x
 VSL_OPT_X
-VUT_OPT_V
+VUT_GLOBAL_OPT_V
