@@ -29,16 +29,26 @@
 
 /* VUT options */
 
-#define VUT_OPT_d \
-	VOPT("d", "[-d]", "Process old log entries and exit",		\
-	    "Process log records at the head of the log and exit."	\
-	)
-
-#define VUT_OPT_D							\
+#define VUT_GLOBAL_OPT_D						\
 	VOPT("D", "[-D]", "Daemonize",					\
 	    "Daemonize."						\
 	)
 
+#define VUT_GLOBAL_OPT_P						\
+	VOPT("P:", "[-P <file>]", "PID file",				\
+		"Write the process' PID to the specified file."		\
+	)
+
+#define VUT_GLOBAL_OPT_V						\
+	VOPT("V", "[-V]", "Version",					\
+	    "Print version information and exit."			\
+	)
+
+#define VUT_OPT_d							\
+	VOPT("d", "[-d]", "Process old log entries and exit",		\
+	    "Process log records at the head of the log and exit."	\
+	)
+
 #define VUT_OPT_g							\
 	VOPT("g:", "[-g <session|request|vxid|raw>]", "Grouping mode (default: vxid)",	\
 	    "The grouping of the log records. The default is to group"	\
@@ -57,20 +67,10 @@
 	)
 
 #define VUT_OPT_n							\
-	VOPT("n:", "[-n <name>]", "Varnish instance name",		\
-	    "Specify the name of the varnishd instance to get logs"	\
-	    " from. If -n is not specified, the host name is used."	\
-	)
-
-#define VUT_OPT_N							\
-	VOPT("N:", "[-N <filename>]", "VSM filename",			\
-	    "Specify the filename of a stale VSM instance. When using"	\
-	    " this option the abandonment checking is disabled."	\
-	)
-
-#define VUT_OPT_P							\
-	VOPT("P:", "[-P <file>]", "PID file",				\
-		"Write the process' PID to the specified file."		\
+	VOPT("n:", "[-n <dir>]", "varnishd working directory",		\
+	    "Specify the varnishd working directory (also known as"	\
+	    " instance name) to get logs from. If -n is not specified,"	\
+	    " the host name is used."					\
 	)
 
 #define VUT_OPT_q							\
@@ -95,8 +95,3 @@
 	    " indefinetely for the Varnish instance to appear. "	\
 	    " Defaults to 5 seconds."					\
 	)
-
-#define VUT_OPT_V							\
-	VOPT("V", "[-V]", "Version",					\
-	    "Print version information and exit."			\
-	)
