@@ -38,35 +38,39 @@
 
 #include "vas.h"
 
-static void __attribute__((__noreturn__))
-VAS_Fail_default(const char *func, const char *file, int line,
+vas_f *VAS_Fail_Func __attribute__((__noreturn__));
+
+void __attribute__((__noreturn__))
+VAS_Fail(const char *func, const char *file, int line,
     const char *cond, enum vas_e kind)
 {
 	int err = errno;
 
-	if (kind == VAS_MISSING) {
-		fprintf(stderr,
-		    "Missing error handling code in %s(), %s line %d:\n"
-		    "  Condition(%s) not true.\n",
-		    func, file, line, cond);
-	} else if (kind == VAS_INCOMPLETE) {
-		fprintf(stderr,
-		    "Incomplete code in %s(), %s line %d:\n",
-		    func, file, line);
-	} else if (kind == VAS_WRONG) {
-		fprintf(stderr,
-		    "Wrong turn in %s(), %s line %d: %s\n",
-		    func, file, line, cond);
+	if (VAS_Fail_Func != NULL) {
+		VAS_Fail_Func(func, file, line, cond, kind);
 	} else {
-		fprintf(stderr,
-		    "Assert error in %s(), %s line %d:\n"
-		    "  Condition(%s) not true.\n",
-		    func, file, line, cond);
+		if (kind == VAS_MISSING) {
+			fprintf(stderr,
+			    "Missing error handling code in %s(), %s line %d:\n"
+			    "  Condition(%s) not true.\n",
+			    func, file, line, cond);
+		} else if (kind == VAS_INCOMPLETE) {
+			fprintf(stderr,
+			    "Incomplete code in %s(), %s line %d:\n",
+			    func, file, line);
+		} else if (kind == VAS_WRONG) {
+			fprintf(stderr,
+			    "Wrong turn in %s(), %s line %d: %s\n",
+			    func, file, line, cond);
+		} else {
+			fprintf(stderr,
+			    "Assert error in %s(), %s line %d:\n"
+			    "  Condition(%s) not true.\n",
+			    func, file, line, cond);
+		}
+		if (err)
+			fprintf(stderr,
+			    "  errno = %d (%s)\n", err, strerror(err));
 	}
-	if (err)
-		fprintf(stderr,
-		    "  errno = %d (%s)\n", err, strerror(err));
 	abort();
 }
-
-vas_f *VAS_Fail __attribute__((__noreturn__)) = VAS_Fail_default;
