@@ -26,7 +26,7 @@
  * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
  * SUCH DAMAGE.
  *
- * const char **VAV_Parse(const char *s, int comment)
+ * const char **VAV_Parse(const char *s, int *argc, int flag)
  *	Parse a command like line into an argv[]
  *	Index zero contains NULL or an error message
  *	"double quotes" and backslash substitution is handled.
@@ -55,7 +55,7 @@ VAV_BackSlash(const char *s, char *res)
 
 	assert(*s == '\\');
 	r = c = 0;
-	switch(s[1]) {
+	switch (s[1]) {
 	case 'n':
 		c = '\n';
 		r = 2;
@@ -112,7 +112,7 @@ VAV_BackSlashDecode(const char *s, const char *e)
 	if (e == NULL)
 		e = strchr(s, '\0');
 	assert(e != NULL);
-	p = calloc((e - s) + 1, 1);
+	p = calloc(1, (e - s) + 1L);
 	if (p == NULL)
 		return (p);
 	for (r = p, q = s; q < e; ) {
@@ -142,7 +142,7 @@ VAV_Parse(const char *s, int *argc, int flag)
 	assert(s != NULL);
 	nargv = 1;
 	largv = 16;
-	argv = calloc(sizeof *argv, largv);
+	argv = calloc(largv, sizeof *argv);
 	if (argv == NULL)
 		return (NULL);
 
@@ -193,7 +193,7 @@ VAV_Parse(const char *s, int *argc, int flag)
 			assert(argv != NULL);
 		}
 		if (flag & ARGV_NOESC) {
-			argv[nargv] = malloc(1 + (s - p));
+			argv[nargv] = malloc(1L + (s - p));
 			assert(argv[nargv] != NULL);
 			memcpy(argv[nargv], p, s - p);
 			argv[nargv][s - p] = '\0';
@@ -242,10 +242,30 @@ Test(const char *str)
 	char **av;
 
 	printf("Test: <%V>\n", str);
-	av = VAV_Parse(str, 0);
+	av = VAV_Parse(str, NULL, 0);
 	VAV_Print(av);
 }
 
+#if defined __linux__
+int
+printf_v(FILE *stream, const struct printf_info *info,
+    const void *const *args)
+{
+	const char *v = *((char **)args[0]);
+	return (fprintf(stream, "%*s",
+	    info->left ? -info->width : info->width, v));
+}
+
+int
+printf_v_info(const struct printf_info *info, size_t n, int *argtypes,
+    int *size)
+{
+	if (n > 0)
+		argtypes[0] = PA_STRING;
+	return (1);
+}
+#endif
+
 int
 main(int argc, char **argv)
 {
@@ -254,7 +274,13 @@ main(int argc, char **argv)
 	(void)argc;
 	(void)argv;
 
+#if defined __FreeBSD__
 	register_printf_render_std("V");
+#elif defined __linux__
+	register_printf_specifier('V', printf_v, printf_v_info);
+#else
+#error Unsupported platform
+#endif
 
 	while (fgets(buf, sizeof buf, stdin))
 		Test(buf);
