@@ -30,16 +30,11 @@
 
 #include "config.h"
 
-#include <sys/types.h>
 #include <sys/socket.h>
 
-#include <netinet/in.h>
-
 #include <netdb.h>
-#include <stdio.h>
 #include <stdlib.h>
 #include <string.h>
-#include <unistd.h>
 
 #include "vas.h"
 #include "vsa.h"
