@@ -71,6 +71,7 @@ tokens = {
 
 	# Special
 	"T_TRUE":       None,
+	"VXID":         "vxid",
 }
 
 #######################################################################
@@ -154,7 +155,7 @@ def file_header(fo):
 	fo.write("""/*
  * NB:  This file is machine generated, DO NOT EDIT!
  *
- * Edit and run generate.py instead
+ * Edit and run lib/libvarnishapi/generate.py instead
  */
 """)
 
