@@ -27,6 +27,8 @@
  *
  */
 
+#ifndef __FLEXELINT__
+
 #include "config.h"
 
 #include <stdint.h>
@@ -48,7 +50,6 @@ static struct SLT tags[SLT__MAX] = {
 #define SLTM(name, flags, sdesc, ldesc)				\
 	[SLT_##name] = { SLT_##name, flags, #name, sdesc, ldesc },
 #include "tbl/vsl_tags.h"
-#undef SLTM
 };
 
 static int
@@ -112,3 +113,5 @@ main(int argc, char *argv[])
 
 	return (0);
 }
+
+#endif // __FLEXELINT__
