@@ -1,5 +1,5 @@
 #!/usr/bin/env python
-#-
+#
 # Copyright (c) 2006 Verdens Gang AS
 # Copyright (c) 2006-2015 Varnish Software AS
 # All rights reserved.
@@ -30,6 +30,11 @@
 # Generate various .c and .h files for the VCL compiler and the interfaces
 # for it.
 
+from __future__ import print_function
+
+import subprocess
+import os
+
 #######################################################################
 # These are our tokens
 
@@ -85,39 +90,39 @@ returns = (
 
 	('recv',
 		"C",
-		('synth', 'pass', 'pipe', 'hash', 'purge', 'vcl')
+		('fail', 'synth', 'pass', 'pipe', 'hash', 'purge', 'vcl')
 	),
 	('pipe',
 		"C",
-		('synth', 'pipe',)
+		('fail', 'synth', 'pipe',)
 	),
 	('pass',
 		"C",
-		('synth', 'restart', 'fetch',)
+		('fail', 'synth', 'restart', 'fetch',)
 	),
 	('hash',
 		"C",
-		('lookup',)
+		('fail', 'lookup',)
 	),
 	('purge',
 		"C",
-		('synth', 'restart',)
+		('fail', 'synth', 'restart',)
 	),
 	('miss',
 		"C",
-		('synth', 'restart', 'pass', 'fetch',)
+		('fail', 'synth', 'restart', 'pass', 'fetch',)
 	),
 	('hit',
 		"C",
-		('synth', 'restart', 'pass', 'miss', 'deliver',)
+		('fail', 'synth', 'restart', 'pass', 'miss', 'deliver',)
 	),
 	('deliver',
 		"C",
-		('synth', 'restart', 'deliver',)
+		('fail', 'synth', 'restart', 'deliver',)
 	),
 	('synth',
 		"C",
-		('restart', 'deliver',)
+		('fail', 'restart', 'deliver',)
 	),
 
 	###############################################################
@@ -125,15 +130,15 @@ returns = (
 
 	('backend_fetch',
 		"B",
-		('fetch', 'abandon')
+		('fail', 'fetch', 'abandon')
 	),
 	('backend_response',
 		"B",
-		('deliver', 'retry', 'abandon')
+		('fail', 'deliver', 'retry', 'abandon', 'pass')
 	),
 	('backend_error',
 		"B",
-		('deliver', 'retry', 'abandon')
+		('fail', 'deliver', 'retry', 'abandon')
 	),
 
 	###############################################################
@@ -160,7 +165,7 @@ returns = (
 sp_variables = [
 	('remote.ip',
 		'IP',
-		('client', 'backend'),
+		('both',),
 		(), """
 		The IP address of the other end of the TCP connection.
 		This can either be the clients IP, or the outgoing IP
@@ -169,7 +174,7 @@ sp_variables = [
 	),
 	('client.ip',
 		'IP',
-		('client', 'backend'),
+		('both',),
 		(), """
 		The client's IP address.
 		"""
@@ -185,14 +190,14 @@ sp_variables = [
 	),
 	('local.ip',
 		'IP',
-		('client', 'backend'),
+		('both',),
 		(), """
 		The IP address of the local end of the TCP connection.
 		"""
 	),
 	('server.ip',
 		'IP',
-		('client', 'backend'),
+		('both',),
 		(), """
 		The IP address of the socket on which the client
 		connection was received.
@@ -211,8 +216,7 @@ sp_variables = [
 		(), """
 		The identity of the server, as set by the -i
 		parameter.  If the -i parameter is not passed to varnishd,
-		server.identity will be set to the name of the instance, as
-		specified by the -n parameter.
+		server.identity will be set to the hostname of the machine.
 		"""
 	),
 	('req',
@@ -229,6 +233,13 @@ sp_variables = [
 		The request type (e.g. "GET", "HEAD").
 		"""
 	),
+	('req.hash',
+		'BLOB',
+		('hit', 'miss', 'pass', 'purge', 'deliver', ),
+		(), """
+		The hash key of this request.
+		"""
+	),
 	('req.url',
 		'STRING',
 		('client',),
@@ -257,6 +268,13 @@ sp_variables = [
 		A count of how many times this request has been restarted.
 		"""
 	),
+	('req.storage',
+		'STEVEDORE',
+		('recv',),
+		('recv',), """
+		The storage backend to use to save this request body.
+		"""
+	),
 	('req.esi_level',
 		'INT',
 		('client',),
@@ -268,6 +286,14 @@ sp_variables = [
 		'DURATION',
 		('client',),
 		('client',), """
+		Upper limit on the object age for cache lookups to return hit.
+
+		Usage of req.ttl should be replaced with a check on
+		obj.ttl in vcl_hit, returning miss when needed, but
+		this currently hits bug #1799, so an additional
+		workaround is required.
+
+		Deprecated and scheduled for removal with varnish release 7.
 		"""
 	),
 	('req.xid',
@@ -299,6 +325,13 @@ sp_variables = [
 		('client', ),
 		('client',), """
 		Set bereq.backend to this if we attempt to fetch.
+		When set to a director, reading this variable returns
+		an actual backend if the director has resolved immediately,
+		or the director otherwise.
+		When used in string context, returns the name of the director
+		or backend, respectively.
+		Note: backend_hint gets reset to the default backend by
+		restarts!
 		"""
 	),
 	('req.hash_ignore_busy',
@@ -380,6 +413,11 @@ sp_variables = [
 		('pipe', 'backend', ),
 		('pipe', 'backend', ), """
 		This is the backend or director we attempt to fetch from.
+		When set to a director, reading this variable returns
+		an actual backend if the director has resolved immediately,
+		or the director otherwise.
+		When used in string context, returns the name of the director
+		or backend, respectively.
 		"""
 	),
 	('bereq.body',
@@ -389,6 +427,13 @@ sp_variables = [
 		The request body.
 		"""
 	),
+	('bereq.hash',
+		'BLOB',
+		('pipe', 'backend', ),
+		(), """
+		The hash key of this request.
+		"""
+	),
 	('bereq.method',
 		'STRING',
 		('pipe', 'backend', ),
@@ -422,8 +467,8 @@ sp_variables = [
 		('backend', ),
 		(), """
 		Indicates whether this request is uncacheable due
-		to a pass in the client side or a hit on an existing
-		uncacheable object (aka hit-for-pass).
+		to a pass in the client side or a hit on an hit-for-pass
+		object.
 		"""
 	),
 	('bereq.connect_timeout',
@@ -449,6 +494,13 @@ sp_variables = [
 		backend.  Not available in pipe mode.
 		"""
 	),
+	('bereq.is_bgfetch',
+		'BOOL',
+		('backend', ),
+		(), """
+		True for background fetches.
+		"""
+	),
 	('beresp',
 		'HTTP',
 		('backend_response', 'backend_error'),
@@ -475,6 +527,9 @@ sp_variables = [
 		('backend_response', 'backend_error'),
 		('backend_response', 'backend_error'), """
 		The HTTP status code returned by the server.
+
+		Status codes >1000 can be set for vcl-internal
+		purposes and will be taken modulo 1000 on delivery.
 		"""
 	),
 	('beresp.reason',
@@ -545,7 +600,7 @@ sp_variables = [
 		Inherited from bereq.uncacheable, see there.
 
 		Setting this variable makes the object uncacheable, which may
-		get stored as a hit-for-pass object in the cache.
+		get stored as a hit-for-miss object in the cache.
 
 		Clearing the variable has no effect and will log the warning
 		"Ignoring attempt to reset beresp.uncacheable".
@@ -592,6 +647,7 @@ sp_variables = [
 		This is the backend we fetched from.  If bereq.backend
 		was set to a director, this will be the backend selected
 		by the director.
+		When used in string context, returns its name.
 		"""
 	),
 	('beresp.backend.name',
@@ -599,42 +655,51 @@ sp_variables = [
 		('backend_response', 'backend_error'),
 		(), """
 		Name of the backend this response was fetched from.
+		Same as beresp.backend.
 		"""
 	),
 	('beresp.backend.ip',
 		'IP',
-		('backend_response', 'backend_error'),
+		('backend_response',),
 		(), """
 		IP of the backend this response was fetched from.
 		"""
 	),
+	('beresp.storage',
+		'STEVEDORE',
+		('backend_response', 'backend_error'),
+		('backend_response', 'backend_error'), """
+		The storage backend to use to save this object.
+		"""
+	),
 	('beresp.storage_hint',
 		'STRING',
 		('backend_response', 'backend_error'),
 		('backend_response', 'backend_error'), """
-		Hint to Varnish that you want to save this object to a
-		particular storage backend.
+		Deprecated. Hint to Varnish that you want to
+		save this object to a particular storage backend.
+		Use beresp.storage instead.
 		"""
 	),
 	('obj.proto',
 		'STRING',
 		('hit',),
 		(), """
-		The HTTP protocol version used when the object was retrieved.
+		The HTTP protocol version stored with the object.
 		"""
 	),
 	('obj.status',
 		'INT',
 		('hit',),
 		(), """
-		The HTTP status code returned by the server.
+		The HTTP status code stored with the object.
 		"""
 	),
 	('obj.reason',
 		'STRING',
 		('hit',),
 		(), """
-		The HTTP status message returned by the server.
+		The HTTP reason phrase stored with the object.
 		"""
 	),
 	('obj.hits',
@@ -684,7 +749,8 @@ sp_variables = [
 		'BOOL',
 		('deliver',),
 		(), """
-		Whether the object is uncacheable (pass or hit-for-pass).
+		Whether the object is uncacheable (pass, hit-for-pass or
+		hit-for-miss).
 		"""
 	),
 	('resp',
@@ -716,6 +782,10 @@ sp_variables = [
 
 		Assigning a HTTP standardized code to resp.status will also
 		set resp.reason to the corresponding status message.
+
+		resp.status 200 will get changed into 304 by core code after
+		a return(deliver) from vcl_deliver for conditional requests
+		to cached content if validation succeeds.
 		"""
 	),
 	('resp.reason',
@@ -773,6 +843,7 @@ stv_variables = (
 
 vcltypes = {
 	'STRING_LIST':	"void*",
+	'SUB':			"void*",
 }
 
 fi = open(join(srcroot, "include/vrt.h"))
@@ -950,10 +1021,17 @@ def file_header(fo):
 	fo.write("""/*
  * NB:  This file is machine generated, DO NOT EDIT!
  *
- * Edit and run generate.py instead.
+ * Edit and run lib/libvcc/generate.py instead.
  */
+
 """)
 
+def lint_start(fo):
+	fo.write('/*lint -save -e525 -e539 */\n\n')
+
+def lint_end(fo):
+	fo.write('\n/*lint -restore */\n')
+
 #######################################################################
 
 polish_tokens(tokens)
@@ -994,9 +1072,9 @@ fo = open(join(buildroot, "include/tbl/vcl_returns.h"), "w")
 
 file_header(fo)
 
-fo.write("\n/*lint -save -e525 -e539 */\n")
+lint_start(fo)
 
-fo.write("\n#ifdef VCL_RET_MAC\n")
+fo.write("#ifdef VCL_RET_MAC\n")
 ll = sorted(returns)
 for i in sorted(rets.keys()):
 	fo.write("VCL_RET_MAC(%s, %s" % (i.lower(), i.upper()))
@@ -1005,7 +1083,8 @@ for i in sorted(rets.keys()):
 		if i in j[2]:
 			fo.write("%sVCL_MET_%s" % (s, j[0].upper()))
 			s = " |\n\t"
-	fo.write("\n)\n")
+	fo.write("\n)\n\n")
+fo.write("#undef VCL_RET_MAC\n")
 fo.write("#endif\n")
 
 fo.write("\n#ifdef VCL_MET_MAC\n")
@@ -1016,9 +1095,10 @@ for i in ll:
 	for j in sorted(i[2]):
 		fo.write("%s(1U << VCL_RET_%s)" % (p, j.upper()))
 		p = " |\n\t"
-	fo.write("\n))\n")
+	fo.write(")\n)\n\n")
+fo.write("#undef VCL_MET_MAC\n")
 fo.write("#endif\n")
-fo.write("\n/*lint -restore */\n")
+lint_end(fo)
 fo.close()
 
 #######################################################################
@@ -1046,7 +1126,7 @@ enum vcl_event_e {
 typedef int vcl_event_f(VRT_CTX, enum vcl_event_e);
 typedef int vcl_init_f(VRT_CTX);
 typedef void vcl_fini_f(VRT_CTX);
-typedef int vcl_func_f(VRT_CTX);
+typedef void vcl_func_f(VRT_CTX);
 """)
 
 
@@ -1121,7 +1201,6 @@ def restrict(fo, spec):
 			assert j in vcls
 			d[j] = True
 	p = ""
-	n = 0
 	l = list(d.keys())
 	l.sort()
 	w = 0
@@ -1149,8 +1228,6 @@ file_header(fo)
 fo.write("""
 #include "config.h"
 
-#include <stdio.h>
-
 #include "vcc_compile.h"
 
 const struct var vcc_vars[] = {
@@ -1221,9 +1298,6 @@ fo.write("""
 
 #include "config.h"
 
-#include <ctype.h>
-#include <stdio.h>
-
 #include "vcc_compile.h"
 """)
 
@@ -1248,11 +1322,12 @@ fo.close()
 ft = open(join(buildroot, "include/tbl/vcc_types.h"), "w")
 file_header(ft)
 
-ft.write("/*lint -save -e525 -e539 */\n")
+lint_start(ft)
 
 for vcltype in sorted(vcltypes.keys()):
 	ft.write("VCC_TYPE(" + vcltype + ")\n")
-ft.write("/*lint -restore */\n")
+ft.write("#undef VCC_TYPE\n")
+lint_end(ft)
 ft.close()
 
 #######################################################################
@@ -1260,6 +1335,7 @@ ft.close()
 fo = open(join(buildroot, "include/tbl/vrt_stv_var.h"), "w")
 
 file_header(fo)
+lint_start(fo)
 
 for i in stv_variables:
 	ct = vcltypes[i[1]]
@@ -1267,6 +1343,8 @@ for i in stv_variables:
 	fo.write(ct + ",\t" + i[2] + ")")
 	fo.write("\n")
 
+fo.write("#undef VRTSTVVAR\n")
+lint_end(fo)
 fo.close()
 
 #######################################################################
@@ -1324,3 +1402,43 @@ for i in stv_variables:
 		fp_vclvar.write("\t%s\n" % j.strip())
 
 fp_vclvar.close()
+
+#######################################################################
+
+if os.path.isdir(os.path.join(srcroot, ".git")):
+	v = subprocess.check_output([
+		"git --git-dir=" + os.path.join(srcroot, ".git") +
+		" show -s --pretty=format:%h"
+	], shell=True, universal_newlines=True)
+	v = v.strip()
+	b = subprocess.check_output([
+		"git --git-dir=" + os.path.join(srcroot, ".git") +
+		" rev-parse --abbrev-ref HEAD"
+	], shell=True, universal_newlines=True)
+	b = b.strip()
+else:
+	b = "NOGIT"
+	v = "NOGIT"
+
+vcsfn = os.path.join(srcroot, "include", "vcs_version.h")
+
+try:
+	i = open(vcsfn).readline()
+except IOError:
+	i = ""
+
+if i != "/* " + v + " */":
+	fo = open(vcsfn, "w")
+	file_header(fo)
+	fo.write('#define VCS_Version "%s"\n' % v)
+	fo.close()
+
+	for i in open(os.path.join(buildroot, "Makefile")):
+		if i[:14] == "PACKAGE_STRING":
+			break
+	i = i.split("=")[1].strip()
+
+	fo = open(os.path.join(srcroot, "include", "vmod_abi.h"), "w")
+	file_header(fo)
+	fo.write('#define VMOD_ABI_Version "%s %s"\n' % (i, v))
+	fo.close()
