@@ -29,14 +29,11 @@
 
 #include "config.h"
 
-#include <sys/types.h>
 #include <sys/socket.h>
 
 #include <netinet/in.h>
 
 #include <netdb.h>
-#include <stddef.h>
-#include <stdio.h>
 #include <stdlib.h>
 #include <string.h>
 
@@ -197,8 +194,8 @@ vcc_acl_try_getaddrinfo(struct vcc *tl, struct acl_e *ae)
 	}
 
 	i4 = i6 = 0;
-	for(res = res0; res != NULL; res = res->ai_next) {
-		switch(res->ai_family) {
+	for (res = res0; res != NULL; res = res->ai_next) {
+		switch (res->ai_family) {
 		case PF_INET:
 			assert(PF_INET < 256);
 			sin4 = (void*)res->ai_addr;
@@ -345,31 +342,36 @@ vcc_acl_entry(struct vcc *tl)
  */
 
 static void
-vcc_acl_emit(struct vcc *tl, const char *acln, int anon)
+vcc_acl_emit(struct vcc *tl, const char *name, const char *rname, int anon)
 {
 	struct acl_e *ae;
 	int depth, l, m, i;
 	unsigned at[VRT_ACL_MAXADDR + 1];
 	struct token *t;
 	struct inifin *ifp;
+	struct vsb *func;
+
+	func = VSB_new_auto();
+	AN(func);
+	VSB_printf(func, "match_acl_%s_", anon ? "anon" : "named");
+	VCC_PrintCName(func, name, NULL);
+	AZ(VSB_finish(func));
 
 	Fh(tl, 0, "\nstatic int __match_proto__(acl_match_f)\n");
-	Fh(tl, 0,
-	    "match_acl_%s_%s(VRT_CTX, const VCL_IP p)\n",
-	    anon ? "anon" : "named", acln);
+	Fh(tl, 0, "%s(VRT_CTX, const VCL_IP p)\n", VSB_data(func));
 	Fh(tl, 0, "{\n");
 	Fh(tl, 0, "\tconst unsigned char *a;\n");
 	Fh(tl, 0, "\tint fam;\n");
 	Fh(tl, 0, "\n");
 	Fh(tl, 0, "\tfam = VRT_VSA_GetPtr(p, &a);\n");
 	Fh(tl, 0, "\tif (fam < 0) {\n");
-	Fh(tl, 0, "\t\tVRT_acl_log(ctx, \"NO_FAM %s\");\n", acln);
+	Fh(tl, 0, "\t\tVRT_acl_log(ctx, \"NO_FAM %s\");\n", name);
 	Fh(tl, 0, "\t\treturn(0);\n");
 	Fh(tl, 0, "\t}\n\n");
 	if (!tl->err_unref && !anon) {
 		ifp = New_IniFin(tl);
 		VSB_printf(ifp->ini,
-			"\tif (0) match_acl_named_%s(0, 0);\n", acln);
+			"\tif (0) %s(0, 0);\n", VSB_data(func));
 	}
 	depth = -1;
 	at[0] = 256;
@@ -417,7 +419,7 @@ vcc_acl_emit(struct vcc *tl, const char *acln, int anon)
 
 		if (!anon) {
 			Fh(tl, 0, "\t%*sVRT_acl_log(ctx, \"%sMATCH %s \" ",
-			    -i, "", ae->not ? "NEG_" : "", acln);
+			    -i, "", ae->not ? "NEG_" : "", name);
 			t = ae->t_addr;
 			do {
 				if (t->tok == CSTR) {
@@ -443,53 +445,52 @@ vcc_acl_emit(struct vcc *tl, const char *acln, int anon)
 
 	/* Deny by default */
 	if (!anon)
-		Fh(tl, 0, "\tVRT_acl_log(ctx, \"NO_MATCH %s\");\n", acln);
+		Fh(tl, 0, "\tVRT_acl_log(ctx, \"NO_MATCH %s\");\n", name);
 	Fh(tl, 0, "\treturn (0);\n}\n");
 
-	if (anon)
-		return;
-
-	/* Emit the struct that will be referenced */
-	Fh(tl, 0, "\nconst struct vrt_acl vrt_acl_named_%s = {\n", acln);
-	Fh(tl, 0, "\t.magic = VRT_ACL_MAGIC,\n");
-	Fh(tl, 0, "\t.match = &match_acl_named_%s,\n", acln);
-	Fh(tl, 0, "};\n\n");
+	if (!anon) {
+		/* Emit the struct that will be referenced */
+		Fh(tl, 0, "\nconst struct vrt_acl %s[] = {{\n", rname);
+		Fh(tl, 0, "\t.magic = VRT_ACL_MAGIC,\n");
+		Fh(tl, 0, "\t.match = &%s,\n", VSB_data(func));
+		Fh(tl, 0, "}};\n\n");
+	}
+	VSB_destroy(&func);
 }
 
 void
-vcc_Acl_Hack(struct vcc *tl, char *b)
+vcc_Acl_Hack(struct vcc *tl, char *b, size_t bl)
 {
-	char acln[32];
+	char name[32];
 	unsigned tcond;
 
 	VTAILQ_INIT(&tl->acl);
 	tcond = tl->t->tok;
 	vcc_NextToken(tl);
-	bprintf(acln, "%u", tl->unique++);
+	bprintf(name, "%u", tl->unique++);
 	vcc_acl_entry(tl);
-	vcc_acl_emit(tl, acln, 1);
-	sprintf(b, "%smatch_acl_anon_%s(ctx, \v1)",
-	    (tcond == T_NEQ ? "!" : ""), acln);
+	vcc_acl_emit(tl, name, name, 1);
+	assert(snprintf(b, bl - 1, "%smatch_acl_anon_%s(ctx, \v1)",
+	    (tcond == T_NEQ ? "!" : ""), name) < bl - 1);
 }
 
 void
 vcc_ParseAcl(struct vcc *tl)
 {
 	struct token *an;
-	char *acln;
+	struct symbol *sym;
 
 	vcc_NextToken(tl);
 	VTAILQ_INIT(&tl->acl);
 
-	vcc_ExpectCid(tl, "ACL");
+	vcc_ExpectVid(tl, "ACL");
 	ERRCHK(tl);
 	an = tl->t;
 	vcc_NextToken(tl);
 
-	acln = TlDupTok(tl, an);
-
-	(void)VCC_HandleSymbol(tl, an, ACL, "&vrt_acl_named_%s", acln);
+	sym = VCC_HandleSymbol(tl, an, ACL, ACL_SYMBOL_PREFIX);
 	ERRCHK(tl);
+	AN(sym);
 
 	SkipToken(tl, '{');
 
@@ -500,5 +501,5 @@ vcc_ParseAcl(struct vcc *tl)
 	}
 	SkipToken(tl, '}');
 
-	vcc_acl_emit(tl, acln, 0);
+	vcc_acl_emit(tl, sym->name, sym->rname, 0);
 }
