@@ -42,13 +42,14 @@
 static void
 parse_call(struct vcc *tl)
 {
+	struct symbol *sym;
 
 	vcc_NextToken(tl);
 	ExpectErr(tl, ID);
 	vcc_AddCall(tl, tl->t);
-	vcc_AddRef(tl, tl->t, SYM_SUB);
-	Fb(tl, 1, "if (VGC_function_%.*s(ctx))\n", PF(tl->t));
-	Fb(tl, 1, "\treturn (1);\n");
+	sym = vcc_AddRef(tl, tl->t, SYM_SUB);
+	VCC_GlobalSymbol(sym, SUB, "VGC_function");
+	Fb(tl, 1, "%s(ctx);\n", sym->rname);
 	vcc_NextToken(tl);
 }
 
@@ -191,21 +192,27 @@ parse_hash_data(struct vcc *tl)
 
 /*--------------------------------------------------------------------*/
 
+static void
+parse_return_pass(struct vcc *tl)
+{
+
+	ExpectErr(tl, '(');
+	vcc_NextToken(tl);
+	Fb(tl, 1, "VRT_hit_for_pass(ctx,\n");
+	tl->indent += INDENT;
+	vcc_Expr(tl, DURATION);
+	ERRCHK(tl);
+	ExpectErr(tl, ')');
+	vcc_NextToken(tl);
+	Fb(tl, 1, ");\n");
+	tl->indent -= INDENT;
+}
+/*--------------------------------------------------------------------*/
+
 static void
 parse_return_synth(struct vcc *tl)
 {
 
-	AN(vcc_IdIs(tl->t, "synth"));
-	vcc_NextToken(tl);
-	if (tl->t->tok == ')') {
-		VSB_printf(tl->sb,
-		    "Syntax has changed, use:\n"
-		    "\treturn(synth(999));\n"
-		    "or\n"
-		    "\treturn(synth(999, \"Response text\"));\n");
-		vcc_ErrWhere(tl, tl->t);
-		return;
-	}
 	ExpectErr(tl, '(');
 	vcc_NextToken(tl);
 	Fb(tl, 1, "VRT_synth(ctx,\n");
@@ -224,11 +231,6 @@ parse_return_synth(struct vcc *tl)
 	ExpectErr(tl, ')');
 	vcc_NextToken(tl);
 	Fb(tl, 1, ");\n");
-	Fb(tl, 1, "VRT_handling(ctx, VCL_RET_SYNTH);\n");
-	Fb(tl, 1, "return (1);\n");
-	vcc_ProcAction(tl->curproc, VCL_RET_SYNTH, tl->t);
-	ExpectErr(tl, ')');
-	vcc_NextToken(tl);
 }
 
 /*--------------------------------------------------------------------*/
@@ -240,8 +242,6 @@ parse_return_vcl(struct vcc *tl)
 	struct inifin *p;
 	char buf[1024];
 
-	AN(vcc_IdIs(tl->t, "vcl"));
-	vcc_NextToken(tl);
 	ExpectErr(tl, '(');
 	vcc_NextToken(tl);
 	ExpectErr(tl, ID);
@@ -253,27 +253,25 @@ parse_return_vcl(struct vcc *tl)
 		return;
 	}
 	if (sym->eval_priv == NULL) {
-		VSB_printf(tl->fi, "%s VCL %.*s */\n",
-		    VCC_INFO_PREFIX, PF(tl->t));
+		VSB_printf(tl->fi, "%s VCL %s */\n", VCC_INFO_PREFIX,
+		    sym->name);
 
 		bprintf(buf, "vgc_vcl_%u", tl->unique++);
 		sym->eval_priv = strdup(buf);
 		AN(sym->eval_priv);
 
 		Fh(tl, 0, "static VCL_VCL %s;", buf);
-		Fh(tl, 0, "\t/* VCL %.*s */\n", PF(tl->t));
+		Fh(tl, 0, "\t/* VCL %s */\n", sym->name);
 
 		p = New_IniFin(tl);
 		AN(p);
-		VSB_printf(p->ini, "\t%s = VRT_vcl_lookup(\"%.*s\");",
-		    buf, PF(tl->t));
+		VSB_printf(p->ini, "\t%s = VRT_vcl_get(ctx, \"%s\");",
+		    buf, sym->name);
+		VSB_printf(p->fin, "\tVRT_vcl_rel(ctx, %s);",
+		    buf);
 	}
-	Fb(tl, 1, "VRT_vcl_select(ctx, %s);\t/* %.*s */\n",
-	    (const char*)sym->eval_priv, PF(tl->t));
-	Fb(tl, 1, "VRT_handling(ctx, VCL_RET_VCL);\n");
-	Fb(tl, 1, "return (1);\n");
-	vcc_NextToken(tl);
-	ExpectErr(tl, ')');
+	Fb(tl, 1, "VRT_vcl_select(ctx, %s);\t/* %s */\n",
+	    (const char*)sym->eval_priv, sym->name);
 	vcc_NextToken(tl);
 	ExpectErr(tl, ')');
 	vcc_NextToken(tl);
@@ -284,39 +282,55 @@ parse_return_vcl(struct vcc *tl)
 static void
 parse_return(struct vcc *tl)
 {
-	int retval = 0;
+	unsigned hand;
+	const char *h;
 
 	vcc_NextToken(tl);
+	if (tl->t->tok == ';' && tl->fb == tl->fc) {
+		/* fb == fc means we're in a subroutine */
+		Fb(tl, 1, "return;\n");
+		return;
+	}
 	ExpectErr(tl, '(');
 	vcc_NextToken(tl);
 	ExpectErr(tl, ID);
 
-	if (vcc_IdIs(tl->t, "synth")) {
-		parse_return_synth(tl);
-		return;
-	}
-	if (vcc_IdIs(tl->t, "vcl")) {
-		parse_return_vcl(tl);
-		return;
-	}
-
-#define VCL_RET_MAC(l, U, B)						\
-	do {								\
-		if (vcc_IdIs(tl->t, #l)) {				\
-			Fb(tl, 1, "VRT_handling(ctx, VCL_RET_" #U ");\n"); \
-			Fb(tl, 1, "return (1);\n");			\
-			vcc_ProcAction(tl->curproc, VCL_RET_##U, tl->t);\
-			retval = 1;					\
-		}							\
-	} while (0);
+	hand = VCL_RET_MAX;
+	h = NULL;
+#define VCL_RET_MAC(l, U, B)				\
+		if (vcc_IdIs(tl->t, #l)) {		\
+			hand = VCL_RET_ ## U;		\
+			h = #U;				\
+		}
 #include "tbl/vcl_returns.h"
-#undef VCL_RET_MAC
-	if (!retval) {
+	if (h == NULL) {
 		VSB_printf(tl->sb, "Expected return action name.\n");
 		vcc_ErrWhere(tl, tl->t);
 		ERRCHK(tl);
 	}
+	assert(hand < VCL_RET_MAX);
+
+	vcc_ProcAction(tl->curproc, hand, tl->t);
 	vcc_NextToken(tl);
+	if (tl->t->tok == '(') {
+		if (hand == VCL_RET_SYNTH)
+			parse_return_synth(tl);
+		else if (hand == VCL_RET_VCL)
+			parse_return_vcl(tl);
+		else if (hand == VCL_RET_PASS)
+			parse_return_pass(tl);
+		else {
+			VSB_printf(tl->sb, "Arguments not allowed.\n");
+			vcc_ErrWhere(tl, tl->t);
+		}
+	} else {
+		if (hand == VCL_RET_SYNTH || hand == VCL_RET_VCL) {
+			VSB_printf(tl->sb, "Missing argument.\n");
+			vcc_ErrWhere(tl, tl->t);
+		}
+	}
+	ERRCHK(tl);
+	Fb(tl, 1, "VRT_handling(ctx, VCL_RET_%s);\n", h);
 	ExpectErr(tl, ')');
 	vcc_NextToken(tl);
 }
@@ -373,7 +387,7 @@ vcc_ParseAction(struct vcc *tl)
 
 	at = tl->t;
 	assert(at->tok == ID);
-	for(atp = action_table; atp->name != NULL; atp++) {
+	for (atp = action_table; atp->name != NULL; atp++) {
 		if (vcc_IdIs(at, atp->name)) {
 			if (atp->bitmask != 0)
 				vcc_AddUses(tl, at, atp->bitmask,
