@@ -52,10 +52,8 @@
 
 #include "config.h"
 
-#include <ctype.h>
 #include <errno.h>
 #include <stdarg.h>
-#include <stdio.h>
 #include <stdlib.h>
 #include <string.h>
 
@@ -68,7 +66,6 @@ struct method method_tab[] = {
 	{ "none", 0U, 0},
 #define VCL_MET_MAC(l,U,t,b)	{ "vcl_"#l, b, VCL_MET_##U },
 #include "tbl/vcl_returns.h"
-#undef VCL_MET_MAC
 	{ NULL, 0U, 0}
 };
 
@@ -80,7 +77,7 @@ TlAlloc(struct vcc *tl, unsigned len)
 	void *p;
 
 	(void)tl;
-	p = calloc(len, 1);
+	p = calloc(1, len);
 	assert(p != NULL);
 	return (p);
 }
@@ -96,20 +93,6 @@ TlDup(struct vcc *tl, const char *s)
 	return (p);
 }
 
-char *
-TlDupTok(struct vcc *tl, const struct token *tok)
-{
-	char *p;
-	int i;
-
-	i = tok->e - tok->b;
-	p = TlAlloc(tl, i + 1);
-	AN(p);
-	memcpy(p, tok->b, i);
-	p[i] = '\0';
-	return (p);
-}
-
 /*--------------------------------------------------------------------*/
 
 struct inifin *
@@ -311,8 +294,7 @@ EmitInitFini(const struct vcc *tl)
 			has_event = 1;
 	}
 
-	Fc(tl, 0, "\t(void)VGC_function_vcl_init(ctx);\n");
-	Fc(tl, 0, "\treturn (*ctx->handling == VCL_RET_OK ? 0: -1);\n");
+	Fc(tl, 0, "\treturn(0);\n");
 	Fc(tl, 0, "}\n");
 
 	/*
@@ -320,8 +302,6 @@ EmitInitFini(const struct vcc *tl)
 	 */
 	Fc(tl, 0, "\nstatic int\nVGC_Discard(VRT_CTX)\n{\n\n");
 
-	Fc(tl, 0, "\t(void)VGC_function_vcl_fini(ctx);\n\n");
-
 	Fc(tl, 0, "\tswitch (vgc_inistep) {\n\n");
 	VTAILQ_FOREACH_REVERSE(p, &tl->inifin, inifinhead, list) {
 		AZ(VSB_finish(p->fin));
@@ -335,7 +315,7 @@ EmitInitFini(const struct vcc *tl)
 		}
 		VSB_destroy(&p->fin);
 	}
-	Fc(tl, 0, "\t}\n\n");
+	Fc(tl, 0, "\t}\n");
 
 	Fc(tl, 0, "\treturn (0);\n");
 	Fc(tl, 0, "}\n");
@@ -371,8 +351,10 @@ EmitInitFini(const struct vcc *tl)
 		VTAILQ_FOREACH_REVERSE(p, &tl->inifin, inifinhead, list) {
 			if (VSB_len(p->event)) {
 				Fc(tl, 0, "\t/* %u */\n", p->n);
-				Fc(tl, 0, "\tif (vgc_warmupstep >= %u &&\n", p->n);
-				Fc(tl, 0, "\t    %s != 0)\n", VSB_data(p->event));
+				Fc(tl, 0,
+				    "\tif (vgc_warmupstep >= %u &&\n", p->n);
+				Fc(tl, 0,
+				    "\t    %s != 0)\n", VSB_data(p->event));
 				Fc(tl, 0, "\t\tretval = 1;\n\n");
 			}
 			VSB_destroy(&p->event);
@@ -415,7 +397,7 @@ EmitStruct(const struct vcc *tl)
 	Fc(tl, 0, "\t.event_vcl = VGC_Event,\n");
 	Fc(tl, 0, "\t.default_director = &%s,\n", tl->default_director);
 	if (tl->default_probe != NULL)
-		Fc(tl, 0, "\t.default_probe = &%s,\n", tl->default_probe);
+		Fc(tl, 0, "\t.default_probe = %s,\n", tl->default_probe);
 	Fc(tl, 0, "\t.ref = VGC_ref,\n");
 	Fc(tl, 0, "\t.nref = VGC_NREFS,\n");
 	Fc(tl, 0, "\t.nsrc = VGC_NSRCS,\n");
@@ -424,7 +406,6 @@ EmitStruct(const struct vcc *tl)
 #define VCL_MET_MAC(l,u,t,b) \
 	Fc(tl, 0, "\t." #l "_func = VGC_function_vcl_" #l ",\n");
 #include "tbl/vcl_returns.h"
-#undef VCL_MET_MAC
 	Fc(tl, 0, "};\n");
 }
 
@@ -437,7 +418,7 @@ vcc_new_source(const char *b, const char *e, const char *name)
 
 	if (e == NULL)
 		e = strchr(b, '\0');
-	sp = calloc(sizeof *sp, 1);
+	sp = calloc(1, sizeof *sp);
 	assert(sp != NULL);
 	sp->name = strdup(name);
 	AN(sp->name);
@@ -556,6 +537,7 @@ vcc_CompileSource(struct vcc *tl, struct source *sp)
 	struct symbol *sym;
 	const struct var *v;
 	struct vsb *vsb;
+	struct inifin *ifp;
 	int i;
 
 	vcc_Expr_Init(tl);
@@ -572,9 +554,9 @@ vcc_CompileSource(struct vcc *tl, struct source *sp)
 		sym->fmt = v->fmt;
 		sym->eval = vcc_Eval_Var;
 		sym->r_methods = v->r_methods;
-		sym->rname = v->rname;
 		sym->w_methods = v->w_methods;
 		sym->lname = v->lname;
+		REPLACE(sym->rname, v->rname);
 	}
 
 	Fh(tl, 0, "/* ---===### VCC generated .h code ###===---*/\n");
@@ -623,7 +605,7 @@ vcc_CompileSource(struct vcc *tl, struct source *sp)
 	}
 
 	/* Configure the default director */
-	vcc_AddRef(tl, tl->t_default_director, SYM_BACKEND);
+	(void)vcc_AddRef(tl, tl->t_default_director, SYM_BACKEND);
 
 	/* Check for orphans */
 	if (vcc_CheckReferences(tl))
@@ -637,14 +619,19 @@ vcc_CompileSource(struct vcc *tl, struct source *sp)
 	if (vcc_CheckUses(tl) || tl->err)
 		return (NULL);
 
+	/* Tie vcl_init/fini in */
+	ifp = New_IniFin(tl);
+	VSB_printf(ifp->ini, "\tVGC_function_vcl_init(ctx);");
+	VSB_printf(ifp->fin, "\t\tVGC_function_vcl_fini(ctx);");
+
 	/* Emit method functions */
 	Fh(tl, 1, "\n");
 	for (i = 1; i < VCL_MET_MAX; i++) {
 		Fh(tl, 1,
-		    "int __match_proto__(vcl_func_f) "
+		    "void __match_proto__(vcl_func_f) "
 		    "VGC_function_%s(VRT_CTX);\n",
 		    method_tab[i].name);
-		Fc(tl, 1, "\nint __match_proto__(vcl_func_f)\n");
+		Fc(tl, 1, "\nvoid __match_proto__(vcl_func_f)\n");
 		Fc(tl, 1,
 		    "VGC_function_%s(VRT_CTX)\n",
 		    method_tab[i].name);
@@ -655,11 +642,9 @@ vcc_CompileSource(struct vcc *tl, struct source *sp)
 		 * in members called from vcl_init, so set OK up front
 		 * and return with whatever was set last.
 		 */
-		if (method_tab[i].bitval == VCL_MET_INIT)
-			Fc(tl, 1, "  VRT_handling(ctx, VCL_RET_OK);\n");
 		Fc(tl, 1, "%s", VSB_data(tl->fm[i]));
 		if (method_tab[i].bitval == VCL_MET_INIT)
-			Fc(tl, 1, "  return (1);\n");
+			Fc(tl, 1, "  return;\n");
 		Fc(tl, 1, "}\n");
 	}
 
@@ -667,6 +652,8 @@ vcc_CompileSource(struct vcc *tl, struct source *sp)
 
 	EmitStruct(tl);
 
+	VCC_XrefTable(tl);
+
 	/* Combine it all */
 
 	vsb = VSB_new_auto();
