@@ -28,8 +28,6 @@
  *
  */
 
-#include <sys/types.h>
-
 #include <stdio.h>
 #include <stdint.h>
 
@@ -51,9 +49,6 @@ struct vsb;
 struct token;
 struct sockaddr_storage;
 
-#define isident1(c) (isalpha(c))
-#define isident(c) (isalpha(c) || isdigit(c) || (c) == '_' || (c) == '-')
-#define isvar(c) (isident(c) || (c) == '.')
 unsigned vcl_fixed_token(const char *p, const char **q);
 extern const char * const vcl_tnames[256];
 void vcl_output_lang_h(struct vsb *sb);
@@ -101,13 +96,11 @@ struct type {
 
 #define VCC_TYPE(foo)		extern const struct type foo[1];
 #include "tbl/vcc_types.h"
-#undef VCC_TYPE
 
 
 enum symkind {
 #define VCC_SYMB(uu, ll)	SYM_##uu,
 #include "tbl/symbol_kind.h"
-#undef VCC_SYMB
 };
 
 typedef void sym_expr_t(struct vcc *tl, struct expr **,
@@ -121,6 +114,7 @@ struct symbol {
 	VTAILQ_ENTRY(symbol)		list;
 	VTAILQ_HEAD(,symbol)		children;
 
+	struct symbol			*parent;
 	const char			*vmod;
 
 	char				*name;
@@ -143,7 +137,7 @@ struct symbol {
 	const char			*extra;
 
 	/* SYM_VAR */
-	const char			*rname;
+	char				*rname;
 	unsigned			r_methods;
 	const char			*lname;
 	unsigned			w_methods;
@@ -190,7 +184,7 @@ struct vcc {
 	int			hindent;
 	unsigned		cnt;
 
-	struct vsb		*fi;		/* VCC info to MGR */
+	struct vsb		*fi;		/* VCC info to MGT */
 	struct vsb		*fc;		/* C-code */
 	struct vsb		*fh;		/* H-code (before C-code) */
 	struct vsb		*fb;		/* Body of current sub
@@ -234,13 +228,12 @@ struct method {
 /* vcc_acl.c */
 
 void vcc_ParseAcl(struct vcc *tl);
-void vcc_Acl_Hack(struct vcc *tl, char *b);
+void vcc_Acl_Hack(struct vcc *tl, char *b, size_t bl);
 
 /* vcc_action.c */
 int vcc_ParseAction(struct vcc *tl);
 
 /* vcc_backend.c */
-#define MAX_BACKEND_NAME	64
 struct fld_spec;
 
 void vcc_ParseProbe(struct vcc *tl);
@@ -270,7 +263,6 @@ void EncToken(struct vsb *sb, const struct token *t);
 int IsMethod(const struct token *t);
 void *TlAlloc(struct vcc *tl, unsigned len);
 char *TlDup(struct vcc *tl, const char *s);
-char *TlDupTok(struct vcc *tl, const struct token *tok);
 
 /* vcc_expr.c */
 double vcc_DoubleVal(struct vcc *tl);
@@ -285,8 +277,9 @@ sym_expr_t vcc_Eval_SymFunc;
 void vcc_Eval_Func(struct vcc *tl, const char *spec,
     const char *extra, const struct symbol *sym);
 enum symkind VCC_HandleKind(vcc_type_t fmt);
+void VCC_GlobalSymbol(struct symbol *, vcc_type_t fmt, const char *pfx);
 struct symbol *VCC_HandleSymbol(struct vcc *, const struct token *,
-    vcc_type_t fmt, const char *str, ...);
+    vcc_type_t fmt, const char *pfx);
 
 /* vcc_obj.c */
 extern const struct var vcc_vars[];
@@ -305,6 +298,7 @@ void Resolve_Sockaddr(struct vcc *tl, const char *host, const char *defport,
 void vcc_stevedore(struct vcc *vcc, const char *stv_name);
 
 /* vcc_symb.c */
+void VCC_PrintCName(struct vsb *vsb, const char *b, const char *e);
 struct symbol *VCC_Symbol(struct vcc *, struct symbol *,
     const char *, const char *, enum symkind, int);
 #define VCC_SymbolTok(vcc, sym, tok, kind, create) \
@@ -322,7 +316,7 @@ void vcc_ErrWhere2(struct vcc *, const struct token *, const struct token *);
 
 void vcc__Expect(struct vcc *tl, unsigned tok, unsigned line);
 int vcc_IdIs(const struct token *t, const char *p);
-void vcc_ExpectCid(struct vcc *tl, const char *what);
+void vcc_ExpectVid(struct vcc *tl, const char *what);
 void vcc_Lexer(struct vcc *tl, struct source *sp);
 void vcc_NextToken(struct vcc *tl);
 void vcc__ErrInternal(struct vcc *tl, const char *func,
@@ -343,9 +337,12 @@ void vcc_ParseImport(struct vcc *tl);
 void vcc_ParseNew(struct vcc *tl);
 
 /* vcc_xref.c */
-int vcc_AddDef(struct vcc *tl, const struct token *t, enum symkind type);
-void vcc_AddRef(struct vcc *tl, const struct token *t, enum symkind type);
+struct symbol *vcc_AddDef(struct vcc *tl, const struct token *t,
+    enum symkind type);
+struct symbol *vcc_AddRef(struct vcc *tl, const struct token *t,
+    enum symkind type);
 int vcc_CheckReferences(struct vcc *tl);
+void VCC_XrefTable(struct vcc *);
 
 void vcc_AddCall(struct vcc *tl, struct token *t);
 struct proc *vcc_AddProc(struct vcc *tl, struct token *t);
@@ -362,3 +359,5 @@ int vcc_CheckUses(struct vcc *tl);
     do { vcc__Expect(a, b, __LINE__); ERRCHK(a);} while (0)
 #define SkipToken(a, b) \
     do { vcc__Expect(a, b, __LINE__); ERRCHK(a); vcc_NextToken(a); } while (0)
+
+#define ACL_SYMBOL_PREFIX "vrt_acl_named"
