@@ -33,7 +33,6 @@
 
 #include <math.h>
 #include <stdarg.h>
-#include <stdio.h>
 #include <stdlib.h>
 #include <string.h>
 
@@ -301,7 +300,7 @@ vcc_expr_edit(vcc_type_t fmt, const char *p, struct expr *e1,
 			continue;
 		}
 		assert(*p == '\v');
-		switch(*++p) {
+		switch (*++p) {
 		case '+': VSB_cat(e->vsb, "\v+"); break;
 		case '-': VSB_cat(e->vsb, "\v-"); break;
 		case '1':
@@ -356,7 +355,7 @@ vcc_expr_fmt(struct vsb *d, int ind, const struct expr *e1)
 			continue;
 		}
 		p++;
-		switch(*p) {
+		switch (*p) {
 		case '+': ind += 2; break;
 		case '-': ind -= 2; break;
 		default:
@@ -376,10 +375,11 @@ vcc_expr_tostring(struct vcc *tl, struct expr **e, vcc_type_t fmt)
 	uint8_t	constant = EXPR_VAR;
 
 	CHECK_OBJ_NOTNULL(*e, EXPR_MAGIC);
-	AN(fmt == STRING || fmt == STRING_LIST);
-	AZ(fmt == (*e)->fmt);
+	assert(fmt == STRING || fmt == STRING_LIST);
+	assert(fmt != (*e)->fmt);
 
-	if ((*e)->fmt == STRING || ((*e)->fmt == STRING_LIST && vcc_isconst(*e))) {
+	if ((*e)->fmt == STRING ||
+	    ((*e)->fmt == STRING_LIST && vcc_isconst(*e))) {
 		(*e)->fmt = fmt;
 		return;
 	}
@@ -432,7 +432,7 @@ vcc_Eval_Regsub(struct vcc *tl, struct expr **e, const struct symbol *sym,
 	vcc_NextToken(tl);
 
 	bprintf(buf, "VRT_regsub(ctx, %d,\v+\n\v1,\n%s", all, p);
-	*e = vcc_expr_edit(STRING, buf, e2, *e);
+	*e = vcc_expr_edit(STRING, buf, e2, NULL);
 
 	SkipToken(tl, ',');
 	vcc_expr0(tl, &e2, STRING);
@@ -468,10 +468,10 @@ vcc_Eval_Handle(struct vcc *tl, struct expr **e, const struct symbol *sym,
 
 	if (sym->fmt != STRING && (fmt == STRING || fmt == STRING_LIST)) {
 		*e = vcc_mk_expr(STRING, "\"%s\"", sym->name);
-		vcc_AddRef(tl, tl->t, sym->kind);
+		(void)vcc_AddRef(tl, tl->t, sym->kind);
 	} else {
-		vcc_ExpectCid(tl, "handle");
-		vcc_AddRef(tl, tl->t, sym->kind);
+		vcc_ExpectVid(tl, "handle");
+		(void)vcc_AddRef(tl, tl->t, sym->kind);
 		*e = vcc_mk_expr(sym->fmt, "%s", sym->rname);
 		(*e)->constant = EXPR_VAR;	/* XXX ? */
 	}
@@ -548,14 +548,14 @@ vcc_do_arg(struct vcc *tl, struct func_arg *fa)
 			if (vcc_IdIs(tl->t, p))
 				break;
 			p += strlen(p) + 1;
-		} while (*p != '\0');
-		if (*p == '\0') {
+		} while (*p != '\1');
+		if (*p == '\1') {
 			VSB_printf(tl->sb, "Wrong enum value.");
 			VSB_printf(tl->sb, "  Expected one of:\n");
 			do {
 				VSB_printf(tl->sb, "\t%s\n", r);
 				r += strlen(r) + 1;
-			} while (*r != '\0');
+			} while (*r != '\0' && *r != '\1');
 			vcc_ErrWhere(tl, tl->t);
 			return;
 		}
@@ -608,7 +608,7 @@ vcc_func(struct vcc *tl, struct expr **e, const char *spec,
 	AN(rfmt);
 	VTAILQ_INIT(&head);
 	while (*p != '\0') {
-		fa = calloc(sizeof *fa, 1);
+		fa = calloc(1, sizeof *fa);
 		AN(fa);
 		VTAILQ_INSERT_TAIL(&head, fa, list);
 		if (!memcmp(p, "PRIV_", 5)) {
@@ -755,12 +755,13 @@ static void
 vcc_expr4(struct vcc *tl, struct expr **e, vcc_type_t fmt)
 {
 	struct expr *e1, *e2;
-	const char *ip;
+	const char *ip, *sign;
 	const struct symbol *sym;
 	enum symkind kind;
 	double d;
 	int i;
 
+	sign = "";
 	*e = NULL;
 	if (tl->t->tok == '(') {
 		SkipToken(tl, '(');
@@ -770,7 +771,7 @@ vcc_expr4(struct vcc *tl, struct expr **e, vcc_type_t fmt)
 		*e = vcc_expr_edit(e2->fmt, "(\v1)", e2, NULL);
 		return;
 	}
-	switch(tl->t->tok) {
+	switch (tl->t->tok) {
 	case ID:
 		/*
 		 * XXX: what if var and func/proc had same name ?
@@ -796,7 +797,7 @@ vcc_expr4(struct vcc *tl, struct expr **e, vcc_type_t fmt)
 			vcc_ErrWhere(tl, tl->t);
 			return;
 		}
-		switch(sym->kind) {
+		switch (sym->kind) {
 		case SYM_VAR:
 		case SYM_FUNC:
 		case SYM_ACL:
@@ -840,45 +841,48 @@ vcc_expr4(struct vcc *tl, struct expr **e, vcc_type_t fmt)
 		e1->constant = EXPR_CONST;
 		vcc_NextToken(tl);
 		*e = e1;
-		break;
+		return;
+	case '-':
+		if (fmt != INT && fmt != REAL && fmt != DURATION)
+			break;
+		vcc_NextToken(tl);
+		ExpectErr(tl, CNUM);
+		sign = "-";
+		/* FALLTHROUGH */
 	case CNUM:
 		/*
 		 * XXX: %g may not have enough decimals by default
 		 * XXX: but %a is ugly, isn't it ?
 		 */
 		assert(fmt != VOID);
-		if (fmt == DURATION) {
-			vcc_Duration(tl, &d);
-			ERRCHK(tl);
-			e1 = vcc_mk_expr(DURATION, "%g", d);
-		} else if (fmt == BYTES) {
+		if (fmt == BYTES) {
 			vcc_ByteVal(tl, &d);
 			ERRCHK(tl);
 			e1 = vcc_mk_expr(BYTES, "%.1f", d);
-			ERRCHK(tl);
-		} else if (fmt == REAL) {
-			e1 = vcc_mk_expr(REAL, "%f", vcc_DoubleVal(tl));
-			ERRCHK(tl);
-		} else if (fmt == INT) {
-			e1 = vcc_mk_expr(INT, "%.*s", PF(tl->t));
-			vcc_NextToken(tl);
 		} else {
 			vcc_NumVal(tl, &d, &i);
-			if (i)
-				e1 = vcc_mk_expr(REAL, "%f", d);
+			ERRCHK(tl);
+			if (tl->t->tok == ID) {
+				e1 = vcc_mk_expr(DURATION, "%s%g",
+				    sign, d * vcc_TimeUnit(tl));
+				ERRCHK(tl);
+			} else if (i || fmt == REAL)
+				e1 = vcc_mk_expr(REAL, "%s%f",
+				    sign, d);
 			else
-				e1 = vcc_mk_expr(INT, "%ld", (long)d);
+				e1 = vcc_mk_expr(INT, "%s%ld",
+				    sign, (unsigned long)d);
 		}
 		e1->constant = EXPR_CONST;
 		*e = e1;
-		break;
+		return;
 	default:
-		VSB_printf(tl->sb, "Unknown token ");
-		vcc_ErrToken(tl, tl->t);
-		VSB_printf(tl->sb, " when looking for %s\n\n", fmt->name);
-		vcc_ErrWhere(tl, tl->t);
 		break;
 	}
+	VSB_printf(tl->sb, "Unknown token ");
+	vcc_ErrToken(tl, tl->t);
+	VSB_printf(tl->sb, " when looking for %s\n\n", fmt->name);
+	vcc_ErrWhere(tl, tl->t);
 }
 
 /*--------------------------------------------------------------------
@@ -983,52 +987,78 @@ vcc_expr_string_add(struct vcc *tl, struct expr **e, struct expr *e2)
 	}
 }
 
+static const struct adds {
+	unsigned	op;
+	vcc_type_t	a;
+	vcc_type_t	b;
+	vcc_type_t	fmt;
+} vcc_adds[] = {
+	{ '+', BYTES,		BYTES,		BYTES },
+	{ '-', BYTES,		BYTES,		BYTES },
+	{ '+', DURATION,	DURATION,	DURATION },
+	{ '-', DURATION,	DURATION,	DURATION },
+	{ '+', INT,		INT,		INT },
+	{ '-', INT,		INT,		INT },
+	{ '+', INT,		REAL,		REAL },
+	{ '-', INT,		REAL,		REAL },
+	{ '+', REAL,		INT,		REAL },
+	{ '-', REAL,		INT,		REAL },
+	{ '+', REAL,		REAL,		REAL },
+	{ '-', REAL,		REAL,		REAL },
+	{ '-', TIME,		TIME,		DURATION },
+	{ '+', TIME,		DURATION,	TIME },
+	{ '-', TIME,		DURATION,	TIME },
+
+	{ EOI, VOID, VOID, VOID }
+};
+
 static void
 vcc_expr_add(struct vcc *tl, struct expr **e, vcc_type_t fmt)
 {
+	const struct adds *ap;
 	struct expr  *e2;
 	vcc_type_t f2;
 	struct token *tk;
+	char buf[128];
 
 	*e = NULL;
 	vcc_expr_mul(tl, e, fmt);
 	ERRCHK(tl);
+
+	if (tl->t->tok != '+' && tl->t->tok != '-')
+		return;
+
 	f2 = (*e)->fmt;
+	for (ap = vcc_adds; ap->op != EOI; ap++)
+		if (ap->a == f2 && ap->op == tl->t->tok)
+			break;
+
+	if (ap->op == EOI &&
+	    (fmt == STRING || fmt == STRING_LIST) &&
+	    f2 != STRING && f2 != STRING_LIST) {
+		vcc_expr_tostring(tl, e, fmt);
+		f2 = (*e)->fmt;
+	}
 
 	while (tl->t->tok == '+' || tl->t->tok == '-') {
 		tk = tl->t;
 		vcc_NextToken(tl);
-		if (f2 == TIME)
-			vcc_expr_mul(tl, &e2, DURATION);
-		else
-			vcc_expr_mul(tl, &e2, f2);
+		vcc_expr_mul(tl, &e2, f2);
 		ERRCHK(tl);
 
-#define ADD_OK(op, a, b, c)						  \
-		if (tk->tok == op[0] && (*e)->fmt == a && e2->fmt == b) { \
-			*e = vcc_expr_edit(c, "(\v1" op "\v2)", *e, e2);  \
-			continue;					  \
-		}							  \
+		for (ap = vcc_adds; ap->op != EOI; ap++) {
+			if (tk->tok == ap->op && (*e)->fmt == ap->a &&
+			    e2->fmt == ap->b)
+				break;
+		}
 
-		ADD_OK("-", TIME,	TIME,		DURATION);
-		ADD_OK("+", TIME,	DURATION,	TIME);
-		ADD_OK("-", TIME,	DURATION,	TIME);
-		ADD_OK("+", DURATION,	DURATION,	DURATION);
-		ADD_OK("-", DURATION,	DURATION,	DURATION);
-		ADD_OK("+", BYTES,	BYTES,		BYTES);
-		ADD_OK("-", BYTES,	BYTES,		BYTES);
-		ADD_OK("+", INT,	INT,		INT);
-		ADD_OK("-", INT,	INT,		INT);
-		ADD_OK("+", REAL,	REAL,		REAL);
-		ADD_OK("-", REAL,	REAL,		REAL);
-		ADD_OK("+", REAL,	INT,		REAL);
-		ADD_OK("-", REAL,	INT,		REAL);
-		ADD_OK("+", INT,	REAL,		REAL);
-		ADD_OK("-", INT,	REAL,		REAL);
+		if (ap->fmt != VOID) {
+			bprintf(buf, "(\v1 %c \v2)", ap->op);
+			*e = vcc_expr_edit(ap->fmt, buf, *e, e2);
+			continue;
+		}
 
-#undef ADD_OK
-
-		if (tk->tok == '+') {
+		if (tk->tok == '+' && ap->op == EOI) {
 			if ((*e)->fmt == STRING && e2->fmt == STRING) {
 				vcc_expr_string_add(tl, e, e2);
 				return;
@@ -1097,8 +1127,13 @@ static const struct cmps {
 	NUM_REL(REAL),
 	NUM_REL(TIME),
 
+	{IP,		T_EQ,	"!VRT_ipcmp(\v1, \v2)" },
+	{IP,		T_NEQ,	"VRT_ipcmp(\v1, \v2)" },
+
 	{STRING,	T_EQ,	"!VRT_strcmp(\v1, \v2)" },
 	{STRING,	T_NEQ,	"VRT_strcmp(\v1, \v2)" },
+	{STRING_LIST,	T_EQ,	"!VRT_strcmp(\v1, \v2)" },
+	{STRING_LIST,	T_NEQ,	"VRT_strcmp(\v1, \v2)" },
 
 	{VOID, 0, NULL}
 };
@@ -1114,6 +1149,7 @@ vcc_expr_cmp(struct vcc *tl, struct expr **e, vcc_type_t fmt)
 	const char *re;
 	const char *not;
 	struct token *tk;
+	struct symbol *sym;
 	enum symkind kind;
 
 	*e = NULL;
@@ -1130,6 +1166,11 @@ vcc_expr_cmp(struct vcc *tl, struct expr **e, vcc_type_t fmt)
 			break;
 	if (cp->fmt != VOID) {
 		vcc_NextToken(tl);
+		if ((*e)->fmt == STRING_LIST) {
+			// XXX: This is not optimal, but we can't pass two
+			// STRING_LIST's to a function anyway...
+			vcc_expr_tostring(tl, e, STRING);
+		}
 		vcc_expr_strfold(tl, &e2, (*e)->fmt);
 		ERRCHK(tl);
 		if (e2->fmt != (*e)->fmt) { /* XXX */
@@ -1161,16 +1202,16 @@ vcc_expr_cmp(struct vcc *tl, struct expr **e, vcc_type_t fmt)
 	    (tl->t->tok == '~' || tl->t->tok == T_NOMATCH)) {
 		not = tl->t->tok == '~' ? "" : "!";
 		vcc_NextToken(tl);
-		vcc_ExpectCid(tl, "ACL");
-		vcc_AddRef(tl, tl->t, SYM_ACL);
-		bprintf(buf, "%smatch_acl_named_%.*s(ctx, \v1)",
-		    not, PF(tl->t));
+		vcc_ExpectVid(tl, "ACL");
+		sym = vcc_AddRef(tl, tl->t, SYM_ACL);
+		VCC_GlobalSymbol(sym, ACL, ACL_SYMBOL_PREFIX);
+		bprintf(buf, "%sVRT_acl_match(ctx, %s, \v1)", not, sym->rname);
 		vcc_NextToken(tl);
 		*e = vcc_expr_edit(BOOL, buf, *e, NULL);
 		return;
 	}
 	if ((*e)->fmt == IP && (tl->t->tok == T_EQ || tl->t->tok == T_NEQ)) {
-		vcc_Acl_Hack(tl, buf);
+		vcc_Acl_Hack(tl, buf, sizeof buf);
 		*e = vcc_expr_edit(BOOL, buf, *e, NULL);
 		return;
 	}
@@ -1181,6 +1222,14 @@ vcc_expr_cmp(struct vcc *tl, struct expr **e, vcc_type_t fmt)
 		e2 = NULL;
 		vcc_expr0(tl, &e2, (*e)->fmt);
 		ERRCHK(tl);
+		if (e2->fmt != (*e)->fmt) {
+			VSB_printf(tl->sb, "Comparison of different types: ");
+			VSB_printf(tl->sb, "%s ", (*e)->fmt->name);
+			vcc_ErrToken(tl, tk);
+			VSB_printf(tl->sb, " %s\n", e2->fmt->name);
+			vcc_ErrWhere(tl, tk);
+			return;
+		}
 		*e = vcc_expr_edit(BOOL, buf, *e, e2);
 		return;
 	}
@@ -1200,12 +1249,12 @@ vcc_expr_cmp(struct vcc *tl, struct expr **e, vcc_type_t fmt)
 	default:
 		break;
 	}
-	if (fmt == BOOL &&
-	    ((*e)->fmt == STRING || (*e)->fmt == BACKEND ||
-	     (*e)->fmt == INT)) {
-		*e = vcc_expr_edit(BOOL, "(\v1 != 0)", *e, NULL);
+	if (fmt != BOOL)
 		return;
-	}
+	if ((*e)->fmt == STRING || (*e)->fmt == BACKEND || (*e)->fmt == INT)
+		*e = vcc_expr_edit(BOOL, "(\v1 != 0)", *e, NULL);
+	else if ((*e)->fmt == DURATION)
+		*e = vcc_expr_edit(BOOL, "(\v1 > 0)", *e, NULL);
 }
 
 /*--------------------------------------------------------------------
@@ -1344,10 +1393,8 @@ vcc_Expr(struct vcc *tl, vcc_type_t fmt)
 		}
 		vcc_expr_fmt(tl->fb, tl->indent, e);
 		VSB_putc(tl->fb, '\n');
-	} else {
-		if (t1 != tl->t)
-			vcc_ErrWhere2(tl, t1, tl->t);
-	}
+	} else if (t1 != tl->t)
+		vcc_ErrWhere2(tl, t1, tl->t);
 	vcc_delete_expr(e);
 }
 
