@@ -29,7 +29,6 @@
 
 #include "config.h"
 
-#include <stdio.h>
 #include <string.h>
 
 #include "vcc_compile.h"
@@ -196,6 +195,7 @@ vcc_Compound(struct vcc *tl)
 			vcc_ErrWhere(tl, tl->t);
 			return;
 		}
+		Fb(tl, 1, "if (*ctx->handling) return;\n");
 	}
 }
 
@@ -208,10 +208,11 @@ vcc_Compound(struct vcc *tl)
 static void
 vcc_ParseFunction(struct vcc *tl)
 {
+	struct symbol *sym;
 	int m, i;
 
 	vcc_NextToken(tl);
-	vcc_ExpectCid(tl, "function");
+	vcc_ExpectVid(tl, "function");
 	ERRCHK(tl);
 
 	m = IsMethod(tl->t);
@@ -228,7 +229,7 @@ vcc_ParseFunction(struct vcc *tl)
 		tl->fb = tl->fm[m];
 		if (tl->mprocs[m] == NULL) {
 			(void)vcc_AddDef(tl, tl->t, SYM_SUB);
-			vcc_AddRef(tl, tl->t, SYM_SUB);
+			(void)vcc_AddRef(tl, tl->t, SYM_SUB);
 			tl->mprocs[m] = vcc_AddProc(tl, tl->t);
 		}
 		tl->curproc = tl->mprocs[m];
@@ -237,31 +238,23 @@ vcc_ParseFunction(struct vcc *tl)
 		Fb(tl, 0, " */\n");
 	} else {
 		tl->fb = tl->fc;
-		i = vcc_AddDef(tl, tl->t, SYM_SUB);
-		if (i > 1) {
+		sym = vcc_AddDef(tl, tl->t, SYM_SUB);
+		VCC_GlobalSymbol(sym, SUB, "VGC_function");
+		if (sym->ndef > 1) {
 			VSB_printf(tl->sb,
-			    "Function '%.*s' redefined\n", PF(tl->t));
+			    "Function '%s' redefined\n", sym->name);
 			vcc_ErrWhere(tl, tl->t);
 			return;
 		}
 		tl->curproc = vcc_AddProc(tl, tl->t);
-		Fh(tl, 0, "int VGC_function_%.*s "
-		    "(VRT_CTX);\n", PF(tl->t));
-		Fc(tl, 1, "\nint __match_proto__(vcl_func_t)\n");
-		Fc(tl, 1, "VGC_function_%.*s(VRT_CTX)\n",
-		    PF(tl->t));
+		Fh(tl, 0, "void %s(VRT_CTX);\n", sym->rname);
+		Fc(tl, 1, "\nvoid __match_proto__(vcl_func_t)\n");
+		Fc(tl, 1, "%s(VRT_CTX)\n", sym->rname);
 	}
 	vcc_NextToken(tl);
 	tl->indent += INDENT;
 	Fb(tl, 1, "{\n");
 	L(tl, vcc_Compound(tl));
-	if (m == -1) {
-		/*
-		 * non-method subroutines must have an explicit non-action
-		 * return in case they just fall through the bottom.
-		 */
-		Fb(tl, 1, "  return(0);\n");
-	}
 	Fb(tl, 1, "}\n");
 	tl->indent -= INDENT;
 	tl->fb = NULL;
@@ -331,6 +324,7 @@ vcc_Parse(struct vcc *tl)
 	struct toplev *tp;
 	struct token *tok;
 
+	AZ(tl->indent);
 	if (tl->t->tok != ID || !vcc_IdIs(tl->t, "vcl")) {
 		VSB_printf(tl->sb,
 		    "VCL version declaration missing\n"
@@ -395,4 +389,5 @@ vcc_Parse(struct vcc *tl)
 			return;
 		}
 	}
+	AZ(tl->indent);
 }
