@@ -30,12 +30,13 @@
 
 #include <ctype.h>
 #include <stdarg.h>
-#include <stdio.h>
 #include <stdlib.h>
 #include <string.h>
 
 #include "vcc_compile.h"
 
+#include "vct.h"
+
 /*--------------------------------------------------------------------*/
 
 enum symkind
@@ -45,6 +46,7 @@ VCC_HandleKind(vcc_type_t fmt)
 	if (fmt == BACKEND)	return(SYM_BACKEND);
 	if (fmt == PROBE)	return(SYM_PROBE);
 	if (fmt == STEVEDORE)	return(SYM_STEVEDORE);
+	if (fmt == SUB)		return(SYM_SUB);
 	if (fmt == INSTANCE)	return(SYM_INSTANCE);
 	return(SYM_NONE);
 }
@@ -52,10 +54,9 @@ VCC_HandleKind(vcc_type_t fmt)
 const char *
 VCC_SymKind(struct vcc *tl, const struct symbol *s)
 {
-	switch(s->kind) {
+	switch (s->kind) {
 #define VCC_SYMB(uu, ll)	case SYM_##uu: return(#ll);
 #include "tbl/symbol_kind.h"
-#undef VCC_SYMB
 	default:
 		ErrInternal(tl);
 		VSB_printf(tl->sb, "Symbol Kind 0x%x\n", s->kind);
@@ -63,6 +64,24 @@ VCC_SymKind(struct vcc *tl, const struct symbol *s)
 	}
 }
 
+void
+VCC_PrintCName(struct vsb *vsb, const char *b, const char *e)
+{
+
+	AN(vsb);
+	AN(b);
+
+	if (e == NULL)
+		e = strchr(b, '\0');
+	assert(b < e);
+
+	for (; b < e; b++)
+		if (vct_isalnum(*b))
+			VSB_putc(vsb, *b);
+		else
+			VSB_printf(vsb, "_%02x_", *b);
+}
+
 static struct symbol *
 vcc_new_symbol(struct vcc *tl, const char *b, const char *e)
 {
@@ -128,7 +147,7 @@ VCC_Symbol(struct vcc *tl, struct symbol *parent,
 		if (q < e)
 			break;
 		if (kind != SYM_NONE && sym->kind != kind)
-			continue;
+			break;
 		if (kind == SYM_NONE && sym->kind == kind)
 			continue;
 		break;
@@ -144,6 +163,7 @@ VCC_Symbol(struct vcc *tl, struct symbol *parent,
 		return (sym);
 	if (sym == NULL) {
 		sym = vcc_new_symbol(tl, b, q);
+		sym->parent = parent;
 		if (sym2 != NULL)
 			VTAILQ_INSERT_BEFORE(sym2, sym, list);
 		else
@@ -178,22 +198,23 @@ VCC_WalkSymbols(struct vcc *tl, symwalk_f *func, enum symkind kind)
 	vcc_walksymbols(tl, tl->symbols, func, kind);
 }
 
-static void
-vcc_global(struct vcc *tl, struct symbol *sym,
-    vcc_type_t fmt, const char *str, va_list ap)
+void
+VCC_GlobalSymbol(struct symbol *sym, vcc_type_t fmt, const char *pfx)
 {
 	struct vsb *vsb;
 
+	CHECK_OBJ_NOTNULL(sym, SYMBOL_MAGIC);
+	AN(pfx);
+
 	vsb = VSB_new_auto();
 	AN(vsb);
-	VSB_vprintf(vsb, str, ap);
+	VSB_printf(vsb, "%s_", pfx);
+	VCC_PrintCName(vsb, sym->name, NULL);
 	AZ(VSB_finish(vsb));
-	if (tl != NULL)
-		sym->rname = TlDup(tl, VSB_data(vsb));
-	else
-		sym->rname = strdup(VSB_data(vsb));
+	REPLACE(sym->rname, VSB_data(vsb));
 	AN(sym->rname);
 	VSB_destroy(&vsb);
+
 	sym->fmt = fmt;
 	sym->kind = VCC_HandleKind(sym->fmt);
 	if (sym->kind != SYM_NONE)
@@ -203,17 +224,15 @@ vcc_global(struct vcc *tl, struct symbol *sym,
 
 #define VCL_MET_MAC(l,u,t,b)   sym->r_methods |= VCL_MET_##u;
 #include "tbl/vcl_returns.h"
-#undef VCL_MET_MAC
 }
 
 struct symbol *
 VCC_HandleSymbol(struct vcc *tl, const struct token *tk, vcc_type_t fmt,
-    const char *str, ...)
+    const char *pfx)
 {
 	struct symbol *sym;
 	enum symkind kind;
 	const char *p;
-	va_list ap;
 
 	kind = VCC_HandleKind(fmt);
 	assert(kind != SYM_NONE);
@@ -229,24 +248,24 @@ VCC_HandleSymbol(struct vcc *tl, const struct token *tk, vcc_type_t fmt,
 		vcc_ErrWhere(tl, sym->def_b);
 		return (sym);
 	} else if (sym != NULL && sym->def_b != NULL) {
-		VSB_printf(tl->sb, "Name '%.*s' already used.\n", PF(tk));
+		VSB_printf(tl->sb, "Name '%.*s' already defined.\n", PF(tk));
 		vcc_ErrWhere(tl, tk);
 		VSB_printf(tl->sb, "First definition:\n");
 		AN(sym->def_b);
 		vcc_ErrWhere(tl, sym->def_b);
 		return (sym);
-	} else if (sym != NULL) {
+	} else if (sym != NULL && sym->kind != kind) {
 		VSB_printf(tl->sb,
-		    "Name %.*s is a reserved name.\n", PF(tk));
+		    "Name %.*s must have type '%s'.\n",
+		    PF(tk), VCC_SymKind(tl, sym));
 		vcc_ErrWhere(tl, tk);
 		return (sym);
 	}
-	sym = VCC_SymbolTok(tl, NULL, tk, kind, 1);
+	if (sym == NULL)
+		sym = VCC_SymbolTok(tl, NULL, tk, kind, 1);
 	AN(sym);
 	AZ(sym->ndef);
-	va_start(ap, str);
-	vcc_global(tl, sym, fmt, str, ap);
-	va_end(ap);
+	VCC_GlobalSymbol(sym, fmt, pfx);
 	sym->ndef = 1;
 	if (sym->def_b == NULL)
 		sym->def_b = tk;
