@@ -30,10 +30,6 @@
 
 #include "config.h"
 
-#include <math.h>
-#include <stdarg.h>
-#include <stdio.h>
-#include <stdlib.h>
 #include <string.h>
 
 #include "vcc_compile.h"
@@ -144,6 +140,11 @@ const struct type STRING_LIST[1] = {{
 	.tostring =		"VRT_CollectString(ctx,\n\v1,\nvrt_magic_string_end)",
 }};
 
+const struct type SUB[1] = {{
+	.magic =		TYPE_MAGIC,
+	.name =			"SUB",
+}};
+
 const struct type TIME[1] = {{
 	.magic =		TYPE_MAGIC,
 	.name =			"TIME",
@@ -166,7 +167,6 @@ VCC_Type(const char *p)
 
 #define VCC_TYPE(foo)	if (!strcmp(p, #foo)) return (foo);
 #include "tbl/vcc_types.h"
-#undef VCC_TYPE
 	return (NULL);
 }
 
