@@ -29,16 +29,13 @@
 
 #include "config.h"
 
-#include <stdio.h>
 #include <string.h>
 #include <stdlib.h>
-#include <sys/types.h>
 #include <sys/socket.h>
 
 #include "vcc_compile.h"
 
 #include "vre.h"
-#include "vrt.h"
 #include "vsa.h"
 #include "vss.h"
 #include "vtcp.h"
@@ -57,7 +54,6 @@ vcc_regexp(struct vcc *tl)
 	Expect(tl, CSTR);
 	if (tl->err)
 		return (NULL);
-	memset(&t, 0, sizeof t);
 	t = VRE_compile(tl->t->dec, 0, &error, &erroroffset);
 	if (t == NULL) {
 		VSB_printf(tl->sb,
@@ -66,7 +62,7 @@ vcc_regexp(struct vcc *tl)
 		return (NULL);
 	}
 	VRE_free(&t);
-	sprintf(buf, "VGC_re_%u", tl->unique++);
+	bprintf(buf, "VGC_re_%u", tl->unique++);
 	p = TlAlloc(tl, strlen(buf) + 1);
 	strcpy(p, buf);
 
@@ -124,7 +120,7 @@ vcc_suckaddr(struct vcc *tl, const char *host, const struct suckaddr *vsa,
 
 	q = TlAlloc(tl, 40);
 	AN(q);
-	sprintf(q, "(const void*)suckaddr_%u", tl->unique);
+	assert(snprintf(q, 40, "(const void*)suckaddr_%u", tl->unique) < 40);
 	*ip = q;
 	tl->unique++;
 }
