@@ -39,8 +39,6 @@
 
 #include "config.h"
 
-#include <stdio.h>
-
 #include "vcc_compile.h"
 
 /*--------------------------------------------------------------------*/
@@ -73,7 +71,7 @@ struct proc {
  * Keep track of definitions and references
  */
 
-void
+struct symbol *
 vcc_AddRef(struct vcc *tl, const struct token *t, enum symkind kind)
 {
 	struct symbol *sym;
@@ -83,9 +81,10 @@ vcc_AddRef(struct vcc *tl, const struct token *t, enum symkind kind)
 		sym->ref_b = t;
 	AN(sym);
 	sym->nref++;
+	return (sym);
 }
 
-int
+struct symbol *
 vcc_AddDef(struct vcc *tl, const struct token *t, enum symkind kind)
 {
 	struct symbol *sym;
@@ -95,7 +94,7 @@ vcc_AddDef(struct vcc *tl, const struct token *t, enum symkind kind)
 		sym->def_b = t;
 	AN(sym);
 	sym->ndef++;
-	return (sym->ndef);
+	return (sym);
 }
 
 /*--------------------------------------------------------------------*/
@@ -231,7 +230,6 @@ vcc_CheckActionRecurse(struct vcc *tl, struct proc *p, unsigned bitmap)
 			vcc_ErrWhere(tl, p->return_tok[VCL_RET_##U]);	\
 		}
 #include "tbl/vcl_returns.h"
-#undef VCL_RET_MAC
 
 		VSB_printf(tl->sb, "\n...in subroutine \"%.*s\"\n",
 		    PF(p->name));
@@ -276,7 +274,6 @@ vcc_checkaction1(struct vcc *tl, const struct symbol *sym)
 			VSB_printf(tl->sb, " \"%s\"", #l);
 
 #include "tbl/vcl_returns.h"
-#undef VCL_RET_MAC
 		VSB_printf(tl->sb, "\n");
 		tl->err = 1;
 	}
@@ -392,3 +389,37 @@ vcc_CheckUses(struct vcc *tl)
 	VCC_WalkSymbols(tl, vcc_checkuses, SYM_SUB);
 	return (tl->err);
 }
+
+/*---------------------------------------------------------------------*/
+
+static void
+vcc_pnam(struct vcc *tl, const struct symbol *sym)
+{
+
+	if (sym->parent != tl->symbols) {
+		vcc_pnam(tl, sym->parent);
+		Fc(tl, 0, ".");
+	}
+	Fc(tl, 0, "%s", sym->name);
+}
+
+static void __match_proto__(symwalk_f)
+vcc_xreftable(struct vcc *tl, const struct symbol *sym)
+{
+
+	Fc(tl, 0, " * %-7s ", VCC_SymKind(tl, sym));
+	Fc(tl, 0, " %-9s ", sym->fmt != NULL ? sym->fmt->name : "");
+	vcc_pnam(tl, sym);
+	if (sym->wildcard != NULL)
+		Fc(tl, 0, "*");
+	Fc(tl, 0, "\n");
+}
+
+void
+VCC_XrefTable(struct vcc *tl)
+{
+
+	Fc(tl, 0, "\n/*\n * Symbol Table\n *\n");
+	VCC_WalkSymbols(tl, vcc_xreftable, SYM_NONE);
+	Fc(tl, 0, "*/\n\n");
+}
