@@ -1,5 +1,5 @@
 /*-
- * Copyright (c) 2010-2015 Varnish Software AS
+ * Copyright (c) 2010-2017 Varnish Software AS
  * All rights reserved.
  *
  * Author: Poul-Henning Kamp <phk@FreeBSD.org>
@@ -28,22 +28,23 @@
 
 #include "config.h"
 
-#include <sys/types.h>
-#include <sys/socket.h>
+#include <sys/stat.h>
+
 #include <netinet/in.h>
 
 #include <ctype.h>
-#include <stdarg.h>
 #include <stdlib.h>
 #include <syslog.h>
 
+#include "cache/cache.h"
+
 #include "vrnd.h"
 #include "vrt.h"
 #include "vtcp.h"
 #include "vsa.h"
 #include "vtim.h"
+#include "vcl.h"
 
-#include "cache/cache.h"
 #include "cache/cache_director.h"
 
 #include "vcc_if.h"
@@ -140,15 +141,11 @@ vmod_log(VRT_CTX, const char *s, ...)
 	va_list ap;
 
 	CHECK_OBJ_NOTNULL(ctx, VRT_CTX_MAGIC);
-	WS_Assert(ctx->ws);
-
-
 	u = WS_Reserve(ctx->ws, 0);
 	t.b = ctx->ws->f;
 	va_start(ap, s);
 	t.e = VRT_StringList(ctx->ws->f, u, s, ap);
 	va_end(ap);
-
 	if (t.e != NULL) {
 		assert(t.e > t.b);
 		t.e--;
@@ -168,27 +165,33 @@ vmod_syslog(VRT_CTX, VCL_INT fac, const char *fmt, ...)
 	txt t;
 
 	CHECK_OBJ_NOTNULL(ctx, VRT_CTX_MAGIC);
+	u = WS_Reserve(ctx->ws, 0);
+	t.b = ctx->ws->f;
 	va_start(ap, fmt);
-	if (ctx->ws != NULL) {
-		u = WS_Reserve(ctx->ws, 0);
-		t.b = ctx->ws->f;
-		t.e = VRT_StringList(ctx->ws->f, u, fmt, ap);
-		if (t.e != NULL)
-			syslog((int)fac, "%s", t.b);
-		WS_Release(ctx->ws, 0);
-	} else
-		vsyslog((int)fac, fmt, ap);
+	t.e = VRT_StringList(ctx->ws->f, u, fmt, ap);
 	va_end(ap);
+	if (t.e != NULL)
+		syslog((int)fac, "%s", t.b);
+	WS_Release(ctx->ws, 0);
+}
+
+VCL_BOOL __match_proto__(td_std_file_exists)
+vmod_file_exists(VRT_CTX, VCL_STRING file_name)
+{
+	struct stat st;
+
+	CHECK_OBJ_NOTNULL(ctx, VRT_CTX_MAGIC);
+	return (stat(file_name, &st) == 0);
 }
 
 VCL_VOID __match_proto__(td_std_collect)
-vmod_collect(VRT_CTX, VCL_HEADER hdr)
+vmod_collect(VRT_CTX, VCL_HEADER hdr, VCL_STRING sep)
 {
 	struct http *hp;
 
 	CHECK_OBJ_NOTNULL(ctx, VRT_CTX_MAGIC);
 	hp = VRT_selecthttp(ctx, hdr->where);
-	http_CollectHdr(hp, hdr->what);
+	http_CollectHdrSep(hp, hdr->what, sep);
 }
 
 VCL_BOOL __match_proto__(td_std_healthy)
@@ -255,3 +258,27 @@ vmod_strstr(VRT_CTX, VCL_STRING s1, VCL_STRING s2)
 		return (NULL);
 	return (strstr(s1, s2));
 }
+
+VCL_STRING __match_proto__(td_std_getenv)
+vmod_getenv(VRT_CTX, VCL_STRING name)
+{
+	CHECK_OBJ_NOTNULL(ctx, VRT_CTX_MAGIC);
+	if (name == NULL || *name == '\0')
+		return (NULL);
+	return (getenv(name));
+}
+
+VCL_VOID __match_proto__(td_std_late_100_continue)
+vmod_late_100_continue(VRT_CTX, VCL_BOOL late)
+{
+	CHECK_OBJ_NOTNULL(ctx, VRT_CTX_MAGIC);
+	if (ctx->method != VCL_MET_RECV) {
+		VSLb(ctx->vsl, SLT_VCL_Error,
+		    "std.late_100_continue() only valid in vcl_recv{}");
+		return;
+	}
+
+	CHECK_OBJ_NOTNULL(ctx->req, REQ_MAGIC);
+	if (ctx->req->want100cont)
+		ctx->req->late100cont = late;
+}
